<?php
/**
 * Application level Controller
 *
 * PHP versions 5
 *
 * @copyright     Copyright (c) 2016, SOUM Corporation.
 * @copyright     Copyright (c) 2011 - 2012, National Institute of Informatics,
 * @license       BSD License (http://creativecommons.org/licenses/BSD/)
 */
require(CONFIGS.'mail.php');

App::import('Core', 'l10n', 'Validation');

class AppController extends Controller {

	var $helpers = array('Session', 'Html', 'Form', 'Javascript');
	var $components = array("Common", "Session", 'EmailJp', 'AutoGroupCreator',
			'TakerAgreement', 'MailUtils', 'AttributeOperator', 'FavoriteUpdator',
			'Announce');
	var $uses = array('Mygroup', 'Account', 'SpAdministrator', 'Group',
			'AccountEppn', 'Invite', 'SpcSort', 'IdpAdministrator');

	public function csrfError() {
		$this->redirect('/'.$this->params['url']['url']);
	}

	protected function ignoreCsrf()
	{
		if (isset($this->ignoreCsrfActions) && in_array($this->action, $this->ignoreCsrfActions)) {
			$this->Csrf->ignore = true;
		}
	}

	/**
	 * before
	 * @param $id
	 */
	function beforeFilter() {
//debug($this->getIdentityProvider());
		$this->accessLog();

		// 画像の場合は事前処理を行わない
		if ($this->params["controller"] === "images") {
			return;
		}

		if ($this->params["controller"] === "connector"
		    && $this->params["action"] === "services") {
			return;
		}

		// 言語設定
		$this->setLanguage(null);
		// Csrfを無効にするかを確認
		$this->ignoreCsrf();
		// 認証処理
		$this->auth();

		// IdPで認証したアカウントが同一かを確認
		$loginKey = $this->Session->read('LoginKey');
		if ($loginKey && $loginKey != $this->getLoginKey()) {
			// CGポータルは除外
			if ($loginKey != 'guestguest') {
				$this->Session->destroy();
			}
		}

		// 未ログイン時のみ自動グループ処理を行う
		//  アカウント紐付け処理時は除外する
		//  招待処理時も除外する
		if (!$this->isLogin()
			&& !($this->params["controller"] === "accounts" && $this->params["action"] === 'integration_step1')
			&& !($this->params["controller"] === "accounts" && $this->params["action"] === 'integration_step2')
			&& !($this->params["controller"] === "invite")
			) {
			// 自動グループ作成
			$this->AutoGroupCreator->service(
				$this->getAccountID(),
				$this->getOrganizationNameEn(), $this->getOrganizationNameJa(),
				$this->getOrganizationalUnitNameEn(), $this->getOrganizationalUnitNameJa(),
				$this->getEntityID());

			// ログイン済みに
			$this->setLoggedIn();

			// 自動グループに入会するか
			$idResult = $this->AutoGroupCreator->servNeedAgree(
				$this->getAccountID(),
				$this->getOrganizationNameEn(), $this->getOrganizationNameJa(),
				$this->getOrganizationalUnitNameEn(), $this->getOrganizationalUnitNameJa(),
				$this->getEntityID(),
				$this->getEPPN(),
				$this->getEPTID());
			if ($idResult) {
				// グループ自動入会のメッセージを表示する
				$this->Session->write('AutoGroupMessage', $idResult);
			}
		}

		// アカウント有無の確認
		if ($this->params["controller"] != "accounts"
			&& $this->params["controller"] != "pages"
			&& $this->params["controller"] != "menus"
			&& $this->params["controller"] != "languages"
			&& $this->params["controller"] != "logout"
			&& $this->params["controller"] != "top"
			&& $this->params["controller"] != "redirect"
			&& $this->params["controller"] != "images") {
			// グループ入会URLなら、アカウント作成後に入会処理をさせる
			// グループ入会URL以外なら、元のページに戻る
			if ($this->params["controller"] === "invite" &&
				($this->params["action"] === "register" || $this->params["action"] === "rgst") ) {
				$this->Session->write('joinUrl', $this->params['url']['url']);
				$inviteGroukey = $this->params['pass'][0];
				$inviteGroup = $this->Group->findByGroupKey($inviteGroukey);
				$this->Session->write('inviteGroupName', $inviteGroup['Group']['name']);
			} elseif (!($this->params["controller"] === "mygroups" && $this->params["action"] === "member")) {
				$this->Session->write('nextPage', '/' . $this->params['url']['url']);
			}

			// アカウントがなければ、アカウント作成画面を表示し、
			// アカウント名とメールアドレスの両方が設定されていなければ、アカウント編集画面を表示
			$accountId = $this->getAccountID();
			if (empty($accountId)
				|| (!$this->Account->hasAccountName($accountId) && !$this->Account->hasMailaddress($accountId))) {
				$this->redirect('/accounts/create');
				return;
			} elseif (!($this->Account->hasAccountName($accountId) && $this->Account->hasMailaddress($accountId))) {
				$this->redirect('/accounts/edit');
				return;
			}
		}

		// 言語・名前設定
		$accountLang = $this->Session->read('accountLanguage');
		if (empty($accountLang)) {
			$account = $this->Account->find('first', array('conditions' => array('id' => $this->getAccountID())));
			$this->setLanguage($account['Account']['language']);
			$this->setAccountName($accountLang, $account);
		}

//		// オプション設定
//		$this->setOption();
//		// IdP管理者設定
//		$this->setIdpAdmin();

		// 同意状況確認
		$this->_checkAgreement();

		// 招待中グループの確認
		$displayedInviting = $this->Session->read('DisplayedInviting');
		if (empty($invitingDispShow)) {
			$invitingList = $this->Invite->findInviting($this->getAccountID());
			$this->set('invitingList', $invitingList);
		}

		// お知らせ設定
		$this->setAnnounce();
	}

	/**
	 * お知らせ設定
	 */
	private function setAnnounce() {
		$info = $this->Announce->getInfo();
		if (empty($info)) {
			return;
		}
		$hashNow = md5($info);
		if (!empty($_COOKIE["accnounce-hash1"])) {
			$hash = explode("-", $_COOKIE["accnounce-hash1"]);
			if ($hashNow == $hash[1]) {
				return;
			}
		}
		if (!empty($_COOKIE["accnounce-hash2"])) {
			$hash = explode("-", $_COOKIE["accnounce-hash2"]);
			if ($hashNow == $hash[1]) {
				return;
			}
		}
		// クッキーのハッシュ値と違ったら表示
		$this->set("accnounce", $info);
		$this->set("accnounceHash", $hashNow);
	}

	protected function _getGroupIdFromURL() {
		if (empty($this->params['pass'])) {
			return null;
		}

		$pass = $this->params['pass'];
		$groupid = $pass[count($pass) - 1];

		if (!empty($groupid) && is_numeric($groupid)) {
			return $groupid;
		}

		return null;
	}

	protected function _checkAccessible($groupid) {
		$targetGroup = $this->Group->find('first', array(
				'fields' => array('id', 'active', 'sp', 'public', 'openmember'),
				'conditions' => array('id' => $groupid)));
		// 存在しないグループおよびSPコネクタは表示しない
		if (empty($targetGroup)) {
			$this->cakeError('notFound', array());
			return;
		}

		// システム管理者は制限なしにアクセス可能
		if ($this->isSuperManager()) {
			return;
		}

		// 削除されたグループおよびSPコネクタは表示しない
		if ($targetGroup['Group']['active'] == 9) {
			$this->cakeError('notFound', array());
			return;
		}
		// 非公開グループはメンバーのみアクセス可能
		if ($targetGroup['Group']['public'] != 1) {

			// /rd/xxx からのアクセス時は検索対象でないグループも閲覧可能
			if ($targetGroup['Group']['public'] == 2) {
				$rdGroupid = $this->Session->read('rd_groupid');
				if (!empty($rdGroupid) && $rdGroupid == $groupid) {
					return;
				}
			}

			$count = $this->Mygroup->find('count', array('conditions' =>array(
					'account_id' => $this->getAccountID(),
					'groupid' => $groupid)));
			if ($count == 0) {
				$this->cakeError('cannotAccess', array('url' => $this->here));
				return;
			}
		}
	}

	protected function _checkAgreement() {
		// 同意
		if (!$this->isAgreed()) {
			$this->setAgreement('true');
			// 同意確認
			if ($this->TakerAgreement->isShowAgree($this->getAccountID())) {
				$this->Session->write('AgreementMessage', true);
			}
		}
	}

	protected function issetRemoteUser()
	{
		if ($this->Common->isDevelopment()) {
			return $this->getEPPN() != '';
		} else {
//			return isset($_SERVER['REMOTE_USER']);
			return isset($_SERVER['Shib-Session-ID']);
		}
	}

//	/**
//	 * オプション情報を設定します
//	 */
//	private function setOption() {
//		$spcSortType = $this->SpcSort->getSortType($this->getAccountID());
//		$this->set('spcSortType', $spcSortType);
//	}

//	/**
//	 * IdP管理者情報を設定します。
//	 */
//	private function setIdpAdmin() {
//		$idpAdmin = $this->Session->read('IdpAdmin');
//		if (empty($idpAdmin)) {
//			$isIdPAdmin = $this->IdpAdministrator->isAdmin(
//				$this->getEPPN(), $this->getEPTID(), $this->getIdentityProvider());
//			if ($isIdPAdmin) {
//				$this->Session->write('IdpAdmin', 'Yes');
//				$this->set('isIdpAdmin', true);
//			} else {
//				$this->Session->write('IdpAdmin', 'No');
//				$this->set('isIdpAdmin', false);
//			}
//		} else {
//			if ($idpAdmin === 'Yes') {
//				$this->set('isIdpAdmin', true);
//			} else {
//				$this->set('isIdpAdmin', false);
//			}
//		}
//	}

	/**
	 * 認証要求処理
	 */
	protected function auth() {
		// ログイン画面
		if ($this->params["controller"] === "pages" && $this->params["url"]["url"] === "/") {
			if ($this->issetRemoteUser()) {
				//$this->redirect('/mygroups/view');
				$this->redirect('/top/index');
			}
			// 認証なしでトップページにアクセスしたら何もしない
			else if (!isset($_SERVER['Shib-Session-ID'])) {
				return;
			}
		} else if ($this->params["controller"] === 'connector' && $this->params["action"] == 'services') {
			return;
		// 利用規約、監視URL、バッチ処理は未認証でも閲覧可能
		} else if ($this->params["controller"] === 'rules'
				|| $this->params["controller"] === 'watch'
				|| $this->params["controller"] === 'batch_inspects'
				|| $this->params["controller"] === 'logout'
				|| $this->params["controller"] === 'languages') {
			return;
		// 未認証なら認証を要求する
		} else {
//			if (!isset($_SERVER['REMOTE_USER'])) {
			if (!isset($_SERVER['Shib-Session-ID'])) {
				$this->Session->destroy();
				if ($this->Common->isProduction())
					$this->redirect(Configure::read("production.loginlink").$this->here);
				else
					$this->redirect(Configure::read("test.loginlink").$this->here);
				return;
			}
		}
		if (!$this->validateAttribute()) {
			if ($this->Common->isDevelopment()) {
				$this->redirect('/dev/env');
			}
			$this->cakeError('attributeNotFound', array());
			return;
		}
	}

	/**
	 * ブラウザ言語を取得する。
	 */
	private function getBrowserLanguage() {
		if (!isset($_SERVER['HTTP_ACCEPT_LANGUAGE'])) {
			return '';
		}
		$languages = explode(',', $_SERVER['HTTP_ACCEPT_LANGUAGE']);
		$languages = array_reverse($languages);
		$result = '';
		foreach ($languages as $language) {
			if (preg_match('/^ja/i', $language)) {
    			$result = 'ja';
	  		} elseif (preg_match('/^en/i', $language)) {
    			$result = 'eng';
  			}
		}
		return $result;
	}

	/**
	 * 表示言語を取得する。
	 * @return eng or ja
	 */
	protected function getLanguage() {
		return Configure::read('Config.language');
	}

	/**
	 * 表示言語を設定する。
	 * @param string $accountLang language eng or ja
	 */
	protected function setLanguage($accountLang) {
		$sessionLang = $this->Session->read('accountLanguage');
		$showLanguage = $this->Session->read('showLanguage');
		$lang = 'eng';
		if (!empty($sessionLang)) {
			$lang = $sessionLang;
		} else if (!empty($accountLang)) {
			$this->Session->write('accountLanguage', $accountLang);
			$lang = $accountLang;
		} else if (!empty($showLanguage)) {
			$lang = $showLanguage;
		} else {
			$browserLang = $this->getBrowserLanguage();
			if (empty($browserLang)) {
				$browserLang = 'eng';
			}
			$lang = $browserLang;
		}
		Configure::write('Config.language', $lang);
		$this->setLanguageCookie($lang);
	}

	/**
	 *
	 * @param $accountLang
	 */
	protected function changeLanguage($accountLang) {
		if ($accountLang != 'ja' && $accountLang != 'eng') {
			return;
		}

		if (isset($_SERVER['Shib-Session-ID'])) {
			$this->Session->write('accountLanguage', $accountLang);
		} else {
			$this->Session->write('showLanguage', $accountLang);
		}
		Configure::write('Config.language', $accountLang);
		$this->setLanguageCookie($accountLang);
	}

	/**
	 * アカウント名を設定
	 */
	protected function setAccountName($lang, $account) {
		$name = $this->getAccountName($lang, $account);
		$this->Session->write('accountName', $name);
	}


	/**
	 * Set language Cookie.
	 * Expiration date is 24 hours.
	 * @param string $lang ja or eng
	 */
	private function setLanguageCookie($lang) {
		if ($lang === 'eng')
			$lang = 'en';
		setcookie('language', $lang, time() + 3600 * 24, '/');
	}

	/**
	 * ログイン済みに設定します。
	 */
	protected function setLoggedIn() {
		$this->Session->write('Login', 'true');
		$loginKey = $this->getEPPN().$this->getEPPN();
		$this->Session->write('LoginKey', $loginKey);

		// MYグループのお気に入り(表示順)を更新します
		$this->FavoriteUpdator->update($this->getAccountID());
	}

	/**
	 * ログインしているかを判定します。
	 */
	private function isLogin() {
		// アカウントがない状態は対象外
		$id = $this->getAccountId();
		if (empty($id)) {
			return true;
		}

		$result =  $this->Session->read('Login');
		if ($result === 'true') {
			return true;
		}
		return false;
	}

	private function getLoginKey() {
		return $this->getEPPN().$this->getEPPN();
	}

	/**
	 * 同意確認済みに設定します。
	 */
	protected function setAgreement($status) {
		$this->Session->write('Agreement', $status);
	}

	/**
	 * 初回アクセス時に同意確認を取っているかを判定します。
	 */
	private function isAgreed() {
		// アカウントがない状態は対象外
		$accountId = $this->getAccountId();
		if (empty($accountId)
			|| (!$this->Account->hasAccountName($accountId) && !$this->Account->hasMailaddress($accountId))) {
			return true;
		}

		$result =  $this->Session->read('Agreement');
		if ($result === 'true') {
			return true;
		}
		return false;
	}

	/**
	 * アカウントIDをセッションに設定する。
	 */
	private function setAccountID() {
		$eppn = $this->getEPPN();
		$eptid = $this->getEPTID();

		$count = $this->AccountEppn->find('count', array(
				'conditions' => array('eppn' => $eppn)));
		// ePPNが複数マッチしたらエラー
		if ($count > 1) {
			$this->log('ePPN ' .$eppn .' is registered with multiple accounts.');
			$this->cakeError('systemError');
			return;
		}
		$accountEppn = $this->AccountEppn->find('first', array(
				'fields' => array('id', 'account_id', 'eppn', 'eptid'),
				'conditions' => array('eppn' => $eppn)));
		$this->Session->write('accountID', $accountEppn['AccountEppn']['account_id']);

		// ePTIDが未登録な場合は登録する
		if (!empty($accountEppn) && empty($accountEppn['AccountEppn']['eptid'])) {
			if (!empty($eptid)) {
				$accountEppn['AccountEppn']['eptid'] = $eptid;
				$this->AccountEppn->save($accountEppn);
			}
		}

		// IdPから受信したePTIDとDB上のePTIDが異なる場合はログに出力する
		if (!empty($accountEppn) && !empty($accountEppn['AccountEppn']['eptid'])) {
			if (!empty($eptid) && $accountEppn['AccountEppn']['eptid'] != $eptid) {
				$this->log('account #' . $accountEppn['AccountEppn']['account_id'] .
					' is logging in through account_eppn #' . $accountEppn['AccountEppn']['id'] .
					', with matched ePPN ' . $eppn .
					', but different ePTID (DB:' . $accountEppn['AccountEppn']['eptid'] . ' and received:' . $eptid . ')', 'warning');
			}
		}
	}

	/**
	 * アカウントIDを取得する
	 */
	protected function getAccountID() {
		$id =  $this->Session->read('accountID');
		if (empty($id)) {
			$this->setAccountID();
			$id = $this->Session->read('accountID');
		}
		return $id;
	}

	/**
	 * システム管理者かどうかを判定する。
	 */
	protected function isSuperManager() {
		$super = $this->Session->read('super');
		if (empty($super)) {
			$this->setSuperAdmin();
			$super = $this->Session->read('super');
		}
		if ($super === 'true') {
			return true;
		}
		return false;
	}

	/**
	 * スーパー管理者は全てのコントローラにアクセス可能(groupIDが1はスーパー管理者)
	 */
	private function setSuperAdmin() {
		$superSession = $this->Session->read('super');
		if (empty($superSession)) {
			$count = $this->Mygroup->find('count', array('conditions' =>array(
				'account_id' => $this->getAccountID(), 'admin' => 1, 'groupid' => 1)));
			if ($count > 0) {
				$this->Session->write('super', 'true');
			} else {
				$this->Session->write('super', 'false');
			}
		}
	}

	/**
	 * 該当グループの管理者のみアクセスを許可する。
	 */
	protected function _validateAdminGroup($groupid) {
		if (is_null($groupid)) {
			return;
		}

		if (!$this->isGroupAdmin($groupid)) {
			$this->cakeError('cannotAccess', array('url' => $this->here));
			return;
		}
	}

	/**
	 * shibbolethで認証されていて、ePPN or ePTIDがない場合にエラーとする。
	 */
	private function validateAttribute() {
		// CG画面は属性チェックは不要
		if ($this->params["controller"] === 'top') {
			return true;
		}
		// 画像も属性チェックは不要
		if ($this->params["controller"] === 'images') {
			return true;
		}

		if (isset($_SERVER['Shib-Session-ID'])) {
			$eppn = $this->getEPPN();
			$eptid = $this->getEPTID();
			//if (empty($eppn) && empty($eptid)) {
			if (empty($eppn)) {
				return false;
			}
		}
		return true;
	}

	/**
	 * SubmitボタンのName属性を取得する。
	 */
	protected function getSubmitName() {
		if (empty($this->params['form'])) {
			return false;
		}
		foreach ($this->params['form'] as $key => $value) {
			// input type="image" のときは _x などの文字列が付与される
			preg_match('/^(.*?)(_[xy])?$/', $key, $matches);
			return $matches[1];
		}
	}

	/**
	 * For Test
	 */
	function redirect($url, $status = null, $exit = true) {
		if (Configure::read('CAKE_TEST') === 'on') {
			$this->set('function', 'redirect');
			$this->set('url', $url);
			$this->set('status', $status);
			$this->set('exit', $exit);
			$this->render(null, null, TESTS.DS.'dummy.ctp');
	  } else {
			parent::redirect($url, $status, $exit);
		}
	}

	/**
	 * For Test
	 */
	function cakeError($method, $messages = array()) {
		if (Configure::read('CAKE_TEST') === 'on') {
			$this->render(null, null, TESTS.DS.'dummy.ctp');
		} else {
			parent::cakeError($method, $messages);
		}
	}

	/**
	 * get ePPN
	 */
	protected function getEPPN() {
		if ($this->Common->isDevelopment()) {
			return empty($_COOKIE["eppn"]) ? "" : $_COOKIE["eppn"];
		} else {
			return empty($_SERVER["eppn"]) ? "" : $_SERVER["eppn"];
		}
	}

	protected function getScope() {
		return $this->AttributeOperator->findScope($this->getEPPN());
	}

	/**
	 * get ePTID
	 */
	protected function getEPTID() {
		if ($this->Common->isDevelopment()) {
			return empty($_COOKIE["persistent-id"]) ? "" : $_COOKIE["persistent-id"];
		} else {
			return empty($_SERVER["persistent-id"]) ? "" : $_SERVER["persistent-id"];
		}
	}


	/**
	 * get Shib-Identity-Provider
	 */
	protected function getIdentityProvider() {
		return $_SERVER["Shib-Identity-Provider"];
	}


	/**
	 * get entityID
	 */
	protected function getEntityID() {
		return $this->getIdentityProvider();
	}

	/**
	 * get Shib-Session-ID
	 */
	protected function getShibSessionId() {
		return $_SERVER["Shib-Session-ID"];
	}

	/**
	 * get Organization Name
	 */
	protected function getOrganizationName() {
		if (empty($_SERVER["jao"])) {
			if (empty($_SERVER["o"])) {
				return "";
			} else {
				return $_SERVER["o"];
			}
		} else {
			return $_SERVER["jao"];
		}
	}

	/**
	 * get Japanese Organization Name
	 */
	protected function getOrganizationNameJa() {
		return empty($_SERVER["jao"]) ? "" : $_SERVER["jao"];
	}

	/**
	 * get English Organization Name
	 */
	protected function getOrganizationNameEn() {
		return empty($_SERVER["o"]) ? "" : $_SERVER["o"];
	}

	/**
	 * get Japanese OrganizationalUnit Name
	 */
	protected function getOrganizationalUnitNameJa() {
		return empty($_SERVER["jaou"]) ? "" : $_SERVER["jaou"];
	}

	/**
	 * get English OrganizationalUnit Name
	 */
	protected function getOrganizationalUnitNameEn() {
		return empty($_SERVER["ou"]) ? "" : $_SERVER["ou"];
	}

	/**
	 * get displayName
	 */
	protected function getDisplayName() {
		if (empty($_SERVER["jaDisplayName"])) {
			if (empty($_SERVER["displayName"])) {
				return "";
			} else {
				return $_SERVER["displayName"];
			}
		} else {
			return $_SERVER["jaDisplayName"];
		}
	}

	/**
	 * get English displayName
	 */
	protected function getDisplayNameJa() {
		return empty($_SERVER["jaDisplayName"]) ? "" : $_SERVER["jaDisplayName"];
	}

	/**
	 * get English displayName
	 */
	protected function getDisplayNameEN() {
		return empty($_SERVER["displayName"]) ? "" : $_SERVER["displayName"];
	}

	/**
	 * get mail
	 */
	protected function getMail() {
		return empty($_SERVER["mail"]) ? '' : $_SERVER["mail"];
	}

	//
	// isServerVariable
	// サーバ変数が存在し、かつ値が一致する場合のみtrueを返す
	// 属性値に、|がある場合は、OR指定。|で分割し、そのなかの１つと一致すればtrueとする。
	// サーバ変数値に、;セミコロンがある場合は、OR指定。;で分割し、そのなかの１つと一致すればtrueとする。
	// サーバ変数値に空白が含まれている場合は、空白をすべて取り除く。
	// 属性名がaffiliationの場合、サーバ変数値の@以降を取り除く。
	//
	// @param $key 属性名
	// @param $val 属性値
	// @return サーバ変数と一致する場合は、true、一致しない場合はfalseを返す。
	//
	protected function isServerVariable($key, $val) {
		if (empty($_SERVER[$key])) {
			return false;
		}

		$server_val = $_SERVER[$key];
		$server_vals = explode(';', $server_val);
		$vals = explode('|', $val);
		foreach($server_vals as $server_val) {

			$server_val = str_replace(' ', '', $server_val);
			if ($key === 'affiliation') {
				$pos = strpos($server_val,"@");
				if ($pos !== false) {
					$server_val = substr($server_val, 0, $pos);
				}
			}
			foreach($vals as $v) {
				if ($key === 'affiliation') {
					$pos = strpos($v,"@");
					if ($pos !== false) {
						$v = substr($v, 0, $pos);
					}
				}
				if ($server_val === $v) {
					return true;
				}
			}
		}
		return false;
	}

	/**
	 * 乱数を取得します。
	 * @param $len 乱数の長さ
	 */
	protected function getRandom($len = 20){
		$strings = "abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ0123456789";
		$string_len = strlen($strings) - 1;
		mt_srand();
		$res = "";
		for ($i = 0; $i < $len; $i++) {
			$res .= $strings{ mt_rand(0, $string_len) };
		}
		return $res;
	}

	/**
	 * 乱数の配列に存在しないユニークな乱数を取得します。
	 * @param $randoms 乱数の配列
	 * @param $len 乱数の長さ
	 */
	public function getUniqRandom($randoms, $len){
		$random = $this->getRandom($len);
		while(true) {
			$duplicate = false;
			foreach($randoms as $val) {
				if ($val === $random) {
					$duplicate = true;
					$random = $this->getRandom($len);
					break;
				}
			}
			if (!$duplicate) {
				return $random;
			}
		}
	}

	/**
	 * アクセスログに出力する。
	 */
	protected function accessLog() {
		$url = $this->here;
		$message = "accountid:".$this->getAccountID()."  ePPN:".$this->getEppn()."  ePTID:".$this->getEPTID()."  ".$url;
		$this->log($message, "access");
	}

	/**
	 * グループ管理者かどうかを判定する。
	 * @param $groupid
	 */
	protected function isGroupAdmin($groupid) {
		if ($this->isSuperManager()) {
			return true;
		}

		$count = $this->Mygroup->find('count', array('conditions' => array(
			'account_id' => $this->getAccountID(),
			'groupid' => $groupid, 'admin' => array(1, 2))));
		if ($count >= 1) {
			return true;
		}
		return false;
	}

	/**
	 * 自分以外の管理者へメールを送信する。
	 * @param $groupid
	 * @param $titleKey
	 * @param $template
	 * @param $Mail
	 * @param $excludedAccountId メール送信を除外するaccount_id
	 * @param $targetAccountId 申請対象者のaccount_id
	 */
	protected function sendMailToOtherAdmin($groupid, $titleKey, $template, $Mail,
		$excludedAccountId = null, $targetAccountId = null) {
		$mygroups = $this->Mygroup->find('all', array(
			'fields' => array('account_id'),
			'conditions' => array('groupid' => $groupid, 'admin' => array(1, 2))));
		// 管理者が1人のみなら何もしない
		if (empty($mygroups)) {
			return;
		}

		// 申請者の名前を取得
		$inspectedName = $this->Account->find('first', array(
			'fields' => array('id', 'name', 'name_en'),
			'conditions' => array('id' => $targetAccountId)));
		$Mail['inspectedName'] = $inspectedName['Account']['name'];
		$Mail['inspectedNameJp'] = $this->getAccountName('ja', $inspectedName);
		$Mail['inspectedNameEn'] = $this->getAccountName('eng', $inspectedName);

		// 処理した管理者の名前を取得
		$handledName = $this->Account->find('first', array(
			'fields' => array('id', 'name', 'name_en'),
			'conditions' => array('id' => $this->getAccountID())));
		$Mail['handledName'] = $handledName['Account']['name'];
		$Mail['handledNameJp'] = $this->getAccountName('ja', $handledName);
		$Mail['handledNameEn'] = $this->getAccountName('eng', $handledName);

		// 管理者連絡先にメール送信
		$group = $this->Group->find('first', array(
					'fields' => array('id', 'contactadmin'),
					'conditions' => array('id' => $groupid)));
		if (!empty($group['Group']['contactadmin'])
			&& Validation::email(trim($group['Group']['contactadmin']))) {
			$this->sendMail($titleKey, trim($group['Group']['contactadmin']), 'both', $template, $Mail);
			return;
		}

		// アカウント情報のメールアドレスにメール送信
		$accountIdList = array();
		// 除外eppnのaccount_idを取得
		foreach ($mygroups as $mygroup) {
			if ($mygroup['Mygroup']['account_id'] === $this->getAccountID()) {
				continue;
			}
			if (!empty($excludedAccountId)) {
				if ($mygroup['Mygroup']['account_id'] === $excludedAccountId) {
					continue;
				}
			}
			array_push($accountIdList, $mygroup['Mygroup']['account_id']);
		}
		if (empty($accountIdList)) {
			return;
		}

		$accountList = $this->Account->find('all', array(
			'fields' => array('mail', 'language'),
			'conditions' => array('id' => $accountIdList)));
		if (empty($accountList)) {
			return;
		}

		$this->sendMailAdmin($accountList, $titleKey, $template, $Mail);
	}

	public function getAccountName($lang, $account) {
		$name = '';
		if ($lang === 'ja' || empty($lang)) {
			if (!empty($account['Account']['name'])) {
				$name = $account['Account']['name'];
			} else {
				$name = $account['Account']['name_en'];
			}
		} else {
			if (!empty($account['Account']['name_en'])) {
				$name = $account['Account']['name_en'];
			} else {
				$name = $account['Account']['name'];
			}
		}
		return $name;
	}


	/**
	 * 管理者全員へメールを送信する。
	 * @param $groupid
	 * @param $titleKey
	 * @param $template
	 * @param $Mail
	 */
	function sendMailAllAdmin($groupid, $titleKey, $template, $Mail) {
		// 管理者連絡先にメール送信
		$group = $this->Group->find('first', array(
					'fields' => array('id', 'contactadmin'),
					'conditions' => array('id' => $groupid)));
		if (!empty($group['Group']['contactadmin'])
			&& Validation::email(trim($group['Group']['contactadmin']))) {
			$this->sendMail($titleKey, trim($group['Group']['contactadmin']), 'both', $template, $Mail);
			return;
		}

		$mygroups = $this->Mygroup->find('all', array(
			'fields' => array('account_id'),
			'conditions' => array('groupid' => $groupid, 'admin' => array(1, 2))));

		$accountIdList = array();
		foreach ($mygroups as $mygroup) {
			array_push($accountIdList, $mygroup['Mygroup']['account_id']);
		}
		$accountList = $this->Account->find('all', array(
			'fields' => array('mail', 'language'),
			'conditions' => array('id' => $accountIdList)));
		$this->sendMailAdmin($accountList, $titleKey, $template, $Mail);
	}

	/**
	 *
	 * @param array $accountList
	 * @param string $titleKey
	 * @param string $template
	 * @param array $Mail
	 */
	private function sendMailAdmin($accountList, $titleKey, $template, $Mail) {
		$addressList = array();
		$langJa = false;
		$langEng = false;
		foreach($accountList as $account) {
			array_push($addressList, $account['Account']['mail']);
			if ($account['Account']['language'] === "ja") {
				$langJa = true;
			} else if ($account['Account']['language'] === "eng") {
				$langEng = true;
			}
		}
		$lang = "ja";
		if ($langJa && !$langEng) {
			$lang = "ja";
		} else if (!$langJa && $langEng) {
			$lang = "eng";
		} else if ($langJa && $langEng) {
			$lang = "both";
		}
		$this->sendMail($titleKey, $addressList, $lang, $template, $Mail);
	}

	/**
	 * メールを送信する。
	 * @param string $titleKey メールタイトルキー
	 * @param array $address メールアドレス 複数可
	 * @param string $lang 受信者の使用言語
	 * @param string　$template テンプレート
	 * @param array $variable テンプレートに渡す変数
	 * @param string $layout レイアウト
	 * @param string $replyTo reply-toのメールアドレス
	 */
	function sendMail($titleKey, $address, $lang,
			$template = 'simple_message', $variable = null, $layout = null, $replyTo = null) {
		if (empty($address)) {
			return;
		}

		$Config = new MailConfig();

		$toList = array();
		if (!is_array($address)) {
			if ($Config->debug['on']) {
				$address = $Config->debug['address'];
			}
			$toList = array(trim($address));
		} else {
			foreach ($address as $to) {
				array_push($toList, trim($to));
			}
		}

		if ($lang === 'eng' || $lang === 'en') {
			$this->EmailJp->template = $template."_eng";
			$this->EmailJp->layout = 'default_eng';
			$title = $Config->titles[$titleKey]['eng'];
		} elseif ($lang === 'ja') {
			$this->EmailJp->template = $template."_jp" ;
			$this->EmailJp->layout = 'default_jp';
			$title = $Config->titles[$titleKey]['ja'];
		} elseif ($lang === 'both') {
			$this->EmailJp->template = $template."_both" ;
			$this->EmailJp->layout = 'default_both';
			$title = $Config->titles[$titleKey]['ja'].'  '.$Config->titles[$titleKey]['eng'];
		} elseif (is_array($lang)) {
			$this->EmailJp->template = $template."_".implode('_', $lang);
			$this->EmailJp->layout = 'default_both';
			$titles = array();
			foreach ($lang as $_lang) {
				$titles[] = $Config->titles[$titleKey][$_lang == 'eng' ? 'eng' : 'ja'];
			}
			$title = implode('  ', $titles);
		} else {
			$this->EmailJp->template = $template."_jp";
			$this->EmailJp->layout = 'default';
			$title = $Config->titles[$titleKey]['ja'].'  '.$Config->titles[$titleKey]['eng'];
		}

		if (!empty($layout)) {
			$this->EmailJp->layout = $layout;
		}
		$this->log('$this->EmailJp->template='.$this->EmailJp->template, 'debug');

		$titleIso = mb_convert_encoding($Config->titlePrefix.$title, "iso-2022-jp") ;
		$this->EmailJp->smtpOptions = $Config->smtpOptions;
		$this->EmailJp->sendAs = "text" ;
		$this->EmailJp->charset = "iso-2022-jp" ;
		$this->EmailJp->from = $Config->from;
		$this->EmailJp->to = $toList;
		$this->EmailJp->subject = $titleIso;
		$this->EmailJp->delivery = "smtp" ;
		$this->EmailJp->lineLength = 2048;
		if ($this->MailUtils->isEmail($replyTo)) {
			$this->EmailJp->replyTo = $replyTo;
		}

		$variable['footer_jp'] = $Config->footer_jp;
		$variable['footer_eng'] = $Config->footer_eng;
		if (!empty($variable)) {
			$this->set('Mail', $variable);
		}

		$result = $this->EmailJp->send();
		$this->mailLog($toList, $title, $result);
		$this->EmailJp->reset();
	}

	/**
	 * メール送信ログを出力する。
	 * @param $toList
	 * @param $eppn
	 * @param $title
	 */
	function mailLog($toList, $title, $result) {
		$addressString = implode(',', $toList);
		$eppn = $this->getEPPN();
		$eptid = $this->getEPTID();
		if (empty($eppn) && empty($eptid)) {
			$message = "to:".$addressString." title:".$title." result:".$result;
			$this->log($message, "mail_batch");
		} else {
			$message ='"accountid:'.$this->getAccountID().'  ePPN:'.$eppn.' ePTID:'.$eptid.'  to:'.$addressString." title:".$title." result:".$result;
			$this->log($message, "mail");
		}
	}

	/**
	 * メッセージ表示欄に複数のメッセージを表示させます。
	 * @param $message
	 */
	function setFlash($message) {
		$before = $this->Session->read('Message.flash');
		if (!empty($before)) {
			$message = $before['message']."<br/>".$message;
		}
		$this->Session->setFlash($message, 'message');
	}

	/**
	 * メッセージを置換します。
	 * @param $msgid
	 * @param $strings
	 */
	function createMessage($msgid, $strings) {
		$msgstr = str_replace(
			array("%REPLACE1", "%REPLACE2", "%REPLACE3", "%REPLACE4"),
			h($strings), __($msgid, true));
		return $msgstr;
	}

	function createMessageUnsafe($msgid, $replace, $search = array('%REPLACE1', '%REPLACE2', '%REPLACE3', '%REPLACE4'))
	{
		return str_replace($search, $replace, __($msgid, true));
	}

	private function getServerValue($key, $default = '')
	{
		return isset($_SERVER[$key]) ? $_SERVER[$key] : $default;
	}
	public function inPersonScopedAffiliation($searchValue)
	{
		$values = $this->getPersonScopedAffiliation();
		if (!is_array($searchValue)) {
			$searchValue = array($searchValue);
		}
		foreach ($searchValue as $search) {
			if (in_array($search, $values)) {
				return true;
			}
		}
		return false;
	}

	private function getPersonScopedAffiliation()
	{
		if ($this->Common->isDevelopment()) {
			return explode(';', $this->getEppn());
		} else {
			return explode(';', $this->getServerValue('unscoped-affiliation'));
		}
	}

	/**
	 * デバッグログを出力する。
	 * @param $message
	 */
	public function debugLog($message) {
		$debug = Configure::read('debug');
		if ($debug) {
			$this->log($message, 'debug');
		}
	}
}
?>
