<?php
/**
 * Application model
 *
 * PHP versions 5
 *
 * @copyright     Copyright (c) 2016, SOUM Corporation.
 * @copyright     Copyright (c) 2011 - 2012, National Institute of Informatics,
 * @license       BSD License (http://creativecommons.org/licenses/BSD/)
 */
class AppModel extends Model {

	/**
	 * メールアドレス一覧のemailバリデーションを行う。
	 * @param array $values
	 */
	function emailList($values) {

		$value = array_shift($values);
		$mails = explode("\n", $value);
		$errorFlg = false;
		$errorMsg = "";
		$count = count($mails);
		$validCnt = 0;
		for ($i = 0; $i < $count; $i++) {
			$mail = $mails[$i];
			if (strlen(trim($mail)) == 0) {
				continue;
			}

			$validCnt++;
			if (!Validation::email(trim($mail))) {
				$errorFlg = true;
				$errorMsg .= trim($mail)."  ";
				continue;
			}
		}
		if ($validCnt == 0) {
			$errorFlg = true;
		}

		if ($errorFlg) {
			if (trim($errorMsg) == "") {
//				return __("validation.no.mailaddress", true);
				return true;
			} else {
				return __("validation.mailaddress.incorrect.format.next", true)."　".$errorMsg;
			}
		}
		return true;
	}

	/**
	 *
	 * @param $field
	 * @param $value
	 */
	function invalidate($field, $value = true) {
		return parent::invalidate($field, __($value, true));
	}

	/**
	 * 乱数を取得します。
	 * @param $len 乱数の長さ
	 */
	protected function getRandom($len = 20, $strings = 'abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ0123456789') {
		$string_len = strlen($strings) - 1;
		mt_srand();
		$res = '';
		for ($i = 0; $i < $len; $i++) {
			$res .= $strings{ mt_rand(0, $string_len) };
		}
		return $res;
	}

	/**
	 * saveAllメソッドの戻り値を評価する。
	 *
	 * @param $results saveAllの戻り値
	 * @return bool 全て成功している場合にはtrueを返す。ひとつでもエラーがある場合にはfalseを返す。
	 */
	function checkSaveAll($results) {
		if (!is_array($results)) {
			return results;
		}
		foreach ($results as $value) {
			if (is_array($value)) {
				if (!$this->checkSaveAll($value)){
					return false;
				}
			}
			if ($value==false) {
				return false;
			}
		}
		return true;
	}
}
