<?php
/**
 * Management Group Controller
 *
 * PHP versions 5
 *
 * @copyright     Copyright (c) 2011 - 2012, National Institute of Informatics,
 * @license       BSD License (http://creativecommons.org/licenses/BSD/)
 */
App::import('Controller', 'Manages');

class ManageGroupsController extends ManagesController {

	var $name = 'ManageGroups';
	var $uses = array('ManageGroup', 'Invite', 'Group', 'SpGroup',
						'AddGroup', 'AddGroupInspect', 'Mygroup', 'GroupSphost', 'SpHost',
						'Account', 'Transaction', 'AutoGroup', 'ManageGroupAuto', 'ProvideAttribute',
						'InspectAttribute', 'ExtendedAttribute', 'GroupExtendedValue', 'AddGroupsSpcLog');
	var $components = array('TakerAgreement', 'Url', 'AgreementOperator', 'AdminGroup', 'GroupInvite',
						'Csrf', 'Callback', 'ParameterReplacer', 'NoticeUtils');
	var $helpers = array('Javascript');

	private function disableValidatePost() {
//		if ($this->action == 'invite') {
//			// 招待の確認ページではCSRFのチェックはしない
//			if ($this->getSubmitName() == 'confirm') {
//				$this->Csrf->validatePost = false;
//			}
//		} elseif ($this->action == 'add_up_sp_confirm') {
//			// Webサービスの選択 -> 確認ページ
//			$this->Csrf->validatePost = false;
//		}
		$this->Csrf->validatePost = false;
	}

	function beforeFilter() {
		$this->disableValidatePost();
		parent::beforeFilter();
		$groupid = $this->_getGroupIdFromURL();
		$this->_checkAccessible($groupid);
		$this->_validateAdminGroup($groupid);
	}

	protected function _checkAccessible($groupid) {
		if (is_null($groupid)) {
			$this->cakeError('notFound', array());
			return;
		}

		if ($this->params['action'] === 'invite'
			|| $this->params['action'] === 'confirm'
			|| $this->params['action'] === 'resend'
			|| $this->params['action'] === 'invite_state'
			|| $this->params['action'] === 'invite_cancel'
			|| $this->params['action'] === 'extended') {
			if (!($this->Group->isGroup($groupid) || $this->Group->isIdPGroup($groupid)
				|| $this->Group->isSpConnector($groupid))) {
				$this->cakeError('notFound', array());
				return;
			}
		} else {
			if (!($this->Group->isGroup($groupid) || $this->Group->isIdPGroup($groupid))) {
				$this->cakeError('notFound', array());
				return;
			}
		}
		parent::_checkAccessible($groupid);
	}

	/**
	 * edit group
	 * @param integer $groupid group ID
	 */
	function edit($groupid) {
		$this->set('groupid', $groupid);

		// 自動グループの属性取得
		$attr = $this->AutoGroup->getGroupAttribute($groupid);
		$this->set('groupAttribute', $attr);
		// タブ化対応
		$this->setFavariteFiveGroup($groupid);

		switch ($this->getSubmitName()) {
			case '': // グループ編集画面を表示
				$this->_editShow($groupid);
				break;
			case 'save': // 保存
				if (empty($this->data)) {
					return;
				}

				if (!$this->groupSave($groupid)) {
					return;
				}
				$this->Session->setFlash(__("message.groups.save", true), 'message');
				$this->redirect('/group_lists/detail/'.$groupid);
				break;
			case 'cancel': // キャンセル
				$this->redirect('/mygroups/view');
				return;
			default:
				break;
		}
	}

	private function _editShow($groupid) {
		$Group = $this->ManageGroup->find('first', array('conditions' => array('id' => $groupid)));
		$Group = $this->convertDb2Form($Group);
		$this->data = $Group;

		// 自動グループかを検索
		$count = $this->AutoGroup->find('count', array(
			'conditions' => array('group_id' => $groupid)));
		$this->data['ManageGroup']['autoGroup'] = false;
		if ($count > 0) {
			$this->data['ManageGroup']['autoGroup'] = true;
			// 修正可能な項目を制限
			$this->data['ManageGroup']['public'] = 0;
			$this->data['ManageGroup']['search'] = 0;
			$this->data['ManageGroup']['inspectjoin'] = 2;
			$this->data['ManageGroup']['openmember'] = 0;
		}


		// 上位グループ
		$topIdList = $this->AddGroup->getTopIdList($groupid, $onlyActive=false);
		$topGroupList = $this->Group->findGroupList($topIdList);
		$this->set('topGroupList', $topGroupList);

		// 下位グループ
		$bottomIdList = $this->AddGroup->getBottomIdList($groupid, $onlyActive=false);
		$bottomGroupList = $this->Group->findGroupList($bottomIdList);
		$this->set('bottomGroupList', $bottomGroupList);

		// 現在独立しているかを設定
		$independence = $Group['ManageGroup']['independence'];
		$this->set('currentIndependence', $Group['ManageGroup']['independence']);
	}

	private function convertDb2Form($Group)
	{
		// 非公開・検索対象とする・検索対象としない -> 公開・非公開, 検索対象にする・しないの設定に変換
		if ($Group['ManageGroup']['public'] == 1) {
			$Group['ManageGroup']['search'] = 1;
		} else {
			$Group['ManageGroup']['search'] = 0;
		}
		if ($Group['ManageGroup']['public'] == 2) {
			$Group['ManageGroup']['public'] = 1;
		}
		return $Group;
	}

	private function groupSave($groupid) {
		$count = $this->AutoGroup->find('count', array(
				'conditions' => array('group_id' => $groupid)));
		if ($count > 0) {
			// 自動グループならグループ名がを取り除いてsaveする
			$group = $this->data;
			unset($group['ManageGroup']['name']);
			unset($group['ManageGroup']['name_en']);
			$group = $this->convertPostData($group);
			$result = $this->ManageGroupAuto->save($group['ManageGroup']);
debug($result);
		} else {
			$group = $this->data;
			$group = $this->convertPostData($group);
			$result = $this->ManageGroup->save($group);
		}
		if (!$result) {
			$this->editAfterValidation($groupid);

			// 現在独立しているかを設定
			$Group = $this->ManageGroup->find('first', array('conditions' => array('id' => $groupid)));
			$independence = $Group['ManageGroup']['independence'];
			$this->set('currentIndependence', $Group['ManageGroup']['independence']);

			$this->Session->setFlash(__("message.input.error", true), 'message');
			return false;
		}

		return true;
	}

	private function convertPostData($group) {
		$group['ManageGroup']['sp'] = 0;
		$group = $this->updateIconIfUploaded($group);
		// グループの公開・非公開: 公開
		// 検索対象に: しない
		// の場合にpublic=2
		if ($group['ManageGroup']['public'] == 1 && $group['ManageGroup']['search'] == 0) {
			$group['ManageGroup']['public'] = 2;
		}
		return $group;
	}

	private function updateIconIfUploaded($group) {
		if (isset($group['ManageGroup']['file']['size'])
			&& $group['ManageGroup']['file']['size'] > 0) {
			$group['ManageGroup']['icon'] = 0;
		}
		return $group;
	}

	private function editAfterValidation($groupid) {
	}

	/**
	 * invite group
	 * @param integer $id group id
	 */
	function invite($id = null) {
		$this->data['ManageGroup']['id'] = $id;
		$idpGroup = $this->AutoGroup->isIdPGroup($id);
		switch ($this->getSubmitName()) {
			case '': // 初期表示(入力画面)
				$less = isset($this->params['url']['less']) ? (int)$this->params['url']['less'] : 0;	// 招待なし入会画面
				if ($less == 0 && isset($this->data['ManageGroup']['forcibleAdmission'])) {
					$less = (int)$this->data['ManageGroup']['forcibleAdmission'];
				}
				if ($less != 1 || !$this->GroupInvite->canForce() || $idpGroup) { $less = 0; }
				$this->data['ManageGroup']['forcibleAdmission'] = $less;

				// メール本文載せる入会URL
				$url = h(Router::url('/', true))."join/".$id."/"."[システムで払い出す番号]";
				$this->data['ManageGroup']['introductionUrl'] = $url;
				$this->set('ManageGroup', $this->data);

				// グループ名、ユーザ名を取得
				$group = $this->Group->find('first', array('fields' => array('name', 'introduction', 'sp'),
					'conditions' => array('id' => $id)));
				$user = $this->Account->find('first', array('fields' => array('name', 'name_en'),
					'conditions' => array('id' => $this->getAccountID())));
				$this->set('jaTmpVal', $this->GroupInvite->getTemplate('ja', $group['Group'], $user['Account'], $less));
				$this->set('engTmpVal', $this->GroupInvite->getTemplate('en', $group['Group'], $user['Account'], $less));
				$this->data['ManageGroup']['UserName'] = $user['Account']['name'];
				$this->data['ManageGroup']['sp'] = $group['Group']['sp'];
				$this->setInputFormData($id);
				$this->setFavariteFiveGroup($id);
				return;
			case 'confirm':
				$less = 0;
				if (isset($this->data['ManageGroup']['forcibleAdmission']) && isset($this->data['ManageGroup']['mailaddress_ml'])) {
					$this->cakeError('illegalAccess');
				}
				if (isset($this->data['ManageGroup']['forcibleAdmission'])) {
					$less = (int)$this->data['ManageGroup']['forcibleAdmission'];
					if ($less != 1 || !$this->GroupInvite->canForce() || $idpGroup) {
						$this->cakeError('illegalAccess');
					}
				}
				$this->data['ManageGroup']['forcibleAdmission'] = $less;
				if (!isset($this->data['ManageGroup']['mailaddress_ml'])) {
					$this->data['ManageGroup']['mailaddress_ml'] = '';
				}

				$this->setFavariteFiveGroup($id);
				$result = $this->toConfirm($id);
				if ($result) {
					$this->render('confirm');
				}
				$this->setInputFormData($id);
				break;
			case 'cancel':
				if (!$this->Group->isSpConnector($id)) {
					$this->redirect('/group_lists/detail/'.$id);
				} else  {
					$this->redirect('/sp_groups/detail/'.$id);
				}
				return;
			default:
				break;
		}
	}

	private function getMymembers($groupid, $forcibleAdmission) {
		return $this->AdminGroup->getMembers($this->getAccountID(), $this->getLanguage(), $groupid, $forcibleAdmission);
	}

	private function setInputFormData($groupid) {
		$forcibleAdmission = $this->GroupInvite->canForce();
		$this->set('forcibleAdmission', $forcibleAdmission);
		if (!isset($this->data['ManageGroup']['forcibleAdmission'])) {
			$this->data['ManageGroup']['forcibleAdmission'] = false;
		}
		$this->data['ManageGroup']['idp'] = $this->AutoGroup->isIdPGroup($groupid);
	}

	private function getGroupNotAgreedMemberIds($group, $memberIds) {
		if ($group['sp']) {
			$notAgreedMemberIds = array();
		} else {
			$notAgreedMemberIds = $this->TakerAgreement->getNotAgreedToMail($memberIds);
		}
		return $notAgreedMemberIds;
	}

	private function removeNotAgreedMemberid($memberIds, $notAgreedMemberIds) {
		// ViewにLogicを入れないようにここで$addMemberListから$notAgreedMemberIds分を削除しておく
		foreach ($notAgreedMemberIds as $notAgreedMemberId) {
			$key = array_search($notAgreedMemberId, $memberIds);
			if ($key === false) {
				throw new Exception('Error');
			}
			unset($memberIds[$key]);
		}
		return $memberIds;
	}

	// 該当アカウントへの最新の管理者入力メールアドレスを取得
	//   取得されるメールアドレスは、管理者が入力したものではない可能性があるため利用に問題あり[CG-2141]
	private function getLatestAdminInput($accountId, $notAgreedMemberIds) {
		$results = array();
		foreach ($notAgreedMemberIds as $notAgreedMemberId) {
			// 再送していない最新の招待
			$invite = $this->Invite->find('first', array(
				'conditions' => array(
					'account_id' => $accountId,
					'invited_account_id' => $notAgreedMemberId,
					'resend_id IS NULL',
				),
				'order' => array('created'),
			));
			if ($invite) {
				$results[$notAgreedMemberId] = $invite['Invite']['address'];
			}
		}
		return $results;
	}

	private function toConfirm($groupid) {
		// バリデーション(save以外でバリデーション)
		$this->ManageGroup->set($this->data);

		// 無効なメールアドレスの確認
		$invalid_addresses = $this->GroupInvite->getInvalidAddresses(explode("\n", $this->data['ManageGroup']['mailaddress']));
		$invalid_addresses_ml = $this->GroupInvite->getInvalidAddresses(array($this->data['ManageGroup']['mailaddress_ml']));
		if ($invalid_addresses || $invalid_addresses_ml) {
			$error_addresses = array_merge($invalid_addresses, $invalid_addresses_ml);
			$this->set('ManageGroup', $this->data);
			$this->Session->setFlash($this->createMessage('message.manage_groups.invite.invalid.email', implode(', ', $error_addresses)), 'message');
			return false;
		}

		// グループを取得
		$group = $this->GroupInvite->findGroup($groupid);
		$accountIds = array();
		if (isset($this->data['ManageGroup']['member'])) {
			$accountIds = array_keys($this->data['ManageGroup']['member']);
		}
		// 確認ページ用のデータ設定をするタイミングで有効なAccountIDの判定も行っているため、ここで行ってこの後emtpyでのチェックにも使用
		$forcibleAdmission = $this->GroupInvite->canForce();
		if ($group['sp']) {
			$forcibleAdmission = false;
		}
		$this->set('forcibleAdmission', $forcibleAdmission);
		$choiceForcibleAdmission = $forcibleAdmission && $this->data['ManageGroup']['forcibleAdmission'];

		// 招待なし入会のメールアドレスのチェック
		// メールアドレスを全部（仮アカウントを除く）を対象としてマッチングをとった結果、0ならエラー、複数マッチしたらエラー、
		// 1件マッチしたものが到達性未確認か登録済みならエラーのチェックを入れ、エラーメッセージを出す。
		$valid_addresses = $this->GroupInvite->getValidAddresses(explode("\n", $this->data['ManageGroup']['mailaddress']));
		if ($choiceForcibleAdmission) {
			$noAccountList = array();
			$manyAccountList = array();
			$noReachabilityList = array();
			$alreadyJoinList = array();
			foreach ($valid_addresses as $address) {
				$accounts = $this->Account->find('all', array(
					'fields' => array('id', 'arrival'),
					'conditions' => array(
						'mail' => $address,
						'is_temp' => 0,
					),
				));
				if (empty($accounts)) {
					$noAccountList[] = $address;
				}
				else if (count($accounts) > 1) {
					$manyAccountList[] = $address;
				}
				else if ($accounts[0]['Account']['arrival'] == 0) {
					$noReachabilityList[] = $address;
				}
				else if ($this->Mygroup->isMember($accounts[0]['Account']['id'], $groupid)) {
					$alreadyJoinList[] = $address;
				}
			}
			$msg = '';
			if (!empty($noAccountList)) {
				$msg = $this->createMessage('message.manage_groups.invite.less.no.account', implode(', ', $noAccountList));
			}
			if (!empty($manyAccountList)) {
				if (!empty($msg)){
					$msg .= '<br/>';
				}
				$msg .= $this->createMessage('message.manage_groups.invite.less.more.account', implode(', ', $manyAccountList));
			}
			if (!empty($noReachabilityList)) {
				if (!empty($msg)){
					$msg .= '<br/>';
				}
				$msg .= $this->createMessage('message.manage_groups.invite.less.reachability', implode(', ', $noReachabilityList));
			}
			if (!empty($alreadyJoinList)) {
				if (!empty($msg)){
					$msg .= '<br/>';
				}
				$msg .= $this->createMessage('message.manage_groups.invite.less.already', implode(', ', $alreadyJoinList));
			}
			if ($msg) {
				$this->set('ManageGroup', $this->data);
				$this->Session->setFlash($msg, 'message');
				return false;
			}
		}

		$memberIds = $this->setConfirmPageData($group, $accountIds, $choiceForcibleAdmission);
		$notAgreedMemberIds = array();
		if (!$choiceForcibleAdmission) { // 強制入会ではない場合は、同意の確認
			$notAgreedMemberIds = $this->getGroupNotAgreedMemberIds($group, $memberIds);
			if ($notAgreedMemberIds) {
				$memberIds = $this->removeNotAgreedMemberid($memberIds, $notAgreedMemberIds);
			}
		}
		$this->set('notAgreedMemberIds', $notAgreedMemberIds);
		$this->set('addMemberList', $memberIds);
		if ($notAgreedMemberIds) {
			$this->data['ManageGroup']['adminInput'] = $this->getLatestAdminInput($this->getAccountID(), $notAgreedMemberIds);
		}

		// メールアドレスをaddr_rpec形式に変換
		$valid_address = implode("\n", $valid_addresses);
		$this->data['ManageGroup']['mailaddress'] = $valid_address;

		$valid_addresses_ml = $this->GroupInvite->getValidAddresses(array($this->data['ManageGroup']['mailaddress_ml']));
		$valid_address_ml = implode("\n", $valid_addresses_ml);
		$this->data['ManageGroup']['mailaddress_ml'] = $valid_address_ml;
		if (empty($valid_address) && empty($valid_address_ml) && empty($memberIds) && empty($notAgreedMemberIds)) {
			$this->set('ManageGroup', $this->data);
			$msg = __("message.manage_groups.invite.input.error", true);
			if ($choiceForcibleAdmission) {
				$msg = __("message.manage_groups.invite.less.input.error", true);
			}
			$this->Session->setFlash($msg, 'message');
			return false;
		}

		// メール本文載せる入会URL
		if ($choiceForcibleAdmission) {
			$url = Router::url('/', true).'group_lists/detail/'.$groupid;
		}
		else {
			if ($this->data['ManageGroup']['defaultLang'] === 'ja') {
				$random = sprintf('[%s]', __("manage_groups.random.value.ja", true));
			} else {
				$random = sprintf('[%s]', __("manage_groups.random.value.eng", true));
			}
			$url = $this->GroupInvite->getInviteUrl($group['group_key'])."/".$random;
		}
		$this->data['ManageGroup']['introductionUrl'] = $url;

		$this->set('ManageGroup', $this->data);
		return true;
	}

	private function getValidMemberIdAndNameList($forcibleAdmission, $groupid, $accountIds) {
		$myMembers = $this->getMymembers($groupid, $forcibleAdmission);
		$checkKeys = $this->GroupInvite->getCheckKeys($forcibleAdmission);
		$memberIds = array();
		$memberNameList = array();
		foreach ($accountIds as $accountId) {
			foreach ($checkKeys as $checkKey) {
				if (isset($myMembers[$checkKey][$accountId])) {
					$memberIds[] = $accountId;
					$memberNameList[$accountId] = $myMembers[$checkKey][$accountId];
				}
			}
		}
		return array(
			'id' => $memberIds,
			'name' => $memberNameList,
		);
	}

	private function setConfirmPageData($group, $accountIds, $choiceForcibleAdmission) {
		if ($accountIds) {
			$id_and_name_list = $this->getValidMemberIdAndNameList($choiceForcibleAdmission, $group['id'], $accountIds);
			$memberIds = $id_and_name_list['id'];
			$memberNameList = $id_and_name_list['name'];
		} else {
			$memberIds = array();
			$memberNameList = array();
		}

		$this->set('addMemberNameList', $memberNameList);
		return $memberIds;
	}

	// 実行時に改めて確認
	private function validateInviteMemberId($groupid, $choiceForcibleAdmission, $memberIdList) {
		$members = $this->getMymembers($groupid, $choiceForcibleAdmission);
		$checkKeys = $this->GroupInvite->getCheckKeys($choiceForcibleAdmission);
		$memberIds = array();
		foreach ($memberIdList as $accountId) {
			foreach ($checkKeys as $checkKey) {
				if (isset($members[$checkKey][$accountId])) {
					$memberIds[] = $accountId;
				}
			}
		}
		return $memberIds;
	}

	private function errorRenderConfirm($groupid) {
		$accountIds = array();
		if (isset($this->data['ManageGroup']['adminInput'])) {
			$accountIds = array_keys($this->data['ManageGroup']['adminInput']);
		}
		if (isset($this->data['ManageGroup']['addMemberIdList'])) {
			$accountIds = array_merge($accountIds, explode(',', $this->data['ManageGroup']['addMemberIdList']));
		}
		$group = $this->GroupInvite->findGroup($groupid);
		$forcibleAdmission = false;
		$this->set('forcibleAdmission', $forcibleAdmission);
		$memberIds = $this->setConfirmPageData($group, $accountIds, $forcibleAdmission);
		$notAgreedMemberIds = $this->getGroupNotAgreedMemberIds($group, $memberIds);
		if ($notAgreedMemberIds) {
			$memberIds = $this->removeNotAgreedMemberid($memberIds, $notAgreedMemberIds);
		}
		$this->set('notAgreedMemberIds', $notAgreedMemberIds);
		$this->set('addMemberList', $memberIds);

		$this->set('ManageGroup', $this->data);
		$this->render('confirm');
	}

	/**
	 * 確認する。
	 * @param integer $id group id
	 */
	function confirm($id = null) {
		$this->data['ManageGroup']['id'] = $id;
		$this->setFavariteFiveGroup($id);
		$idpGroup = $this->AutoGroup->isIdPGroup($id);
		switch ($this->getSubmitName()) {
			case 'send': // メール送信
				$less = 0;
				if (isset($this->data['ManageGroup']['forcibleAdmission']) && isset($this->data['ManageGroup']['mailaddress_ml'])) {
					$this->cakeError('illegalAccess');
				}
				if (isset($this->data['ManageGroup']['forcibleAdmission'])) {
					$less = (int)$this->data['ManageGroup']['forcibleAdmission'];
					if ($less != 1 || !$this->GroupInvite->canForce() || $idpGroup) {
						$this->cakeError('illegalAccess');
					}
				}
				$this->data['ManageGroup']['forcibleAdmission'] = $less;
				if (!isset($this->data['ManageGroup']['mailaddress_ml'])) {
					$this->data['ManageGroup']['mailaddress_ml'] = '';
				}

				$adminInput = array();
				if (isset($this->data['ManageGroup']['adminInput'])) {
					$invalid_addresses = $this->GroupInvite->getInvalidAddresses($this->data['ManageGroup']['adminInput']);
					if ($invalid_addresses) {
						$this->Session->setFlash($this->createMessage('message.manage_groups.invite.invalid.email', implode(', ', $invalid_addresses)), 'message');
						return $this->errorRenderConfirm($id);
					}
					$adminInput = $this->data['ManageGroup']['adminInput'];
				}

				$group = $this->GroupInvite->findGroup($id);
				$forcibleAdmission = $this->GroupInvite->canForce();
				if ($group['sp']) {
					$forcibleAdmission = false;
				}
				$choiceForcibleAdmission = $forcibleAdmission && $this->data['ManageGroup']['forcibleAdmission'];

				$addMemberIdList = array();
				if (isset($this->data['ManageGroup']['addMemberIdList'])) {
					$addMemberIdList = explode(',', $this->data['ManageGroup']['addMemberIdList']);

					// グループへの招待の場合は同意の確認
					if (!$group['sp'] && !$choiceForcibleAdmission) {
						// 同意していないアカウントが混じる可能性がありうるのでチェック
						if ($this->TakerAgreement->getNotAgreedToMail($addMemberIdList)) {
							$this->cakeError('systemError');
							return;
						}
					}
				}
				if ($adminInput) {
					$addMemberIdList = array_merge($addMemberIdList, array_keys($adminInput));
				}

				if ($addMemberIdList) {
					// 選択したアカウントの最終チェック
					$addMemberIdList = $this->validateInviteMemberId($id, $choiceForcibleAdmission, $addMemberIdList);
					if (!$this->GroupInvite->validateMygroupMember($this->getAccountID(), $addMemberIdList)) {
						$this->set('ManageGroup', $this->data);
						$this->Session->setFlash(__("errors.illegal.access.message", true), 'message');
						$this->setInputFormData($id);
						$this->render('invite');
						return;
					}
				}

				$this->Transaction->begin();
				$result = $this->GroupInvite->execute(array(
					'accountId' => $this->getAccountID(),
					'group' => $group,
					'addresses' => explode("\n", $this->data['ManageGroup']['mailaddress']),
					'addressMl' => $this->data['ManageGroup']['mailaddress_ml'],
					'memberIdList' => $addMemberIdList,
					'adminInput' => $adminInput,
					'body' => $this->data['ManageGroup']['introduction'],
					'lang' => $this->data['ManageGroup']['defaultLang'],
					'accountLang' => $this->getLanguage(),
					'forcibleAdmission' => $choiceForcibleAdmission,
				));
				if (!$result) {
					$this->Transaction->rollback();
					$this->cakeError('systemError');
					return;
				}
				$this->Transaction->commit();
				if ($choiceForcibleAdmission) {
					$this->Session->setFlash(__("message.manage_groups.sent.mail.less", true), 'message');
				}
				else  {
					$this->Session->setFlash(__("message.manage_groups.sent.mail", true), 'message');
				}
				if (!$group['sp']) {
					$this->redirect('/group_lists/detail/'.$id);
				} else {
					$this->redirect('/sp_groups/detail/'.$id);
				}
				break;
			case 'back':
				$this->set('ManageGroup', $this->data);

				// addMemberIdListから除いたadminInputを戻す
				$addMemberIdList = array();
				if (isset($this->data['ManageGroup']['addMemberIdList'])) {
					$addMemberIdList = explode(',', $this->data['ManageGroup']['addMemberIdList']);
				}
				if (isset($this->data['ManageGroup']['adminInput'])) {
					$addMemberIdList = array_merge($addMemberIdList, array_keys($this->data['ManageGroup']['adminInput']));
				}
				$memberIdKeys = array();
				foreach ($addMemberIdList as $memberId) {
					$memberIdKeys[$memberId] = '1';
				}
				$this->data['ManageGroup']['member'] = $memberIdKeys;

				$this->setInputFormData($id);
				$this->render('invite');
				break;
			default:
				$this->redirect('/admin/manage_groups/invite/'.$id);	// 確認画面から戻るボタンで戻った後のタブと言語切り替え対応
				break;
		}
	}

	/**
	 * 招待メールを再送する。
	 * @param integer $inviteID
	 * @param integer $groupid
	 */
	function resend($inviteID, $groupid) {
		// 再送可能なIDかを検証
		$invite = $this->Invite->find('first', array(
			'conditions' => array('id' => $inviteID, 'groupid' => $groupid, 'status' => 0),
		));
		if (empty($invite)) {
			$this->cakeError('illegalAccess', array());
			return;
		}

		$group = $this->Group->find('first', array(
			'fields' => array('id', 'sp', 'group_key', 'contactadmin'),
			'conditions' => array('id' => $groupid)));
		if (empty($group)) {
			$this->cakeError('illegalAccess', array());
			return;
		}

		$orgInvite = array(
			'Invite' => array(
				'id' => $invite['Invite']['id'],
				'status' => 2,
			),
		);
		$this->Invite->save($orgInvite);

		$result = $this->GroupInvite->saveInvite(
			$this->getAccountID(),
			$invite['Invite']['address'], $groupid, $invite['Invite']['term'], $invite['Invite']['body'],
			$invite['Invite']['language'], $invite['Invite']['temp_account_id'],
			$invite['Invite']['id'], $invite['Invite']['invited_account_id']
		);

		$inviteAddress = $invite['Invite']['address'];
		// アカウントが選択された場合は、アカウントのメールアドレスへ送信する
		if ($invite['Invite']['invited_account_id']) {
			$account = $this->Account->find('first', array(
				'conditions' => array('id' => $invite['Invite']['invited_account_id']),
			));
			if ($account) {
				$inviteAddress = $account['Account']['mail'];
			}
		}

		$url = $this->GroupInvite->getInviteUrl($group['Group']['group_key']);
		$random = $result['Invite']['random'];

		// 再送者のメールアドレスをRely-Toに設定
		$replyTo = '';
		$account = $this->Account->find('first', array('conditions' => array('id' => $this->getAccountID())));
		if (isset($account['Account']['mail'])) {
			$replyTo = $account['Account']['mail'];
		}

		$this->GroupInvite->sendInviteMail(
			$url, $random, $inviteAddress, $replyTo, $group['Group']['sp'],
			$invite['Invite']['body'], $invite['Invite']['language'],
			$this->getLanguage()
		);

		$this->Session->setFlash(__("message.manage_groups.sent.mail", true), 'message');
		$this->redirect('/admin/manage_groups/invite_state/'.$groupid.'?'.$this->getInviteQuery());
	}

	private function getInviteQuery() {
		$page = isset($this->params['url']['page']) ? (int)$this->params['url']['page'] : 1;
		if ($page <= 0) { $page = 1; }
		$filter = isset($this->params['url']['filter']) ? (int)($this->params['url']['filter']) : 0;
		if ($filter < 0 || $filter > 4) { $filter = 0; }
		$word = isset($this->params['url']['word']) ? trim($this->params['url']['word']) : '';
		return 'word='.$word.'&page='.$page.'&filter='.$filter;
	}

	/**
	 * グループ招待状況を表示する。
	 * @param integer $groupid
	 */
	function invite_state($groupid) {
		if (!ctype_digit(strval($groupid))) {
			$this->cakeError('notFound', array());
			return;
		}

		$page = isset($this->params['url']['page']) ? (int)$this->params['url']['page'] : 1;
		if ($page <= 0) { $page = 1; }
		$filter = isset($this->params['url']['filter']) ? (int)($this->params['url']['filter']) : 0;
		if ($filter < 0 || $filter > 4) { $filter = 0; }
		$this->set('filter', $filter);
		$word = isset($this->params['url']['word']) ? trim($this->params['url']['word']) : '';
		$this->set('word', $word);

		$conditions = array('groupid' => $groupid);
		$inviteAllCount = $this->Invite->find('count', array(
			'conditions' => $conditions));
		$this->set('inviteAllCount', $inviteAllCount);

		if ($filter == 1) { // 招待中
			$start = date("Y-m-d H:i:s", strtotime("-1 week"));
			$conditions = array('groupid' => $groupid, 'status' => 0,
				array('OR' => array('term' => 'onetime', "created >=" => $start)));
		}
		else if ($filter == 2) { // 入会あり
			$conditions = array('groupid' => $groupid, 
				array('OR' => array('status' => '1', "counter >=" => 1)));
		}
		else if ($filter == 3 || $filter == 4) { // 入会済み(退会済みを除く)と入会済みかつ退会済み
			$sub = "(select account_id from mygroups where groupid = $groupid)";
			$invSub = 'invited_account_id in ' . $sub;
			$joinSub = 'joined_account_id in ' . $sub;
			if ($filter == 3) {
				$conditions = array('groupid' => $groupid, 'status' => 1,
					array('OR' => array($invSub, $joinSub)));
			}
			else {
				$conditions = array('groupid' => $groupid, 'status' => 1,
					array( 'OR' => array(
							array('NOT' => array($invSub)),
							array('NOT' => array($joinSub)))));
			}
		}

		$word = str_replace('　', ' ', $word);
		$word = trim($word);
		$word = preg_replace('/\s+/', ' ', $word);
		if ($word != '') {
			$word = str_replace(array('\\', '%', '_'), array('\\\\', '\%', '\_'), $word);	// '\'と'%'と'_'のエスケープ
			$words = explode(' ',$word);
			$sub = "(select id from accounts where id in (select invited_account_id from invites where groupid=$groupid and invited_account_id is not null"
				." union"
				." select joined_account_id from invites where groupid=$groupid and joined_account_id is not null)";
			foreach ($words as $wd) {
				$wd = mysql_real_escape_string($wd);
				if ($this->getLanguage() === 'ja') {
					$sub .= ' and ((name like \'%' . $wd . '%\') or ((IFNULL(name, \'\') = \'\') and (name_en like \'%' . $wd . '%\')))';
				}
				else {
					$sub .= ' and ((name_en like \'%' . $wd . '%\') or ((IFNULL(name_en, \'\') = \'\') and (name like \'%' . $wd . '%\')))';
				}
			}
			$sub .= ')';
			$invSub = 'invited_account_id in ' . $sub;
			$joinSub = 'joined_account_id in ' . $sub;
			
			$wordCond = array('invited_account_id' => NULL);
			$wordCond['AND'] = array();
			foreach ($words as $wd) {
				$wordCond['AND'][] = array('address like' => '%'.$wd.'%');
			}
			$conditions['OR'] = array(
				$invSub,
				$joinSub,
				'AND' => $wordCond);
		}

		$inviteCount = $this->Invite->find('count', array(
			'conditions' => $conditions));

		$invites = $this->Invite->find('all', array(
			'fields' => array('id', 'address', 'status', 'modified', 'created', 'term', 'counter',
				'resend_id', 'LENGTH(body)', 'invited_account_id', 'forcible', 'joined_account_id'),
			'conditions' => $conditions,
			'order' => array('created' => 'desc'),
			'limit' => Configure::read("page.statelimit"),
			'page' => $page));

		// 既存グループから招待した人の名前を設定
		$this->getAccountNames($invites, 'invited_account_id', 'name');

		// メールアドレス（個人）から招待した人の名前を設定
		$this->getAccountNames($invites, 'joined_account_id', 'joined_name');

		$manageGroup['ManageGroup']['id'] = $groupid;
		$this->set('ManageGroup', $manageGroup);
		$this->data['ManageGroup']['id'] = $groupid;
		$this->set('invites', $invites);

		// ページ制御
		$this->controllPage($page, $inviteCount, Configure::read("page.statelimit"));
		$this->set('page', $page);

		$group = $this->Group->find('first', array('fields' => array('sp'),
				'conditions' => array('id' => $groupid)));
		$this->data['ManageGroup']['sp'] = $group['Group']['sp'];
		$this->data['ManageGroup']['idp'] = $this->Group->isIdPGroup($groupid);
		$this->setFavariteFiveGroup($groupid);
	}

	private function getAccountNames(&$invites, $columnKey, $outputKey) {
		$invitedAccountIdList = array();
		foreach ($invites as $invite) {
			if (!empty($invite['Invite'][$columnKey])) {
				array_push($invitedAccountIdList, $invite['Invite'][$columnKey]);
			}
		}
		if (!empty($invitedAccountIdList)) {
			$invitedAccounts = $this->getInvitedAccounts($invitedAccountIdList);
			foreach ($invites as $invite_key => $invite) {
				$invitedAccountId = $invite['Invite'][$columnKey];
				if (empty($invitedAccountId)) {
					continue;
				}
				if (!isset($invitedAccounts[$invitedAccountId])) {	// 	アカウント削除済み
					if ($outputKey == 'name') {
						$invites[$invite_key]['Invite'][$outputKey] = '-';
					}
					continue;
				}

				$account = $invitedAccounts[$invitedAccountId];
				if ($this->getLanguage() === 'ja')
					$name = $account['name'] != '' ? $account['name'] : $account['name_en'];
				else
					$name = $account['name_en'] != '' ? $account['name_en'] : $account['name'];

				$invites[$invite_key]['Invite'][$outputKey] = $name;
			}
		}
	}

	private function getInvitedAccounts($idList) {
		$accountList = $this->Account->find('all', array('conditions' => array('id' => $idList)));
		$results = array();
		foreach ($accountList as $account) {
			$results[$account['Account']['id']] = $account['Account'];
		}
		return $results;
	}

	/**
	 * グループを再開する。
	 * @param integer $id group ID
	 */
	function resume($id) {
		switch ($this->getSubmitName()) {
			case '':
				$this->_show($id);
				break;
			case 'resume':
				$group = $this->ManageGroup->find('first', array('conditions' => array('ManageGroup.id' => $id)));
				$group['ManageGroup']['active'] = 0;

				if ($this->ManageGroup->save($group)) {
					$this->Session->setFlash(__("message.manage_groups.opened.group", true), 'message');
					$this->redirect('/group_lists/detail/'.$id);
				}
				break;
			case 'cancel':
				$this->redirect('/group_lists/detail/'.$id);
				return;
			default:
				break;
		}
	}

	/**
	 * グループを休止する。
	 * @param integer $id group ID
	 */
	function suspend($id) {
		switch ($this->getSubmitName()) {
			case '':
				$this->_show($id);
				break;
			case 'suspend':
				$group = $this->ManageGroup->find('first', array('conditions' => array('ManageGroup.id' => $id)));
				$group['ManageGroup']['active'] = 1;

				if ($this->ManageGroup->save($group)) {
					$this->Session->setFlash(__("message.manage_groups.suspend.group", true), 'message');
					$this->redirect('/group_lists/detail/'.$id);
				}
				break;
			case 'cancel':
				$this->redirect('/group_lists/detail/'.$id);
				return;
			default:
				break;
		}
	}

	/**
	 * グループを削除する。
	 * @param $groupid
	 */
	function delete($groupid) {
		// タブ化対応
		$this->setFavariteFiveGroup($groupid);

		switch ($this->getSubmitName()) {
			case '':
				$this->_show($groupid);
				break;
			case 'delete':
				$this->Transaction->begin();
				// グループ削除
				if (!$this->NoticeUtils->deleteGroup($groupid)) {
					$this->Transaction->rollback();
					$this->log('ManageGroupsController::delete deleteGroup() Error');
					$this->cakeError('systemError');
					return;
				}
				$this->Transaction->commit();

				$this->Session->setFlash(__("message.manage_groups.del.group", true), 'message');
				$this->redirect('/mygroups/view');
				break;
			case 'cancel':
				$this->redirect('/group_lists/detail/'.$groupid);
				return;
			default:
				break;
		}
	}

	/**
	 * グループを表示する。
	 * @param @param integer $id group ID
	 */
	private function _show($id) {
		$this->set('ManageGroup', $this->ManageGroup->find('first',
			array('conditions' => array('ManageGroup.id' => $id)))
		);
	}

	/**
	 * 「上にSPコネクタを追加」を表示する。
	 * @param integer $groupid 自身のグループID
	 */
	function add_up_sp($groupid) {
		// 自グループ情報を設定
		$ManageGroup = $this->ManageGroup->find('first', array(
			'fields' => array('group_key', 'id', 'independence', 'sp_type'),
			'conditions' => array('id' => $groupid)));
		$this->set('ManageGroup', $ManageGroup);
		$this->data['ManageGroup']['id'] = $groupid;

		// このグループの上になることが可能なSPコネクタを検索する。
		//   SPコネクタは、down_permission = 1 なら上になれ、
		//   既にこのグループの上グループになっていたら追加できない。
		// sp = 1, down_permission = 1
		// 自分のグループじゃない
		// SPコネクタ
		$idpGroup = $this->AutoGroup->isIdPGroup($groupid);
		if (!$idpGroup) {
			$sgroups = $this->ManageGroup->find("all", array(
				'fields' => array('id', 'group_key', 'name', 'name_en', 'introduction', 'introduction_en', 'down_permission', 'spc_icon'),
				'conditions' => array(
					'active' => 0,
					'sp' => 1,
					'(public = 1 OR id IN (select groupid from mygroups where account_id=? and admin in (1,2)))' => $this->getAccountID(),
					'down_permission' => array(1, 2),
					'id NOT IN (select top_id from add_groups where bottom_id = ?)' => $groupid,
					'NOT' => array('sp_type' => 1),
				),
				'order' => array('name')
			));
		} else {
			$sgroups = $this->ManageGroup->find("all", array(
				'fields' => array('id', 'group_key', 'name', 'name_en', 'introduction', 'introduction_en', 'down_permission', 'spc_icon'),
				'conditions' => array(
					'active' => 0,
					'sp' => 1,
					'(public = 1 OR id IN (select groupid from mygroups where account_id=? and admin in (1,2)))' => $this->getAccountID(),
					'down_permission' => array(1, 2),
					'id NOT IN (select top_id from add_groups where bottom_id = ?)' => $groupid,
				),
				'order' => array('name')
			));
		}

		$this->setAddGroupView($sgroups, 'SpGroupOptions', 'up');

		$spcIdList = $this->getSpcIdList($sgroups);
		$groupSpHosts = $this->getGroupSpHostsByGroupId($spcIdList);
		$this->set('groupSpHosts', $groupSpHosts);
		// タブ化対応
		$this->setFavariteFiveGroup($groupid);

		// 機関グループ判定
		$isIdpGroup = $this->AutoGroup->isIdPGroup($groupid);
		$this->set('isIdpGroup', $isIdpGroup);
	}

	private function getSpcIdList($sgroups)
	{
		$spcIdList = array();
		foreach ($sgroups as $entity) {
			$spcIdList[] = $entity['ManageGroup']['id'];
		}
		return $spcIdList;
	}

	private function getGroupSpHostsByGroupId($group_id)
	{
		$sp_hosts = $this->GroupSphost->find('all', array(
			'joins' => array(array(
				'type' => 'INNER', 'table' => 'sp_hosts', 'alias' => 'SpHosts',
				'conditions' => array(
					'GroupSphost.sp_id = SpHosts.id',
				),
			)),
			'fields' => array('GroupSphost.group_id', 'GroupSphost.sp_id', 'SpHosts.service_name', 'SpHosts.service_name_en'),
			'conditions' => array('group_id' => $group_id))
		);

		# array(
		#	$group_id => array(
		#		$sp_id => array(
		#			'service_name' => '',
		#			'service_name_en' => '',
		#		),
		#	),
		# );
		if (!is_array($group_id)) {
			$group_id = array($group_id);
		}
		$groupSpHosts = array();
		foreach ($group_id as $id) {
			$groupSpHosts[$id] = array();
		}

		$is_ja = $this->getLanguage() === 'ja';
		foreach ($sp_hosts as $entity) {
			if ($is_ja) {
				if (!empty($entity['SpHosts']['service_name']))
					$entity['SpHosts']['service_name_en'] = '';
			} else {
				if (!empty($entity['SpHosts']['service_name_en']))
					$entity['SpHosts']['service_name'] = '';
			}
			$group_id = $entity['GroupSphost']['group_id'];
			$groupSpHosts[$group_id][$entity['GroupSphost']['sp_id']] = $entity['SpHosts'];
		}
		return $groupSpHosts;
	}

	/**
	 * 上下グループを表示する。
	 * @param $groupid
	 */
	function updown($groupid) {
		$group = $this->ManageGroup->find('first', array('fields' => array('independence'),
			'conditions' => array('id' => $groupid)));

		$group['ManageGroup']['id'] = $groupid;
		$this->set("ManageGroup", $group);

		// 上SPコネクタを検索
		$topGroups = $this->Group->find('all', array(
			'fields' => array('id', 'name', 'name_en', 'lead_url', 'introduction', 'introduction_en', 'spc_icon',
				'active'),
			'conditions' => array(
				'active' => array(0, 1),
				'sp' => 1,
				'id IN (select top_id from add_groups where bottom_id = ?)' => $groupid),
			'order' => array('name'),
		));
		foreach ($topGroups as $key => $group) {
			if ($this->ExtendedAttribute->hasAny(array('spc_id' => $group['Group']['id'], 'kind' => 1))) {
				$topGroups[$key]['Group']['extended'] = 1;
			}
		}
		$this->set('topGroups', $topGroups);
		// タブ化対応
		$this->setFavariteFiveGroup($groupid);
	}

	/**
	 * グループを上に接続できるかを確認する。
	 * 共通       → 公開されている or 自分か管理者なら追加可能、接続されていないなら接続可能
	 * SPコネクタ →
	 * グループ   →   下位グループになっていない なら追加可能
	 * @param unknown_type $myGroupid
	 * @param unknown_type $selectedGroupId
	 * @param unknown_type $sp
	 * @return 接続できるかどうか
	 */
	private function validate_add_save_up($myGroupid, $selectedGroupId, $sp) {
		if (!$sp) {
			// 下位グループに申請しているかを確認
			$count = $this->AddGroupInspect->find('count', array('conditions' => array(
				'top_id' => $myGroupid,
				'join' => 0,
				'kind' => 1,  // 自分が上になりたい
				'status' => 0
			)));
			if ($count > 0) {
				$this->Session->setFlash(__('message.manage_groups.not.add.up.group.applying', true), 'message');
				$this->redirect('/admin/manage_groups/updown/'.$myGroupid);
				return;
			}

			// 自グループが上位グループなら、上位グループと接続できない
			$count = $this->AddGroup->find('count', array('conditions' => array(
				'top_id' => $myGroupid, 'top' => 0)));
			if ($count > 0)
				return false;
		}

		$groupIdList = array();
		if (!is_array($selectedGroupId)) {
			$groupIdList = array($selectedGroupId);
		} else {
			foreach ($selectedGroupId as $id) {
				array_push($groupIdList, $id);
			}
		}

		foreach ($groupIdList as $groupId) {
			// 同じグループIDは接続できない
			if ($myGroupid == $groupId) {
				return false;
			}

			$group = $this->Group->find('first', array(
				'fields' => array('id', 'public', 'active', 'down_permission', 'independence', 'sp'),
				'conditions' => array('id' => $groupId)));
			$mygroup = $this->Mygroup->find('first', array('fields' => array('groupid', 'admin'),
				'conditions' => array('groupid' => $groupId)));
			// 公開していなく、管理者でもない
			if ($group['Group']['public'] != 1) {
				if ($mygroup['Mygroup']['admin'] == 0) {
					return false;
				}
			}

			// 既に削除されていたら接続できない
			if ($group['Group']['active'] == 9) {
				return false;
			}

			// 独立していたら接続できない(グループのみ)
			if ($group['Group']['sp'] == 0 && $group['Group']['independence'] == 1) {
				return false;
			}

			// 上位グループなることを許可していないなら接続できない(グループのみ)
			if ($group['Group']['sp'] == 0 && $group['Group']['down_permission'] == 0) {
				return false;
			}

			// 既に選択したSPコネクタ(グループ)と接続しているか確認
			$count = $this->AddGroup->find('count', array(
				'conditions' => array('top_id' => $groupId, 'bottom_id' => $myGroupid)));
			if ($count > 0) {
				return false;
			}

			// グループ
			if (!$sp) {
				// 下位グループになっているか確認(SPコネクタの下位は問題ない)
				$count = $this->AddGroup->find('count', array(
					'conditions' => array('bottom_id' => $groupId, 'top' => 0)));
				if ($count > 0) {
					return false;
				}
			}
		}

		return true;
	}

	/**
	 * SPコネクタの接続確認を行う。
	 * @param $myGroupid
	 */
	function add_up_sp_confirm($myGroupid) {
		if ($this->getSubmitName() == "cancel") {
			$this->redirect('/admin/manage_groups/updown/'.$myGroupid);
			return;
		}

		$selectedGroupIdList = array();
		if (!empty($this->data['ManageGroup']['spGroup'])) {
			if (!is_array($this->data['ManageGroup']['spGroup'])) {
				$selectedGroupIdList = array($this->data['ManageGroup']['spGroup']);
			} else {
				foreach ($this->data['ManageGroup']['spGroup'] as $id) {
					array_push($selectedGroupIdList, $id);
				}
			}
		}
		if (empty($selectedGroupIdList) || empty($this->data['ManageGroup']['spGroup'])) {
			$this->redirect('/admin/manage_groups/add_up_sp/'.$myGroupid);
			return;
		}

		// 機関グループの場合は、いきなり接続させる
		$isIdpGroup = $this->AutoGroup->isIdPGroup($myGroupid);
		if ($isIdpGroup) {
			// SPコネクタと機関グループを接続
			$this->Transaction->begin();
			foreach ($selectedGroupIdList as $selectedGroupId) {
				// SPコネクタなのかをチェック
				$isSpc = $this->SpGroup->isSpConnector($selectedGroupId)
					&& $this->SpGroup->isActive($selectedGroupId);
				if (!$isSpc) {
					continue;
				}

				$this->AddGroup->create();
				$addGroup = array();
				$addGroup['AddGroup']['top_id'] = $selectedGroupId;
				$addGroup['AddGroup']['bottom_id'] = $myGroupid;
				$addGroup['AddGroup']['top'] = 1;
				$addGroup['AddGroup']['bottom'] = 0; // 下になるのは自分なので、グループ
				$result = $this->AddGroup->save($addGroup);

				// サービスの接続ログ保存
				$this->AddGroupsSpcLog->create();
				$sLog = array(
					'kind' => 1,
					'group_id' => $myGroupid,
					'spc_id' => $selectedGroupId,
					'add_group_id' => $this->AddGroup->getLastInsertID(),
					'account_id' => $this->getAccountID(),
				);
				$result = $this->AddGroupsSpcLog->save(array('AddGroupsSpcLog' => $sLog));
			}
			$this->Transaction->commit();
			$this->redirect('/admin/manage_groups/updown/'.$myGroupid);
			return;
		}

		$selectedIdString = '';
		foreach ($selectedGroupIdList as $selectedGroupId) {
			// そもそも追加できるSPコネクタなのかをチェック
			if (!$this->validate_add_save_up($myGroupid, $selectedGroupId, true)) {
				$spConnector = $this->Group->find('first', array(
					'fields' => array('id', 'name'),
					'conditions' => array('id' => $selectedGroupId))
				);
				$msgstr = $this->createMessage("message.manage.cannot.connect.sp", $spConnector['Group']['name']);
				$this->Session->setFlash($msgstr, 'message');
				$this->redirect('/admin/manage_groups/add_up_sp/'.$myGroupid);
				return;
			}

			// 既に申請したいたら、何もしない
			$count = $this->AddGroupInspect->find('count', array('conditions' => array(
				'top_id' => $selectedGroupId,
				'bottom_id' => $myGroupid,
				'join' => 0,
				'kind' => 0,  // 自分が下になりたい
				'status' => 0
			)));
			if ($count > 0) {
				$spConnector = $this->Group->find('first', array(
					'fields' => array('id', 'name'),
					'conditions' => array('id' => $selectedGroupId))
				);
				$msgstr = $this->createMessage("message.manage.already.connect.application", $spConnector['Group']['name']);
				$this->Session->setFlash($msgstr, 'message');
				$this->redirect('/admin/manage_groups/add_up_sp/'.$myGroupid);
				return;
			}

			if (!empty($selectedIdString)) {
				$selectedIdString = $selectedIdString.',';
			}
			$selectedIdString = $selectedIdString.$selectedGroupId;
		}


		// 選択したSPコネクタに問題がないので確認画面を表示
		$provideAttributeList = $this->TakerAgreement->serveSpcAgreementInfo(
			$selectedGroupIdList, $this->getLanguage());
		$this->set('provideAttributeList', $provideAttributeList);
		$this->data['ManageGroup']['id'] = $myGroupid;
		$this->data['ManageGroup']['selectedSpc'] = $selectedIdString;
		$this->data['ManageGroup']['spc'] = 'true';

		// タブ化対応
		$this->setFavariteFiveGroup($myGroupid);

		// 非mAP対応用SPコネクタのサービス名表示表
//		$groupSpHosts = $this->getGroupSpHostsByGroupId($selectedGroupIdList);
//		$this->set('groupSpHosts', $groupSpHosts);
		$nomAPserviceList = array();
		foreach ($selectedGroupIdList as $gid) {
			$non = false;
			foreach ($provideAttributeList as $provide) {
				if ($gid == $provide['ProvideAttribute']['group_id']) {
					$non = true;
					break;
				}
			}
			if ($non) {
				continue;
			}

			$serviceList = $this->GroupSphost->getServiceNameList($gid);
			$nomAPserviceList[$gid] = array(
				'name' => $serviceList,
				'spc_icon' => $this->Group->getIcon($gid),
			);
		}
		$this->set('nomAPserviceList', $nomAPserviceList);

		$this->render('add_up_sp_agreement');
	}

	/**
	 * 該当SPコネクタを自分の上に追加する。
	 * 追加になるグループIDなら追加し、追加するのに申請の必要があったら申請する。
	 * @param $myGroupid
	 */
	function add_up_sp_save($myGroupid) {
		if ($this->getSubmitName() == "cancel") {
			$this->redirect('/admin/manage_groups/updown/'.$myGroupid);
			return;
		}

		$selectedSpcIdList = explode(',', $this->data['ManageGroup']['selectedSpc']);
		if (empty($selectedSpcIdList)) {
			$this->redirect('/admin/manage_groups/updown/'.$myGroupid);
			return;
		}
		if (empty($this->data['ManageGroup']['spc'])) {
			$this->redirect('/admin/manage_groups/updown/'.$myGroupid);
			return;
		}

		// 通常グループ(IdPグループ以外)ではmAP非対応グループは接続できない
		$idpGroup = $this->AutoGroup->isIdPGroup($myGroupid);

		foreach ($selectedSpcIdList as $selectedSpcId) {
			// 通常グループ(IdPグループ以外)ではmAP非対応グループは接続できない
			if (!$idpGroup) {
				$spConnector = $this->Group->find('first', array(
						'fields' => array('id', 'name', 'sp_type'),
						'conditions' => array('id' => $selectedSpcId))
				);
				if ($spConnector['Group']['sp_type'] == 1) {
					$this->cakeError('cannotAccess', array('url' => $this->here));
					return;
				}
			}
			// そもそも追加できるSPコネクタなのかをチェック
			if (!$this->validate_add_save_up($myGroupid, $selectedSpcId, true)) {
				$spConnector = $this->Group->find('first', array(
					'fields' => array('id', 'name'),
					'conditions' => array('id' => $selectedSpcId))
				);
				$msgstr = $this->createMessage("message.manage.cannot.connect.sp", $spConnector['Group']['name']);
				$this->Session->setFlash($msgstr, 'message');
				$this->redirect('/admin/manage_groups/updown/'.$myGroupid);
				return;
			}

			// 既に申請したいたら、何もしない
			$count = $this->AddGroupInspect->find('count', array('conditions' => array(
				'top_id' => $selectedSpcId,
				'bottom_id' => $myGroupid,
				'join' => 0,
				'kind' => 0,  // 自分が下になりたい
				'status' => 0
			)));
			if ($count > 0) {
				$spConnector = $this->Group->find('first', array(
					'fields' => array('id', 'name'),
					'conditions' => array('id' => $selectedSpcId))
				);
				$msgstr = $this->createMessage("message.manage.already.connect.application", $spConnector['Group']['name']);
				$this->Session->setFlash($msgstr, 'message');
				$this->redirect('/admin/manage_groups/add_up_sp/'.$myGroupid);
				return;
			}
		}

		foreach ($selectedSpcIdList as $selectedSpcId) {
			$selectedSpcInfo = $this->Group->find("first", array('fields' => array('id', 'name', 'down_permission', 'sp'),
					'conditions' => array('id' => $selectedSpcId)));

			$this->Transaction->begin();
			// SPコネクタを接続
			$resultConnect = $this->upSave($myGroupid, $selectedSpcInfo, $selectedSpcId, 1);
			if (!$resultConnect) {
				$this->Transaction->rollback();
				$this->cakeError('systemError');
				return;
			}
			if ($selectedSpcInfo['Group']['down_permission'] == 2) {
				$this->setFlash(__("message.manage_groups.request.up.sp.application", true).' ('.
					h($selectedSpcInfo['Group']['name']).')', 'message');
			} else {
				$msg = sprintf(__('message.manage_groups.done.up.sp.group', true), h($selectedSpcInfo['Group']['name']));
				$this->setFlash($msg, 'message');
			}

			// 接続申請しないで接続の場合
			//   @note 接続申請ありで接続の場合、接続時の情報を保存(inspect_attributes)しておき
			//         申請が承認された時点で同意処理を行う
			if ($selectedSpcInfo['Group']['down_permission'] == 1) {
				// 利用属性を登録
				$attributeList = $this->getCheckAttributeList($this->data['ManageGroup'], $selectedSpcId);
//				$this->debugLog('ManageGroupsController::add_up_sp_save $selectedSpcId = '.$selectedSpcId.' $attributeList = '.print_r($attributeList, true));
				$resultAttribute = $this->upSaveAttribute($myGroupid, $attributeList, $selectedSpcId);


				if (!$resultAttribute) {
					$this->Transaction->rollback();
					$this->cakeError('systemError');
					return;
				}

				// 下位グループに同意データを登録、下位グループ管理者がOKすると、下位グループメンバーに連絡される
				$agreeid = $this->AgreementOperator->registerBottomAdminByConnectSPC($myGroupid, $selectedSpcId);
				if (!$agreeid) {
					$this->Transaction->rollback();
					$this->cakeError('systemError');
					return;
				}
			}
			$this->Transaction->commit();

			// 上でコミットするため同一のif文がある
			// メール送信は別スレットなので、上でコミットしないとメール本文が意図した内容にならない
			if (!$idpGroup && $selectedSpcInfo['Group']['down_permission'] == 1) {
				// グループメンバーに同意する必要があることをメールする
				$attributePath = $this->getSelectCheckAttribute($this->data['ManageGroup'], $selectedSpcId);
//				$this->debugLog('ManageGroupsController::add_up_sp_save $selectedSpcId = '.$selectedSpcId.' $attributePath = '.print_r($attributePath, true));
//				$this->debugLog("ManageGroupsController::add_up_sp_save url = /map/batch_send_mails/agreement_mail/$myGroupid/0/$selectedSpcId/$attributePath");
				$this->Url->get("/map/batch_send_mails/agreement_mail/$myGroupid/0/$selectedSpcId$attributePath");

				// 下位グループ(複数)の管理者へ連絡
				$this->AgreementOperator->sendMailBottomAdmin(
					$myGroupid, $selectedSpcId, $agreeid, 'add_group_conntact_bottom_group_template');

				// 接続したSPコネクタの属性に同意していなかったらメッセージ表示
				$result = $this->TakerAgreement->needSpcConnectAgree($selectedSpcId, $myGroupid, $this->getAccountID());
				if (!empty($result)) {
					$this->setAgreement('false');
				}
			}
			// 追加先の管理者へメールをする

			$this->sedMailForAddGroup($selectedSpcId, $myGroupid, 'down',
				$selectedSpcInfo['Group']['down_permission'], true);

			if ($selectedSpcInfo['Group']['down_permission'] == 1) {
				// calbackUrlを取得
				$group = $this->Group->findGroup($myGroupid);
				$groupKey = $group['Group']['group_key'];
				$serviceInfoList = $this->GroupSphost->findList($selectedSpcId);
				foreach ($serviceInfoList as $serviceInfo) {
					$spid = $serviceInfo['GroupSphost']['sp_id'];
					$callbackUrl = $serviceInfo['GroupSphost']['callback_url'];
					if (empty($callbackUrl)) {
						continue;
					}
					$callbackUrl = $this->ParameterReplacer->replaceGroupKey($groupKey, $callbackUrl);
//					$callbackUrl = $this->ParameterReplacer->replaceEntityId($callbackUrl);
					$account = $this->Account->find('first', array('conditions' => array('id' => $this->getAccountID())));
					$callbackUrl = $this->ParameterReplacer->replaceAdminEmail($account['Account']['mail'], $callbackUrl);
					$this->log('$callbackUrl = ' . $callbackUrl, "debug");
					// サービスのAPI呼び出し
					$json = $this->Callback->call($spid, $callbackUrl);
					if (!isset($json) || ($json['result'] != 'OK' && $json['result'] != 'ok')) {
						$msg = sprintf(__('message.failed.service.api2', true), h($serviceInfo['GroupSphost']['service_name']));
						$this->Session->setFlash($msg, 'message');
					}
				}
			}
		}

		// 次画面遷移、承認あり
		if (count($selectedSpcIdList) == 1) {
			if ($selectedSpcInfo['Group']['down_permission'] == 2) {
				$this->redirect('/admin/manage_inspects/group_inspect_sp_state/'.$myGroupid);
			}
		}

		// 次画面遷移、複数のSPコネクタを選択または、承認なしの場合
		$this->redirect('/admin/manage_groups/updown/'.$myGroupid);
	}

	/**
	 * 選択した属性をパスにして返します。
	 * @param unknown_type $spAttributeList
	 */
	private function getSelectAttribute($spAttributeList, $selectedGroupId) {
		$return = '';
		if (isset($spAttributeList['ismemberof'.$selectedGroupId][0]))
			$return = $return.'/1';
		else
			$return = $return.'/0';

		if (isset($spAttributeList['eptid'.$selectedGroupId][0]))
			$return = $return.'/1';
		else
			$return = $return.'/0';

		if (isset($spAttributeList['name'.$selectedGroupId][0]))
			$return = $return.'/1';
		else
			$return = $return.'/0';

		if (isset($spAttributeList['mail'.$selectedGroupId][0]))
			$return = $return.'/1';
		else
			$return = $return.'/0';

		if (isset($spAttributeList['eppn'.$selectedGroupId][0]))
			$return = $return.'/1';
		else
			$return = $return.'/0';

		if (isset($spAttributeList['idp'.$selectedGroupId][0]))
			$return = $return.'/1';
		else
			$return = $return.'/0';

		if (isset($spAttributeList['introduction'.$selectedGroupId][0]))
			$return = $return.'/1';
		else
			$return = $return.'/0';

		if (isset($spAttributeList['language'.$selectedGroupId][0]))
			$return = $return.'/1';
		else
			$return = $return.'/0';

		return $return;
	}

	/**
	 * チェックを入れた属性をパスにして返します。
	 * @param unknown_type $spAttributeList
	 */
	private function getSelectCheckAttribute($spAttributeList, $selectedGroupId) {
		$return = '';
		if (isset($spAttributeList['ismemberof'.$selectedGroupId][0])
			&& $spAttributeList['ismemberof'.$selectedGroupId][0] > 0)
			$return = $return.'/1';
		else
			$return = $return.'/0';

		if (isset($spAttributeList['eptid'.$selectedGroupId][0])
			&& $spAttributeList['eptid'.$selectedGroupId][0] > 0)
			$return = $return.'/1';
		else
			$return = $return.'/0';

		if (isset($spAttributeList['name'.$selectedGroupId][0])
			&& $spAttributeList['name'.$selectedGroupId][0] > 0)
			$return = $return.'/1';
		else
			$return = $return.'/0';

		if (isset($spAttributeList['mail'.$selectedGroupId][0])
			&& $spAttributeList['mail'.$selectedGroupId][0] > 0)
			$return = $return.'/1';
		else
			$return = $return.'/0';

		if (isset($spAttributeList['eppn'.$selectedGroupId][0])
			&& $spAttributeList['eppn'.$selectedGroupId][0] > 0)
			$return = $return.'/1';
		else
			$return = $return.'/0';

		if (isset($spAttributeList['idp'.$selectedGroupId][0])
			&& $spAttributeList['idp'.$selectedGroupId][0] > 0)
			$return = $return.'/1';
		else
			$return = $return.'/0';

		if (isset($spAttributeList['introduction'.$selectedGroupId][0])
			&& $spAttributeList['introduction'.$selectedGroupId][0] > 0)
			$return = $return.'/1';
		else
			$return = $return.'/0';

		if (isset($spAttributeList['language'.$selectedGroupId][0])
			&& $spAttributeList['language'.$selectedGroupId][0] > 0)
			$return = $return.'/1';
		else
			$return = $return.'/0';

		return $return;
	}

	/**
	 * 該当グループを自分の上に追加する
	 * @param unknown_type $myGroupid 自分のグループID
	 * @param unknown_type $selectedGroupInfo 選択したグループ情報
	 * @param unknown_type $selectedGroupId 選択したグループID
	 * @param unknown_type $sp 上になるグループ(0:グループ単体, 1:SPコネクタ, SPコネクタと接続しているグループ
	 * @param $agreement メンバーから接続の同意を得ているか true：得ている
	 * @param $spAttributeList SPコネクタの利用属性に対する同意情報
	 * @param $selectedSpcIdList 接続しているSPコネクタID一覧(上位グループに接続時に使用する)
	 * @return 登録結果 true：成功, false：失敗
	 *
	 */
	private function upSave($myGroupid, $selectedGroupInfo, $selectedGroupId, $sp, $selectedSpcIdList = array()) {
		if ($selectedGroupInfo['Group']['down_permission'] == 2) {
			// 選択したグループが、このグループを下グループに追加するのに承認が必要なら申請する
			$groupInspect = array();
			$groupInspect['AddGroupInspect']['account_id'] = $this->getAccountID();
			$groupInspect['AddGroupInspect']['top_id'] = $selectedGroupId;
			$groupInspect['AddGroupInspect']['bottom_id'] = $myGroupid;
			$groupInspect['AddGroupInspect']['to_id'] = $selectedGroupId;   // 申請先グループID
			$groupInspect['AddGroupInspect']['from_id'] = $myGroupid; // 申請元グループID
			$groupInspect['AddGroupInspect']['join'] = 0;
			$groupInspect['AddGroupInspect']['kind'] = 0; // 下になりたい申請
			$groupInspect['AddGroupInspect']['status'] = 0;
			$groupInspect['AddGroupInspect']['top'] = $sp;    // 上になるのはSPコネクタ or グループ
			$groupInspect['AddGroupInspect']['bottom'] = 0; // 下になるのは自分なので、グループ
			$this->AddGroupInspect->create();
			$result = $this->AddGroupInspect->save($groupInspect);
			if (!$result) {
				return false;
			}
			$inspectId = $this->AddGroupInspect->getLastInsertID();

			// 接続申請したなら、その時の属性を保存しておく
			if (empty($selectedSpcIdList)) {
				// SPコネクタとの接続時
				$attributeList = $this->getCheckAttributeList($this->data['ManageGroup'], $selectedGroupId);

				$entity = $this->InspectAttribute->generateEntity($attributeList, $inspectId, $myGroupid, $selectedGroupId);
			} else {
				// 上位グループとの接続時
				$entity = array();
				foreach ($selectedSpcIdList as $id) {
					$attributeList = $this->getCheckAttributeList($this->data['ManageGroup'], $id);
					$attr = $this->InspectAttribute->generateEntity($attributeList, $inspectId, $myGroupid, $selectedGroupId);
					if (empty($entity)) {
						$entity = $attr;
					} else {
						$entity = $this->InspectAttribute->marge($entity, $attr);
					}
				}
			}
			$this->InspectAttribute->create();
			$result = $this->InspectAttribute->save($entity);
			if (!$result) {
				return false;
			}

			return true;
		}

		// 承認が不要なら登録する
		$this->AddGroup->create();
		$addGroup = array();
		$addGroup['AddGroup']['top_id'] = $selectedGroupId;
		$addGroup['AddGroup']['bottom_id'] = $myGroupid;
		$addGroup['AddGroup']['top'] = $sp;
		$addGroup['AddGroup']['bottom'] = 0; // 下になるのは自分なので、グループ
		$result = $this->AddGroup->save($addGroup);
		if (!$result) {
			return false;
		}
		return true;
	}

	/**
	 * メンバー同意済み属性にチェックを入れた属性を取得する
	 * @param unknown_type $spAttributeList
	 * @param unknown_type $selectedSpcId
	 */
	private function getCheckAttributeList($spAttributeList, $selectedSpcId) {
		if (empty($selectedSpcId)) {
			return true;
		}

		$managerAttributeList = array(
			'ismemberof' => false,
			'eptid' => false,
			'name' => false,
			'mail' => false,
			'eppn' => false,
			'idp' => false,
			'introduction' => false,
			'language' => false,
		);
		$select = false;
		if (isset($spAttributeList['ismemberof'.$selectedSpcId][0])
			&& $spAttributeList['ismemberof'.$selectedSpcId][0] > 0) {
			$managerAttributeList['ismemberof'] = true;
		}
		if (isset($spAttributeList['eptid'.$selectedSpcId][0])
			&& $spAttributeList['eptid'.$selectedSpcId][0] > 0) {
			$managerAttributeList['eptid'] = true;
		}
		if (isset($spAttributeList['name'.$selectedSpcId][0])
			&& $spAttributeList['name'.$selectedSpcId][0] > 0) {
			$managerAttributeList['name'] = true;
		}
		if (isset($spAttributeList['mail'.$selectedSpcId][0])
			&& $spAttributeList['mail'.$selectedSpcId][0] > 0) {
			$managerAttributeList['mail'] = true;
		}
		if (isset($spAttributeList['eppn'.$selectedSpcId][0])
			&& $spAttributeList['eppn'.$selectedSpcId][0] > 0) {
			$managerAttributeList['eppn'] = true;
		}
		if (isset($spAttributeList['organization'.$selectedSpcId][0])
			&& $spAttributeList['organization'.$selectedSpcId][0] > 0) {
			$managerAttributeList['organization'] = true;
		}
		if (isset($spAttributeList['idp'.$selectedSpcId][0])
			&& $spAttributeList['idp'.$selectedSpcId][0] > 0) {
			$managerAttributeList['idp'] = true;
		}
		if (isset($spAttributeList['introduction'.$selectedSpcId][0])
			&& $spAttributeList['introduction'.$selectedSpcId][0] > 0) {
			$managerAttributeList['introduction'] = true;
		}
		if (isset($spAttributeList['language'.$selectedSpcId][0])
			&& $spAttributeList['language'.$selectedSpcId][0] > 0) {
			$managerAttributeList['language'] = true;
		}

		return $managerAttributeList;
	}

	/**
	 * グループメンバーをSPコネクタの利用属性に同意させる。
	 * @param $myGroupId
	 * @param $managerAttributeList
	 * @param $spcId
	 * @param $topGroupId
	 */
	private function upSaveAttribute($myGroupId, $managerAttributeList, $spcId, $topGroupId = 0) {

debug($managerAttributeList);
		if (empty($managerAttributeList)) {
			return true;
		}

		// SPコネクタへの接続
		// SPコネクタの利用属性で、該当SPコネクタに対して自グループのメンバーを同意済みにする
		$result = $this->TakerAgreement->takeAgreementForManager(
			$spcId, $myGroupId, $managerAttributeList, $topGroupId, $this->getAccountID());
		if (!$result) {
			return false;
		}

		return true;
	}

	/**
	 * 該当グループを自分の下に追加する。
	 * @param unknown_type $myGroupid 自分のグループID
	 * @param unknown_type $selectedGroupInfo 選択したグループ情報
	 * @param unknown_type $selectedGroupId 選択したグループID(SPコネクタ or グループ)
	 */
	private function downSave($myGroupid, $selectedGroupInfo, $selectedGroupId) {
		if ($selectedGroupInfo['Group']['up_permission'] == 2) {
			// 選択したグループが、このグループを上グループに追加するのに承認が必要なら申請する
			$groupInspect = array(
				'account_id' => $this->getAccountID(),
				'top_id' => $myGroupid,
				'bottom_id' => $selectedGroupId,
				'to_id' => $selectedGroupId,   // 申請先グループID
				'from_id' => $myGroupid, // 申請元グループID
				'join' => 0,
				'kind' => 1, // 上になりたい申請
				'status' => 0,
				'top' => 0,    // 上になるのは自分なので、グループ
				'bottom' => 0, // 下になるのグループのみ
			);
			$this->AddGroupInspect->create();
			$this->AddGroupInspect->save(array('AddGroupInspect' => $groupInspect));
		} else {
			// 承認が不要なら登録する
			$addGroup = array(
				'top_id' => $myGroupid,
				'bottom_id' => $selectedGroupId,
				'top' => 0,    // 上になるのは自分なのでグループ
				'bottom' => 0, // 下になるのグループのみ
			);
			$this->AddGroup->create();
			$this->AddGroup->save(array('AddGroup' => $addGroup));
		}
	}

	/**
	 * グループ間の接続を削除する。
	 * @param $updown
	 * @param $pairGroupid
	 * @param $myGroupid
	 */
	function delete_connection($updown, $pairGroupid, $myGroupid) {
		if ($updown === 'up') {
			$this->AddGroup->deleteAll(array(
				'top_id' => $pairGroupid, 'bottom_id' => $myGroupid), false);
		} elseif ($updown === 'down') {
			$this->AddGroup->deleteAll(array(
				'top_id' => $myGroupid, 'bottom_id' => $pairGroupid), false);
		}

		$groupName = $this->Group->getGroupName($pairGroupid, 'ja');
		$message = sprintf(__("message.group.disconnect", true), $groupName);
		$this->Session->setFlash($message, 'message');
		$this->redirect('/admin/manage_groups/edit/'.$myGroupid);
	}

	/**
	 * SPコネクタとの接続を削除する。
	 * @param $updown
	 * @param $pairGroupid
	 * @param $myGroupid
	 */
	function delete_connection_sp($updown, $pairGroupid, $myGroupid) {
		if ($updown === 'up') {
			$this->Transaction->begin();
			$result = $this->GroupExtendedValue->deleteAll(array(
					'spc_id' => $pairGroupid, 'group_id' => $myGroupid), false);
			if (!$result) {
				$this->Transaction->rollback();
				$this->cakeError('systemError');
				return;
			}

			// サービスの接続解除ログ保存
			$addGroup = $this->AddGroup->find('first', array('fields' => array('id'), 'conditions' => array(
				'top_id' => $pairGroupid, 'bottom_id' => $myGroupid, 'top' => 1)));
			$this->AddGroupsSpcLog->create();
			$sLog = array(
				'kind' => 0,
				'group_id' => $myGroupid,
				'spc_id' => $pairGroupid,
				'add_group_id' => $addGroup['AddGroup']['id'],
				'account_id' => $this->getAccountID(),
			);
			$result = $this->AddGroupsSpcLog->save(array('AddGroupsSpcLog' => $sLog));
			if (!$result) {
				$this->Transaction->rollback();
				$this->cakeError('systemError');
				return;
			}

			$result = $this->AddGroup->deleteAll(array(
					'top_id' => $pairGroupid, 'bottom_id' => $myGroupid), false);
			if (!$result) {
				$this->Transaction->rollback();
				$this->cakeError('systemError');
				return;
			}
			$this->Transaction->commit();
		} else {
			$this->cakeError('notFound', array());
			return;
		}

		$groupName = $this->Group->getGroupName($pairGroupid, 'ja');
		$message = sprintf(__("message.group.disconnect.sp", true), $groupName);
		$this->Session->setFlash($message, 'message');
		$this->redirect('/admin/manage_groups/updown/'.$myGroupid);
	}

	/**
	 * 「上にグループを追加」を表示する。
	 * @param $page
	 * @param $groupid
	 */
	function add_up($groupid) {
		// 自グループ情報を設定
		$ManageGroup = $this->ManageGroup->find('first', array(
			'fields' => array('group_key', 'id', 'independence'),
			'conditions' => array('id' => $groupid),
		));
		$this->set('ManageGroup', $ManageGroup);
		$this->data['ManageGroup']['id'] = $groupid;

		if ($ManageGroup['ManageGroup']['independence'] != 0) {
			$this->cakeError('illegalAccess');
			return;
		}

		// "自グループ"が既にあるグループの上グループになっていると上に追加できない
		$this->set("message", "");
		$count = $this->AddGroup->find('count', array(
			'conditions' => array('top_id' => $groupid)));
		if ($count > 0) {
			$this->set("message", __("message.manage_groups.not.add.up.group", true));
			$this->set("PublicGroupOptions", array());
			$this->set("selectedGroups", array());
			$this->render('add_up');
			return;
		}

		if ($count == 0) {
			// "自グループ"が既にグループの上グループになる申請をしていると上にグループを追加できない
			$inspectCount = $this->AddGroupInspect->find('count', array('conditions' =>
				array('from_id' => $groupid, 'top_id' => $groupid, 'bottom' => 0, 'status' => 0)));
			if ($inspectCount > 0) {
				$this->set("message", __("message.manage_groups.not.add.up.group.applying", true));
				$this->set("PublicGroupOptions", array());
				$this->set("selectedGroups", array());
				$this->render('add_up');
				return;
			}
		}

		// このグループの上になることが可能なグループを検索
		// 条件
		//   グループは、down_permission = 1 なら上になれる。
		//   既にグループの下グループになっていると、上グループになれない。
		//   sp = 0, down_permission = 1
		//   自分のグループじゃない
		$groupCount = $this->ManageGroup->find('count', array(
			'conditions' => array(
				'active' => 0,
				'sp' => 0,
				'independence' => 0,
				'down_permission' => array(1, 2),
				'(public = 1 OR id IN (select groupid from mygroups where account_id=? and admin in (1,2)))' => $this->getAccountID(),
				'NOT' => array('id' => array($groupid)),
				'id NOT IN (select top_id from add_groups where bottom_id = ?)' => $groupid,
				'id NOT IN (select bottom_id from add_groups where top = 0 and bottom_id is not null)',
			)));

		$pgroups = $this->ManageGroup->find('all', array(
			'fields' => array('id', 'group_key', 'name', 'name_en', 'introduction', 'introduction_en', 'down_permission'),
			'conditions' => array(
				'active' => 0,
				'sp' => 0,
				'independence' => 0,
				'down_permission' => array(1, 2),
				'(public = 1 OR id IN (select groupid from mygroups where account_id=? and admin in (1,2)))' => $this->getAccountID(),
				'NOT' => array('id' => array($groupid)),
				'id NOT IN (select top_id from add_groups where bottom_id = ?)' => $groupid,
				'id NOT IN (select bottom_id from add_groups where top = 0 and bottom_id is not null)',),
			'order' => array('name', 'name_en'),
		));

		$this->setAddGroupView($pgroups, 'PublicGroupOptions', 'up');
		$this->render('add_up');
	}

	/**
	 * 「下にグループを追加」を表示する。
	 * @param integer $page page number
	 * @param integer $groupid group ID
	 */
	function add_down($groupid) {
		// 自グループ情報を設定
		$ManageGroup = $this->ManageGroup->find('first', array(
			'fields' => array('group_key', 'id', 'independence'),
			'conditions' => array('id' => $groupid)));
		$this->set('ManageGroup', $ManageGroup);
		$this->data['ManageGroup']['id'] = $groupid;

		if ($ManageGroup['ManageGroup']['independence'] != 0) {
			$this->cakeError('illegalAccess');
			return;
		}

		// "自グループ"が既にあるグループの下グループになっていると下に追加できない
		$count = $this->AddGroup->find('count', array(
			'conditions' => array('bottom_id' => $groupid, 'top' => 0)));
		if ($count > 0) {
			$this->set("message", __("manage_groups.not.connect.group.already.exist", true));
			$this->set("PublicGroupOptions", array());
			$this->set("selectedGroups", array());
			$this->render('add_down');
			return;
		}
		if ($count == 0) {
			// "自グループ"が既にグループの下グループになる申請をしていると下にグループを追加できない
			$inspectCount = $this->AddGroupInspect->find('count', array('conditions' =>
				array('from_id' => $groupid, 'bottom_id' => $groupid, 'top' => 0, 'status' => 0)));
			if ($inspectCount > 0) {
				$this->set("message", __("manage_groups.not.connect.group.alreadyapplication", true));
				$this->set("PublicGroupOptions", array());
				$this->set("selectedGroups", array());
				$this->render('add_down');
				return;
			}
		}
		$this->set("message", "");

		// このグループの下になることが可能なグループを検索
		// 条件
		//  *グループは、up_permission = 1 でなら上になれる
		//  *既に上グループになっていると、下グループになれない。
		//  *sp = 0, up_permission = 1
		//  *自分のグループじゃない
		//  *既に下グループになっていない
		//  *グループ
		$groupCount = $this->ManageGroup->find("count", array(
			'conditions' => array(
				'active' => 0,
				'sp' => 0,
				'up_permission' => array(1, 2),
				'independence' => 0,
				'(public = 1 OR id IN (select groupid from mygroups where account_id=? and admin in (1,2)))' => $this->getAccountID(),
				'NOT' => array('id' => array($groupid)),
				'id NOT IN (select top_id from add_groups where top_id is not null)',
				'id NOT IN (select bottom_id from add_groups where top_id = ?)' => $groupid
			),
		));

		$pgroups = $this->ManageGroup->find("all", array(
			'fields' => array('id', 'group_key', 'name', 'name_en', 'introduction', 'introduction_en', 'up_permission'),
			'conditions' => array(
				'active' => 0,
				'sp' => 0,
				'up_permission' => array(1, 2),
				'independence' => 0,
				'(public = 1 OR id IN (select groupid from mygroups where account_id=? and admin in (1,2)))' => $this->getAccountID(),
				'NOT' => array('id' => array($groupid)),
				'id NOT IN (select top_id from add_groups where top_id is not null)',
				'id NOT IN (select bottom_id from add_groups where top_id = ?)' => $groupid),
			'order' => array('group_key')
		));

		$this->setAddGroupView($pgroups, 'PublicGroupOptions', 'down');
		$this->render('add_down');
	}

	/**
	 * 該当グループを自分の下に追加する。
	 * 追加できるグループIDなら追加し、追加時に申請する必要があるのなら申請する。
	 * @param integer $myGroupid 該当グループID(下になるグループID)
	 */
	function add_down_save($myGroupid) {

		if ($this->getSubmitName() == "cancel") {
			$this->redirect('/admin/manage_groups/updown/'.$myGroupid);
			return;
		}

		$selectedGroupId = $this->data['ManageGroup']['publicGroup'];
		if (empty($selectedGroupId)) {
			$this->redirect('/admin/manage_groups/updown/'.$myGroupid);
			return;
		}

		// そもそも追加できるグループなのかをチェック
		if (!$this->validate_add_save_down($myGroupid, $selectedGroupId, false)) {
			$this->Session->setFlash(__("message.manage.cannot.connect.group", true), 'message');
			$this->redirect('/admin/manage_groups/updown/'.$myGroupid);
			return;
		}

		// 既に申請したいたら、何もしない
		$count = $this->AddGroupInspect->find('count', array('conditions' => array(
			'top_id' => $myGroupid,
			'bottom_id' => $selectedGroupId,
			'join' => 0,
			'kind' => 1,  // 自分が上になりたい
			'status' => 0
		)));
		if ($count > 0) {
			$group = $this->Group->find('first', array('fields' => array('id', 'name'), 'conditions' => array('id' => $selectedGroupId)));
			$msgstr = $this->createMessage("message.manage.already.connect.application", $group['Group']['name']);
			$this->Session->setFlash($msgstr, 'message');
			$this->redirect('/admin/manage_groups/add_down/'.$myGroupid);
			return;
		}


		// 選択したグループ情報を取得(該当グループは、上にこのグループを入れる際に申請するかのup_permissionを取得)
		$selectedGroupInfo = $this->Group->find("first", array(
			'fields' => array('id', 'name', 'up_permission'),
			'conditions' => array('id' => $selectedGroupId)));

		// グループを追加する
		if (!empty($this->data['ManageGroup']['publicGroup'])) {
			$this->downSave($myGroupid, $selectedGroupInfo, $this->data['ManageGroup']['publicGroup'], 0);
		}

		if ($selectedGroupInfo['Group']['up_permission'] == 2) {
			$this->Session->setFlash(
				__("message.manage_groups.request.down.application", true).' ('.
				$selectedGroupInfo['Group']['name'].')', 'message');
		} else {
			$this->Session->setFlash(__("message.manage_groups.done.down.group", true).
				' ('.$selectedGroupInfo['Group']['name'].')', 'message');
		}

		// 追加先の管理者へメールをする
		$this->sedMailForAddGroup($selectedGroupId, $myGroupid, 'down', $selectedGroupInfo['Group']['up_permission']);

		// 次画面遷移
		if ($selectedGroupInfo['Group']['up_permission'] == 2) {
			$this->redirect('/admin/manage_inspects/group_inspect_state/'.$myGroupid);
		} else {
			$this->redirect('/admin/manage_groups/updown/'.$myGroupid);
		}
	}

	/**
	 * 上位グループがSPコネクタと接続していなかったら接続する。
	 * 上位グループがSPコネクタと接続していたら接続確認を行う。
	 * @param $myGroupid
	 */
	function add_up_group_confrim($myGroupid) {
		if ($this->getSubmitName() == "cancel") {
			$this->redirect('/admin/manage_groups/edit/'.$myGroupid);
			return;
		}

		$selectedGroupId = $this->data['ManageGroup']['publicGroup'];
		if (empty($selectedGroupId)) {
			$this->redirect('/admin/manage_groups/edit/'.$myGroupid);
			return;
		}

		// 追加できるグループなのかをチェック
		if (!$this->validate_add_save_up($myGroupid, $selectedGroupId, false)) {
			$this->Session->setFlash(__("message.manage.cannot.connect.group", true), 'message');
			$this->redirect('/admin/manage_groups/edit/'.$myGroupid);
			return;
		}

		// 既に申請したいたら、何もしない
		$count = $this->AddGroupInspect->find('count', array('conditions' => array(
			'top_id' => $selectedGroupId,
			'bottom_id' => $myGroupid,
			'join' => 0,
			'kind' => 0,  // 自分が下になりたい
			'status' => 0
		)));
		if ($count > 0) {
			$group = $this->Group->find('first', array('fields' => array('id', 'name'), 'conditions' => array('id' => $selectedGroupId)));
			$msgstr = $this->createMessage("message.manage.already.connect.application", $group['Group']['name']);
			$this->Session->setFlash($msgstr, 'message');
			$this->redirect('/admin/manage_groups/add_up/'.$myGroupid);
			return;
		}

		// 選択したグループ(上位グループ)にSPコネクタが接続されているか確認
		// 接続されていたら、SPコネクタと同様な動作にする。
		$spcIdList = $this->AddGroup->find('list', array(
			'fields' => array('id', 'top_id'),
			'conditions' => array('bottom_id' => $selectedGroupId, 'top' => 1)));
		if (empty($spcIdList)) {
			// 直接接続する
			$selectedGroupInfo = $this->Group->find("first", array(
				'fields' => array('id', 'name', 'down_permission', 'sp'),
				'conditions' => array('id' => $selectedGroupId)));
			$this->upSave($myGroupid, $selectedGroupInfo, $selectedGroupId, 0, array());

			if ($selectedGroupInfo['Group']['down_permission'] == 2) {
				$this->Session->setFlash(__("message.manage_groups.request.up.application", true).' ('.
					$selectedGroupInfo['Group']['name'].')', 'message');
			} else {
				$this->Session->setFlash(__("message.manage_groups.done.up.group", true).' ('.
					$selectedGroupInfo['Group']['name'].')', 'message');
			}

			// 追加先の管理者へメールをする
			$this->sedMailForAddGroup($selectedGroupId, $myGroupid, 'down',
				$selectedGroupInfo['Group']['down_permission'], 0);

			if ($selectedGroupInfo['Group']['down_permission'] == 2) {
				// 承認あり
				$this->redirect('/admin/manage_inspects/group_inspect_state/'.$myGroupid);
			} else {
				// 承認なし
				$this->redirect('/admin/manage_groups/edit/'.$myGroupid);
			}

			return;
		}


		// 確認画面を表示する
		$selectedSpcIdString = '';
		foreach ($spcIdList as $spcid) {
			if (!empty($spcid)) {
				$selectedSpcIdString = $selectedSpcIdString.',';
			}
			$selectedSpcIdString = $selectedSpcIdString.$spcid;
		}

		// 選択したSPコネクタに問題がないので確認画面を表示
		$provideAttributeList = $this->TakerAgreement->serveSpcAgreementInfo(
			$spcIdList, $this->getLanguage());
		$this->set('provideAttributeList', $provideAttributeList);
		$this->data['ManageGroup']['id'] = $myGroupid;
		$this->data['ManageGroup']['selectedGroup'] = $selectedGroupId;
		$this->data['ManageGroup']['selectedSpc'] = $selectedSpcIdString;
		$this->data['ManageGroup']['spc'] = 'false';
		$this->render('add_up_group_agreement');
	}

	/**
	 * 該当グループを自分の上に追加する。
	 * 追加になるグループIDなら追加し、追加するのに申請の必要があったら申請する。
	 * ※SPコネクタに接続している上位グループへの接続のみ呼ばれる。
	 * @param $myGroupid
	 */
	function add_up_group_save($myGroupid) {
		if ($this->getSubmitName() == "cancel") {
			$this->redirect('/admin/manage_groups/edit/'.$myGroupid);
			return;
		}

		// 選択した上位グループID
		$selectedGroupId = $this->data['ManageGroup']['selectedGroup'];
		if (empty($selectedGroupId)) {
			$this->redirect('/admin/manage_groups/edit/'.$myGroupid);
			return;
		}

		// 追加できるグループなのかをチェック
		if (!$this->validate_add_save_up($myGroupid, $selectedGroupId, false)) {
			$this->Session->setFlash(__("message.manage.cannot.connect.group", true), 'message');
			$this->redirect('/admin/manage_groups/updown/'.$myGroupid);
			return;
		}

		// 既に申請したいたら、何もしない
		$count = $this->AddGroupInspect->find('count', array('conditions' => array(
			'top_id' => $selectedGroupId,
			'bottom_id' => $myGroupid,
			'join' => 0,
			'kind' => 0,  // 自分が下になりたい
			'status' => 0
		)));
		if ($count > 0) {
			$group = $this->Group->find('first', array('fields' => array('id', 'name'), 'conditions' => array('id' => $selectedGroupId)));
			$msgstr = $this->createMessage("message.manage.already.connect.application", $group['Group']['name']);
			$this->Session->setFlash($msgstr, 'message');
			$this->redirect('/admin/manage_groups/add_up/'.$myGroupid);
			return;
		}

		$selectedGroupInfo = $this->Group->find("first", array(
			'fields' => array('id', 'name', 'down_permission', 'sp'),
			'conditions' => array('id' => $selectedGroupId)));

		// 選択したグループに接続しているSPコネクタIDを取得
		$selectedSpcIdList = explode(',', $this->data['ManageGroup']['selectedSpc']);

		$this->Transaction->begin();
		// グループ接続、接続申請、接続申請した属性を登録
		$resultConnect = $this->upSave($myGroupid, $selectedGroupInfo, $selectedGroupId, 0, $selectedSpcIdList);
		if (!$resultConnect) {
			$this->Transaction->rollback();
			$this->cakeError('systemError');
			return;
		}

		$resultAttribute = true;
		foreach ($selectedSpcIdList as $spcid) {
			if (empty($spcid)) {
				continue;
			}
			// SPコネクタIDが上位グループに接続していないといけない
			$count = $this->AddGroup->find('count', array('conditions' => array(
				'top_id' => $spcid, 'bottom_id' => $selectedGroupId)));
			if ($count <= 0) {
				continue;
			}

			// 接続申請なしでの接続できる場合、利用属性を登録
			if ($selectedGroupInfo['Group']['down_permission'] == 1) {
				// 利用属性を登録
				$attributeList = $this->getCheckAttributeList($this->data['ManageGroup'], $spcid);
				$resultAttribute = $this->upSaveAttribute($myGroupid, $attributeList, $spcid, $selectedGroupId);
				if (!$resultAttribute)
					break;
			}
		}
		if (!$resultAttribute) {
			$this->Transaction->rollback();
			$this->cakeError('systemError');
			return;
		}
		$this->Transaction->commit();

		if ($selectedGroupInfo['Group']['down_permission'] == 2) {
			$this->Session->setFlash(__("message.manage_groups.request.up.application", true).' ('.
				$selectedGroupInfo['Group']['name'].')', 'message');
		} else {
			$this->Session->setFlash(__("message.manage_groups.done.up.group", true).' ('.
				$selectedGroupInfo['Group']['name'].')', 'message');
		}

		// 追加先の管理者へメールをする
		$this->sedMailForAddGroup($selectedGroupId, $myGroupid, 'down',
			$selectedGroupInfo['Group']['down_permission'], 0);

		// 接続申請なしでの接続できる場合、メンバーにメールする
		if ($selectedGroupInfo['Group']['down_permission'] == 1) {
			foreach ($selectedSpcIdList as $spcid) {
				if (empty($spcid)) {
					continue;
				}
				// 上位グループがSPコネクタに接続していないといけない
				$count = $this->AddGroup->find('count', array('conditions' => array(
					'top_id' => $spcid, 'bottom_id' => $selectedGroupId)));
				if ($count <= 0) {
					continue;
				}

				// グループメンバーに同意する必要があることをメールする
				$attributePath = $this->getSelectCheckAttribute($this->data['ManageGroup'], $spcid);
				$this->Url->get("/map/batch_send_mails/agreement_mail/$myGroupid/$selectedGroupId/$spcid/$attributePath");
			}
		}

		if ($selectedGroupInfo['Group']['down_permission'] == 2) {
			// 承認あり
			$this->redirect('/admin/manage_inspects/group_inspect_state/'.$myGroupid);
		} else {
			// 承認なし
			$this->redirect('/admin/manage_groups/edit/'.$myGroupid);
		}
	}

	/**
	 * 招待メールをキャンセルする。
	 * @param integer $inviteID
	 * @param integer $groupid
	 */
	function invite_cancel($inviteID, $groupid) {
		$this->Transaction->begin();

		// 招待を無効に
		$result = $this->Invite->updateInvalid($inviteID);
		if (!$result) {
			$this->Transaction->rollback();
			$this->cakeError('systemError');
			return;
		}

		// 仮アカウントを削除
		$invite = $this->Invite->findById($inviteID);
		$tempAccountId = $invite['Invite']['temp_account_id'];
		$result = $this->Account->deleteTemporaryAccount($tempAccountId, $groupid);
		if (!$result) {
			// 仮アカウントがない場合もある
			$this->log('仮アカウントがありませんでした。');
		}
		$this->Mygroup->deleteByAccountId($tempAccountId, $groupid);
		$this->Transaction->commit();

		$this->Session->setFlash(__("message.manage.group.invite.cancel", true), 'message');
		$this->redirect('/admin/manage_groups/invite_state/'.$groupid.'?'.$this->getInviteQuery());
	}

	/**
	 * 拡張属性のグループ属性の値を入力する。
	 * @param integer $spcid
	 * @param integer $groupid
	 */
	function extended($spcid, $groupid) {
		if (!$this->Group->hasAny(array('active' => array(0, 1), 'sp' => 1,
			'id IN (select top_id from add_groups where bottom_id = ?)' => $groupid, 'id' => $spcid))) {
			// spcid がグループに関係無い
			$this->log('spcid notin splist!', "debug");
			$this->redirect('/admin/manage_groups/updown/'.$groupid);
			return;
		}

		if (!$this->ExtendedAttribute->hasAny(array('spc_id' => $spcid, 'kind' => 1))) {
			// グループ属性が無い
			$this->log('spcid has not group attr!', "debug");
			$this->redirect('/admin/manage_groups/updown/'.$groupid);
			return;
		}
			// タブ化対応
		$this->setFavariteFiveGroup($groupid);

		switch ($this->getSubmitName()) {
			case '':
				$this->_extendedShow($spcid, $groupid);
				$this->set('errorPoint', array('eaid' => 0, 'idx' => 0));
				break;
			case 'save':
				if (empty($this->data)) {
					return;
				}
				$this->Transaction->begin();

				// 既に登録されている id のリストを取得する
				$exatidlist = $this->ExtendedAttribute->find('list', array(
					'fields' => array('id', 'id'),
					'conditions' => array('spc_id' => $spcid, 'kind' => 1)));
				$list = $this->GroupExtendedValue->find('list', array(
					'fields' => array('id', 'extended_attribute_id'),
					'conditions' => array('group_id' => $groupid, 'extended_attribute_id' => $exatidlist)));
				$this->log($list, "debug");
				$this->log('input data = ', "debug");
				$this->log($this->data['ExtendedValue'], "debug");
				$valueArray = array();
				$idx = array();
				foreach ($this->data['ExtendedValue'] as $key => $value) {
					$this->log('key = ' . $key, "debug");
					// 新規 value_<extended_attribute_id>_<index>_0
					// 既存 value_<extended_attribute_id>_<index>_<group_extended_value_id>
					$index = explode('_', $key);
					$this->log('index = ', "debug");
					$this->log($index, "debug");
					$eaid = $index[1];
					if (isset($idx[$eaid])) {
						$idx[$eaid]++;
					}
					else {
						$idx[$eaid] = 0;
					}
					$evalue = array();
					$evalue['GroupExtendedValue']['group_id'] = $groupid;
					$evalue['GroupExtendedValue']['extended_attribute_id'] = $eaid;
					$evalue['GroupExtendedValue']['value'] = $value;
					if (!empty($index[3])) {  // 既存データ
						if (array_key_exists($index[3], $list)) {
							// アップデート対象
							$list[$index[3]] = 0;
						}
						$beforevValue = $this->GroupExtendedValue->find('first', array(
							'fields' => array('value'),
							'conditions' => array('id' => $index[3])));
						if ($beforevValue['GroupExtendedValue']['value'] == $value) {  // 変更無し
							continue;
						}
						$evalue['GroupExtendedValue']['id'] = $index[3];
						$this->log('Update id = ' . $index[3], "debug");
						$result = $this->GroupExtendedValue->save($evalue);
						if (!$result) {
							$this->Transaction->rollback();
							$this->_extendedShow($spcid, $groupid);
							$this->set('errorPoint', array('eaid' => $index[1], 'idx' => $idx[$eaid]));
							$this->Session->setFlash(__("message.input.error", true), 'message');
							return;
						}
					} else {  // 新規
						$this->log('New! create', "debug");
						$this->GroupExtendedValue->create();
						$result = $this->GroupExtendedValue->save($evalue);
						if (!$result) {
							$this->Transaction->rollback();
							$this->_extendedShow($spcid, $groupid);
							$this->set('errorPoint', array('eaid' => $index[1], 'idx' => $idx[$eaid]));
							$this->Session->setFlash(__("message.input.error", true), 'message');
							return;
						}
					}
				}
				// 削除処理
				$this->log('Delete list = ', "debug");
				$this->log($list, "debug");
				foreach ($list as $key => $attr) {
					if ($attr != 0) {
						// 既存だが今回の更新データに id が無かった
						$this->log('Delete id = ' . $key, "debug");
						$this->GroupExtendedValue->delete($key, false);
					}
				}
				$this->Transaction->commit();

				$this->Session->setFlash(__("manage_groups.extended.save", true), 'message');
				$this->redirect('/admin/manage_groups/updown/' . $groupid);
				break;
			case 'cancel':
				$this->redirect('/admin/manage_groups/updown/' . $groupid);
				return;
			default:
				break;
		}
	}

	private function _extendedShow($spcid, $groupid) {
		$groupAttrs = $this->ExtendedAttribute->find('all', array(
			'fields' => array('id', 'name', 'friendly_name'),
			'conditions' => array('spc_id' => $spcid, 'kind' => 1),
			'order' => array('created' => 'desc')));

		$this->log('groupAttrs = ', "debug");
		$this->log($groupAttrs, "debug");

		$spgroup = $this->Group->find('first', array('fields' => array('id', 'name'),
			'conditions' => array('active' => array(0, 1), 'sp' => 1,
				'id IN (select top_id from add_groups where bottom_id = ?)' => $groupid,
				'id' => $spcid)));

		$manageGroup = array();
		$manageGroup['id'] = $groupid;
		$manageGroup['sp'] = $spcid;
		$manageGroup['name'] = $spgroup['Group']['name'];
		$this->set('ManageGroup', $manageGroup);
		$this->set('groupAttrs', $groupAttrs);

		$extendedValues = array();
		if (empty($this->data['ExtendedValue'])) {   // 新規表示時
			foreach ($groupAttrs as $attr) {
				$values = $this->GroupExtendedValue->find('all', array(
					'fields' => array('id', 'value'),
					'conditions' => array('group_id' => $groupid,
						'extended_attribute_id' => $attr['ExtendedAttribute']['id'])));
				$key = $attr['ExtendedAttribute']['id'];
				$extendedValues[$key] = $values;
			}
		} else {  // エラー時の再表示
			foreach ($this->data['ExtendedValue'] as $key => $value) {
				$index = explode('_', $key);
				$evalue = array();
				$evalue['GroupExtendedValue']['id'] = $index[3];
				$evalue['GroupExtendedValue']['value'] = $value;
				$extendedValues[$index[1]][] = $evalue;
			}
		}
		$this->log('ExtendedAttribute = ', "debug");
		$this->log($extendedValues, "debug");
		$this->set('extendedValues', $extendedValues);

		return;
	}
}
?>
