<?php
/**
 * Management Inspection Controller
 *
 * PHP versions 5
 *
 * @copyright     Copyright (c) 2011 - 2012, National Institute of Informatics,
 * @license       BSD License (http://creativecommons.org/licenses/BSD/)
 */
App::import('Controller', 'Manages');

class ManageInspectsController extends ManagesController {

	var $name = 'ManageInspects';
	var $uses = array('ManageInspect', 'Account', 'Inspect', 'Group', 'Mygroup', 'ManageGroup',
			'AddGroupInspect', 'AddGroup', 'Organization', 'AccountEppn', 'AgreementEntrustment',
			'InspectAttribute', 'Transaction', 'GroupSphost');
	var $components = array('IdpTranslator', 'Url', 'ParameterReplacer', 'Callback', 'AgreementOperator');

	function beforeFilter() {
		parent::beforeFilter();
		$groupid = $this->_getGroupIdFromURL();
		$this->_checkAccessible($groupid);
		$this->_validateAdminGroup($groupid);
	}

	protected function _checkAccessible($groupid) {
		if (is_null($groupid)) {
			$this->cakeError('notFound', array());
			return;
		}
		if ($this->params['action'] === 'history'
			|| $this->params['action'] === 'join'
			|| $this->params['action'] === 'joinAllow'
			|| $this->params['action'] === 'joinDeny'
			|| $this->params['action'] === 'quit'
			|| $this->params['action'] === 'quitAllow'
			|| $this->params['action'] === 'quitDeny') {
			if (!($this->Group->isGroup($groupid) || $this->Group->isIdPGroup($groupid)
				|| $this->Group->isSpConnector($groupid))) {
				$this->cakeError('notFound', array());
				return;
			}
		} else {
			if (!($this->Group->isGroup($groupid) || $this->Group->isIdPGroup($groupid))) {
				$this->cakeError('notFound', array());
				return;
			}
		}
		parent::_checkAccessible($groupid);
	}

	/**
	 * 入会審査を表示します。
	 * @param integer $groupid group ID
	 */
	function join($groupid) {
		if (!is_numeric($groupid)) {
			$this->Session->setFlash(__("errors.illegal.access.message", true), 'message');
			$this->redirect('/mygroups/view');
		}
		$this->inpectSearch($groupid, '0');

		// ２日分の履歴検索
		if (empty($this->viewVars['users'])) {
			$this->inpectSearch2days($groupid, 0);
		}
		// タブ化対応
		$this->setFavariteFiveGroup($groupid);
	}

	/**
	 * 退会審査を表示します。
	 * @param integer $groupid group id
	 */
	function quit($groupid) {
		if (!is_numeric($groupid)) {
			$this->Session->setFlash(__("errors.illegal.access.message", true), 'message');
			$this->redirect('/mygroups/view');
		}
		$this->inpectSearch($groupid, '1');

		// ２日分の履歴検索
		if (empty($this->viewVars['users'])) {
			$this->inpectSearch2days($groupid, 1);
		}
	}

	/**
	 * 入会申請を許可します。
	 * @param integer $inspectid
	 * @param integer $groupid
	 */
	function joinAllow($inspectid, $groupid) {
		$this->validateInspect($inspectid, $groupid, true);
		$inspect = $this->getInspect($inspectid);
		// 既に処理済みなら何もしない
		if ($inspect['Inspect']['status'] == 1) {
			$this->Session->setFlash(__("message.already.processed", true), 'message');
			$this->redirect('/admin/manage_inspects/join/'.$groupid);
			return;
		}

		// 入会承認処理済に
		$inspectChg = array();
		$inspectChg['Inspect']['id'] = $inspectid;
		$inspectChg['Inspect']['status'] = 1;
		$inspectChg['Inspect']['result'] = 'approval';
		$this->Inspect->save($inspectChg);

		// そのグループがオープン状態か検索
		$Group = $this->Group->find('first', array('fields' => array('id', 'name', 'name_en', 'sp'),
			'conditions' => array('Group.id' => $groupid)));

		// 既に入会していたら、
		$alreadyJoin = $this->Mygroup->find('count', array('conditions' => array(
			'account_id' => $inspect['Inspect']['account_id'],
			'groupid' => $groupid)));
		if ($alreadyJoin) {
			$this->Session->setFlash(__("group_lists.message.already.join", true), 'message');
			$this->redirect('/group_lists/members/'.$groupid);
			return;
		}

		// MYグループに登録
		$Mygroup = array(
			'Mygroup' => array(
				'account_id' => $inspect['Inspect']['account_id'],
				'groupid' => $groupid,
				'admin' => 0,
			)
		);
		if ($Group['Group']['sp'] == 1) {
			$Mygroup['Mygroup']['admin'] = 1;
		}
		$this->Mygroup->save($Mygroup);


		// 申請者情報を取得
		$Account = $this->Account->find('first', array('conditions' => array(
				'id' => $inspect['Inspect']['account_id'])));
		if ($Group['Group']['sp'] == 0) {
			// グループ　入会承認結果をメールで送信
			$Mail = array(
				'groupName' => $Group['Group']['name'],
				'groupNameJp' => $this->getGroupNameForMail('ja', $Group),
				'groupNameEn' => $this->getGroupNameForMail('eng', $Group),
				'url' => Router::url('/invite/join_finish/', true).$Group['Group']['id'],
			);
			// 申請者にメール送信
			$this->sendMail("join.group", $Account['Account']['mail'], $Account['Account']['language'],
					'join_group_approval_ok_template', $Mail);

			// 自分以外の管理者にメール送信
			$Mail['url'] = Router::url('/admin/manage_inspects/history/1/', true).$Group['Group']['id'];
			$this->sendMailToOtherAdmin($groupid, 'join.group.approval',
					'join_group_approval_ok_to_admin_template', $Mail, null, $inspect['Inspect']['account_id']);

			$this->Session->setFlash(__("message.manage_inspects.approve.join", true), 'message');
		} elseif ($Group['Group']['sp'] == 1) {
			// SPコネクタ　入会承認結果をメールで送信
			$Mail = array(
				'spConnectorName' => $Group['Group']['name'],
				'spConnectorNameJp' => $this->getGroupNameForMail('ja', $Group),
				'spConnectorNameEn' => $this->getGroupNameForMail('eng', $Group),
				'url' => Router::url('/sp_groups/detail/', true).$Group['Group']['id'],
			);

			// 申請者にメール送信
			$this->sendMail("join.sp.admin", $Account['Account']['mail'], $Account['Account']['language'],
					'join_sp_approval_ok_template', $Mail);

			// 自分以外の管理者にメール送信
			$Mail['url'] = Router::url('/admin/manage_inspects/history/1/', true).$Group['Group']['id'];
			$this->sendMailToOtherAdmin($groupid, 'join.group.approval',
					'join_sp_approval_ok_to_admin_template', $Mail, $Account['Account']['id'], $inspect['Inspect']['account_id']);

			$this->Session->setFlash(__("message.manage_inspects.approve.admin", true), 'message');
		}

		// 申請データがあるか確認
		$count = $this->ManageInspect->find('count', array(
			'conditions' => array('groupid' => $groupid, 'kind' => 0, 'status' => 0)));
		if ($count > 0) {
			$this->redirect('/admin/manage_inspects/join/'.$groupid);
		} else {
			$this->redirect('/admin/manage_inspects/history/1/'.$groupid);
		}
	}

	/**
	 *
	 * @param integer $inspectid
	 * @param integer $groupid
	 * @param boolean $isJoin
	 */
	private function validateInspect($inspectid, $groupid, $isJoin) {
		$inspect = $this->getInspect($inspectid);
		if (empty($inspect)) {
			$this->Session->setFlash(__("message.manage_inspects.no.information", true), 'message');
			$this->redirect('/mygroups/view');
		}

		// 既に処理済みなら何もしない
		if ($inspect['Inspect']['status'] == 1) {
			$this->Session->setFlash(__("message.already.processed", true), 'message');
			if ($isJoin) {
				$this->redirect('/admin/manage_inspects/join/'.$groupid);
			} else {
				$this->redirect('/admin/manage_inspects/quit/'.$groupid);
			}
		}
	}

	/**
	 * 対象申請データを取得する。
	 * @param integer $inspectid
	 */
	private function getInspect($inspectid) {
		$inspect = $this->Inspect->find('first', array('fields' => array('id', 'account_id', 'status'),
			'conditions' => array('id' => $inspectid)));
		return $inspect;
	}

	/**
	 * 入会申請を拒否する。
	 * @param integer $inspectid
	 * @param integer $groupid
	 */
	function joinDeny($inspectid, $groupid) {
		$this->validateInspect($inspectid, $groupid, true);
		$inspect = $this->getInspect($inspectid);

		// 入会審査処理済に
		$inspectChg['Inspect']['id'] = $inspectid;
		$inspectChg['Inspect']['status'] = 1;
		$inspectChg['Inspect']['result'] = 'rejection';
		$this->Inspect->save($inspectChg);

		$Account = $this->Account->find('first', array('conditions' => array(
			'id' => $inspect['Inspect']['account_id'])));

		$Group = $this->Group->find('first', array('fields' => array('id', 'name', 'name_en', 'sp'),
			'conditions' => array('Group.id' => $groupid)));

		// 入会審査結果をメールで送信
		if ($Group['Group']['sp'] == 0) {
			$Mail = array(
				'groupName' => $Group['Group']['name'],
				'groupNameJp' => $this->getGroupNameForMail('ja', $Group),
				'groupNameEn' => $this->getGroupNameForMail('eng', $Group),
				'url' => Router::url('/group_lists/detail/', true).$Group['Group']['id'],
			);
			$this->sendMail("join.group", $Account['Account']['mail'], $Account['Account']['language'],
					'join_group_approval_ng_template', $Mail);

			// 自分以外の管理者にメール送信
			$Mail['url'] = Router::url('/admin/manage_inspects/history/1/', true).$Group['Group']['id'];
			$this->sendMailToOtherAdmin($groupid, 'join.group.approval',
					'join_group_approval_ng_to_admin_template', $Mail, null, $inspect['Inspect']['account_id']);

			$this->Session->setFlash(__("message.manage_inspects.reject.group", true), 'message');
		} elseif ($Group['Group']['sp'] == 1) {
			$Mail = array(
				'spConnectorName' => $Group['Group']['name'],
				'url' => Router::url('/sp_groups/detail/', true).$Group['Group']['id'],
			);
			$this->sendMail("join.sp.admin", $Account['Account']['mail'], $Account['Account']['language'],
					'join_sp_approval_ng_template', $Mail);

			// 自分以外の管理者にメール送信
			$Mail['url'] = Router::url('/admin/manage_inspects/history/1/', true).$Group['Group']['id'];
			$this->sendMailToOtherAdmin($groupid, 'join.group.approval',
					'join_sp_approval_ng_to_admin_template', $Mail, $Account['Account']['id'], $inspect['Inspect']['account_id']);

			$this->Session->setFlash(__("message.manage_inspects.reject.admin", true), 'message');
		}

		// 申請データがあるか確認
		$count = $this->ManageInspect->find('count', array(
			'conditions' => array('groupid' => $groupid, 'kind' => 0, 'status' => 0)));
		if ($count > 0) {
			$this->redirect('/admin/manage_inspects/join/'.$groupid);
		} else {
			$this->redirect('/admin/manage_inspects/history/1/'.$groupid);
		}
	}

	/**
	 * 退会申請を許可する
	 * @param $inspectid
	 * @param $groupid
	 */
	function quitAllow($inspectid, $groupid) {
		$this->validateInspect($inspectid, $groupid, false);
		$inspect = $this->getInspect($inspectid);

		// 退会審査処理済に
		$inspectChg['Inspect']['id'] = $inspectid;
		$inspectChg['Inspect']['status'] = 1;
		$inspectChg['Inspect']['result'] = 'approval';
		$this->Inspect->save($inspectChg);

		$Account = $this->Account->find('first', array('conditions' => array(
				'id' => $inspect['Inspect']['account_id'])));

		// Myグループ情報を削除
		$this->Mygroup->deleteAll(array('groupid' => $groupid, 'account_id' => $Account['Account']['id']), false);
		$this->AgreementEntrustment->deleteAll(array('group_id' => $groupid, 'account_id' => $Account['Account']['id']), false);

		// 退会審査結果をメール送信
		$Group = $this->Group->find('first', array('conditions' => array('id' => $groupid)));
		$Mail = array(
			'groupName' => $Group['Group']['name'],
			'groupNameJp' => $this->getGroupNameForMail('ja', $Group),
			'groupNameEn' => $this->getGroupNameForMail('eng', $Group),
			'url' => Router::url('/group_lists/detail/', true).$Group['Group']['id'],
		);
		$this->sendMail("quit.group", $Account['Account']['mail'], $Account['Account']['language'],
				'quit_group_approval_ok_template', $Mail);

		// 自分以外の管理者にメール送信
		$Mail['url'] = Router::url('/admin/manage_inspects/history/1/', true).$Group['Group']['id'];
		$this->sendMailToOtherAdmin($groupid, 'quit.group.approval',
				'quit_group_approval_ok_to_admin_template', $Mail, null, $inspect['Inspect']['account_id']);

		$this->Session->setFlash(__("message.manage_inspects.approve.group.quit", true), 'message');

		// 申請データがあるか確認
		$count = $this->ManageInspect->find('count', array(
			'conditions' => array('groupid' => $groupid, 'kind' => 1, 'status' => 0)));
		if ($count > 0) {
			$this->redirect('/admin/manage_inspects/quit/'.$groupid);
		} else {
			$this->redirect('/admin/manage_inspects/history/1/'.$groupid);
		}
	}

	/**
	 * 退会申請を拒否する。
	 * @param $inspectid
	 * @param $groupid
	 */
	function quitDeny($inspectid, $groupid) {
		$this->validateInspect($inspectid, $groupid, false);
		$inspect = $this->getInspect($inspectid);

		// 入会審査処理済に
		$inspectChg['Inspect']['id'] = $inspectid;
		$inspectChg['Inspect']['status'] = 1;
		$inspectChg['Inspect']['result'] = 'rejection';
		$this->Inspect->save($inspectChg);

		$Account = $this->Account->find('first', array('conditions' => array(
			'id' => $inspect['Inspect']['account_id'])));

		// 退会審査結果をメールで送信
		$Group = $this->Group->find('first', array('conditions' => array('id' => $groupid)));

		$Mail = array(
			'groupName' => $Group['Group']['name'],
			'groupNameJp' => $this->getGroupNameForMail('ja', $Group),
			'groupNameEn' => $this->getGroupNameForMail('eng', $Group),
			'url' => Router::url('/group_lists/detail/', true).$Group['Group']['id'],
		);
		$this->sendMail("quit.group", $Account['Account']['mail'], $Account['Account']['language'],
				'quit_group_approval_ng_template', $Mail);

		// 自分以外の管理者にメール送信
		$Mail['url'] = Router::url('/admin/manage_inspects/history/1/', true).$Group['Group']['id'];
		$this->sendMailToOtherAdmin($groupid, 'quit.group.approval',
				'quit_group_approval_ng_to_admin_template', $Mail, null, $inspect['Inspect']['account_id']);

		$this->Session->setFlash(__("message.manage_inspects.reject.group.quit", true), 'message');

		// 申請データがあるか確認
		$count = $this->ManageInspect->find('count', array(
			'conditions' => array('groupid' => $groupid, 'kind' => 1, 'status' => 0)));
		if ($count > 0) {
			$this->redirect('/admin/manage_inspects/quit/'.$groupid);
		} else {
			$this->redirect('/admin/manage_inspects/history/1/'.$groupid);
		}
	}

	/**
	 * 審査対象者を検索する。
	 * @param integer $groupid
	 * @param integer or array $kind 0:入会 1:退会 array(0, 1):入会と退会
	 * @param integer $status 0：未審査 1:審査済み
	 * @param integer $page
	 */
	private function inpectSearch($groupid, $kind, $status = 0, $page = 0) {
		// 入会審査者を検索
		$options = array(
			'conditions' => array('kind' => $kind, 'status' => $status, 'groupid' => $groupid),
			'order' => array("ManageInspect.modified" => "DESC"),
		);
		if (!empty($page)) {
			$options = array(
				'conditions' => array('kind' => $kind, 'status' => $status, 'groupid' => $groupid),
				'order' => array("ManageInspect.modified" => "DESC"),
				'limit' => Configure::read("page.historylimit"),
				'page' => $page,
			);
		}
		$users = $this->ManageInspect->find('all', $options);

		if (count($users) > 0) {
			// 入会対象者の情報を取得
			$accountIdList = array();
			for ($i = 0; $i < count($users); $i++) {
				array_push($accountIdList, $users[$i]['ManageInspect']['account_id']);
			}
			$accountIdList = array_unique($accountIdList);
			$accounts = $this->Account->find('all',	array('conditions' => array('Account.id' => $accountIdList)));
			//$accounts = $this->setIdP($accounts);
			$this->IdpTranslator->setIdP($accounts, $this->getLanguage());

			for ($i = 0; $i < count($users); $i++) {
				$users[$i]['ManageInspect']['userid'] = "";
				foreach ($accounts as $account) {
					if ($users[$i]['ManageInspect']['account_id'] == $account['Account']['id']) {
						$lang_account = array($account);
						$this->removeUnnecessaryLanguage($lang_account);
						$account = $lang_account[0];
						$users[$i]['ManageInspect']['name'] = $account['Account']['name'].$account['Account']['name_en'];
						$users[$i]['ManageInspect']['university'] = $account['Account']['university'].$account['Account']['university_en'];
						$users[$i]['ManageInspect']['introduction'] = $account['Account']['introduction'].$account['Account']['introduction_en'];
						$users[$i]['ManageInspect']['userid'] = $account['Account']['id'];
						$users[$i]['ManageInspect']['idp'] = $account['Account']['idp'];
						$users[$i]['ManageInspect']['arrival'] = $account['Account']['arrival'];
						break;
					}
				}
				// for delete account
				if ($users[$i]['ManageInspect']['userid'] === "") {
					$users[$i]['ManageInspect']['name'] = '-';
					$users[$i]['ManageInspect']['university'] = '-';
					$users[$i]['ManageInspect']['introduction'] = '-';
					$users[$i]['ManageInspect']['idp'] = '-';
				}
			}
		}

		$group = $this->Group->find('first', array('fields' => array('name', 'sp'),
			'conditions' => array('id' => $groupid)));
		$this->set('sp', $group['Group']['sp']);
		$this->data['ManageInspect']['groupid'] = $groupid;
		$this->set('ManageInspect', $this->data);
		$this->set('users', $users);
	}

	/**
	 * 過去2日分の審査対象者を検索する。
	 * @param integer $groupid
	 * @param integer or array $kind 0:入会 1:退会 array(0, 1):入会と退会
	 * @param integer $status 0：未審査 1:審査済み
	 */
	private function inpectSearch2days($groupid, $kind, $status = 1) {
		// 入会審査者を検索
		$date = date('Y-m-d H:i:s', strtotime('-48 hour ' . date("Y-m-d H:i:s")));
		$options = array(
			'conditions' => array(
				'kind' => $kind, 'status' => $status, 'groupid' => $groupid, 'modified > \''.$date.'\''),
			'limit' => Configure::read("page.historylimit"),
			'order' => array("modified" => "DESC"),
		);
		$users = $this->ManageInspect->find('all', $options);

		if (count($users) > 0) {
			// 入会対象者の情報を取得
			$accoutIdList = array();
			for ($i = 0; $i < count($users); $i++) {
				$accoutIdList[$i] = $users[$i]['ManageInspect']['account_id'];
			}
			$accountIdList = array_unique($accountIdList);
			$accounts = $this->Account->find('all',	array('conditions' => array('id' => $accoutIdList)));
			$this->IdpTranslator->setIdP($accounts, $this->getLanguage());
			for ($i = 0; $i < count($users); $i++) {
				foreach ($accounts as $account) {
					if ($users[$i]['ManageInspect']['account_id'] == $account['Account']['id']) {
						$users[$i]['ManageInspect']['name'] = $account['Account']['name'];
						$users[$i]['ManageInspect']['university'] = $account['Account']['university'];
						$users[$i]['ManageInspect']['introduction'] = $account['Account']['introduction'];
						$users[$i]['ManageInspect']['userid'] = $account['Account']['id'];
						$users[$i]['ManageInspect']['idp'] = $account['Account']['idp'];
						$users[$i]['ManageInspect']['arrival'] = $account['Account']['arrival'];
						break;
					}
				}
			}
		}
		$this->set('historys', $users);
	}

	/**
	 * SPコネクタ追加審査を表示する。
	 * @param $groupid
	 */
	function group_inspect_sp($groupid) {
		$this->data['ManageInspect']['id'] = $groupid;
		$upGroups = $this->findInspectGroup($groupid, 0);
		$this->removeUnnecessaryLanguage($upGroups);
		$this->set('upGroups', $upGroups);

		if (empty($upGroups)) {
			$historys = $this->findInspectGroup2days($groupid, array(1, 2));
			$this->removeUnnecessaryLanguage($historys);
			$this->set('historys', $historys);
		}
		// タブ化対応
		$this->setFavariteFiveGroup($groupid);
		$this->render('group_inspect_sp');
	}


	/**
	 *
	 * @param integer $page
	 * @param integer $groupid
	 */
	function group_inspect_sp_history($page, $groupid= 0) {
		if ($groupid == 0) {
			$groupid = $page;
		}
		$this->data['ManageInspect']['id'] = $groupid;
		$upGroups = $this->findInspectGroup($groupid, array(1, 2));
		$this->removeUnnecessaryLanguage($upGroups);
		$this->set('upGroups', $upGroups);
		// タブ化対応
		$this->setFavariteFiveGroup($groupid);
	}

	/**
	 *
	 * @param integer $groupid
	 * @param integer $status
	 */
	private function findInspectGroup($groupid, $status) {
		$upGroups = $this->ManageGroup->find('all',
			array(
				'fields' => array('ManageGroup.id',
					'ManageGroup.group_key',
					'ManageGroup.name', 'ManageGroup.name_en',
					'ManageGroup.introduction', 'ManageGroup.introduction_en',
					'AddGroupInspect.id', 'AddGroupInspect.kind', 'AddGroupInspect.status',
					'AddGroupInspect.created', 'AddGroupInspect.modified'),
				'joins' => array(
					array(
					'type' => 'INNER', 'table' => 'add_group_inspects', 'alias' => 'AddGroupInspect',
					'conditions' => array(
						'ManageGroup.id = AddGroupInspect.from_id', // 申請元のグループ情報をとりたい
						'AddGroupInspect.bottom_id' => $groupid,
						'AddGroupInspect.to_id' => $groupid,
						'AddGroupInspect.top' => 1, // SPコネクタからの申請
						'AddGroupInspect.kind' => 1,
						'AddGroupInspect.status' => $status)
					)
				),
				'order' => array('AddGroupInspect.modified DESC'),
				'limit' => Configure::read("page.statelimit"),
			)
		);
		return $upGroups;
	}

	/**
	 *
	 * @param integer $groupid
	 * @param integer $status
	 */
	private function findInspectGroup2days($groupid, $status) {
		$date = date('Y-m-d H:i:s', strtotime('-48 hour ' . date("Y-m-d H:i:s")));
		$upGroups = $this->ManageGroup->find('all',
			array(
				'fields' => array('ManageGroup.id', 'ManageGroup.group_key',
					'ManageGroup.name', 'ManageGroup.name_en',
					'ManageGroup.introduction', 'ManageGroup.introduction_en',
					'AddGroupInspect.id', 'AddGroupInspect.kind', 'AddGroupInspect.status',
					'AddGroupInspect.created', 'AddGroupInspect.modified'),
				'joins' => array(
					array(
					'type' => 'INNER', 'table' => 'add_group_inspects', 'alias' => 'AddGroupInspect',
					'conditions' => array(
						'ManageGroup.id = AddGroupInspect.from_id', // 申請元のグループ情報をとりたい
						'AddGroupInspect.bottom_id' => $groupid,
						'AddGroupInspect.to_id' => $groupid,
						'AddGroupInspect.top' => 1, // SPコネクタからの申請
						'AddGroupInspect.kind' => 1,
						'AddGroupInspect.status' => $status,
						'AddGroupInspect.modified > \''.$date.'\''
						)
					)
				),
				'order' => array('AddGroupInspect.modified DESC'),
				'limit' => Configure::read("page.historylimit"),
			)
		);
		return $upGroups;
	}

	/**
	 * SPコネクタ追加を許可する。
	 * @param $updown up:このグループの上になりたい down:このグループの下になりたい
	 * @param $inspectId 申請ID
	 * @param $groupid group ID
	 */
	function inspect_allow_sp($updown, $inspectId, $groupid) {
		$this->allow($inspectId, $groupid, true);
		$this->Session->setFlash(__("message.manage_inspects.approve.sp.add", true), 'message');

		// 申請情報を取得
		$addGroupInspect = $this->AddGroupInspect->find('first', array(
			'conditions' => array('id' => $inspectId, 'top' => 1)));
		$groupSphostList = array();
		$fromGroupId = '';
		if (!empty($addGroupInspect)) {
			$spcid = $addGroupInspect['AddGroupInspect']['top_id'];
			$fromGroupId = $addGroupInspect['AddGroupInspect']['bottom_id'];
			$groupSphostList = $this->GroupSphost->find('all', array('conditions' => array('group_id' => $spcid)));
		}

		foreach ($groupSphostList as $groupSphost) {
			if (empty($fromGroupId)) {
				continue;
			}
			$spid = $groupSphost['GroupSphost']['sp_id'];
			$callbackUrl = $groupSphost['GroupSphost']['callback_url'];
			if (empty($callbackUrl)) {
				continue;
			}
			$group = $this->Group->find('first', array('conditions' => array('id' => $fromGroupId)));
			$groupKey = $group['Group']['group_key'];
			$callbackUrl = $this->ParameterReplacer->replaceGroupKey($groupKey, $callbackUrl);
//			$callbackUrl = $this->ParameterReplacer->replaceEntityId($callbackUrl);
			// グループ管理者のIDを取得
			$mygroups = $this->Mygroup->find('first', array(
				'conditions' => array('groupid' => $fromGroupId, 'admin' => array(1, 2)),
				'order' => array('id')));
			$account = $this->Account->find('first', array('conditions' => array('id' => $mygroups['Mygroup']['account_id'])));
			$callbackUrl = $this->ParameterReplacer->replaceAdminEmail($account['Account']['mail'], $callbackUrl);
			// サービスのAPI呼び出し
			$json = $this->Callback->call($spid, $callbackUrl);
			if (!isset($json) || ($json['result'] != 'OK' && $json['result'] != 'ok')) {
				$this->Session->setFlash(__("message.failed.service.api", true), 'message');
			}
		}



		// 未処理のデータを検索
		$count = $this->AddGroupInspect->find('count', array(
			'conditions' => array(
				'to_id' => $groupid,
				'top' => 1,
				'status' => 0
				)));
		if ($count > 0) {
			$this->redirect('/admin/manage_inspects/group_inspect_sp/'.$groupid);
		} else {
			$this->redirect('/admin/manage_inspects/group_inspect_sp_history/1/'.$groupid);
		}
	}

	/**
	 *
	 * @param integer $inspectId
	 * @param integer $groupid
	 * @param integer $sp
	 */
	private function allow($inspectId, $groupid, $sp) {
		// 申請情報を取得
		$inspect = $this->AddGroupInspect->find('first', array(
			'fields' => array('id', 'top_id', 'bottom_id', 'top', 'bottom', 'from_id', 'status', 'account_id'),
			'conditions' => array('id' => $inspectId, 'status' => 0)));
		if (empty($inspect)) {
			$this->Session->setFlash(__("message.manage_inspects.no.information", true), 'message');
			if (!$sp) {
				$this->redirect('/admin/manage_inspects/group_inspect/'.$groupid);
			} else {
				$this->redirect('/admin/manage_inspects/group_inspect_sp/'.$groupid);
			}
		}

		// 処理済に
		$inspect['AddGroupInspect']['status'] = 1;
		$this->AddGroupInspect->save($inspect);

		// 承認することでグループの階層に問題がないかを確認する
		//   自グループを上位グループ(SPコネクタならOK)に接続する際に、
		//   上位グループが下位グループでないことを確認
		if ($groupid == $inspect['AddGroupInspect']['bottom_id'] && $inspect['AddGroupInspect']['top'] == 0) {
			$count = $this->AddGroup->find('count', array('conditions' => array(
				'bottom_id' => $inspect['AddGroupInspect']['top_id'], 'top' => 0)));
			if ($count > 0) {
				$this->Session->setFlash(__("manage_groups.not.connect.group.already.exist", true), 'message');
				$this->redirect('/admin/manage_groups/updown/'.$groupid);
			}
		}

		// 承認することでグループの階層に問題がないかを確認する
		//   自グループを下位グループに接続する際に、
		//   下位グループが上位グループでないことを確認
		if ($groupid == $inspect['AddGroupInspect']['top_id']) {
			$count = $this->AddGroup->find('count', array('conditions' => array(
				'top_id' => $inspect['AddGroupInspect']['bottom_id'], 'top' => 0)));
			if ($count > 0) {
				$this->Session->setFlash(__("message.manage_groups.not.add.up.group", true), 'message');
				$this->redirect('/admin/manage_groups/updown/'.$groupid);
			}
		}

		$this->Transaction->begin();

		// グループを追加
		$addGroup = array(
			'AddGroup' => array(
				'top_id' => $inspect['AddGroupInspect']['top_id'],
				'bottom_id' => $inspect['AddGroupInspect']['bottom_id'],
				'top' => $inspect['AddGroupInspect']['top'],
				'bottom' => $inspect['AddGroupInspect']['bottom'],
			)
		);
		$result = $this->AddGroup->save($addGroup);
		if (!$result) {
			$this->Transaction->rollback();
			$this->cakeError('systemError');
			return;
		}
		// 申請元のグループ管理者へ結果をメールする
		$this->sendMailAllowDeny($groupid, $inspect['AddGroupInspect']['from_id'], true, $sp);

		// #上位グループが下位グループからの接続申請
		// 上位グループ管理者が承認した
		if (!$sp && $groupid == $inspect['AddGroupInspect']['top_id']) {
			$fromGroupId = $inspect['AddGroupInspect']['bottom_id']; // 下位グループID
			// 上位グループが接続しているSPコネクタを取得
			$spcIdList = $this->AddGroup->getConnectedSPCIdList($groupid, $onlyActive=false);

			// SPへの属性提供が管理者によって同意ずみに設定されていたら、接続先のメンバーを同意状態にする
			$inspectAttribute = $this->InspectAttribute->findAttributeByInspectId($inspectId);
			if (!empty($inspectAttribute)) {
				// 処理済に更新
				$entity = array();
				$entity['id'] = $inspectAttribute['id'];
				$entity['status'] = 1;
				$result = $this->InspectAttribute->save($entity);
				if (!$result) {
					$this->Transaction->rollback();
					$this->cakeError('systemError');
					return;
				}
			}

			foreach ($spcIdList as $spcid) {
				// 上位グループが下位グループからの接続申請をOKにするので、
				// 接続先(下位グループ)のメンバーの属性を同意済みにする
				// $groupid : 上位グループID
				$result = $this->TakerAgreement->takeAgreementForManager($spcid, $fromGroupId, $inspectAttribute,
					$groupid, $inspect['AddGroupInspect']['account_id']);
				if (!$result) {
					$this->Transaction->rollback();
					$this->cakeError('systemError');
					return;
				}
			}
			$this->Transaction->commit();

			foreach ($spcIdList as $spcid) {
				// グループメンバーに同意する必要があることをメールする
				$attributePath = $this->InspectAttribute->getSelectAttributePath($inspectAttribute);
				$this->Url->get("/map/batch_send_mails/agreement_mail/$fromGroupId/$groupid/$spcid$attributePath");
			}
		// #下位グループが上位グループからの接続申請
		// 下位グループ管理者が承認した
		} elseif (!$sp && $groupid == $inspect['AddGroupInspect']['bottom_id']) {
			// 上位グループが接続しているSPコネクタを取得
			$bottomGroupId = $groupid;
			$topGroupId = $inspect['AddGroupInspect']['top_id'];
			$spcIdList = $this->AddGroup->getConnectedSPCIdList($topGroupId, $onlyActive=false);
			$attrList = $this->InspectAttribute->getEmptyAttrList();
			foreach ($spcIdList as $spcid) {
				// 下位グループが上位グループからの接続申請をOKにするので、
				// 自グループ(下位グループ)の委任しているメンバーの属性を同意済みにする
				// $groupid : 上位グループID
				$result = $this->TakerAgreement->takeAgreementForManager($spcid, $bottomGroupId, $attrList,
					$topGroupId, $this->getAccountID());
				if (!$result) {
					$this->Transaction->rollback();
					$this->cakeError('systemError');
					return;
				}
			}
			$this->Transaction->commit();

			foreach ($spcIdList as $spcid) {
				// グループメンバーに同意する必要があることをメールする
				$attributePath = $this->InspectAttribute->getSelectAttributePath($attrList);
				$this->Url->get("/map/batch_send_mails/agreement_mail/$groupid/$topGroupId/$spcid$attributePath");
			}

		// #SPコネクタからきた上位グループへの接続申請
		// 上位グループ管理者が承認した
		} elseif ($sp) {
			$spcid = $inspect['AddGroupInspect']['from_id'];

			// 自グループ(上位グループ)の委任しているメンバーの属性を同意済みにする
			$attrList = $this->InspectAttribute->getEmptyAttrList();
			$result = $this->TakerAgreement->takeAgreementForManager($spcid, $groupid, $attrList, 0,
				$this->getAccountID());
			if (!$result) {
				$this->Transaction->rollback();
				$this->cakeError('systemError');
				return;
			}
			// 下位グループに同意データを登録、下位グループ管理者がOKすると、下位グループメンバーに連絡される
			$agreeid = $this->AgreementOperator->registerBottomAdminByConnectSPC($groupid, $spcid);
			if (!$agreeid) {
				$this->Transaction->rollback();
				$this->cakeError('systemError');
				return;
			}
			$this->Transaction->commit();

			// 自グループのメンバーに同意する必要があることをメールする
			$inspectAttribute = array();
			$attributePath = $this->InspectAttribute->getSelectAttributePath($inspectAttribute);
			$this->Url->get("/map/batch_send_mails/agreement_mail/$groupid/0/$spcid$attributePath");

			// 下位グループの管理者へ連絡
			$this->AgreementOperator->sendMailBottomAdmin(
				$groupid, $spcid, $agreeid, 'add_group_conntact_bottom_group_template');
		}
		$this->Transaction->commit();
	}

	/**
	 * SPコネクタ追加を拒否する。
	 * @param $updown up:このグループの上になりたい down:このグループの下になりたい
	 * @param $inspectId 申請ID
	 * @param $groupid group ID
	 */
	function inspect_deny_sp($updown, $inspectId, $groupid) {
		$this->deny($inspectId, $groupid, true);

		$this->Session->setFlash(__("message.manage_inspects.reject.sp.add", true), 'message');

		// 未処理のデータを検索
		$count = $this->AddGroupInspect->find('count', array(
			'conditions' => array(
				'to_id' => $groupid,
				'top' => 1,
				'status' => 0
				)));
		if ($count > 0) {
			$this->redirect('/admin/manage_inspects/group_inspect_sp/'.$groupid);
		} else {
			$this->redirect('/admin/manage_inspects/group_inspect_sp_history/1/'.$groupid);
		}
	}

	/**
	 *
	 * @param integer $inspectId
	 * @param integer $groupid
	 * @param boolean $sp
	 */
	private function deny($inspectId, $groupid, $sp) {
		// 申請情報を取得
		$inspect = $this->AddGroupInspect->find('first', array(
			'fields' => array('id', 'top_id', 'bottom_id', 'top', 'bottom', 'from_id', 'status'),
			'conditions' => array('id' => $inspectId, 'status' => 0)));
		if (empty($inspect)) {
			$this->Session->setFlash(__("message.manage_inspects.no.information", true), 'message');
			if (!$sp) {
				$this->redirect('/admin/manage_inspects/group_inspect/'.$groupid);
			} else {
				$this->redirect('/admin/manage_inspects/group_inspect_sp/'.$groupid);
			}
		}

		// 処理済に
		$inspect['AddGroupInspect']['status'] = 2;
		$this->AddGroupInspect->save($inspect);

		// 申請元のグループ管理者へ結果をメールする
		$this->sendMailAllowDeny($groupid, $inspect['AddGroupInspect']['from_id'], false, $sp);
	}

	/**
	 * グループ追加の申請状況を表示する。
	 * @param $groupid
	 */
	function group_inspect_sp_state($groupid) {
		$this->data['ManageInspect']['id'] = $groupid;

		// グループ追加審査情報を検索
		$groups = $this->ManageGroup->find('all',
			array(
				'fields' => array('ManageGroup.id', 'ManageGroup.name', 'ManageGroup.name_en',
					'ManageGroup.introduction', 'ManageGroup.introduction_en',
					'AddGroupInspect.id', 'AddGroupInspect.kind', 'AddGroupInspect.status', 'AddGroupInspect.created'),
				'joins' => array(
					array(
				    'type' => 'INNER', 'table' => 'add_group_inspects', 'alias' => 'AddGroupInspect',
					'conditions' => array(
						'ManageGroup.id = AddGroupInspect.to_id', // 申請先のグループ情報をとりたい
						'AddGroupInspect.from_id' => $groupid,
						'AddGroupInspect.top' => 1)
					)
				),
				'order' => array('AddGroupInspect.created DESC'),
				'limit' => Configure::read("page.statelimit"),
			)
		);
		$this->removeUnnecessaryLanguage($groups);
		$this->set('groups', $groups);
		// タブ化対応
		$this->setFavariteFiveGroup($groupid);
	}

	/**
	 * SPコネクタ追加の申請を取り消す。
	 * @param $inspectid 申請ID
	 * @param $groupid group ID
	 */
	function group_inspect_cancel_sp($inspectid, $groupid) {
		$this->validateAddGroupInspect($inspectid, $groupid);

		$inspect['AddGroupInspect']['id'] = $inspectid;
		$inspect['AddGroupInspect']['status'] = 3;
		$this->AddGroupInspect->save($inspect);

		$this->Session->setFlash(__("message.manage_inspects.cancel.application", true), 'message');

		// 申請中のデータがあるかを確認
		$count = $this->AddGroupInspect->find('count', array(
			'conditions' => array('from_id' =>$groupid, 'status' => 0)));
		if ($count > 0) {
			$this->redirect('/admin/manage_inspects/group_inspect_sp_state/'.$groupid);
		} else {
			$this->redirect('/group_lists/detail/'.$groupid);
		}
	}

	/**
	 *
	 * @param integer $inspectid
	 * @param integer $groupid
	 * @param boolean $sp
	 */
	private function validateAddGroupInspect($inspectid, $groupid) {
		$groupInspect = $this->AddGroupInspect->find('first', array(
			'fields' => array('id', 'status'),
			'conditions' => array('id' => $inspectid)));

		if (empty($groupInspect)) {
			$this->Session->setFlash(__("message.manage_inspects.no.information", true), 'message');
			$this->redirect('/mygroups/view');
		}

		// 既に処理済みなら何もしない
		if ($groupInspect['AddGroupInspect']['status'] != 0) {
			$this->Session->setFlash(__("message.already.processed", true), 'message');
			$this->redirect('/admin/manage_inspects/group_inspect_sp_state/'.$groupid);
		}
	}

	/**
	 * 審査履歴を表示する。
	 * @param $page
	 * @param $groupid
	 */
	function history($page = 1, $groupid) {
		if (!is_numeric($groupid)) {
			$this->Session->setFlash(__("errors.illegal.access.message", true), 'message');
			$this->redirect('/mygroups/view');
		}

		$this->inpectSearch($groupid, array(0, 1), 1, $page);

		$count = $this->ManageInspect->find('count',
			array('conditions' => array('groupid' => $groupid, 'status' => 1)));

		// ページ制御
		$this->controllPage($page, $count, Configure::read("page.historylimit"));

		$this->data['ManageInspect']['page'] = $page;
		// タブ化対応
		$this->setFavariteFiveGroup($groupid);
	}

	/**
	 * グループ追加の申請状況を表示する。
	 * @param $groupid
	 */
	function group_inspect_state($groupid) {
		$this->data['ManageInspect']['id'] = $groupid;

		// グループ追加審査情報を検索
		$groups = $this->ManageGroup->find('all',
			array(
				'fields' => array('ManageGroup.id',
					'ManageGroup.name', 'ManageGroup.name_en',
					'ManageGroup.introduction', 'ManageGroup.introduction_en',
					'AddGroupInspect.id', 'AddGroupInspect.kind', 'AddGroupInspect.status', 'AddGroupInspect.created'),
				'joins' => array(
					array(
				    'type' => 'INNER', 'table' => 'add_group_inspects', 'alias' => 'AddGroupInspect',
					'conditions' => array(
						'ManageGroup.id = AddGroupInspect.to_id', // 申請先のグループ情報をとりたい
						'AddGroupInspect.from_id' => $groupid,
						'AddGroupInspect.top' => 0)
					)
				),
				'order' => array('AddGroupInspect.created DESC'),
				'limit' => Configure::read("page.statelimit"),
			)
		);
		$this->removeUnnecessaryLanguage($groups);
		$this->set('groups', $groups);
		// タブ化対応
		$this->setFavariteFiveGroup($groupid);
	}

	/**
	 * グループ追加審査を表示する。
	 * @param $groupid
	 */
	function group_inspect($groupid) {
		$this->data['ManageInspect']['id'] = $groupid;

		// 上下になりたいグループからの申請
		$groups = $this->findInspectGroupList($groupid, 0);
		$this->removeUnnecessaryLanguage($groups);
		$this->set('Groups', $groups);

		$historys = $this->findInspectGroupList2days($groupid, array(1, 2));
		$this->removeUnnecessaryLanguage($historys);
		$this->set('historys', $historys);
		// タブ化対応
		$this->setFavariteFiveGroup($groupid);
		$this->render('group_inspect');
	}

	/**
	 *
	 * @param integer $groupid
	 * @param integer $status
	 */
	private function findInspectGroupList($groupid, $status) {
		// 上下になりたいグループからの申請
		$groups = $this->ManageGroup->find('all',
			array(
				'fields' => array('ManageGroup.id',
					'ManageGroup.name', 'ManageGroup.name_en',
					'ManageGroup.introduction', 'ManageGroup.introduction_en',
					'AddGroupInspect.id', 'AddGroupInspect.kind', 'AddGroupInspect.status', 'AddGroupInspect.created', 'AddGroupInspect.modified'),
				'joins' => array(
					array(
				    'type' => 'INNER', 'table' => 'add_group_inspects', 'alias' => 'AddGroupInspect',
					'conditions' => array(
						'ManageGroup.id = AddGroupInspect.from_id', // 申請元のグループ情報をとりたい
						'AddGroupInspect.to_id' => $groupid,
						'AddGroupInspect.top' => 0,
						'AddGroupInspect.status' => $status,
						)
					)
				),
				'order' => array('AddGroupInspect.created DESC'),
				'limit' => Configure::read("page.statelimit"),
			)
		);
		return $groups;
	}

	/**
	 *
	 * @param integer $groupid
	 * @param integer $status
	 */
	private function findInspectGroupList2days($groupid, $status) {
		// 上下になりたいグループからの申請
		$date = date('Y-m-d H:i:s', strtotime('-48 hour ' . date("Y-m-d H:i:s")));
		$groups = $this->ManageGroup->find('all',
			array(
				'fields' => array('ManageGroup.id',
					 'ManageGroup.name', 'ManageGroup.name_en',
					 'ManageGroup.introduction', 'ManageGroup.introduction_en',
					 'AddGroupInspect.id', 'AddGroupInspect.kind', 'AddGroupInspect.status', 'AddGroupInspect.created', 'AddGroupInspect.modified'),
				'joins' => array(
					array(
				    'type' => 'INNER', 'table' => 'add_group_inspects', 'alias' => 'AddGroupInspect',
					'conditions' => array(
						'ManageGroup.id = AddGroupInspect.from_id', // 申請元のグループ情報をとりたい
						'AddGroupInspect.to_id' => $groupid,
						'AddGroupInspect.top' => 0,
						'AddGroupInspect.status' => $status,
						'AddGroupInspect.modified > \''.$date.'\'',
						)
					)
				),
				'order' => array('AddGroupInspect.status ASC', 'AddGroupInspect.created DESC'),
				'limit' => Configure::read("page.historylimit"),
			)
		);
		return $groups;
	}

	/**
	 * グループ追加を許可する。
	 * @param $updown up:このグループの上になりたい down:このグループの下になりたい
	 * @param $inspectId 申請ID
	 * @param $groupid group ID
	 */
	function inspect_allow($updown, $inspectId, $groupid) {
		$this->allow($inspectId, $groupid, false);

		$this->Session->setFlash(__("message.manage_inspects.approve.group.add", true), 'message');

		// 未処理のデータを検索
		$count = $this->AddGroupInspect->find('count', array(
			'conditions' => array(
				'to_id' => $groupid,
				'top' => 0,
				'status' => 0
				)));
		if ($count > 0) {
			$this->redirect('/admin/manage_inspects/group_inspect/'.$groupid);
		} else {
			$this->redirect('/admin/manage_inspects/group_inspect_history/'.$groupid);
		}
	}

	/**
	 * グループ追加を拒否する。
	 * @param $updown up:このグループの上になりたい down:このグループの下になりたい
	 * @param $inspectId 申請ID
	 * @param $groupid group ID
	 */
	function inspect_deny($updown, $inspectId, $groupid) {
		$this->deny($inspectId, $groupid, false);

		$this->Session->setFlash(__("message.manage_inspects.reject.group.add", true), 'message');

		// 未処理のデータを検索
		$count = $this->AddGroupInspect->find('count', array(
			'conditions' => array(
				'to_id' => $groupid,
				'top' => 0,
				'status' => 0
				)));
		if ($count > 0) {
			$this->redirect('/admin/manage_inspects/group_inspect/'.$groupid);
		} else {
			$this->redirect('/admin/manage_inspects/group_inspect_history/'.$groupid);
		}
		$this->redirect('/admin/manage_inspects/group_inspect/'.$groupid);
	}

	/**
	 *
	 * @param $groupid
	 */
	function group_inspect_history($groupid) {
		$this->data['ManageInspect']['id'] = $groupid;
		// 上下になりたいグループからの申請
		$groups = $this->findInspectGroupList($groupid, array(1, 2));
		$this->removeUnnecessaryLanguage($groups);
		$this->set('Groups', $groups);
		// タブ化対応
		$this->setFavariteFiveGroup($groupid);
	}

	/**
	 * グループ追加の申請を取り消す。
	 * @param $inspectid 申請ID
	 * @param $groupid group ID
	 */
	function group_inspect_cancel($inspectid, $groupid) {
		$this->validateAddGroupInspect($inspectid, $groupid, false);
 
		$inspect['AddGroupInspect']['id'] = $inspectid;
		$inspect['AddGroupInspect']['status'] = 3;
		$this->AddGroupInspect->save($inspect);
 
		$this->Session->setFlash(__("message.manage_inspects.cancel.application", true), 'message');
 
		// 申請中のデータがあるかを確認
		$count = $this->AddGroupInspect->find('count', array(
			'conditions' => array('from_id' =>$groupid, 'status' => 0)));
		if ($count > 0) {
			$this->redirect('/admin/manage_inspects/group_inspect_state/'.$groupid);
		} else {
			$this->redirect('/group_lists/detail/'.$groupid);
		}
	}
}
?>
