<?php
/**
 * メールをまとめて送信します。
 *
 * PHP versions 5
 *
 * @copyright     Copyright (c) 2011 - 2012, National Institute of Informatics,
 * @license       BSD License (http://creativecommons.org/licenses/BSD/)
 *
 * 実行例）
 * http://localhost/map/batch_send_mails/agreement_mail/グループID/SPコネクタID?mapurl=https%3a%2f%2fmap%2egakunin%2enii%2eac%2ejp%2fmap%2f
 * https://map.gakunin.nii.ac.jp/map/batch_send_mails/agreement_mail/グループID/SPコネクタID?mapurl=https%3a%2f%2fmap%2egakunin%2enii%2eac%2ejp%2fmap%2f
 * https://test-map.gakunin.nii.ac.jp/map/batch_send_mails/agreement_mail/グループID/SPコネクタID?mapurl=https%3a%2f%2fmap%2egakunin%2enii%2eac%2ejp%2fmap%2f
 */
class BatchSendMailsController extends AppController {
	var $name = 'BatchSendMails';
	var $uses = array('Mygroup', 'Account', 'Group', 'GroupSphost', 'ProvideAttribute',
		'AgreementEntrustment', 'Agreement', 'AgreementIsmemberof');
	var $components = array('TakerAgreement');

	function beforeFilter() {
		// サーバ自身からのアクセスのみを許可する
		if ($_SERVER['REMOTE_ADDR'] != $_SERVER['SERVER_ADDR']) {
			$this->cakeError('cannotAccess', array());
			return;
		}
		$this->autoRender = false;
		$this->accessLog();
	}

	/**
	 * グループメンバー全員(管理者込み)にグループ接続時の同意メールを送信する。
	 * @param $groupid グループID
	 * @param $upgroupid 上位グループID
	 * @param $spcid SPコネクタID
	 * @param $agreedForManager 管理者によっで同意された属性一覧
	 */
	function agreement_mail($groupid, $upgroupid, $spcid,
			$ismemberof = 0, $eptid = 0, $name = 0, $mail = 0, $eppn = 0, $idp = 0, $introduction = 0, $language = 0) {
		// グループ名を取得
		$group = $this->Group->find('first', array('fields' => array('id', 'group_key', 'name', 'name_en'),
				'conditions' => array('id' => $groupid)));
		$groupNameJp = $group['Group']['name'];
		if (empty($groupNameJp)) {
			$groupNameJp = $group['Group']['name_en'];
		}
		$groupNameEn = $group['Group']['name_en'];;
		if (empty($groupNameEn)) {
			$groupNameEn = $group['Group']['name'];
		}

		// SPコネクタ名を取得
		$spcNameJp = '';
		$spc = $this->Group->find('first', array(
			'fields' => array('id', 'name', 'name_en', 'introduction', 'introduction_en'),
			'conditions' => array('id' => $spcid)));
		$spcNameJp = $spc['Group']['name'];
		if (empty($spcNameJp)) {
			$spcNameJp = $spc['Group']['name_en'];
		}
		$spcNameEn = $spc['Group']['name_en'];
		if (empty($spcNameEn)) {
			$spcNameEn = $spc['Group']['name'];
		}

		$Mail['groupNameJp'] = $groupNameJp;
		$Mail['groupNameEn'] = $groupNameEn;
		$Mail['spcNameJp'] = $spcNameJp;
		$Mail['spcNameEn'] = $spcNameEn;
		$Mail['spcid'] = $spc['Group']['id'];
		$Mail['serviceNameListJp'] = array();
		$Mail['serviceNameListEn'] = array();
		// SPコネクタに紐すくサービス名一覧を取得
		$groupSpHostList = $this->GroupSphost->find('all', array(
				'conditions' => array('group_id' => $spcid)));
		foreach ($groupSpHostList as $groupSpHost) {
			$serviceName = $this->GroupSphost->getServiceName('ja', $groupSpHost);
			array_push($Mail['serviceNameListJp'], $serviceName);
			$serviceName = $this->GroupSphost->getServiceName('eng', $groupSpHost);
			array_push($Mail['serviceNameListEn'], $serviceName);
		}

		// SPコネクタの必要属性を取得
		$provideAttribute = $this->ProvideAttribute->find('first', array(
				'conditions' => array('group_id' => $spcid)));
		$attributeList = $this->getProvideAttribute($provideAttribute);
		$this->set('attributeList', $attributeList);

//		// 管理者が選択した属性を取得
//		$attributeAdminList = $this->getSelectAttribute($ismemberof, $eptid, $name, $mail, $idp, $introduction, $language);
//		$this->set('attributeAdminList', $attributeAdminList);

		// メンバーを取得
		$accountIdList = $this->Mygroup->find('list', array(
				'fields' => array('account_id', 'account_id'),
				'conditions' => array('groupid' => $groupid)));
		$accountList = $this->Account->find('all', array(
						'conditions' => array('id' => $accountIdList)));
		foreach ($accountList as $account) {
			$this->debugLog('BatchSendMailsController::agreement_mail $account => '.var_export($account, true));
			if ($account['Account']['is_temp'] > 0) {
				// 仮アカウントにはメールを送信しない
				$this->log('BatchSendMailsController::agreement_mail'
						.' don\'t send mail to account id:'.$account['Account']['id']
						.' for a temporary account.', 'info');
				continue;
			}


			$accountid = $account['Account']['id'];
			// 管理者によって、強制的に同意の変更があった属性を取得
			$agreeIsmemberof = $this->AgreementIsmemberof->find('first', array('conditions' => array(
				'account_id' => $accountid, 'spc_id' => $spcid, 'ismemberof' => 1,
				'OR' => array('group_id' => array($groupid, $upgroupid)))));
			$agreement = $this->Agreement->find('first', array('conditions' => array(
				'account_id' => $accountid, 'group_id' => $spcid)));
			$this->debugLog('BatchSendMailsController::agreement_mail $agreeIsmemberof => '.var_export($agreeIsmemberof, true));
			$this->debugLog('BatchSendMailsController::agreement_mail $agreement => '.var_export($agreement, true));
			$changeList = $this->getChangeAttribute(
				$provideAttribute, $agreeIsmemberof, $agreement,
				$ismemberof, $eptid, $name, $mail, $eppn, $idp, $introduction, $language);
			$changeShowList = $this->getShowAttribute($changeList);
			$this->set('changeList', $changeShowList);
			$this->debugLog('BatchSendMailsController::agreement_mail $changeList => '.var_export($changeList, true));
			$this->debugLog('BatchSendMailsController::agreement_mail $changeShowList => '.var_export($changeShowList, true));

			// 委任の状態を取得
			$entrustment = $this->AgreementEntrustment->find('first', array('conditions' => array(
				'group_id' => $groupid, 'account_id' => $accountid)));
			$entrustList = array();
			$needAttrList = array();
			if ($entrustment) {	// 委任している人
				$entrustList = $this->getChangeEntrustAttribute($provideAttribute, $agreeIsmemberof, $agreement,
					$changeList);
				$entrustShowList = $this->getShowAttribute($entrustList);
				$this->set('entrustList', $entrustShowList);
				$this->debugLog('BatchSendMailsController::agreement_mail $entrustList => '.var_export($entrustList, true));
				$this->debugLog('BatchSendMailsController::agreement_mail $entrustShowList => '.var_export($entrustShowList, true));
			} else {
				if (!$upgroupid) {
					$needAttrList = $this->TakerAgreement->needSpcConnectAgree($spcid, $groupid, $accountid);
				} else {
					// 上位グループとSPコネクタの同意情報
					$needAttrList = $this->TakerAgreement->needSpcConnectAgree($spcid, $upgroupid, $accountid);
				}
				$needAttrShowList = $this->getShowAttribute($needAttrList);
				$this->set('needAttrList', $needAttrShowList);
				$this->debugLog('BatchSendMailsController::agreement_mail $needAttrList => '.var_export($needAttrList, true));
				$this->debugLog('BatchSendMailsController::agreement_mail $needAttrShowList => '.var_export($needAttrShowList, true));
			}
			if (empty($changeList) && empty($needAttrList) && empty($entrustList)) {
				// 強制で変更された属性がなく、同意する属性もなく、委任で変更された属性もないなら、メールしない
				$this->log('BatchSendMailsController::agreement_mail'
						.' don\'t send mail to '.$account['Account']['mail']
						.' because of having no attribute that have been forced to change'
						.' or that user need to agree on', 'info');
				continue;
			}

			// 委任している人
			if ($entrustment) {
				// 通知メール：③
				$this->sendMail('agreement.use.attribute', $account['Account']['mail'],
					$account['Account']['language'], 'add_group_agreement_entrustment_template', $Mail);
			// 委任していない人
			} else {
				// 同意の必要なし
				if (empty($needAttrList)) {
					// 通知メール：②
					// 管理者の強制だと$changeListに値が入るので、view側で強制ありかどうかを表示する
					$this->sendMail('agreement.use.attribute', $account['Account']['mail'],
						$account['Account']['language'], 'add_group_agreement_nothing_template', $Mail);
				// 同意の必要あり
				} else {
					// 通知メール：①, ④
					// 管理者の強制だと$changeListに値が入るので、view側で強制ありかどうかを表示する
					$this->sendMail('agreement.use.attribute', $account['Account']['mail'],
						$account['Account']['language'], 'add_group_agreement_need_template', $Mail);
				}
			}
		}
	}

	private function getChangeAttribute($provideAttribute, $ismemberof, $agreement,
		$pIsmemberof, $pEptid, $pName, $pMail, $pEppn, $pIdp, $pIntroduction, $pLanguage) {
		if (empty($provideAttribute)) {
			return array();
		}

		$changeList = array();
		if ($pIsmemberof == 1 && $provideAttribute['ProvideAttribute']['ismemberof']) {
			if (!empty($ismemberof)) {
				if ($ismemberof['AgreementIsmemberof']['ismemberof']
					&& !$ismemberof['AgreementIsmemberof']['before_ismemberof']) {
					$changeList['ismemberof'] = true;
				}
			}
		}
		if (empty($agreement)) {
			return $changeList;
		}
		if ($pEptid == 1 && $provideAttribute['ProvideAttribute']['eptid']) {
			if ($agreement['Agreement']['eptid']
				&& !$agreement['Agreement']['before_eptid']) {
				$changeList['eptid'] = true;
			}
		}
		if ($pName == 1 && $provideAttribute['ProvideAttribute']['name']) {
			if ($agreement['Agreement']['name']
				&& !$agreement['Agreement']['before_name']) {
				$changeList['name'] = true;
			}
		}
		if ($pMail && $provideAttribute['ProvideAttribute']['mail']) {
			if ($agreement['Agreement']['mail']
				&& !$agreement['Agreement']['before_mail']) {
				$changeList['mail'] = true;
			}
		}
		if ($pEppn && $provideAttribute['ProvideAttribute']['eppn']) {
			if ($agreement['Agreement']['eppn']
				&& !$agreement['Agreement']['before_eppn']) {
				$changeList['eppn'] = true;
			}
		}
		if ($pIdp && $provideAttribute['ProvideAttribute']['idp']) {
			if ($agreement['Agreement']['idp']
				&& !$agreement['Agreement']['before_idp']) {
				$changeList['idp'] = true;
			}
		}
		if ($pIntroduction && $provideAttribute['ProvideAttribute']['introduction']) {
			if ($agreement['Agreement']['introduction']
				&& !$agreement['Agreement']['before_introduction']) {
				$changeList['introduction'] = true;
			}
		}
		if ($pLanguage && $provideAttribute['ProvideAttribute']['language']) {
			if ($agreement['Agreement']['language']
				&& !$agreement['Agreement']['before_language']) {
				$changeList['language'] = true;
			}
		}
		return $changeList;
	}

	private function getProvideAttribute($provideAttribute) {
		$attributeList['ja'] = array();
		$attributeList['eng'] = array();
		if (empty($provideAttribute))
			return $attributeList;

		if ($provideAttribute['ProvideAttribute']['ismemberof']) {
			$attributeList['ja']['ismemberof'] = __("common.group.id.jp", true);
			$attributeList['eng']['ismemberof'] = __("common.group.id.en", true);
		}
		if ($provideAttribute['ProvideAttribute']['eptid']) {
			$attributeList['ja']['eptid'] = __("common.eptid.jp", true);
			$attributeList['eng']['eptid'] = __("common.eptid.en", true);
		}
		if ($provideAttribute['ProvideAttribute']['name']) {
			$attributeList['ja']['name'] = __("common.name.jp", true);
			$attributeList['eng']['name'] = __("common.name.en", true);
		}
		if ($provideAttribute['ProvideAttribute']['mail']) {
			$attributeList['ja']['mail'] = __("common.mailaddress.jp", true);
			$attributeList['eng']['mail'] = __("common.mailaddress.en", true);
		}
		if ($provideAttribute['ProvideAttribute']['eppn']) {
			$attributeList['ja']['eppn'] = __("common.eppn", true);
			$attributeList['eng']['eppn'] = __("common.eppn", true);
		}
		if ($provideAttribute['ProvideAttribute']['idp']) {
			$attributeList['ja']['idp'] = __("common.idp.jp", true);
			$attributeList['eng']['idp'] = __("common.idp.en", true);
		}
		if ($provideAttribute['ProvideAttribute']['introduction']) {
			$attributeList['ja']['introduction'] = __("common.my.introduction.jp", true);
			$attributeList['eng']['introduction'] = __("common.my.introduction.en", true);
		}
		if ($provideAttribute['ProvideAttribute']['language']) {
			$attributeList['ja']['language'] = __("common.language.jp", true);
			$attributeList['eng']['language'] = __("common.language.en", true);
		}

		return $attributeList;
	}

	private function getSelectAttribute($ismemberof, $eptid, $name, $mail, $eppn, $idp, $introduction, $language) {
		$attributeList['ja'] = array();
		$attributeList['eng'] = array();
		if ($ismemberof) {
			$attributeList['ja']['ismemberof'] = __("common.group.id.jp", true);
			$attributeList['eng']['ismemberof'] = __("common.group.id.en", true);
		}
		if ($eptid) {
			$attributeList['ja']['eptid'] = __("common.eptid.jp", true);
			$attributeList['eng']['eptid'] = __("common.eptid.en", true);
		}
		if ($name) {
			$attributeList['ja']['name'] = __("common.name.jp", true);
			$attributeList['eng']['name'] = __("common.name.en", true);
		}
		if ($mail) {
			$attributeList['ja']['mail'] = __("common.mailaddress.jp", true);
			$attributeList['eng']['mail'] = __("common.mailaddress.en", true);
		}
		if ($eppn) {
			$attributeList['ja']['eppn'] = __("common.eppn", true);
			$attributeList['eng']['eppn'] = __("common.eppn", true);
		}
		if ($idp) {
			$attributeList['ja']['idp'] = __("common.idp.jp", true);
			$attributeList['eng']['idp'] = __("common.idp.en", true);
		}
		if ($introduction) {
			$attributeList['ja']['introduction'] = __("common.my.introduction.jp", true);
			$attributeList['eng']['introduction'] = __("common.my.introduction.en", true);
		}
		if ($language) {
			$attributeList['ja']['language'] = __("common.language.jp", true);
			$attributeList['eng']['language'] = __("common.language.en", true);
		}
		return $attributeList;
	}

	private function getShowAttribute($changeList) {
		$attributeList['ja'] = array();
		$attributeList['eng'] = array();
		if (isset($changeList['ismemberof'])) {
			$attributeList['ja']['ismemberof'] = __("common.group.id.jp", true);
			$attributeList['eng']['ismemberof'] = __("common.group.id.en", true);
		}
		if (isset($changeList['eptid'])) {
			$attributeList['ja']['eptid'] = __("common.eptid.jp", true);
			$attributeList['eng']['eptid'] = __("common.eptid.en", true);
		}
		if (isset($changeList['name'])) {
			$attributeList['ja']['name'] = __("common.name.jp", true);
			$attributeList['eng']['name'] = __("common.name.en", true);
		}
		if (isset($changeList['mail'])) {
			$attributeList['ja']['mail'] = __("common.mailaddress.jp", true);
			$attributeList['eng']['mail'] = __("common.mailaddress.en", true);
		}
		if (isset($changeList['eppn'])) {
			$attributeList['ja']['eppn'] = __("common.eppn", true);
			$attributeList['eng']['eppn'] = __("common.eppn", true);
		}
		if (isset($changeList['idp'])) {
			$attributeList['ja']['idp'] = __("common.idp.jp", true);
			$attributeList['eng']['idp'] = __("common.idp.en", true);
		}
		if (isset($changeList['introduction'])) {
			$attributeList['ja']['introduction'] = __("common.my.introduction.jp", true);
			$attributeList['eng']['introduction'] = __("common.my.introduction.en", true);
		}
		if (isset($changeList['language'])) {
			$attributeList['ja']['language'] = __("common.language.jp", true);
			$attributeList['eng']['language'] = __("common.language.en", true);
		}
		return $attributeList;
	}

	private function getChangeEntrustAttribute($provideAttribute, $ismemberof, $agreement, $changeList) {
		if (empty($provideAttribute)) {
			return array();
		}
		// $changeListでtrueになってなくて、現在1でbeforeが0の属性は委任で設定されたはず
		$entrustList = array();
		if ($provideAttribute['ProvideAttribute']['ismemberof'] && !isset($changeList['ismemberof'])
			&& !empty($ismemberof) && $ismemberof['AgreementIsmemberof']['ismemberof']
			&& !$ismemberof['AgreementIsmemberof']['before_ismemberof']) {
			$entrustList['ismemberof'] = true;
		}
		if (empty($agreement)) {
			return $entrustList;
		}
		if ($provideAttribute['ProvideAttribute']['eptid'] && !isset($changeList['eptid'])
			&& $agreement['Agreement']['eptid'] && !$agreement['Agreement']['before_eptid']) {
			$entrustList['eptid'] = true;
		}
		if ($provideAttribute['ProvideAttribute']['name'] && !isset($changeList['name'])
			&& $agreement['Agreement']['name'] && !$agreement['Agreement']['before_name']) {
			$entrustList['name'] = true;
		}
		if ($provideAttribute['ProvideAttribute']['mail'] && !isset($changeList['mail'])
			&& $agreement['Agreement']['mail'] && !$agreement['Agreement']['before_mail']) {
			$entrustList['mail'] = true;
		}
		if ($provideAttribute['ProvideAttribute']['eppn'] && !isset($changeList['eppn'])
			&& $agreement['Agreement']['eppn'] && !$agreement['Agreement']['before_eppn']) {
			$entrustList['eppn'] = true;
		}
		if ($provideAttribute['ProvideAttribute']['idp'] && !isset($changeList['idp'])
			&& $agreement['Agreement']['idp'] && !$agreement['Agreement']['before_idp']) {
			$entrustList['idp'] = true;
		}
		if ($provideAttribute['ProvideAttribute']['introduction'] && !isset($changeList['introduction'])
			&& $agreement['Agreement']['introduction'] && !$agreement['Agreement']['before_introduction']) {
			$entrustList['introduction'] = true;
		}
		if ($provideAttribute['ProvideAttribute']['language'] && !isset($changeList['language'])
			&& $agreement['Agreement']['language'] && !$agreement['Agreement']['before_language']) {
			$entrustList['language'] = true;
		}

		return $entrustList;
	}
}
?>
