<?php
/**
 * 2年間利用がなかったユーザに対してのアカウント削除予告メール通知
 *
 * PHP versions 5
 *
 * @copyright     Copyright (c) 2021, National Institute of Informatics,
 * @license       BSD License (http://creativecommons.org/licenses/BSD/)
 *
 * 実行例）
 * http://localhost/map/batch_send_notice_mails/batch
 * https://map.gakunin.nii.ac.jp/map/batch_send_notice_mails/batch
 * https://test-map.gakunin.nii.ac.jp/map/batch_send_notice_mails/batch
 * テスト用）
 * http://localhost/map/batch_send_notice_mails/batch?nomail=1&date=2021-10-11
 */
class BatchSendNoticeMailsController extends AppController {
	var $name = 'BatchSendNoticeMails';
	var $uses = array('AccountEppn', 'Account', 'Group', 'SentNoticeMailLog', 'Mygroup', 'Organization');
	var $components = array('Common','IdpTranslator', 'NoticeUtils');

	var $nomail = 0;	// テスト用メール送信不可フラグ
	var $additionalTextList = array(
		'ja' => array(
			'',
			"あなたのアカウントには複数のePPNが紐付けられており、一部の紐付けのみが解除予定です。\n上記ePPNをご確認の上利用期間延長を希望する紐付けをお間違えのないようご注意ください。",
			"あなたのアカウントには複数のePPNが紐付けられていますが、すべてのePPNが解除対象となっています。",
			"本メールアドレス（本通知メールの宛先欄をご参照ください）は複数のアカウントに登録されております。\n上記ePPNをご確認の上利用期間延長を行うアカウントをお間違えのないようご注意ください。",
			"本メールアドレス（本通知メールの宛先欄をご参照ください）は複数のアカウントに登録されていますが、\nすべてのアカウントが削除対象になっています。",
			"以下のグループについてあなた以外に管理者がいません（／いなくなる予定です）。\n有効期限を延長しない場合は必ず削除日までに当該グループに別の管理者を\n割り当てるか、当該グループの削除を行ってください。\n%s"
		),
		'en' => array(
			'',
			"Your account has multiple ePPNs linked to it, and only some of them are scheduled to be unlinked. \nPlease check the ePPNs above to make sure you have the right one.",
			"Your account has multiple ePPNs linked to it, but all of them are marked for unlinking.",
			"This email address (please refer to the address field of this email) is registered to multiple accounts. \nPlease check the ePPN above to make sure you have the correct account for your subscription extension.",
			"This email address (please refer to the address field of this email) is registered to \nmultiple accounts, all of which are marked for deletion.",
			"The following groups do not have (or will not have) any administrators other than you. \nIf you do not want to extend your subscription, please make sure to assign another \nadministrator to the group or delete the group before the deletion date of your account.\n%s"
		)
	);
	var $addEppn = 0;	// 複数ePPN用追加文言
	var $addMail = 0;	// 同一メール追加文言
	var $addAdmin = 0;	// 単独管理者追加文言
	var $soloAdminGroups = array();	// 単独管理者グループIDリスト
	var $delete = true;	// アカウント削除あり
	
	function beforeFilter() {
		// サーバ自身からのアクセスのみを許可する
		if ($_SERVER['REMOTE_ADDR'] != $_SERVER['SERVER_ADDR']) {
			$this->cakeError('cannotAccess', array());
			return;
		}
		$this->autoRender = false;
		$this->accessLog();
	}

	/* 日々のバッチ処理 */
	function batch() {
		if ($this->params['url']['url'] != 'batch_send_notice_mails/batch') {
			return;
		}

		$this->log('BatchSendNoticeMailsController start', "batch");

		if (isset($this->params['url']['nomail'])) {
			$this->log('BatchSendNoticeMailsController * No email will be sent for testing purposes.*', "batch");
			$this->nomail = 1;
		}
		$startTime = date('Y-m-d H:i:s');
		if (isset($this->params['url']['date'])) {
			$startTime = $this->params['url']['date'] . ' 00:00:00';
			$this->log("BatchSendNoticeMailsController * The start time($startTime) has been specified.*", "batch");
			$this->debugLog("'BatchSendNoticeMailsController::batch startTime = $startTime");
		}

		// キャッシュを無効化
		$this->AccountEppn->cacheQueries = false;
		$this->Account->cacheQueries = false;
		$this->Group->cacheQueries = false;
		$this->SentNoticeMailLog->cacheQueries = false;
		$this->Mygroup->cacheQueries = false;
		$this->Organization->cacheQueries = false;

		// メール送信後確認ページ以外にアクセスのあったアカウントの送信ログ削除
		$idList = $this->AccountEppn->find('list', array(
			'fields' => array('id'),
			'conditions' => array(
				'id in (select account_eppn_id from sent_notice_mail_logs where AccountEppn.id = sent_notice_mail_logs.account_eppn_id and last_login_approx > sent_notice_mail_logs.sent)'
			)
		));
		$this->log('BatchSendNoticeMailsController : Number of accounts accessed after the email was sent :' . count($idList), "batch");
		if ($idList) {
			$this->log('BatchSendNoticeMailsController : Ids accessed after the email was sent (' . implode(",", $idList) . ")", "batch");
			$result = $this->SentNoticeMailLog->deleteAll(array('account_eppn_id' => $idList), false);
			if (!$result) {
				$error = $this->SentNoticeMailLog->getDataSource()->error;
				$this->log('BatchSendNoticeMailsController ERROR SentNoticeMailLog deleteAll Error '.$error, "batch");
				return;
			}
		}

		// 複数のePPNと紐付けられているアカウント洗い出し
		$linkedAccounts = $this->AccountEppn->getLinkedAccounts();
		$this->debugLog('BatchSendNoticeMailsController : linkedAccounts :' . print_r($linkedAccounts, true));

		// 同一メールアドレスの別アカウント洗い出し
		$sameMails = $this->AccountEppn->getSameMails();
		$this->debugLog('BatchSendNoticeMailsController : sameMails :' . print_r($sameMails, true));

		$this->execute(2, $linkedAccounts, $sameMails, $startTime);	// 2回目
		$this->execute(1, $linkedAccounts, $sameMails, $startTime);	// 1回目

		$this->log('BatchSendNoticeMailsController end', "batch");
	}

	function reset() {
		$this->addEppn = 0;	// 複数ePPN用追加文言
		$this->addMail = 0;	// 同一メール追加文言
		$this->addAdmin = 0;	// 単独管理者追加文言
		$this->soloAdminGroups = array();	// 単独管理者グループIDリスト
		$this->delete = true;	// アカウント削除あり
	}

	private function execute($kind, $linkedAccounts, $sameMails, $startTime, $manualEppnId = 0) {
		$conditions = array();
		if ($kind == 3) {	// 手動送信
			$conditions = array('id' => $manualEppnId);
		}
		else {
			$months = 22;
			if ($kind == 2) {
				$months = 23;
			}

			// $kind回目：$monthsか月目（2年を経過する2or1か月前）のメール通知
			if ($kind == 1) {	// 1回目のメールは送信記録が無い場合のみ送信
				$conditions = array('last_login_approx <= (? - INTERVAL ? MONTH)' => array($startTime, $months),
                                	'id not in (select account_eppn_id from sent_notice_mail_logs)',
                                	"account_id not in (select id from accounts where mail = '' or mail is null)");
			}
			else {
				$conditions = array('last_login_approx <= (? - INTERVAL ? MONTH)' => array($startTime, $months),
                                	'id not in (select account_eppn_id from sent_notice_mail_logs where kind = ?)' => array($kind),
                                	"account_id not in (select id from accounts where mail = '' or mail is null)");
			}
		}
		$accountEppns = $this->AccountEppn->find('all', array(
			'fields' => array('id', 'account_id', 'last_login_approx'),
			'conditions' => $conditions,
			'order' => array('id DESC')
		));
		if (!$accountEppns) {
			if ($kind == 3) {
				$this->debugLog("BatchSendNoticeMailsController::execute No account is available to send email. kind=$kind, manualEppnId=$manualEppnId");
			}
			else {
				$this->log("BatchSendNoticeMailsController : No account has been used for than $months months.", "batch");
			}
			return;
		}
		if ($kind != 3) {
			$this->log("BatchSendNoticeMailsController : Number of accounts that have not been used for more than $months months :" . count($accountEppns), "batch");
		}
		// 既に削除対象になっているアカウントのリスト
		$deleteIdList = $this->SentNoticeMailLog->find('list', array(
			'fields' => array('account_eppn_id'),
		));
		$deleteIdList = array_unique($deleteIdList);
		$this->debugLog("BatchSendNoticeMailsController::execute kind=$kind, deleteIdList(SentNoticeMailLog) => ".var_export($deleteIdList, true));

		// 新規に削除対象になったアカウントの追加
		foreach ($accountEppns as $key => $aEppn) {
			$deleteIdList[] = $aEppn['AccountEppn']['id'];
		}
		$deleteIdList = array_unique($deleteIdList);
		$this->debugLog("BatchSendNoticeMailsController::execute deleteIdList(SentNoticeMailLog+accountEppns) => ".var_export($deleteIdList, true));
		// 複数のePPNと紐付けられているアカウントの削除対象チェック
		foreach ($linkedAccounts as $key => $lAccount) {
			$idKey = array_search($lAccount['AccountEppn']['id'], $deleteIdList);
			if ($idKey !== false) {	// 削除対象
				$linkedAccounts[$key]['delete'] = 1;
			}
		}
		// 同一メールアドレスのアカウントの削除対象チェック
		foreach ($sameMails as $key => $sMail) {
			$idKey = array_search($sMail['AccountEppn']['id'], $deleteIdList);
			if ($idKey !== false) {	// 削除対象
				$sameMails[$key]['delete'] = 1;
			}
		}
		$this->debugLog("BatchSendNoticeMailsController::execute sameMails=> ".var_export($sameMails, true));
//		$this->debugLog("BatchSendNoticeMailsController::execute kind=$kind, accountEppns => ".var_export($accountEppns, true));

		foreach ($accountEppns as $key => &$aEppn) {
			$eppnId = $aEppn['AccountEppn']['id'];
			$accountId = $aEppn['AccountEppn']['account_id'];
			$eppns = array($eppnId);
			$accountIds = array($accountId);
			$this->debugLog("BatchSendNoticeMailsController::execute key=$key, aEppn => ".var_export($aEppn, true));
			if (empty($aEppn['firstLinkedEppn'])) {	// 既に一緒にメールされていない
				$sameAccountKeys = array();
				$addEppn = $this->getAddEppn($eppnId, $linkedAccounts, $accountId, $sameAccountKeys);
				if ($addEppn != 0) {	// 複数のePPNと紐付けられている
					$this->addEppn = $addEppn;
					if ($addEppn == 1) {	// 一部削除対象
						$this->delete = false;
						$accountIds = array();	// アカウント削除ではないので、削除対象から外す
					}
//			$this->debugLog("BatchSendNoticeMailsController::execute accountIds => ".var_export($accountIds, true));
//			$this->debugLog("BatchSendNoticeMailsController::execute key=$key, sameAccountKeys => ".var_export($sameAccountKeys, true));
					// 一緒にメール送信されるアカウントのaccount_eppn_idをセットする
					foreach ($sameAccountKeys as $saKey) {
						$aeId = $linkedAccounts[$saKey]['AccountEppn']['id'];
						$idKey =  $this->hasSameId($aeId, $accountEppns);
//			$this->debugLog("BatchSendNoticeMailsController::execute idKey=$idKey, aeId = $aeId, accountEppns => ".var_export($accountEppns, true));
						if ($idKey !== false) {	// 削除対象
							$accountEppns[$idKey]['firstLinkedEppn'] = $eppnId;
							$eppns[] = $aeId;
						}
					}
				}

				$idKey = $this->hasSameId($eppnId, $sameMails);
//			$this->debugLog("BatchSendNoticeMailsController::execute sameMail check eppnId=$eppnId, idKey=$idKey");
				if ($idKey !== false) {	// 同一メールアドレスの別アカウントが存在する
					$sameMailKeys = $this->getSameMailKeys($sameMails[$idKey]['Account']['mail'], $sameMails);
					$count = 0;
//			$this->debugLog("BatchSendNoticeMailsController::execute key=$key, sameMailKeys => ".var_export($sameMailKeys, true));
					foreach ($sameMailKeys as $saKey) {
						if (!empty($sameMails[$saKey]['delete'])) {
							$count++;
						}
					}
					if ($count == count($sameMailKeys)) {	// 全部削除対象
						$this->addMail = 4;
					}
					else {
						$this->addMail = 3;	// 一部削除対象
					}
					// 一緒にメール送信されるアカウントのaccount_eppn_idをセットする
					foreach ($sameMailKeys as $saKey) {
						$aeId = $sameMails[$saKey]['AccountEppn']['id'];
						$idKey =  $this->hasSameId($aeId, $accountEppns);
//			$this->debugLog("BatchSendNoticeMailsController::execute idKey=$idKey, aeId = $aeId, accountEppns => ".var_export($accountEppns, true));
						if ($idKey !== false) {	// 削除対象
							$accountEppns[$idKey]['firstLinkedEppn'] = $eppnId;
							$aId = $accountEppns[$idKey]['AccountEppn']['account_id'];
							$eppns[] =  $aeId;
							if ($aId != $accountId) {
								// 同一メールアドレスで別リンキングがあった場合の対応
								$sameAccountKeys = array();
								$addEppn = $this->getAddEppn($aeId, $linkedAccounts, $aId,
									$sameAccountKeys);
								if ($addEppn == 1) {	// 一部紐づけ解除
									$this->addEppn = 1;
								}
								else {
									$this->delete = true;
									$accountIds[] = $aId;
									if ($addEppn == 2 && $this->addEppn == 0) {
										$this->addEppn = 2;
									}
								}
							}
						}
					}
//			$this->debugLog("BatchSendNoticeMailsController::execute mail accountIds => ".var_export($accountIds, true));
				}
			}
			if (!empty($aEppn['firstLinkedEppn']) && $aEppn['firstLinkedEppn'] != $eppnId) {
				// 既に一緒にメールされている
				$this->debugLog("BatchSendNoticeMailsController::execute : id=$eppnId has already been emailed with id=" . $aEppn['firstLinkedEppn']);
				$this->log("BatchSendNoticeMailsController  account_eppn_id($eppnId) has already been emailed with id=" . $aEppn['firstLinkedEppn'], "batch");
				continue;
			}	
			if ($this->delete) {	// 削除あり
				// 単独管理者グループ洗い出し
				$accountIds = array_unique($accountIds);
				$this->debugLog("BatchSendNoticeMailsController::execute accountIds => ".var_export($accountIds, true));
				foreach ($accountIds as $aId) {
					$saGroups = $this->NoticeUtils->getSoloAdminGroups($aId, $deleteIdList);
					$this->soloAdminGroups = array_merge($this->soloAdminGroups, $saGroups);
				}
				$this->soloAdminGroups = array_unique($this->soloAdminGroups);
				$this->debugLog("BatchSendNoticeMailsController::execute accountId=$accountId, soloAdminGroups => ".var_export($this->soloAdminGroups, true));
				if (!empty($this->soloAdminGroups)) {
					$this->addAdmin = 5;	// 単独管理者文言追加
				}
			}
			// メール送信
			$eppns = array_unique($eppns);
			if ($kind != 3) {
				$this->log("BatchSendNoticeMailsController sendNoticeMail account_id = $accountId, account_eppn_id = (".implode(",",$eppns).")", "batch");
			}
			$result = $this->sendNoticeMail($kind, $accountId, $aEppn['AccountEppn']['last_login_approx'], $eppns);
			if (!$result) {
				return;	
			}
			$this->reset();
			if ($kind != 3) {
				sleep(1);
			}
		}
	}

	private function sendNoticeMail($kind, $accountId, $last_login_approx, $eppns) {
		$account = $this->Account->find('first', array(
			'fields' => array('name', 'name_en', 'mail', 'language'),
			'conditions' => array('id' => $accountId)
		));
		// 削除・解除の文章変化
		if (!$this->delete) {	// 一部ePPN紐付け解除のみ
			$deleteUnlinkJa = __("mail.unlink.ja", true);
			$planDeleteUnlinkJa = __("mail.plan.unlink.ja", true);
			$deleteUnlinkEn = __("mail.unlink.en", true);
			$planDeleteUnlinkEn = __("mail.plan.unlink.en", true);
		}
		else {
			$deleteUnlinkJa = __("mail.delete.ja", true);
			$planDeleteUnlinkJa = __("mail.plan.delete.ja", true);
			$deleteUnlinkEn = __("mail.delete.en", true);
			$planDeleteUnlinkEn = __("mail.plan.delete.en", true);
		}
		$Mail = array(
			'accountNameJa' => $account['Account']['name'],
			'accountNameEn' => $account['Account']['name_en'],
			'deleteUnlinkJa' => $deleteUnlinkJa,
			'deleteUnlinkEn' => $deleteUnlinkEn,
			'planDeleteUnlinkJa' => $planDeleteUnlinkJa,
			'planDeleteUnlinkEn' => $planDeleteUnlinkEn,
			
		);
		$targetTime = strtotime($last_login_approx);
		$Mail['deleteDate'] = date('Y/m/d', strtotime('+2 year', $targetTime));
		$accountEppnList = $this->AccountEppn->find('all', array(
			'fields' => array('id', 'eppn', 'entityid'),
			'conditions' => array('id' => $eppns),
			'order' => array('id DESC')
		));
		$language = $account['Account']['language'];
		if ($language == 'eng') {
			$language = 'en';
		}
		$this->IdpTranslator->appendIdpName($accountEppnList);
		if ($this->Common->isProduction()) {
			$url = Configure::read("production.loginlink");
		}
		else {
			$url = Configure::read("test.loginlink");
		}
		$url = $url.'/map/accounts/check/%s&entityID=%s';
		$eppnList = array();
		foreach ($accountEppnList as $key => $accountEppn) {
			$random = $this->SentNoticeMailLog->createSentNoticeMailLog($accountEppn['AccountEppn']['id'], $kind);
			if (!$random) {
				$this->log('BatchSendNoticeMailsController ERROR createSentNoticeMailLog Error ; No email is sent. accountId='.$accountId, "batch");
				return false;
			}
			$eppnList[$key]['eppn'] = $accountEppn['AccountEppn']['eppn']; 
			$eppnList[$key]['idpNameJa'] = $accountEppn['AccountEppn']['idpJa']; 
			$eppnList[$key]['idpNameEn'] = $accountEppn['AccountEppn']['idpEn']; 
			$eppnList[$key]['url'] = sprintf($url, $random, $accountEppn['AccountEppn']['entityid']);
		}
		$mail = $account['Account']['mail'];
		$this->set('eppnList', $eppnList);
		$this->debugLog('BatchSendNoticeMailsController::sendNoticeMail $eppnList => '.var_export($eppnList, true));

		$this->log("BatchSendNoticeMailsController : sendMail : mail=$mail, lang=$language", "debug");
		$additionalText = $this->getAdditionalText();
		$this->debugLog("BatchSendNoticeMailsController::sendNoticeMail addEppn = $this->addEppn, addMail = $this->addMail, addAdmin = $this->addAdmin");
		$this->set('additionalText', $additionalText);
		$this->debugLog('BatchSendNoticeMailsController::sendNoticeMail $additionalText => '.var_export($additionalText[$language], true));

		$this->debugLog('BatchSendNoticeMailsController::sendNoticeMail $Mail => '.var_export($Mail, true));
		if ($this->nomail == 0) {
			$this->sendMail('account.confirmation', $mail, $language, 'notice_account_delete_template', $Mail);
		}

		return true;
	}

	private function getAdditionalText() {
		$additionalText = array(
			'ja' => array(),
			'en' => array()
		);
		if ($this->addEppn) {
			$additionalText['ja'][] = $this->additionalTextList['ja'][$this->addEppn];
			$additionalText['en'][] = $this->additionalTextList['en'][$this->addEppn];
		}
		if ($this->addMail) {
			$additionalText['ja'][] = $this->additionalTextList['ja'][$this->addMail];
			$additionalText['en'][] = $this->additionalTextList['en'][$this->addMail];
		}
		if ($this->addAdmin) {
			$soloAdminGroupNames = implode(", ", $this->Group->findGroupNameList($this->soloAdminGroups));
			$additionalText['ja'][] = sprintf($this->additionalTextList['ja'][$this->addAdmin], $soloAdminGroupNames);
			$additionalText['en'][] = sprintf($this->additionalTextList['en'][$this->addAdmin], $soloAdminGroupNames);
		}
		return $additionalText;
	}

	private function hasSameId($id, $accountEppns) {
		foreach ($accountEppns as $key => $aEpps) {
			if ($aEpps['AccountEppn']['id'] == $id) {
				return $key;
			}
		}
		return false;
	}

	private function getSameMailKeys($mail, $sameMails) {
		$sameMailKeys = array();
		foreach ($sameMails as $key => $sMail) {
			if ($sMail['Account']['mail'] == $mail) {
				$sameMailKeys[] = $key;
			}
		}
		return $sameMailKeys;
	}

	private function getAddEppn($eppnId, $linkedAccounts, $accountId, &$sameAccountKeys) {
		$idKey = $this->hasSameId($eppnId, $linkedAccounts);
		if ($idKey !== false) {	// 複数のePPNと紐付けられている
			$sameAccountKeys = $this->NoticeUtils->getSameAccountKeys($accountId, $linkedAccounts);
			$count = 0;
			foreach ($sameAccountKeys as $saKey) {
				if (!empty($linkedAccounts[$saKey]['delete'])) {
					$count++;
				}
			}
			if ($count == count($sameAccountKeys)) {	// 全部削除対象
				return 2;
			}
			else {
				return 1;	// 一部削除対象
			}
		}
		return 0;	// 紐づけ無し
	}

	/* 手動送信 */
	function manual($eppnId) {
		$startTime = date('Y-m-d H:i:s');
		// 複数のePPNと紐付けられているアカウント洗い出し
		$linkedAccounts = $this->AccountEppn->getLinkedAccounts();
		$this->debugLog('BatchSendNoticeMailsController::manual : linkedAccounts :' . print_r($linkedAccounts, true));

		// 同一メールアドレスの別アカウント洗い出し
		$sameMails = $this->AccountEppn->getSameMails();
		$this->debugLog('BatchSendNoticeMailsController::manual : sameMails :' . print_r($sameMails, true));

		$this->execute(3, $linkedAccounts, $sameMails, $startTime, $eppnId);	// 手動送信
	}
}
?>
