<?php
/**
 * AdminGroup Componet
 *
 * PHP versions 5
 *
 * @copyright     Copyright (c) 2011 - 2012, National Institute of Informatics,
 * @license       BSD License (http://creativecommons.org/licenses/BSD/)
 */
class AdminGroupComponent extends Object
{
	var $Group;
	var $Mygroup;
	var $Account;

	function initialize(&$controller) {
		$this->controller =& $controller;
		$this->Group = ClassRegistry::init('Group');
		$this->Mygroup = ClassRegistry::init('Mygroup');
		$this->Account = ClassRegistry::init('Account');
	}

	/**
	 * 管理しているグループを検索
	 * @param integer $groupid
	 */
	public function find($account_id, $groupid = null)
	{
		$adminGroupIdList = $this->Mygroup->find('list', array(
			'fields' => array('groupid'),
			'conditions' => array(
				'account_id' => $account_id,
				'admin' => array(1, 2)
			)
		));

		$conditions = array(
			'id' => $adminGroupIdList,
			'sp' => 0,
		);
		if (!is_null($groupid)) {
			$conditions['NOT'] = array('id' => $groupid);
		}
		$adminGroupList = $this->Group->find('list', array(
			'fields' => array('id', 'group_key'),
			'conditions' => $conditions,
		));
		return $adminGroupList;
	}

	public function getMembers($accountId, $language, $groupid = null, $forcibleAdmission = false)
	{
		$adminGroupIdList = $this->Mygroup->find('list', array(
			'fields' => array('groupid'),
			'conditions' => array(
				'account_id' => $accountId,
				'admin' => array(1, 2)
			)
		));

		if (!is_null($groupid)) {
			$key = array_search($groupid, $adminGroupIdList);
			if ($key !== false) {
				unset($adminGroupIdList[$key]);
			}
		}

		$accounts = $this->Account->find('all', array(
			'fields' => array('id', 'mail', 'name', 'name_en'),
			'joins' => array(
				array(
					'type' => 'INNER', 'table' => 'mygroups', 'alias' => 'Mygroup',
					'conditions' => array(
						'Mygroup.account_id = Account.id',
					),
				),
				array(
					'type' => 'INNER', 'table' => 'groups', 'alias' => 'Group',
					'conditions' => array(
						'Mygroup.groupid = Group.id',
					),
				),
			),
			'conditions' => array(
				'Mygroup.groupid' => $adminGroupIdList,
				'Group.sp' => 0,
				'Account.is_temp' => 0,
			),
			'order' => array('name', 'name_en'),
		));

		$MygroupAccountIds = array();
		if (!is_null($groupid)) {
			$MygroupAccountIds = $this->Mygroup->find('list', array(
				'fields' => array('account_id'),
				'conditions' => array(
					'groupid' => $groupid,
				)
			));
		}

		$results = array(
			'valid' => array(),
			'empty' => array(),
		);
		foreach ($accounts as $account) {
			if ($account['Account']['id'] == $accountId) {
				continue;
			} elseif (in_array($account['Account']['id'], $MygroupAccountIds)) {
				continue;
			}
			$key = empty($account['Account']['mail']) ? 'empty' : 'valid';

			if ($language == 'ja') {
				$name = $account['Account']['name'] != '' ? $account['Account']['name'] : $account['Account']['name_en'];
			} else {
				$name = $account['Account']['name_en'] != '' ? $account['Account']['name_en'] : $account['Account']['name'];
			}
			$results[$key][$account['Account']['id']] = $name;
		}
		if (!$forcibleAdmission) {
			unset($results['empty']);
		}
		return $results;
	}
}
