<?php
/**
 * Agreement Operator Component
 *
 * PHP versions 5
 *
 * @copyright     Copyright (c) 2016, SOUM Corporation.
 * @copyright     Copyright (c) 2011 - 2012, National Institute of Informatics,
 * @license       BSD License (http://creativecommons.org/licenses/BSD/)
 */
class AgreementOperatorComponent extends Object {

	function initialize(&$controller) {
		$this->controller =& $controller;
		$this->Transaction = ClassRegistry::init('Transaction');
		$this->Account = ClassRegistry::init('Account');
		$this->AddGroup = ClassRegistry::init('AddGroup');
		$this->Group = ClassRegistry::init('Group');
		$this->Mygroup = ClassRegistry::init('Mygroup');
		$this->GroupSphost = ClassRegistry::init('GroupSphost');
		$this->Agreement = ClassRegistry::init('Agreement');
		$this->AgreementIsmemberof = ClassRegistry::init('AgreementIsmemberof');
		$this->AgreementConnect = ClassRegistry::init('AgreementConnect');
		$this->AgreementEntrustment = ClassRegistry::init('AgreementEntrustment');
		$this->ProvideAttribute = ClassRegistry::init('ProvideAttribute');
		$this->SpHost = ClassRegistry::init('SpHost');
	}

   	/**
   	 * メンバーの同意状況を取得する
   	 */
   	function members($page, $groupid, $accountid = null) {
   		// このグループが接続している上位グループIDを取得
		$topGroupIdList = $this->AddGroup->find('list', array(
			'fields' => array('top_id', 'top_id'),
			'conditions' => array('top' => 0, 'bottom_id' => $groupid)));
		// このグループと上位グループが接続しているSPコネクタIDを取得
		$idList = $topGroupIdList;
		array_push($idList, $groupid);
		$spcIdList = $this->AddGroup->find('list', array(
			'fields' => array('top_id', 'top_id'),
			'conditions' => array(
				'top' => 1,
				'bottom_id' => $idList)));
		if (empty($spcIdList)) {
			return array();
		}
		// $spcIdListからプライベートサービスを削除
		$spcIdList = $this->Group->find('list', array(
			'fields' => array('id', 'id'),
			'conditions' => array(
				'id' => array_values($spcIdList),
				'sp' => array(0,1))));
		if (empty($spcIdList)) {
			return array();
		}

		// メンバー情報を取得
		if (empty($accountid)) {
			$accountList = $this->Account->find('all', array(
				'conditions' => array('id in (select account_id from mygroups where groupid = ?)' => $groupid),
				'order' => array('id'),
				'limit' => Configure::read("page.memberlimit"),
				'page' => $page,
			));
		} else {
			$accountList = $this->Account->find('all', array(
				'conditions' => array(
					'id in (select account_id from mygroups where groupid = ?)' => $groupid,
					'id' => $accountid),
				'order' => array('id'),
				'limit' => Configure::read("page.memberlimit"),
				'page' => $page,
			));
		}

		// 不要な項目を取り除く
		$this->controller->removeUnnecessaryLanguage($accountList);

		for ($i = 0; $i < count($accountList); $i++) {
			$id = $accountList[$i]['Account']['id'];
			$lang = $accountList[$i]['Account']['language'];
			$agreeData = $this->controller->TakerAgreement->servData($id, $lang, $spcIdList, array(), true);
			if (empty($agreeData))
				$accountList[$i]['Account']['agreement_list'] = array();
			else
				$accountList[$i]['Account']['agreement_list'] = $agreeData['Agreement'];
			if ($this->Mygroup->isAdminOnly($id, $groupid)) {	// 管理者権限のみ
				foreach ($accountList[$i]['Account']['agreement_list'] as &$agreement) {
					if (!$this->AddGroup->isDirectConnectSPConnetor($agreement['group_id'], $groupid)) {
						// 上位グループ経由なので同意状況を消す
						unset($agreement['eptid']);
						unset($agreement['name']);
						unset($agreement['mail']);
						unset($agreement['eppn']);
						$agreement['admin_only'] = true;
					}
				}
			}
		}

		return $accountList;
   	}

   	/**
   	 * メンバーの同意状況を取得する。(同意していない人の昇順)
   	 */
   	function sortMembers($page, $groupid, $accountid = null) {
   		$accountList = array();
		$accountList = $this->members($page, $groupid, $accountid);
//              $this->log('sortMembers : after member() accountList = ' . print_r($accountList, true), 'debug');

		// 自グループIDを含む上位グループIDを取得
		$topGroupIdList = $this->AddGroup->getTopIdMyIdList($groupid, $onlyActive=false);
		$this->log('sortMembers : topGroupIdList = ' . print_r($topGroupIdList, true), 'debug');
		// 自グループと上位グループ以外を取り除く
		for ($i = 0; $i < count($accountList); $i++) {
			for ($j = 0; $j < count($accountList[$i]['Account']['agreement_list']); $j++) {
				$nameIdList = $accountList[$i]['Account']['agreement_list'][$j]['top_group_name_list'];
				foreach ($nameIdList as $nameId => $name) {
					$idKey = array_search($nameId, $topGroupIdList);
					if ($idKey === false) {
						unset($accountList[$i]['Account']['agreement_list'][$j]['top_group_name_list'][$nameId]);
					}
				}
			}
		}
//                $this->log('sortMembers : accountList = ' . print_r($accountList, true), 'debug');

		// 委任状態取得
		foreach ($accountList as &$account) {
			$account['entrustment'] = false;
			$count = $this->AgreementEntrustment->find('count', array(
				'conditions' => array('group_id' => $groupid, 'account_id' => $account['Account']['id'])));
			if ($count > 0) {
				$account['entrustment'] = true;
			}
		}
		unset($account);

		$yesList = array();
		$noList = array();
		foreach ($accountList as $account) {
			$agree = true;
			foreach ($account['Account']['agreement_list'] as $agreement) {
				if (isset($agreement['eptid']) && !$agreement['eptid']) {
					$agree = false;
				}
				if (isset($agreement['name']) && !$agreement['name']) {
					$agree = false;
				}
				if (isset($agreement['mail']) && !$agreement['mail']) {
					$agree = false;
				}
				if (isset($agreement['eppn']) && !$agreement['eppn']) {
					$agree = false;
				}

				// グループID
				if ($agree && !isset($agreement['admin_only'])) {
					$groupidAgree = true;
					foreach ($agreement['top_group_name_list'] as $key => $value) {
						// top_group_name_list(自グループと上位グループ以外を取り除いた
						// 接続グループのIDと名前のリスト)に存在するのに
						// top_group_select_id_list(同意済みグループIDのリスト)に無い
						if (!isset($agreement['top_group_select_id_list'][$key])) {
							$groupidAgree = false;
							break;
						}
					}
					if (!$groupidAgree) {
						$agree = false;
					}
				}
			}
			if ($agree) {
				$account['needAgreement'] = false;
				array_push($yesList, $account);
			} else {
				$account['needAgreement'] = true;
				array_push($noList, $account);
			}
		}

		$list = array_merge($noList, $yesList);
		return $list;
   	}

	function updateAgreementList($groupid, $lang) {
		$list = $this->AgreementConnect->find('all', array('conditions' => array(
			'OR' => array('top_group_id' => $groupid , 'bottom_group_id' => $groupid), 'status' => 0)));

		// 追加属性の設定
		for ($i = 0; $i < count($list); $i++) {
			$attributes = $list[$i]['AgreementConnect']['add_attributes'];
			if (empty($attributes))
				continue;

			$list[$i]['attribute_list'] = array();
			$attributeList = $this->explode(',', $attributes);
			foreach ($attributeList as $attr) {
				if ($attr === 'ismemberof') {
					array_push($list[$i]['attribute_list'], __("common.group.id", true));
				} else if ($attr === 'eptid') {
					array_push($list[$i]['attribute_list'], __("common.eptid", true));
				} else if ($attr === 'name') {
					array_push($list[$i]['attribute_list'], __("common.name", true));
				} else if ($attr === 'mail') {
					array_push($list[$i]['attribute_list'], __("common.mailaddress", true));
				} else if ($attr === 'eppn') {
					array_push($list[$i]['attribute_list'], __("common.eppn", true));
				}
			}
		}

		// 追加対象SPの設定
		for ($i = 0; $i < count($list); $i++) {
			$sps = $list[$i]['AgreementConnect']['add_sps'];
			if (empty($sps))
				continue;

			$list[$i]['sp_list'] = array();
			$spList = $this->explode(',', $sps);
			foreach ($spList as $sp) {
				array_push($list[$i]['sp_list'], $sp);
			}
		}

		// SPコネクタ名の設定
		$spcidList = $this->AgreementConnect->find('list', array(
			'fields' => array('spc_id', 'spc_id'),
			'conditions' => array(
					'OR' => array('top_group_id' => $groupid, 'bottom_group_id' => $groupid), 'status' => 0)));
		$spcNameList = $this->Group->find('all', array(
			'fields' => array('id', 'name', 'name_en'),
			'conditions' => array('id' => $spcidList)));
		for ($i = 0; $i < count($list); $i++) {
			$id = $list[$i]['AgreementConnect']['spc_id'];
			if (empty($id))
				continue;

			$list[$i]['spc_name'] = '';
			foreach ($spcNameList as $spc) {
				if ($id != $spc['Group']['id'])
					continue;
				if ($lang === 'ja') {
					if (!empty($spc['Group']['name']))
						$list[$i]['spc_name'] = $spc['Group']['name'];
					else
						$list[$i]['spc_name'] = $spc['Group']['name_en'];
				} else {
					if (!empty($spc['Group']['name_en']))
						$list[$i]['spc_name'] = $spc['Group']['name_en'];
					else
						$list[$i]['spc_name'] = $spc['Group']['name'];
				}
			}
		}
		return $list;
	}

	/**
	 * SPコネクタの利用属性変更に同意するを選択
	 * @param $agreeid agreement_connects.id
	 * @param $groupid 自グループID
	 * @param $operatorId 操作者のアカウントID
	 * @return バリデーション結果
	 */
	function updateAgreeOK($agreeid, $groupid, $operatorId) {
		$result = $this->validateAgreement($agreeid, $groupid);
		if (!$result) {
			return false;
		}

		$this->Transaction->begin();
		$result = $this->executeAgreeOK($agreeid, $groupid, $operatorId);
		if (!$result) {
			$this->Transaction->rollback();
			$this->cakeError('systemError');
		}

		$this->Transaction->commit();
		return true;
	}

	/**
	 * @param $agreeid agreement_connects.id
	 * @param $groupid 自グループID
	 * @param $operatorId 操作者のアカウントID
	 */
	private function executeAgreeOK($agreeid, $groupid, $operatorId) {
		// 同意済みにする
		$connect = array(
			'id' => $agreeid,
			'status' => 1,
		);
		$result = $this->AgreementConnect->save($connect);
		if (!$result) {
			return false;
		}

		// 委任しているメンバーの利用属性を同意済みにする
		$agreeAccountIdList = $this->AgreementEntrustment->find('list', array(
			'fields' => array('account_id', 'account_id'),
			'conditions' => array('group_id' => $groupid)));
		// 操作した本人も追加
		$agreeAccountIdList += array($operatorId => $operatorId);
		// SPコネクタIDを取得
		$connect = $this->AgreementConnect->find('first', array(
			'conditions' => array('id' => $agreeid)));
		$spcid = $connect['AgreementConnect']['spc_id'];
		$attributeList = $this->AgreementConnect->getAttributeList($agreeid);
		foreach ($agreeAccountIdList as $accountid) {
			// agreements
			// 利用属性全てを現状のレコードにマージする
			$agreement = $this->Agreement->find('first', array(
				'conditions' => array('account_id' => $accountid, 'group_id' => $spcid)));
			$entity = array();
			if (empty($agreement)) {
				// insert
				$entity = $attributeList;
				$entity['account_id'] = $accountid;
				$entity['group_id'] = $spcid;
				$this->Agreement->create();
				$result = $this->Agreement->save($entity);
				if (!$result) {
					return false;
				}
			} else {
				// update
				$entity = $attributeList;
				$entity['id'] = $agreement['Agreement']['id'];
				$result = $this->Agreement->save($entity);
				if (!$result) {
					return false;
				}
			}

			// グループIDが利用属性でなければ次のアカウントへ
			if (!isset($attributeList['ismemberof'])) {
				continue;
			}

			//agreement_ismemberofs
			// このグループが登録されてなかったら登録する
			$ismemberof = $this->AgreementIsmemberof->find('first', array(
				'fields' => array('id', 'account_id', 'spc_id', 'group_id'),
				'conditions' => array(
					'account_id' => $accountid,
					'spc_id' => $spcid,
					'group_id' => $groupid)));
			if (empty($ismemberof)) {
				$this->AgreementIsmemberof->create();
				$ismemberof['AgreementIsmemberof']['account_id'] = $accountid;
				$ismemberof['AgreementIsmemberof']['spc_id'] = $spcid;
				$ismemberof['AgreementIsmemberof']['group_id'] = $groupid;
			}
			$ismemberof['AgreementIsmemberof']['ismemberof'] = 1;
			$result = $this->AgreementIsmemberof->save($ismemberof);
			if (!$result) {
				return false;
			}

			// このグループの上位グループがこのSPコネクタに接続していたら、
			// 上位グループのグループIDも同意済みにする
			$upGroupIdList = $this->AddGroup->getTopIdList($groupid, $onlyActive=false);
			if (!empty($upGroupIdList)) {
				$connUpGroupIdList = $this->AddGroup->getGroupIdListConnectedSpc($spcid, $upGroupIdList);
				foreach ($upGroupIdList as $id) {
					$ismemberof = $this->AgreementIsmemberof->find('first', array(
						'fields' => array('id', 'account_id', 'spc_id', 'group_id'),
						'conditions' => array(
							'account_id' => $accountid,
							'spc_id' => $spcid,
							'group_id' => $id)));
					if (empty($ismemberof)) {
						$this->AgreementIsmemberof->create();
						$ismemberof['AgreementIsmemberof']['account_id'] = $accountid;
						$ismemberof['AgreementIsmemberof']['spc_id'] = $spcid;
						$ismemberof['AgreementIsmemberof']['group_id'] = $id;
					}
					$ismemberof['AgreementIsmemberof']['ismemberof'] = 1;
					$result = $this->AgreementIsmemberof->save($ismemberof);
					if (!$result) {
						return false;
					}
				}
			}
		}

		// 下位グループに同意データを登録
		$result = $this->registerAgreeBottomGroup($agreeid, $groupid);
		if (!$result) {
			return false;
		}

		// メンバーにメール
		$this->sendMailMembers($groupid, $agreeAccountIdList, $spcid, $connect, $operatorId);


		// 下位グループの管理者にメールする
		$this->sendMailBottomAdmin($groupid, $spcid, $agreeid);
		return true;
	}

	/**
	 * 下位グループに同意データを登録する
	 * @param $agreeid AgreementConnect.id
	 */
	private function registerAgreeBottomGroup($agreeid, $topGroupid) {
		// 下位グループを検索
		$groupIdList = $this->AddGroup->find('list', array(
			'fields' => array('bottom_id', 'bottom_id'),
			'conditions' => array('top_id' => $topGroupid)));
		if (empty($groupIdList)) {
			return true;
		}

		$agreementConnect = $this->AgreementConnect->find('first', array(
			'conditions' => array('id' => $agreeid)));
		foreach($groupIdList as $groupid) {
//			$count = $this->AgreementConnect->find('count', array('conditions' => array(
//				'spc_id' => $agreementConnect['AgreementConnect']['spc_id'],
//				'bottom_group_id' => $groupid)));
//			if ($count > 0)
//				continue;
			$entity = array(
				'spc_id' => $agreementConnect['AgreementConnect']['spc_id'],
				'add_attributes' => $agreementConnect['AgreementConnect']['add_attributes'],
				'add_sps' => $agreementConnect['AgreementConnect']['add_sps'],
				'bottom_group_id' => $groupid,
				'status' => 0,
			);
			$this->AgreementConnect->create();
			$result = $this->AgreementConnect->save($entity);
			if (!$result) {
				return false;
			}
		}
		return true;
	}

	/**
	 * @param $groupid 自グループID
	 * @param $spcid SPコネクタID
	 * @param $connect AgreementConnect
	 * @param $operatorId 操作者のアカウントID
	 */
	private function sendMailMembers($groupid, $agreeAccountIdList, $spcid, $connect, $operatorId) {
		// メール情報の取得
		$group = $this->Group->find('first', array('conditions' => array('id' => $groupid)));
		$name = $this->Group->getGroupNameJp($group);
		$nameEn = $this->Group->getGroupNameEn($group);
		$serviceNameList = $this->GroupSphost->getServiceNameList($spcid);
		$Mail = array(
			'groupNameJp' => $name,
			'groupNameEn' => $nameEn,
			'serviceNameListJp' => $serviceNameList['jp'],
			'serviceNameListEn' => $serviceNameList['en'],
			'url' => Router::url('/agreements/', true),
		);

		// グループメンバー全員のIDを取得
		$accountIdList = $this->Mygroup->find('list', array(
			'fields' => array('account_id', 'account_id'),
			'conditions' => array('groupid' => $groupid)));

		// 委任していないメンバーにメールする
		$memberIdList = $this->Mygroup->find('list', array(
			'fields' => array('account_id', 'account_id'),
			'conditions' => array('groupid' => $groupid, 'NOT' => array('account_id' => $agreeAccountIdList))));
		$accountList = $this->Account->find('all', array(
			'conditions' => array('id' => $memberIdList)));
		foreach ($accountList as $account) {
			if ($account['Account']['is_temp'] > 0) {
				// 仮アカウントにはメールを送信しない
				continue;
			}
			if ($account['Account']['id'] == $operatorId) {
				// 承認操作した本人にはメールを送信しない
				continue;
			}

			// 同意していない属性を取得する
			$accountid = $account['Account']['id'];
			$needAttrList = $this->getNeedAttribute($accountid, $groupid, $spcid, $connect);
			$this->controller->set('needAttrList', $needAttrList);
			$this->controller->sendMail('agreement.use.attribute',
					$account['Account']['mail'],
					$account['Account']['language'], 'agree_spc_update_member_template', $Mail);
		}

		// 委任しているメンバーにメールをする
		$accountList = $this->Account->find('all', array(
			'conditions' => array('id' => $agreeAccountIdList)));
		foreach ($accountList as $account) {
			if ($account['Account']['is_temp'] > 0) {
				// 仮アカウントにはメールを送信しない
				continue;
			}
			if ($account['Account']['id'] == $operatorId) {
				// 承認操作した本人にはメールを送信しない
				continue;
			}

			$this->controller->sendMail('agreement.use.attribute',
					$account['Account']['mail'],
					$account['Account']['language'], 'agree_spc_update_delegate_member_template', $Mail);
		}
	}

	private function getNeedAttribute($accountid, $groupid, $spcid, $connect) {
		// 同意情報を取得する
		$agreementList = $this->Agreement->find('first', array('conditions' => array(
			'group_id' => $spcid,
			'account_id' => $accountid)));
		$ismemberofList = $this->AgreementIsmemberof->find('first', array('conditions' => array(
			'spc_id' => $spcid,
			'group_id' => $groupid,
			'account_id' => $accountid)));

		$list = array();
		$attributes = $connect['AgreementConnect']['add_attributes'];
		if (empty($attributes)) {
			return $list;
		}
		$attributeList = $this->explode(',', $attributes);
		foreach ($attributeList as $attr) {
			if ($attr === 'ismemberof') {
				if (empty($ismemberofList) || !$ismemberofList['AgreementIsmemberof']['ismemberof']) {
					array_push($list, 'ismemberof');
				}
			}
			$agreement_keys = array('eptid', 'name', 'mail', 'eppn', 'idp', 'introduction', 'language', 'organization');
			foreach ($agreement_keys as $agreement_key) {
				if ($attr === $agreement_key) {
					if (empty($agreementList) || !$agreementList['Agreement'][$agreement_key]) {
						array_push($list, $agreement_key);
					}
				}
			}
		}

		$showAttrList = $this->Agreement->getAttributeShowList($list);
		return $showAttrList;
	}

	/**
	 * 下位グループの管理者にメールする
	 * @param integer $groupid
	 * @param integer $spcid
	 * @param integer $agreeid
	 * @param string $template ﾒｰﾙテンプレート名
	 */
	function sendMailBottomAdmin($topGroupid, $spcid, $agreeid,
		$template = 'agree_spc_update_bottom_group_template') {
		// 下位グループを検索
		$groupIdList = $this->AddGroup->find('list', array(
			'fields' => array('bottom_id', 'bottom_id'),
			'conditions' => array('top_id' => $topGroupid)));
		if (empty($groupIdList)) {
			return;
		}

		// SPコネクタ情報を検索
		$spconnector = $this->Group->find('first', array('conditions' => array('id' => $spcid)));
		// 追加属性情報を設定
		$agreementConnect = $this->AgreementConnect->find('first', array('conditions' => array('id' => $agreeid)));
		$attributeList = $this->AgreementConnect->getAddAttributeNameList($agreementConnect['AgreementConnect']['add_attributes']);
		$Mail = array(
			'spcNameJp' => $this->Group->getGroupNameJp($spconnector),
			'spcNameEn' => $this->Group->getGroupNameEn($spconnector),
			'attributeListJp' => $attributeList['jp'],
			'attributeListEn' => $attributeList['en'],
			'targetSpHostList' => $this->explode(',', $agreementConnect['AgreementConnect']['add_sps']),
		);
		// 追加対象SP情報を設定
		foreach($groupIdList as $groupid) {
			// グループ情報を設定
			$group = $this->Group->find('first', array('conditions' => array('id' => $groupid)));
			$Mail['groupNameJp'] = $this->Group->getGroupNameJp($group);
			$Mail['groupNameEn'] = $this->Group->getGroupNameEn($group);
			$Mail['url'] = Router::url('/admin/manage_agreements/update/', true).$groupid;
			$this->controller->sendMailAllAdmin($groupid, 'agreement.use.attribute', $template, $Mail);
		}
	}

	/**
	 * 同意しないを選択
	 * @param $agreeid
	 * @param $groupid
	 * @return バリデーション結果
	 */
	function updateAgreeNG($agreeid, $groupid) {
		$result = $this->validateAgreement($agreeid, $groupid);
		if (!$result) {
			return false;
		}

		// 処理済みにする
		$connect = array(
			'id' => $agreeid,
			'status' => 2,
		);
		$this->AgreementConnect->save($connect);
		return true;
	}

	/**
	 * 同意処理をする必要があるかを検証する
	 */
	private function validateAgreement($agreeid, $groupid) {
		$connect = $this->AgreementConnect->find('first', array(
				'conditions' => array('id' => $agreeid)));
		if (empty($connect)) {
			$this->controller->Session->setFlash(__("message.manage_inspects.no.information", true), 'message');
			return false;
		}

		if ($connect['AgreementConnect']['top_group_id'] != $groupid
			&& $connect['AgreementConnect']['bottom_group_id'] != $groupid) {
			$this->controller->Session->setFlash(__("message.manage_inspects.no.information", true), 'message');
			return false;
		}

		if ($connect['AgreementConnect']['status'] != 0) {
			$this->controller->Session->setFlash(__("message.already.processed", true), 'message');
			return false;
		}

		return true;
	}

	/**
	 * SPコネクタ接続時、下位グループに同意データを登録する。
	 * @param $topGroupid 上位グループID
	 * @param $spcid SPコネクタID
	 * @reutn 同意ID
	 */
	public function registerBottomAdminByConnectSPC($topGroupid, $spcid) {
		// SPコネクタに利用属性が設定されていなかったら何もしない
		$count = $this->ProvideAttribute->find('count', array('conditions' => array(
			'group_id' => $spcid,
			'OR' => array('ismemberof' => 1, 'eptid' => 1, 'name' => 1, 'mail' => 1, 'eppn' => 1,
					'idp' => 1, 'introduction' => 1, 'language' => 1, 'organization' => 1))));
		if ($count == 0)
			return true;

		// 利用属性一覧を取得
		$attributeList = $this->ProvideAttribute->getProviteAttributeList($spcid);

		// 下位グループを検索
		$groupIdList = $this->AddGroup->find('list', array(
			'fields' => array('bottom_id', 'bottom_id'),
			'conditions' => array('top_id' => $topGroupid)));
		if (empty($groupIdList)) {
			// 下位グループが無ければ何もしない
			return true;
		}

		$addAttributeList = $this->AgreementConnect->generateAddAttributes($attributeList);
		$addAttributeString = implode(',', $addAttributeList);
		// 追加になった対象SPを設定
		$spIdList = $this->GroupSphost->find('list', array(
			'fields' => array('sp_id', 'sp_id'),
			'conditions' => array('group_id' => $spcid)));
		$spHostList = $this->SpHost->find('all', array('conditions' => array('id' => $spIdList)));
		$groupSphostList = $this->GroupSphost->find('all', array('conditions' => array('group_id' => $spcid)));
		$targetSpList = $this->AgreementConnect->getTargetSpList($spHostList, $groupSphostList);
		$addSpList = $targetSpList['addSpList'];
		$addSpString = implode(',', $addSpList);

		$connectId = 0;
		foreach ($groupIdList as $groupid) {
			$entity = array(
				'spc_id' => $spcid,
				'add_attributes' => $addAttributeString,
				'add_sps' => $addSpString,
				'bottom_group_id' => $groupid,
				'status' => 0,
			);
			$this->AgreementConnect->create();
			$result = $this->AgreementConnect->save($entity);
			if (!$result) {
				return false;
			}
			$connectId = $this->AgreementConnect->getLastInsertID();
		}

		// 複数の下位グループにAgreementConnectを登録しているが、登録したデータが同じなので
		// １つのAgreementConnect.idを返せばよい
		return $connectId;
	}

	/**
	 * 同意している属性を取得する
	 * @param $groupid グループID
	 * @param $accountid アカウントID
	 * @return array['spc_id']['group_id']
	 */
	function getAgreedList($groupid, $accountid) {
		// このグループが接続しているSPコネクタIDを取得する
		// そのSPコネクタに対して、直接接続しているグループのグループIDに同意を取っているかを確認する
		$spcIdList = $this->AddGroup->getConnectedSPCIdList($groupid, $onlyActive=false);
		$groupIdList = $this->AddGroup->getTopIdMyIdList($groupid, $onlyActive=false);
		// array['SPコネクタID']['グループID']
		$spcGroupList = array();
		$directGroupIdList = array();
		foreach ($spcIdList as $spcid) {
			// SPコネクタに直に接続しているグループID一覧
			$idList = $this->AddGroup->getGroupIdListConnectedSpc($spcid, $groupIdList);
			$spcGroupList[$spcid] = array();
			foreach ($idList as $id) {
				array_push($spcGroupList[$spcid], $id);
				array_push($directGroupIdList, $id);
			}
		}
		if (empty($spcGroupList)) {
			return array();
		}

		// SPコネクタ一覧に直に接続しているグループID一覧
		$directGroupIdList = array_unique($directGroupIdList);

		// SPコネクタに対して同意を取っているかを取得する
		$spAttrList = array();
		foreach ($spcGroupList as $spcid => $value) {
			$attrList = $this->Agreement->getAgreedList($spcid, $accountid);
			$spAttrList[$spcid] = array();
			$spAttrList[$spcid] = $attrList;
		}

		// ユーザがこのグループのismemberofに同意しているかを確認
		$ismemberList = $this->AgreementIsmemberof->find('list', array(
			'fields' => array('spc_id', 'group_id'),
			'conditions' => array(
				'account_id' => $accountid, 'spc_id' => $spcIdList, 'group_id' => $directGroupIdList, 'ismemberof' => 1)));
		// ismemberof属性と他の属性をひとつにする
		$agreedAttrList = array();
		foreach ($spAttrList as $spcid => $list) {
			$agreedAttrList[$spcid] = $list;
			$agreedAttrList[$spcid]['ismemberof'] = false;
			foreach ($ismemberList as $key => $value) {
				if ($spcid != $key) {
					continue;
				}
				$agreedAttrList[$spcid]['ismemberof'] = true;
				break;
			}
		}

		return $agreedAttrList;
	}

	private function explode($delimiter, $string) {
		if (empty($string)) {
			return array();
		}
		return explode($delimiter, $string);
	}
}
?>
