<?php
/**
 * ExtendedAttributeUtils Component
 *
 * PHP versions 5
 *
 * @copyright     Copyright (c) 2019, National Institute of Informatics,
 * @license       BSD License (http://creativecommons.org/licenses/BSD/)
 */
class ExtendedAttributeUtilsComponent extends Object {
	var $ExtendedAttribute;

	function initialize(&$controller) {
		$this->controller =& $controller;
		$this->ExtendedAttribute = ClassRegistry::init('ExtendedAttribute');
	}

	/**
	 * 拡張属性の再表示用設定
	 * @param $modelName モデル名
	 */
	public function extendedShow($modelName)
	{
		$this->log('extendedShow : in', "debug");
		if (empty($this->controller->data[$modelName]['attr_extended']) ||
				$this->controller->data[$modelName]['attr_extended'][0] == 0) {
			$this->log('extendedShow : NO check!', "debug");
			return; // 拡張属性のチェックがなければ抜ける
		}

		$extendedAttributes = array();
		$extendedAttrNames = array();
		$attrCount = 0;
		foreach ($this->controller->data['ExtendedAttrNames'] as $attr_name) {
			$new_attr_name = 'extended_attr' . ($attrCount + 1);
			$id = 0;
			if (!empty($this->controller->data['ExtendedAttributes'][$attr_name . '_id'])) {
				$id = $this->controller->data['ExtendedAttributes'][$attr_name . '_id'];
			}
			$extendedAttrNames[$attrCount] = $new_attr_name;
			$extendedAttributes[$new_attr_name . '_id'] = $id;
			$extendedAttributes[$new_attr_name . '_kind'] =
				$this->controller->data['ExtendedAttributes'][$attr_name . '_kind'];
			$extendedAttributes[$new_attr_name . '_name'] =
				$this->controller->data['ExtendedAttributes'][$attr_name . '_name'];
			$extendedAttributes[$new_attr_name . '_friendly_name'] =
				$this->controller->data['ExtendedAttributes'][$attr_name . '_friendly_name'];
			if (isset($this->controller->data['ExtendedAttributes'][$attr_name . '_value'])) {
				$extendedAttributes[$new_attr_name . '_value'] =
					$this->controller->data['ExtendedAttributes'][$attr_name . '_value'];
			}
			else {
				$extendedAttributes[$new_attr_name . '_value'] = null;
			}
			$attrCount++;
		}
		$this->controller->set('extendedAttrNames', $extendedAttrNames);
		$this->controller->set('extendedAttributes', $extendedAttributes);

		return;
	}

	/**
	 * 入力された拡張属性を保存する。
	 * @param $myGroupid グループ ID
	 * @param $modelName モデル名
	 */
	public function saveExtended($myGroupid, $modelName)
	{
		// 拡張属性を登録
		// 既に登録されている id のリストを取得する
		$list = $this->ExtendedAttribute->find('list', array('fields' => array('id', 'spc_id'),
			'conditions' => array('spc_id' => $myGroupid)));
		$this->log($list, "debug");
		$this->log('saveExtended : inputData = ', "debug");
		$this->log($this->controller->data, "debug");
		if (empty($this->controller->data[$modelName]['attr_extended']) ||
				$this->controller->data[$modelName]['attr_extended'][0] == 0) {
			if (empty($list)) {
				// 既存データが無くて拡張属性のチェックがなければ抜ける
				return true;
			}
			// 削除処理
			$this->log('Delete All exetended_attributes data spc_id = ' . $myGroupid, "debug");
			$this->ExtendedAttribute->deleteAll(array('spc_id' => $myGroupid), true);   // GroupExtendedValue も削除
			return true;
		}
		$this->log('input data = ', "debug");
		$this->log($this->controller->data['ExtendedAttributes'], "debug");
		$errorPoint = 0;
		foreach ($this->controller->data['ExtendedAttrNames'] as $attr_name) {
			$this->log('saveExtended : attr_name = ' . $attr_name, "debug");
			$id = 0;
			if (!empty($this->controller->data['ExtendedAttributes'][$attr_name . '_id'])) {
				$id = $this->controller->data['ExtendedAttributes'][$attr_name . '_id'];
			}
			$attribute = array();
			if ($id != 0 && (array_key_exists($id, $list))) {
				// アップデート対象
				$list[$id] = 0;
			}
			$attribute['ExtendedAttribute']['spc_id'] = $myGroupid;

			if (isset($this->controller->data['ExtendedAttributes'][$attr_name . '_kind'])) {
				$attribute['ExtendedAttribute']['kind'] = $this->controller->data['ExtendedAttributes'][$attr_name . '_kind'];
			}
			if (isset($this->controller->data['ExtendedAttributes'][$attr_name . '_name'])) {
				$attribute['ExtendedAttribute']['name'] = $this->controller->data['ExtendedAttributes'][$attr_name . '_name'];
			}
			if (isset($this->controller->data['ExtendedAttributes'][$attr_name . '_friendly_name'])) {
				$attribute['ExtendedAttribute']['friendly_name'] = $this->controller->data['ExtendedAttributes'][$attr_name . '_friendly_name'];
			}
			if (isset($this->controller->data['ExtendedAttributes'][$attr_name . '_value'])) {
				$attribute['ExtendedAttribute']['value'] = $this->controller->data['ExtendedAttributes'][$attr_name . '_value'];
			}
			if ($attribute['ExtendedAttribute']['kind'] == 1) {        // グル―プで入力
				$attribute['ExtendedAttribute']['value'] = 'group';
			}
			if ($attribute['ExtendedAttribute']['kind'] == 2) {        // 属性選択
				if (!$this->_checkProvideAttribute($attribute['ExtendedAttribute']['value'],
						$this->controller->data[$modelName])) {
					$this->controller->set('errorPoint', $errorPoint);
					$this->ExtendedAttribute->validationErrors['value'] =
						__("validation.attribute.incorrect.selected", true);
					return false;
				}
			}
			if ($id == 0) {
				$this->log('New! create', "debug");
				$this->ExtendedAttribute->create();
				$result = $this->ExtendedAttribute->save($attribute);
				if (!$result) {
					$this->controller->set('errorPoint', $errorPoint);
					return false;
				}
			} else {
				// 変更があるかを確認
				$beforeAttribute = $this->ExtendedAttribute->find('first', array(
					'fields' => array('id', 'spc_id', 'kind', 'name', 'friendly_name', 'value'),
					'conditions' => array('id' => $id)));
				$diff = $this->_diffExtended($beforeAttribute['ExtendedAttribute'],
					$attribute['ExtendedAttribute']);
				if ($diff) {
					// 違いがあれば更新する
					$this->log('Update id = ' . $id, "debug");
					$attribute['ExtendedAttribute']['id'] = $id;
					$result = $this->ExtendedAttribute->save($attribute);
					if (!$result) {
						$this->controller->set('errorPoint', $errorPoint);
						return false;
					}
				}
			}
			$errorPoint++;
		}
		// 削除処理
		$this->log('Delete list = ', "debug");
		$this->log($list, "debug");
		foreach ($list as $key => $group) {
			if ($group != 0) {
				// 既存だが今回の更新に id が無かった
				$this->log('Delete id = ' . $key, "debug");
				$this->ExtendedAttribute->delete($key, true);   // GroupExtendedValue も削除
			}
		}

		return true;
	}

	/**
	 * 拡張属性の属性選択のバリデーション
	 * @param $myGroupid グループ id
	 */
	private function _checkProvideAttribute($value, $selectAttr)
	{
		$result = false;
		$attrname = 'attr_' . $value;
		$this->log('checkProvideAttribute : attrname = ' . $attrname, "debug");
		$this->log('checkProvideAttribute : data = ', "debug");
		$this->log($selectAttr, "debug");
		if (!empty($selectAttr[$attrname]) && $selectAttr[$attrname][0] == 1) {
			$result = true;
		}
		return $result;
	}

	/**
	 * 拡張属性の変更を調べる。
	 * @param $beforeAttribute 既存データ
	 * @param $attribute 入力データ
	 */
	private function _diffExtended($beforeAttribute, $attribute)
	{
		$this->log('before = ', "debug");
		$this->log($beforeAttribute, "debug");
		$this->log('input = ', "debug");
		$this->log($attribute, "debug");
		if ($beforeAttribute['kind'] != $attribute['kind']) {
			return true;
		}
		if ($beforeAttribute['name'] != $attribute['name']) {
			return true;
		}
		if ($beforeAttribute['friendly_name'] != $attribute['friendly_name']) {
			return true;
		}
		if ($beforeAttribute['value'] != $attribute['value']) {
			return true;
		}
		return false;
	}
}
?>
