<?php
/**
 * GroupInvite Componet
 *
 * PHP versions 5
 *
 * @copyright     Copyright (c) 2011 - 2012, National Institute of Informatics,
 * @license       BSD License (http://creativecommons.org/licenses/BSD/)
 */
class GroupInviteComponent extends Object
{
	var $Mygroup;
	var $Account;
	var $Group;
	var $AddGroup;
	var $Invite;
	var $components = array('TakerAgreement', 'JoinOperator', 'AdminGroup');

	function initialize(&$controller) {
		$this->controller =& $controller;
		$this->Mygroup = ClassRegistry::init('Mygroup');
		$this->Account = ClassRegistry::init('Account');
		$this->Group = ClassRegistry::init('Group');
		$this->AddGroup = ClassRegistry::init('AddGroup');
		$this->Invite = ClassRegistry::init('Invite');
	}

	public function getCheckKeys($forcibleAdmission) {
		$checkKeys = array('valid');
		if ($forcibleAdmission) {
			$checkKeys[] = 'empty';
		}
		return $checkKeys;
	}

	public function canForce() {
		return $this->controller->inPersonScopedAffiliation(array('faculty', 'staff'));
	}

	public function getTemplate($lang, $group, $account, $less = 0) {
		if ($lang == 'ja') {
			return $this->getTemplateJa($group, $account, $less);
		} else {
			return $this->getTemplateEn($group, $account, $less);
		}
	}

	private function getCommonReplacePairs($group) {
		if ($this->controller->Common->isProduction()) {
			$systemName = __("default.map", true);
		} else {
			$systemName = __("default.test", true);
		}
		return array(
			'%SYSTEM_NAME%' => $systemName,
			'%GROUP_NAME%' => $group['name'],
			'%INTRODUCTION%' => $group['introduction']
		);
	}

	private function getTemplateJa($group, $account, $less) {
		$account_name = $account['name'] != '' ? $account['name'] : $account['name_en'];
		$group_or_sp = $group['sp'] ? __("common.sp", true) : __("common.group", true);
		if ($less) {
			$template = $this->getTemplateJaLess();
		} else {
			$template = $this->getTemplateJaTemplate();
		}

		return strtr($template, array(
			'%ACCOUNT_NAME%' => $account_name,
			'%GROUP_OR_SP%' => $group_or_sp,
		) + $this->getCommonReplacePairs($group));
	}

	private function getTemplateEn($group, $account, $less) {
		$account_name = $account['name_en'] != '' ? $account['name_en'] : $account['name'];
		$group_or_sp = $group['sp'] ? 'SP Connector' : 'Group';
		if ($less) {
			$template = $this->getTemplateEnLess();
		} else {
			$template = $this->getTemplateEnTemplate();
		}

		return strtr($template, array(
			'%ACCOUNT_NAME%' => $account_name,
			'%GROUP_OR_SP%' => $group_or_sp,
		) + $this->getCommonReplacePairs($group));
	}

	private function getTemplateJaTemplate() {
		return <<<_TEMPLATE_
%ACCOUNT_NAME% です。
学認クラウドゲートウェイサービスがグループ管理機能を提供しており、そこで
%GROUP_NAME% %GROUP_OR_SP%を管理しています。
この %GROUP_NAME% %GROUP_OR_SP%へ招待いたしますので、
是非、参加をお願いします。

<%GROUP_OR_SP%の紹介>
%INTRODUCTION%


下記のURLにて、入会処理をしてください。
宜しくお願い致します。

_TEMPLATE_;
	}

	private function getTemplateEnTemplate() {
		return <<<_TEMPLATE_
My name is %ACCOUNT_NAME%
I am an administrator of %GROUP_NAME% %GROUP_OR_SP%, one of the groups in ‘%SYSTEM_NAME%’, the group management system of the GakuNin Federation.
I would like to invite you to %GROUP_NAME% %GROUP_OR_SP%.
I am looking forward to you joining our group.

<About the %GROUP_OR_SP%>
%INTRODUCTION%


Please follow the membership procedure in the following URL.
Thank you.

_TEMPLATE_;
	}

	private function getTemplateJaLess() {	// 招待なし入会用テンプレート
		return <<<_TEMPLATE_
%ACCOUNT_NAME% です。
学認クラウドゲートウェイサービスがグループ管理機能を提供しており、そこで
%GROUP_NAME% グループを管理しています。
あなたをこの %GROUP_NAME% グループのメンバーにいたしました。

<グループの紹介>
%INTRODUCTION%


なお、当グループに必要な属性送信には自動的に同意した状態となっております。
以下のURLからご確認ください。

_TEMPLATE_;
	}

	private function getTemplateEnLess() {
		return <<<_TEMPLATE_
My name is %ACCOUNT_NAME%
I am an administrator of %GROUP_NAME% Group, one of the groups in ‘%SYSTEM_NAME%’, the group management system of the GakuNin Federation.
I have made you a member of this %GROUP_NAME% Group.

<About the Group>
%INTRODUCTION%


In addition, you are considered to agree to send attributes required to act as a group member.
Please check the status in the following URL.
Thank you.

_TEMPLATE_;
	}

	// $params = array(
	//   'accountId' => $accountId,
	//   'group' => $group, # GroupInvite->findGroup()
	//   'addresses' => array($mail, $mail),
	//   'addressMl' => $mail,
	//   'memberIdList' => array($accountId, $accountId) // adminInputのaccountIdも含む
	//   'adminInput' => array($accountId => $mail)
	//   'body' => $body, // 招待メール本文
	//   'lang' => $lang, // 招待メール言語
	//   'accountLang' => $accountLang, // 管理者の言語
	public function execute($params) {
		$params += array(
			'forcibleAdmission' => false,
		);
		$addresses = $this->getValidAddresses($params['addresses']);
		$params['addresses'] = array_unique($addresses);

		if ($params['forcibleAdmission']) {
			return $this->executeForce($params);
		} else {
			return $this->executeNormal($params);
		}
	}

	private function getAddressAccounts($params) {
		//	複雑なので実行時に条件分岐をさせずに同じ値でもそれぞれのキーに入れています
		//	$address_accounts = array(
		//		array(
		//			'inviteDbMail' => $mail or $adminInput,
		//			'inviteMail' => $mail
		//			'accountMail' => $mail or null or $adminInput,
		//			'term' => 'time' or 'onetime',
		//			'account' => null or $account,
		//			'isAdminInput' => false or true,
		//		),
		//	);

		// メールアドレス（個人）
		$address_accounts = array();
		foreach ($params['addresses'] as $address) {
			$address_accounts[] = array(
				'inviteDbMail' => $address,
				'inviteMail' => $address,
				'accountMail' => $address,
				'term' => 'onetime',
				'account' => null,
				'isAdminInput' => false,
			);
		}

		// メールアドレス（メーリングリスト）
		$addressesMl = $this->getValidAddresses(array($params['addressMl']));
		$addressMl = implode("\n", $addressesMl);
		if ($addressMl) {
			$address_accounts[] = array(
				'inviteDbMail' => $addressMl,
				'inviteMail' => $addressMl,
				'accountMail' => null,
				'term' => 'time',
				'account' => null,
				'isAdminInput' => false,
			);
		}

		$memberIdList = $params['memberIdList'];
		$adminInput = $params['adminInput'];
		if ($memberIdList) {
			$accountList = $this->Account->find('all', array(
				'fields' => array('id', 'mail', 'name', 'name_en', 'language'),
				'conditions' => array('id' => $memberIdList),
			));
			foreach ($accountList as $account) {
				// 名前選択（同意済み）
				$value = array(
					'inviteDbMail' => $account['Account']['mail'],
					'inviteMail' => $account['Account']['mail'],
					'accountMail' => $account['Account']['mail'],
					'term' => 'onetime',
					'account' => $account['Account'],
					'isAdminInput' => false,
				);
				// 名前選択（同意なしで管理者入力）
				if (isset($adminInput[$account['Account']['id']])) {
					if (!empty($adminInput[$account['Account']['id']])) {
						$value['inviteDbMail'] = $adminInput[$account['Account']['id']];
						$value['accountMail'] = $adminInput[$account['Account']['id']];
						$value['isAdminInput'] = true;
					} else {
						// メールアドレス入力なし = 仮アカウントなしで招待
						$value['accountMail'] = null;
						$value['isAdminInput'] = false;
					}
				}
				$address_accounts[] = $value;
			}
		}
		return $address_accounts;
	}

	public function findGroup($groupid)
	{
		$group = $this->Group->find('first', array('conditions' => array('id' => $groupid)));
		if (empty($group)) {
			throw new Exception('find group error');
		}
		return $group['Group'];
	}
	public function findGroupByGroupKey($groupKey)
	{
		$group = $this->Group->find('first', array('conditions' => array('group_key' => $groupKey)));
		if (empty($group)) {
			throw new Exception('find group error');
		}
		return $group['Group'];
	}

	private function executeNormal($params) {
		$accountId = $params['accountId'];
		$group = $params['group'];
		$body = $params['body'];
		$lang = $params['lang'];
		$accountLang = $params['accountLang'];

		// 招待主のメールアドレスをRely-Toに設定
		$replyTo = '';
		$account = $this->Account->find('first', array('conditions' => array('id' => $accountId)));
		if (isset($account['Account']['mail'])) {
			$replyTo = $account['Account']['mail'];
		}

		// メールアドレスの取得
		$address_accounts = $this->getAddressAccounts($params);
		foreach ($address_accounts as $address_account_key => $address_account) {
			$inviteDbMail = $address_account['inviteDbMail'];
			$accountMail = $address_account['accountMail'];
			$term = $address_account['term'];
			$invitedAccount = $address_account['account'];

			$tempAccountId = null;
			// tempAccountId取得のため有効なものも含めてgroup_idとaddress or invitedAccountIdで該当するInviteをすべて取得
			$invites = $this->Invite->getByGroupId_User($group['id'], $inviteDbMail, $invitedAccount['id']);
			if ($invites) {
				foreach ($invites as $invite) {
					if (isset($invite['Invite']['temp_account_id'])) {
						if ($invite['Invite']['status'] == 2) {
							// 無効データを無視する
							continue;
						}
						$tempAccountId = $invite['Invite']['temp_account_id'];
					}
					// Inviteを無効にできるものは無効にする
					$this->disableInvite($invite);
				}
			}

			// SPコネクタの場合 or メーリングリストへの招待は仮アカウントは作らない
			if ($group['sp'] || is_null($accountMail)) {
				$tempAccountId = null;
			} else {
				if ($tempAccountId) {
					// すでに仮アカウントがあり、アカウントから選んだ場合は念のため名前を更新する
					if ($invitedAccount) {
						$this->updateTempAccount($tempAccountId, $invitedAccount, $accountMail);
					}
				} else {
					// 仮アカウント登録
					$tempAccountId = $this->registerTempAccount($group['id'], $accountMail, $invitedAccount, $address_account['isAdminInput']);
				}
			}

			$result = $this->saveInvite(
				$accountId, $inviteDbMail, $group['id'],
				$term, $body, $lang,
				$tempAccountId,
				null,
				$invitedAccount['id']
			);
			if (!$result) {
				$this->controller->log('ManageGroupsController systemError _saveInvite faild', 'error');
				return false;
			}
			$address_accounts[$address_account_key]['random'] = $result['Invite']['random'];
		}

		// メール送信
		$url = $this->getInviteUrl($group['group_key']);
		if (empty($body)) { // manage_agenciesの場合
			$lang = array('jp', 'eng');
			if ($accountLang == 'eng') {
				$lang = array_reverse($lang);
			}
			$account = $this->Account->find('first', array('fields' => array('name', 'name_en'),
					'conditions' => array('id' => $accountId)));
			$variables = array(
				'body' => $this->getTemplate('ja', $group, $account['Account']),
				'bodyEN' => $this->getTemplate('en', $group, $account['Account']),
			);
		} else {
			$variables = $body;
		}

		$account = $this->Account->find('first', array('fields' => array('name', 'name_en'),
				'conditions' => array('id' => $accountId)));
		foreach ($address_accounts as $address_account) {
			$mailLang = $lang;
			if (empty($body)) {
				// 名前から選択時はその人の使用言語を利用する
				if (isset($address_account['account']['language'])) {
					$mailLang = $address_account['account']['language'];

					$variables = $this->getTemplate('ja', $group, $account['Account']);
					if ($mailLang === 'eng' || $mailLang === 'en') {
						$variables = $this->getTemplate('en', $group, $account['Account']);
					}
				}
			}
			$this->controller->log('executeNormal $mailLang='.var_export($mailLang, true), 'debug');

			$this->sendInviteMail(
				$url, $address_account['random'], $address_account['inviteMail'], $replyTo,
				$group['sp'], $variables, $mailLang, $accountLang
			);
		}
		return true;
	}

	public function getInviteUrl($group_key)
	{
		return Router::url('/', true).'invite/rgst/'.$group_key;
	}

	private function updateTempAccount($tempAccountId, $invitedAccount, $accountMail) {
		$updateAccount = $invitedAccount;
		$updateAccount['id'] = $tempAccountId;
		$updateAccount['mail'] = $accountMail;
		$this->Account->save(array('Account' => $updateAccount));
	}

	private function disableInvite($invite) {
		if ($invite['Invite']['status'] != 0) {
			return;
		}
		$orgInvite = array(
			'Invite' => array(
				'id' => $invite['Invite']['id'],
				'status' => 2,
			),
		);
		$this->Invite->save($orgInvite);
	}

	private function getAddress2AccountIds($addresses) {
		$address2accountIds = array();
		foreach ($addresses as $address) {
			$account = $this->Account->find('first', array(
				'fields' => array('id'),
				'conditions' => array(
					'mail' => $address,
					'is_temp' => 0,
				),
				'order' => array('id'),
			));
			if ($account) {
				$address2accountIds[$address] = $account['Account']['id'];
			}
		}
		return $address2accountIds;
	}

	private function convertMemberIdList($params) {
		if (!$params['addresses']) {
			return $params;
		}
		$address2accountIds = $this->getAddress2AccountIds($params['addresses']);
		if (!$address2accountIds) {
			return $params;
		}

		$memberIdList = $params['memberIdList'];
		$addresses = $params['addresses'];
		foreach ($address2accountIds as $address => $accountId) {
			$key = array_search($address, $addresses);
			if ($key !== false) {
				unset($addresses[$key]);
				$memberIdList[] = $accountId;
			}
		}
		// 重複している可能性があるのでuniqueに
		$memberIdList = array_unique($memberIdList);

		// Mygroupに登録済みのアカウントを除く
		$mygroups = $this->Mygroup->find('all', array(
			'fields' => array('account_id'),
			'conditions' => array('groupid' => $params['group']['id'])
		));
		foreach ($mygroups as $mygroup) {
			$key = array_search($mygroup['Mygroup']['account_id'], $memberIdList);
			if ($key !== false) {
				unset($memberIdList[$key]);
			}
		}

		$params['memberIdList'] = $memberIdList;
		$params['addresses'] = $addresses;
		return $params;
	}

	private function executeForce($params) {
		$orgAddresses = $params['addresses'];
		$params = $this->convertMemberIdList($params);

		if (!$this->joinGroup($params, $orgAddresses)) {
			return false;
		}
		// 通常招待へフォールバックしない
		return true;
	}

	public function joinGroupDB($groupid, $accountId) {
		// グループに参加させる
		$mygroup = array(
			'account_id' => $accountId,
			'groupid' => $groupid,
			'admin' => 0,
		);
		$this->Mygroup->create();
		if (!$this->Mygroup->save($mygroup)) {
			throw new Exception('Mygroup save error');
		}
		// 利用属性を取得
		$spcIdList = $this->AddGroup->getConnectedSPCIdList($groupid, $onlyActive=false);
		$attributeList = $this->TakerAgreement->getAttributeListForSave($spcIdList);
		// 利用属性を登録
		$result = $this->TakerAgreement->saveAttribute(
				$groupid, $attributeList, $spcIdList, $accountId);
		if (!$result) {
			throw new Exception('saveAttribute error');
		}

		// 委任登録はしない
	}

	private function joinGroup($params, $orgAddresses) {
		if (!$params['memberIdList']) {
			return true;
		}

		foreach ($params['memberIdList'] as $accountId) {
			$this->joinGroupDB($params['group']['id'], $accountId);
		}
		$accountList = $this->Account->find('all', array(
			'fields' => array('id', 'mail', 'language'),
			'conditions' => array('id' => $params['memberIdList']),
		));

		// 招待なし入会を実行した管理者のメールアドレスをRely-Toに設定
		$replyTo = '';
		$repAccount = $this->Account->find('first', array('conditions' => array('id' => $params['accountId'])));
		if (isset($repAccount['Account']['mail'])) {
			$replyTo = $repAccount['Account']['mail'];
		}

		foreach ($accountList as $account) {
			$inviteDbMail = $account['Account']['mail'];
			$inviteAccountId = $account['Account']['id'];
			$joinedAccountId = null;
			if (array_search($inviteDbMail, $orgAddresses) !== false) {
				$joinedAccountId = $inviteAccountId;
				$inviteAccountId = null;	// メールアドレス（個人）で招待
			}
			$result = $this->saveInvite(
				$params['accountId'], $inviteDbMail, $params['group']['id'],
				'onetime', $params['body'], $params['lang'],
				null, null, $inviteAccountId,
				1, 1, $joinedAccountId
			);
			if (!$result) {
				$this->controller->log('ManageGroupsController systemError _saveInvite faild', 'error');
				return false;
			}

			if (empty($account['Account']['mail'])) {
				continue;
			}
			$this->sendJoinMail($account['Account'], $params['group'], $params['body'], $replyTo, $params['lang']);
		}

		return true;
	}

	private function sendJoinMail($account, $group, $body, $replyTo, $lang) {
		$url = Router::url('/', true).'group_lists/detail/'.$group['id'];

		$variables = array(
			'body' => $body,
			'url' => $url,
		);

		$this->controller->sendMail('join.group', $account['mail'], $lang, 'joined_group_template', $variables, null, $replyTo);
	}


	/**
	 * 招待情報を保存します。
	 * @param $accountId
	 * @param $address
	 * @param $random
	 * @param $group_id
	 * @param $term
	 * @param $body
	 * @param $lang
	 * @param $tempAccountId
	 * @param $inviteId
	 * @param $inviteAccountId 招待される人のアカウントID
	 * @param $status
	 * @param $forcible 強制入会フラグ
	 * @param $joinedAccountId メールアドレス（個人）で入会した人のアカウントID
	 */
	public function saveInvite($accountId, $address, $groupid, $term, $body,
		$lang, $tempAccountId, $inviteId, $inviteAccountId, $status = 0, $forcible = 0, $joinedAccountId = null) {
		if (empty($address) && !$forcible) {
			return;
		}
		$counter = 0;
		if ($forcible) {
			$counter = 1;
		}

		$Invite = array(
			'Invite' => array(
				'groupid' => $groupid,
				'address' => $address,
				'term' => $term,
				'status' => $status,
				'counter' => $counter,
				'body' => $body,
				'language' => $lang,
				'resend_id' => $inviteId,
				'account_id' => $accountId,
				'invited_account_id' => $inviteAccountId,
				'temp_account_id' => $tempAccountId,
				'forcible' => $forcible,
				'joined_account_id' => $joinedAccountId,
			),
		);
		$this->Invite->create();
		return $this->Invite->save($Invite);
	}

	private function registerTempAccount($groupid, $address, $invitedAccount, $isAdminInput) {
		// 仮アカウント作成
		$accountId = $this->Account->createTemporaryAccount($groupid, $address, $invitedAccount, $isAdminInput);
		if (!$accountId) {
			throw new Exception('createTemporaryAccount error');
		}

		// グループへの仮入会
		$mygroup = array(
			'account_id' => $accountId,
			'groupid' => $groupid,
			'admin' => 0,
		);
		$this->Mygroup->create();
		$result = $this->Mygroup->save($mygroup);
		if (!$result) {
			throw new Exception('Mygroup create error');
		}

		// 利用属性を登録
		// 自グループID＋上位グループIDを検索
		$groupIdList = $this->AddGroup->getTopIdMyIdList($groupid, $onlyActive=false);
		foreach ($groupIdList as $gid) {
			// このグループに接続しているSPコネクタのIDを取得
			$spcIdList = $this->AddGroup->getConnectedSPCIdList($gid, $onlyActive=false);
			$attributeList = $this->TakerAgreement->getAttributeListForSave($spcIdList);
			$mailAttributeList = array();
			foreach ($spcIdList as $spcId) {
				if (isset($attributeList['mail'.$spcId])) {
					$mailAttributeList['mail'.$spcId] = $attributeList['mail'.$spcId];
				}
				if (isset($attributeList['ismemberof'.$spcId])) {
					$mailAttributeList['ismemberof'.$spcId] = $attributeList['ismemberof'.$spcId];
				}
				if (isset($attributeList['eptid'.$spcId])) {
					$mailAttributeList['eptid'.$spcId] = $attributeList['eptid'.$spcId];
				}
			}
			// 利用属性を登録
			$result = $this->TakerAgreement->saveAttribute(
					$gid, $mailAttributeList, $spcIdList, $accountId);
			if (!$result) {
				throw new Exception('saveAttribute error');
			}
		}

		// 委任登録
		$entrustment = true;
		$result = $this->JoinOperator->entrust(
			$groupid, $accountId, $entrustment);
		if (!$result) {
			throw new Exception('entrustment error');
		}
		return $accountId;
	}

	/**
	 * 招待メールを送信する
	 * @param string $url
	 * @param string $random
	 * @param string $address
	 * @param boolean $sp
	 * @param mixed $variables array or string
	 * @param stirng $defaultLang
	 */
	public function sendInviteMail($url, $random, $address, $replyTo, $sp, $variables, $defaultLang, $accountLang) {
		$name = __("common.group", true);
		$nameEN= __("mail.group.en", true);
		if ($sp) {
			$name = __("common.sp", true);
			$nameEN = __("mail.sp.en", true);
		}
		if (is_string($variables)) {
			$variables = array(
				'body' => $variables,
			);
		}
		$variables += array(
			'groupNameOrSpName' => $name,
			'groupNameOrSpNameEN' => $nameEN,
			'url' => $url ."/". $random,
			'language' => $accountLang,
		);
		$this->controller->sendMail("invite.mail", $address, $defaultLang, 'invite_template', $variables, null, $replyTo);
	}

	public function validateMygroupMember($accountId, $member_ids) {
		if (empty($member_ids)) {
			return false;
		}
		// 管理しているグループのメンバーかを確認
		$adminGroupIdList = $this->Mygroup->find('list', array('fields' => array('groupid'),
			'conditions' => array(
			'account_id' => $accountId,
			'admin' => array(1, 2)))
		);
		$groupMemberCount = $this->Mygroup->find('count', array(
			'fields' => array('count(DISTINCT account_id) as count'),
			'conditions' => array('groupid' => $adminGroupIdList, 'account_id' => $member_ids))
		);

		if (count($member_ids) != $groupMemberCount) {
			return false;
		}
		return true;
	}

	public function getInvalidAddresses($addresses) {
		$validated_addresses = $this->validateEmailAddresses($addresses);
		return $validated_addresses['invalid'];
	}

	public function getValidAddresses($addresses) {
		$validated_addresses = $this->validateEmailAddresses($addresses);
		return $validated_addresses['valid'];
	}

	private function validateEmailAddresses($addresses) {
		$results = array(
			'valid'   => array(),
			'invalid' => array(),
		);
		foreach ($addresses as $address_key => $address) {
			$address = $this->trim($address);
			if ($address == '') { continue; }

			$address = $this->getAddrRpec($address);
			$key = Validation::email($address) ? 'valid' : 'invalid';
			$results[$key][$address_key] = $address;
		}
		return $results;
	}

	private function trim($string) {
		return trim(str_replace("\xc2\xa0", '', $string));
	}

	/**
	 * addr-spec形式のメールアドレスを返す
	 * @param string $email email address
	 * @return string email address
	 */
	private function getAddrRpec($email) {
//		$local1 = "[|a-zA-Z0-9!#$%&\'*+\/=?^_`{|}~-][a-zA-Z0-9!#$%&\'*+\/=?^_`{|}~-]+(?:\.[a-zA-Z0-9!#$%&\'*+\/=?^_`{|}~-]+)*";
//		$local2 = "[\"][|a-zA-Z0-9!#$%&\'*+\/=?^_`{|}~-][a-zA-Z0-9!#$%&\'*+\/=?^_`{|}~-]+(?:\.[a-zA-Z0-9!#$%&\'*+\/=?^_`{|}~-]+)*[\"]";
		$local1 = "[|a-zA-Z0-9!#$%&\'*+\/=?^_`{|}~-][\.a-zA-Z0-9!#$%&\'*+\/=?^_`{|}~-]+(?:\.[a-zA-Z0-9!#$%&\'*+\/=?^_`{|}~-]+)*";
		$local2 = "[\"][|a-zA-Z0-9!#$%&\'*+\/=?^_`{|}~-][\.a-zA-Z0-9!#$%&\'*+\/=?^_`{|}~-]+(?:\.[a-zA-Z0-9!#$%&\'*+\/=?^_`{|}~-]+)*[\"]";
		$domain = "(?:[_\p{L}0-9][-_\p{L}0-9]*\.)*(?:[\p{L}0-9][-\p{L}0-9]{0,62})\.(?:(?:[a-z]{2}\.)?[a-z]{2,})";
		$match1 = $local1."[@]".$domain;
		$match2 = $local2."[@]".$domain;

		if (preg_match("/($match1)/", $email, $matches) ) {
			return $matches[0];
		} else if (preg_match("/($match2)/", $email, $matches) ) {
			return $matches[0];
		}
		return $email;
	}
}
