<?php
/**
 * SP Connector Component
 *
 * PHP versions 5
 *
 * @copyright     Copyright (c) 2011 - 2012, National Institute of Informatics,
 * @license       BSD License (http://creativecommons.org/licenses/BSD/)
 */
class SpcUpdatorComponent extends Object {

	var $Group;
	var $SpHost;
	var $GroupSphost;
	var $AddGroup;
	var $AutoGroup;
	var $AgreementConnect;

   	function initialize(&$controller) {
		$this->controller =& $controller;
		$this->Group = ClassRegistry::init('Group');
		$this->AddGroup = ClassRegistry::init('AddGroup');
		$this->AutoGroup = ClassRegistry::init('AutoGroup');
		$this->SpHost = ClassRegistry::init('SpHost');
		$this->GroupSphost = ClassRegistry::init('GroupSphost');
		$this->AgreementConnect = ClassRegistry::init('AgreementConnect');;
   	}

   	/**
	 * SPコネクタに接続している上位グループの管理者へ
	 * 利用属性情報または対象SPが増えた場合に新たに同意する旨を連絡する。
	 * @param $spcid SPコネクタID
	 */
	function sendMailGroup($spcid, $beforeAttribute, $beforeTargetSp, $afterAttribute, $afterTargetSp) {
		// 新たに設定された属性を設定
		$addAttributes = array();
		if (!$beforeAttribute['ProvideAttribute']['ismemberof'] && $afterAttribute['ProvideAttribute']['ismemberof'])
			$addAttributes['ismemberof'] = true;
		if (!$beforeAttribute['ProvideAttribute']['eptid'] && $afterAttribute['ProvideAttribute']['eptid'])
			$addAttributes['eptid'] = true;
		if (!$beforeAttribute['ProvideAttribute']['name'] && $afterAttribute['ProvideAttribute']['name'])
			$addAttributes['name'] = true;
		if (!$beforeAttribute['ProvideAttribute']['mail'] && $afterAttribute['ProvideAttribute']['mail'])
			$addAttributes['mail'] = true;
		if (!$beforeAttribute['ProvideAttribute']['eppn'] && $afterAttribute['ProvideAttribute']['eppn'])
			$addAttributes['eppn'] = true;
		if (!$beforeAttribute['ProvideAttribute']['idp'] && $afterAttribute['ProvideAttribute']['idp'])
			$addAttributes['idp'] = true;
		if (!$beforeAttribute['ProvideAttribute']['introduction'] && $afterAttribute['ProvideAttribute']['introduction'])
			$addAttributes['introduction'] = true;
		if (!$beforeAttribute['ProvideAttribute']['language'] && $afterAttribute['ProvideAttribute']['language'])
			$addAttributes['language'] = true;
		if (!$beforeAttribute['ProvideAttribute']['organization'] && $afterAttribute['ProvideAttribute']['organization'])
			$addAttributes['organization'] = true;

		// 新たに設定された対象SPのIDを設定
		$addTargetSpIdList = array();
		foreach ($afterTargetSp as $after) {
			$add = true;
			foreach ($beforeTargetSp as $before) {
				if ($after === $before) {
					$add = false;
					break;
				}
			}
			if ($add)
				array_push($addTargetSpIdList, $after);
		}

		if (empty($addAttributes) && empty($addTargetSpIdList))
			return true;

		$result  = $this->sendMailTopGroupAdmin($spcid, $addAttributes, $addTargetSpIdList);
		return $result;
	}

	/**
	 * SPコネクタに接続している上位グループの管理者へメールする
	 * @param $spcid SPコネクタID
	 * @param $addAttributeList 追加された属性一覧
	 * @param $addTargetSpIdList 追加された対象SPのID一覧
	 */
	private function sendMailTopGroupAdmin($spcid, $addAttributes, $addTargetSpIdList) {
		// SPコネクタに接続している上位グループIDを検索
		$groupIdList = $this->AddGroup->find('list', array(
			'fields' => array('bottom_id', 'bottom_id'),
			'conditions' => array('top_id' => $spcid, 'top' => 1)));
		$groupIdList = $this->AutoGroup->getGroupIdListWithOutIdPGroup($groupIdList);
		if (empty($groupIdList))
			return true;

		$result = $this->sendMailConnectGroupAdmin($spcid, $addAttributes, $addTargetSpIdList, $groupIdList);
		return $result;
	}

	/**
	 * SPコネクタに接続しているグループの管理者へメールする
	 * @param $spcid SPコネクタID
	 * @param $addAttributeList 追加された属性一覧
	 * @param $addTargetSpIdList 追加された対象SPのID一覧
	 * @param $groupIdList メールするグループID一覧
	 */
	private function sendMailConnectGroupAdmin($spcid, $addAttributes, $addTargetSpIdList, $groupIdList) {
		$spc = $this->Group->find('first', array('conditions' => array('id' => $spcid)));
		$Mail = array();
		$Mail['spcNameJp'] = $this->Group->getGroupNameJp($spc);
		$Mail['spcNameEn'] = $this->Group->getGroupNameEn($spc);


		// 追加になった属性を設定
		$Mail['attributeListJp'] = array();
		$Mail['attributeListEn'] = array();
		$addAttributeList = array();
		if (isset($addAttributes['ismemberof']) && $addAttributes['ismemberof']) {
			array_push($Mail['attributeListJp'], __("common.group.id.ja", true));
			array_push($Mail['attributeListEn'], __("common.group.id.eng", true));
		}
		if (isset($addAttributes['eptid']) && $addAttributes['eptid']) {
			array_push($Mail['attributeListJp'], __("common.eptid", true));
			array_push($Mail['attributeListEn'], __("common.eptid", true));
		}
		if (isset($addAttributes['name']) && $addAttributes['name']) {
			array_push($Mail['attributeListJp'], __("common.name.jp", true));
			array_push($Mail['attributeListEn'], __("common.name.en", true));
		}
		if (isset($addAttributes['mail']) && $addAttributes['mail']) {
			array_push($Mail['attributeListJp'], __("common.mailaddress.ja", true));
			array_push($Mail['attributeListEn'], __("common.mailaddress.eng", true));
		}
		if (isset($addAttributes['eppn']) && $addAttributes['eppn']) {
			array_push($Mail['attributeListJp'], __("common.eppn", true));
			array_push($Mail['attributeListEn'], __("common.eppn", true));
		}
		$addAttributeList = $this->AgreementConnect->generateAddAttributes($addAttributes);

		// 追加になった対象SPを設定
		// ホスト名を検索
		$spHostList = $this->SpHost->find('all', array('conditions' => array('id' => $addTargetSpIdList)));
		// サービス名を検索
		$groupSphostList = $this->GroupSphost->find('all', array(
			'conditions' => array('group_id' => $spcid, 'sp_id' => $addTargetSpIdList)));
		$targetSpList = $this->AgreementConnect->getTargetSpList($spHostList, $groupSphostList);
    	$Mail['targetSpListJp'] = $targetSpList['targetSpListJp'];
    	$Mail['targetSpListEn'] = $targetSpList['targetSpListEn'];
    	$addSpList = $targetSpList['addSpList'];

    	// グループを検索
		$groupList = $this->Group->find('all', array(
			'fields' => array('id', 'name', 'name_en'),
			'conditions' => array('id' => $groupIdList)));

		// グループ接続同意に登録
		$addAttributeString = implode(',', $addAttributeList);
		$addSpString = implode(',', $addSpList);
		foreach ($groupIdList as $groupid) {
			$agreementConnect = array();
			$agreementConnect['spc_id'] = $spcid;
			$agreementConnect['top_group_id'] = $groupid;
			if (!empty($addAttributeString))
				$agreementConnect['add_attributes'] = $addAttributeString;
			if (!empty($addSpString))
				$agreementConnect['add_sps'] = $addSpString;
			$this->AgreementConnect->create();
			$result = $this->AgreementConnect->save($agreementConnect);
			if (!$result) {
				return false;
			}
		}

		// グループ管理者へメールする
		foreach ($groupIdList as $groupid) {
			$Mail['url'] = Router::url('/admin/manage_agreements/update/', true).$groupid;
			foreach ($groupList as $group) {
				if ($groupid === $group['Group']['id']) {
					if (!empty($group['Group']['name']))
						$Mail['groupNameJp'] = $group['Group']['name'];
					else
						$Mail['groupNameJp'] = $group['Group']['name_en'];

					if (!empty($group['Group']['name_en']))
						$Mail['groupNameEn'] = $group['Group']['name_en'];
					else
						$Mail['groupNameEn'] = $group['Group']['name'];
					break;
				}
			}

			$this->controller->sendMailAllAdmin($groupid, 'agreement.use.attribute', 'agree_spc_update_template', $Mail);
		}

		return true;
	}
}
?>
