<?php
/**
 * TakerAgreement Component
 *
 * PHP versions 5
 *
 * @copyright     Copyright (c) 2016, SOUM Corporation.
 * @copyright     Copyright (c) 2011 - 2012, National Institute of Informatics,
 * @license       BSD License (http://creativecommons.org/licenses/BSD/)
 */
class TakerAgreementComponent extends Object {

	var $Agreement;
	var $Mygroup;
	var $AddGroup;
	var $ProvideAttribute;
	var $Group;
	var $GroupSphost;
	var $SpHost;
	var $AgreementIsmemberof;
	var $AgreementEntrustment;
	var $AutoGroup;

	function initialize() {
		$this->Agreement = ClassRegistry::init('Agreement');
		$this->Mygroup = ClassRegistry::init('Mygroup');
		$this->AddGroup = ClassRegistry::init('AddGroup');
		$this->ProvideAttribute = ClassRegistry::init('ProvideAttribute');
		$this->Group = ClassRegistry::init('Group');
		$this->GroupSphost = ClassRegistry::init('GroupSphost');
		$this->SpHost = ClassRegistry::init('SpHost');
		$this->AgreementIsmemberof = ClassRegistry::init('AgreementIsmemberof');
		$this->AgreementEntrustment = ClassRegistry::init('AgreementEntrustment');
		$this->AutoGroup = ClassRegistry::init('AutoGroup');
	}

	/**
	 * 同意する必要があるかを調べ、必要がある場合は同意情報を返却する。
	 * @param $accountid
	 */
	function isShowAgree($accountid) {
		$result = $this->isNeedAgreement($accountid);
		return $result;
	}

	/**
	 * 同意情報を取得する。
	 * @param $accountid アカウントID
	 * @param $lang 使用言語
	 * @param $spcIdList 対象となるSPコネクタID一覧
	 * @param $groupIdList 対象となるグループID一覧
	 * @param $merge true:集約する false:集約しない
	 * @param $onlyMember true:メンバーのグループに関してのみ false:所属しているグループすべての情報を取得する
	 */
	function servData($accountid, $lang, $spcIdList = array(), $groupIdList = array(), $merge = false, $onlyMember = true) {
		$data = array();
		$result = $this->search($accountid, $lang, $spcIdList, $groupIdList, $merge, $onlyMember);
		$data['Agreement'] = $result;
		return $data;
	}

	/**
	 * 同意情報を保存する。
	 */
	function saveAgreed($data, $accountid) {
		$internalSpIdList = explode(',', $data['internal_spid_list']);

		// 同一のaccount_idのレコードを削除する
		$result = $this->Agreement->deleteAll(array('account_id' => $accountid), false);
		if (!$result)
			return false;
		$result = $this->AgreementIsmemberof->deleteAll(array('account_id' => $accountid), false);
		if (!$result)
			return false;
		foreach ($internalSpIdList as $internalId) {
			// SPIDに紐ずくSPコネクタIDを取得
			$spidList = explode('_', $internalId);
			$internalSpcIdList = explode(',', $data['internal_spcid_list'.$internalId]);
			// $internalSpcIdListにSPコネクタIDは入っているが再度データベースを確認してSPコネクタIDを取得する
			$spcIdList = $this->GroupSphost->find('list', array(
					'fields' => array('group_id', 'group_id'),
					'conditions' => array('sp_id' => $spidList, 'group_id' => $internalSpcIdList)));
			// SPCで対象SPが１つのSPコネクタを取得したい
			//$spcIdListのなかから $spidList のみのspcを取得したい

			foreach ($spcIdList as $spcId) {
				$agreement = $this->generateEntity($accountid, $spcId);
				$select = false;
				if (isset($data['eptid'.$internalId][0])) {
					$agreement['eptid'] = true;
					$select = true;
				}
				if (isset($data['name'.$internalId][0])) {
					$agreement['name'] = true;
					$select = true;
				}
				if (isset($data['mail'.$internalId][0])) {
					$agreement['mail'] = true;
					$select = true;
				}
				if (isset($data['eppn'.$internalId][0])) {
					$agreement['eppn'] = true;
					$select = true;
				}
				if (isset($data['organization'.$internalId][0])) {
					$agreement['organization'] = true;
					$select = true;
				}
				if (isset($data['idp'.$internalId][0])) {
					$agreement['idp'] = true;
					$select = true;
				}
				if (isset($data['introduction'.$internalId][0])) {
					$agreement['introduction'] = true;
					$select = true;
				}
				if (isset($data['language'.$internalId][0])) {
					$agreement['language'] = true;
					$select = true;
				}
				if ($select) {
					$this->Agreement->create();
					$this->Agreement->save($agreement);
					$agreement_id = $this->Agreement->getLastInsertID();
				}

				// ismemberofを登録
				if (!empty($data['ismemberof'.$internalId])) {
					foreach ($data['ismemberof'.$internalId] as $topGroupid) {
						$ismemberof = array();
						$ismemberof['account_id'] = $accountid;
						$ismemberof['spc_id'] = $spcId;
						$ismemberof['group_id'] = $topGroupid;
						$ismemberof['ismemberof'] = 1;
						$ismemberof['before_ismemberof'] = 0;
						$this->AgreementIsmemberof->create();
						$result = $this->AgreementIsmemberof->save($ismemberof);
						if (!$result) {
							return false;
						}
					}
				}
			}
		}

		return true;
	}

	function generateEntity($accountid, $spcid) {
		$agreement = array();
		$agreement['account_id'] = $accountid;
		$agreement['group_id'] = $spcid;
		$agreement['eptid'] = false;
		$agreement['name'] = false;
		$agreement['mail'] = false;
		$agreement['eppn'] = false;
		$agreement['organization'] = false;
		$agreement['idp'] = false;
		$agreement['language'] = false;
		$agreement['introduction'] = false;

		$agreement['before_eptid'] = false;
		$agreement['before_name'] = false;
		$agreement['before_mail'] = false;
		$agreement['before_eppn'] = false;
		$agreement['before_organization'] = false;
		$agreement['before_idp'] = false;
		$agreement['before_language'] = false;
		$agreement['before_introduction'] = false;

		return $agreement;
	}

	/**
	 * 管理者による各メンバーの同意済み設定を行う。
	 * @param $spcid SPコネクタID
	 * @param $groupid グループID
	 * @param $managerSelectedAttributeList 管理者が選択した利用属性一覧
	 * @param $topGroupId 上位グループID
	 * @param $operatorId 操作者（申請者）のアカウントID
	 */
	function takeAgreementForManager($spcid, $groupid, $managerAttributeList, $topGroupId = 0, $operatorId = 0) {
		// SPコネクタの提供属性を検索
		$provideAttribute = $this->ProvideAttribute->find('first', array(
			'conditions' => array('group_id' => $spcid)));

		// メンバーのアカウントIDを検索
		$accountIdList = $this->Mygroup->find('list', array(
			'fields' => array('account_id'),
			'conditions' => array('groupid' => $groupid)));
		// メンバーがSPコネクタに同意を取っているかを検索
		$agreedAccoutIdList = $this->Agreement->find('list', array(
			'fields' => array('account_id', 'id'),
			'conditions' => array('account_id' => $accountIdList, 'group_id' => $spcid)));

		// 委任しているメンバーを検索
		$entrustAccountIdList = $this->AgreementEntrustment->find('list', array(
				'fields' => array('account_id', 'account_id'),
				'conditions' => array('group_id' => $groupid)));
		// 操作した本人も追加
		$entrustAccountIdList += array($operatorId => $operatorId);

		foreach ($accountIdList as $accountid) {
			if (isset($agreedAccoutIdList[$accountid])) {
				// 同意情報があるなら更新する
				$agreement = array();
				$agreement['id'] = $agreedAccoutIdList[$accountid];
				$agreement['account_id'] = $accountid;
				$agreement['group_id'] = $spcid;

				$beforeAgrement = $this->Agreement->find('first', array(
						'conditions' => array('account_id' => $accountid, 'group_id' => $spcid)));
				if (!empty($beforeAgrement)) {
					$agreement['before_eptid'] = $beforeAgrement['Agreement']['eptid'];
					$agreement['before_name'] = $beforeAgrement['Agreement']['name'];
					$agreement['before_mail'] = $beforeAgrement['Agreement']['mail'];
					$agreement['before_eppn'] = $beforeAgrement['Agreement']['eppn'];
					$agreement['before_organization'] = $beforeAgrement['Agreement']['organization'];
					$agreement['before_idp'] = $beforeAgrement['Agreement']['idp'];
					$agreement['before_introduction'] = $beforeAgrement['Agreement']['introduction'];
					$agreement['before_language'] = $beforeAgrement['Agreement']['language'];
				}
			} else {
				// 同意情報がないなら登録
				$agreement = $this->generateEntity($accountid, $spcid);
				$this->Agreement->create();
				// before_xxxxの初期化はgenerateEntity()でやっているので、削除
			}

			// 委任している場合は無条件で全ての属性を同意
			$entrust = false;
			foreach ($entrustAccountIdList as $id) {
				if ($accountid != $id)
					continue;
				if ($provideAttribute['ProvideAttribute']['eptid']) {
					$agreement['eptid'] = true;
				}
				if ($provideAttribute['ProvideAttribute']['name']) {
					$agreement['name'] = true;
				}
				if ($provideAttribute['ProvideAttribute']['mail']) {
					$agreement['mail'] = true;
				}
				if ($provideAttribute['ProvideAttribute']['eppn']) {
					$agreement['eppn'] = true;
				}
				if ($provideAttribute['ProvideAttribute']['organization']) {
					$agreement['organization'] = true;
				}
				if ($provideAttribute['ProvideAttribute']['idp']) {
					$agreement['idp'] = true;
				}
				if ($provideAttribute['ProvideAttribute']['introduction']) {
					$agreement['introduction'] = true;
				}
				if ($provideAttribute['ProvideAttribute']['language']) {
					$agreement['language'] = true;
				}
				$entrust = true;
				break;
			}
			if (!$entrust) {
				if ($provideAttribute['ProvideAttribute']['eptid'] && $managerAttributeList['eptid']) {
					$agreement['eptid'] = true;
				}
				if ($provideAttribute['ProvideAttribute']['name'] && $managerAttributeList['name']) {
					$agreement['name'] = true;
				}
				if ($provideAttribute['ProvideAttribute']['mail'] && $managerAttributeList['mail']) {
					$agreement['mail'] = true;
				}
				if ($provideAttribute['ProvideAttribute']['eppn'] && $managerAttributeList['eppn']) {
					$agreement['eppn'] = true;
				}
				if ($provideAttribute['ProvideAttribute']['organization'] && $managerAttributeList['organization']) {
					$agreement['organization'] = true;
				}
				if ($provideAttribute['ProvideAttribute']['idp'] && $managerAttributeList['idp']) {
					$agreement['idp'] = true;
				}
				if ($provideAttribute['ProvideAttribute']['introduction'] && $managerAttributeList['introduction']) {
					$agreement['introduction'] = true;
				}
				if ($provideAttribute['ProvideAttribute']['language'] && $managerAttributeList['language']) {
					$agreement['language'] = true;
				}
			}

			// 操作した本人にはメールしないようにするため、before_を同じにして同意の変化が無いことにする
			if ($accountid == $operatorId) {
				$agreement_keys = array('eptid', 'name', 'mail', 'eppn', 'idp', 'introduction', 'language',
							'organization');
				foreach ($agreement_keys as $agreement_key) {
					$keystr = (string)$agreement_key;
					if (isset($agreement[$keystr])) {
						$before_key = 'before_' . $agreement_key;
						$agreement[$before_key] = $agreement[$keystr];
					}
				}
			}

			$result = $this->Agreement->save($agreement);
			if (!$result) {
				return false;
			}

			// ismemberof
			if (!$provideAttribute['ProvideAttribute']['ismemberof'] ||
				(!$entrust && !$managerAttributeList['ismemberof'])) {
					continue;
			}

			if ($topGroupId > 0) {
				$gid = $topGroupId;
			} else {
				$gid = $groupid;
			}

			$ismemberof = $this->AgreementIsmemberof->find('first', array(
				'fields' => array('id', 'ismemberof'),
				'conditions' => array('account_id' => $accountid, 'spc_id' => $spcid, 'group_id' => $gid)));
			$ismemberof['AgreementIsmemberof']['account_id'] = $accountid;
			$ismemberof['AgreementIsmemberof']['spc_id'] = $spcid;
			$ismemberof['AgreementIsmemberof']['group_id'] = $gid;
			// 操作した本人にはメールしないようにするため、before_を同じにして同意の変化が無いことにする
			if ($accountid == $operatorId) {
				$ismemberof['AgreementIsmemberof']['before_ismemberof'] = 1;
			}
			elseif (isset($ismemberof['AgreementIsmemberof']['ismemberof'])) {
				$ismemberof['AgreementIsmemberof']['before_ismemberof'] = $ismemberof['AgreementIsmemberof']['ismemberof'];
			}
			$ismemberof['AgreementIsmemberof']['ismemberof'] = 1;
			$this->AgreementIsmemberof->create($ismemberof);
			$result = $this->AgreementIsmemberof->save($ismemberof);
			if (!$result) {
				return false;
			}
		}
		return true;
	}

	/**
	 * SPコネクタの利用属性同意情報を取得する。
	 * @param $spcIdList
	 */
	function serveSpcAgreementInfo($spcIdList, $lang) {
		$provideAttributeList = $this->ProvideAttribute->find('all', array(
			'conditions' => array('group_id' => $spcIdList)));
		$spcList = $this->Group->find('all', array(
			'fields' => array('id', 'name', 'name_en'),
			'conditions' => array('id' => $spcIdList)));
		$groupSpHostList = $this->GroupSphost->find('all', array(
			'conditions' => array('group_id' => $spcIdList)));
		for ($i = 0; $i < count($provideAttributeList); $i++) {
			// SPコネクタ名を設定
			foreach ($spcList as $spc) {
				if ($provideAttributeList[$i]['ProvideAttribute']['group_id'] == $spc['Group']['id']) {
					$provideAttributeList[$i]['ProvideAttribute']['spc_name'] = $spc['Group']['name'];
					$provideAttributeList[$i]['ProvideAttribute']['spc_name_en'] = $spc['Group']['name_en'];

					// サービス名を設定
					if (!isset($provideAttributeList[$i]['spc_service_name_list'])) {
						$provideAttributeList[$i]['spc_service_name_list'] = array();
					}
					foreach ($groupSpHostList as $groupSpHost) {
						if ($groupSpHost['GroupSphost']['group_id'] != $spc['Group']['id']) {
							continue;
						}

						// サービス名 spc_service_name
						$serviceName = $this->GroupSphost->getServiceName($lang, $groupSpHost);
						array_push($provideAttributeList[$i]['spc_service_name_list'], $serviceName);
					}

					// アイコンを設定
					$provideAttributeList[$i]['spc_icon'] = $this->Group->getIcon($spc['Group']['id']);

					break;
				}
			}
		}

		return $provideAttributeList;
	}


	/**
	 * SPコネクタの利用属性に同意する
	 * @param integer $groupid グループID
	 * @param integer $spAttributeList 入力データ
	 * @param array $spcIdList SPコネクタID一覧
	 * @param integer $accountid アカウントID
	 */
	function saveAttribute($groupid, $spAttributeList, $spcIdList, $accountid) {
		if (empty($spcIdList)) {
			return true;
		}

		foreach ($spcIdList as $selectedspcId) {
			$agreement = array();
			$agreement['ismemberof'] = false;
			$agreement['eptid'] = false;
			$agreement['name'] = false;
			$agreement['mail'] = false;
			$agreement['eppn'] = false;
			$agreement['organization'] = false;
			$agreement['idp'] = false;
			$agreement['introduction'] = false;
			$agreement['language'] = false;
			$select = false;
			if (isset($spAttributeList['ismemberof'.$selectedspcId][0])) {
				$select = true;
			}
			if (isset($spAttributeList['eptid'.$selectedspcId][0])) {
				$agreement['eptid'] = true;
				$select = true;
			}
			if (isset($spAttributeList['name'.$selectedspcId][0])) {
				$agreement['name'] = true;
				$select = true;
			}
			if (isset($spAttributeList['mail'.$selectedspcId][0])) {
				$agreement['mail'] = true;
				$select = true;
			}
			if (isset($spAttributeList['eppn'.$selectedspcId][0])) {
				$agreement['eppn'] = true;
				$select = true;
			}
			if (isset($spAttributeList['organization'.$selectedspcId][0])) {
				$agreement['organization'] = true;
				$select = true;
			}
			if (isset($spAttributeList['idp'.$selectedspcId][0])) {
				$agreement['idp'] = true;
				$select = true;
			}
			if (isset($spAttributeList['introduction'.$selectedspcId][0])) {
				$agreement['introduction'] = true;
				$select = true;
			}
			if (isset($spAttributeList['language'.$selectedspcId][0])) {
				$agreement['language'] = true;
				$select = true;
			}
			if (!$select) {
				continue;
			}

			$agreement['account_id'] = $accountid;
			$agreement['group_id'] = $selectedspcId;
			$existAgreement = $this->Agreement->find('first', array('conditions' => array(
				'account_id' => $accountid, 'group_id' => $selectedspcId)));
			$result = false;
			if (empty($existAgreement)) {
				$this->Agreement->create();
				$result = $this->Agreement->save($agreement);
			} else {
				$agreement['id'] = $existAgreement['Agreement']['id'];
				$result = $this->Agreement->save($agreement);
			}
			if (!$result)
				return false;


			// ismemberofを登録
			if (!isset($spAttributeList['ismemberof'.$selectedspcId][0])) {
				return true;
			}
			$count = $this->AgreementIsmemberof->find('count', array('conditions' => array(
				'account_id' => $accountid, 'spc_id' => $selectedspcId, 'group_id' => $groupid)));
			if ($count > 0) {
				continue;
			}
			$ismemberof = array();
			$ismemberof['account_id'] = $accountid;
			$ismemberof['spc_id'] = $selectedspcId;
			$ismemberof['group_id'] = $groupid;
			$ismemberof['ismemberof'] = 1;
			$this->AgreementIsmemberof->create();
			$result = $this->AgreementIsmemberof->save($ismemberof);
			if (!$result)
				return false;
		}
		return true;
	}

	/**
	 * 接続したSPコネクタの属性に同意する必要があるかを判定する。
	 * @return 同意の必要がある属性一覧
	 */
	function needSpcConnectAgree($spcid, $groupid, $accountid) {
		// SPコネクタの属性を取得
		$spAttribute = $this->ProvideAttribute->find('first', array(
			'conditions' => array('group_id' => $spcid)));
		if (empty($spAttribute)) {
			return array();
		}

		$allIdList = $this->AddGroup->getTopIdMyIdList($groupid, $onlyActive=false);
		// SPコネクタに接続しているグループIDに限定する
		$groupIdList = $this->AddGroup->getGroupIdListConnectedSpc($spcid, $allIdList);
		// このアカウントの同意した属性を取得
		$isMemberOfList = $this->AgreementIsmemberof->find('all', array(
			'conditions' => array('account_id' => $accountid, 'spc_id' => $spcid, 'group_id' => $groupIdList)));
		$agreement = $this->Agreement->find('first', array(
			'conditions' => array('account_id' => $accountid, 'group_id' => $spcid)));

		// 比較
		$needAttributeList = array();
		if ($spAttribute['ProvideAttribute']['ismemberof']) {
			if (empty($isMemberOfList)) {
				$needAttributeList['ismemberof'] = true;
			}
			foreach ($isMemberOfList as $memberof) {
				if (!$memberof['AgreementIsmemberof']['ismemberof']) {
					$needAttributeList['ismemberof'] = true;
					break;
				}
			}
		}
		if ($spAttribute['ProvideAttribute']['eptid']) {
			if (empty($agreement)) {
				$needAttributeList['eptid'] = true;
			}
			if (!$agreement['Agreement']['eptid']) {
				$needAttributeList['eptid'] = true;
			}
		}
		if ($spAttribute['ProvideAttribute']['name']) {
			if (empty($agreement)) {
				$needAttributeList['name'] = true;
			}
			if (!$agreement['Agreement']['name']) {
				$needAttributeList['name'] = true;
			}
		}
		if ($spAttribute['ProvideAttribute']['mail']) {
			if (empty($agreement)) {
				$needAttributeList['mail'] = true;
			}
			if (!$agreement['Agreement']['mail']) {
				$needAttributeList['mail'] = true;
			}
		}
		if ($spAttribute['ProvideAttribute']['eppn']) {
			if (empty($agreement)) {
				$needAttributeList['eppn'] = true;
			}
			if (!$agreement['Agreement']['eppn']) {
				$needAttributeList['eppn'] = true;
			}
		}
		if ($spAttribute['ProvideAttribute']['idp']) {
			if (empty($agreement)) {
				$needAttributeList['idp'] = true;
			}
			if (!$agreement['Agreement']['idp']) {
				$needAttributeList['idp'] = true;
			}
		}
		if ($spAttribute['ProvideAttribute']['introduction']) {
			if (empty($agreement)) {
				$needAttributeList['introduction'] = true;
			}
			if (!$agreement['Agreement']['introduction']) {
				$needAttributeList['introduction'] = true;
			}
		}
		if ($spAttribute['ProvideAttribute']['language']) {
			if (empty($agreement)) {
				$needAttributeList['language'] = true;
			}
			if (!$agreement['Agreement']['language']) {
				$needAttributeList['language'] = true;
			}
		}
		if ($spAttribute['ProvideAttribute']['organization']) {
			if (empty($agreement)) {
				$needAttributeList['organization'] = true;
			}
			if (!$agreement['Agreement']['organization']) {
				$needAttributeList['organization'] = true;
			}
		}

		return $needAttributeList;
	}

	/**
	 * SPコネクタの利用属性から属性登録用のリストを生成します。
	 * @param $spcIdList SPコネクタID一覧
	 */
	function getAttributeListForSave($spcIdList) {
		$provideList = $this->ProvideAttribute->find('all', array('conditions' => array('group_id' => $spcIdList)));
		$attributeList = array();
		foreach ($provideList as $provide) {
			$id = $provide['ProvideAttribute']['group_id'];
			if ($provide['ProvideAttribute']['ismemberof']) {
				$attributeList['ismemberof'.$id][0] = true;
			}
			if ($provide['ProvideAttribute']['eptid']) {
				$attributeList['eptid'.$id][0] = true;
			}
			if ($provide['ProvideAttribute']['name']) {
				$attributeList['name'.$id][0] = true;
			}
			if ($provide['ProvideAttribute']['mail']) {
				$attributeList['mail'.$id][0] = true;
			}
			if ($provide['ProvideAttribute']['eppn']) {
				$attributeList['eppn'.$id][0] = true;
			}
			if ($provide['ProvideAttribute']['idp']) {
				$attributeList['idp'.$id][0] = true;
			}
			if ($provide['ProvideAttribute']['introduction']) {
				$attributeList['introduction'.$id][0] = true;
			}
			if ($provide['ProvideAttribute']['language']) {
				$attributeList['language'.$id][0] = true;
			}
			if ($provide['ProvideAttribute']['organization']) {
				$attributeList['organization'.$id][0] = true;
			}
		}
		return $attributeList;
	}

	/**
	 * 属性(isMemberOf, ePTID)情報提供の同意を取る必要があるかを調べる。
	 * @param $accountid アカウントID
	 */
	private function isNeedAgreement($accountid) {
		$resultList = $this->search($accountid, '', array(), array());
//debug($resultList);
		foreach ($resultList as $result) {
			// 同意項目が１つもなければそのサービスの同意は必要ない(CG用)
			if ($result['nothing_agree_item']) {
				continue;
			}

			if (isset($result['eptid']) && !$result['eptid']) {
				return true;
			}
			if (isset($result['name']) && !$result['name']) {
				return true;
			}
			if (isset($result['mail']) && !$result['mail']) {
				return true;
			}
			if (isset($result['eppn']) && !$result['eppn']) {
				return true;
			}
			if (isset($result['organization']) && !$result['organization']) {
				return true;
			}
			if (isset($result['idp']) && !$result['idp']) {
				return true;
			}
			if (isset($result['introduction']) && !$result['introduction']) {
				return true;
			}
			if (isset($result['language']) && !$result['language']) {
				return true;
			}

			// ismemberof
			if (!$result['all_group_check']) {
				return true;
			}
		}
		return false;
	}

	/**
	 * @param $accountid アカウントID
	 * @param $onlyMember true:メンバーのグループに関してのみ false:所属しているグループすべての情報を取得する
	 */
	private function getConnectSpcIdList($accountid, $onlyMember = true) {
		$groupIdList = array();

		// 休止中グループも取得する
		$onlyActive = false;

		if ($onlyMember) {
			// メンバーとして所属しているグループ(機関グループを除く)を取得
			$memberGroupIdList = $this->Mygroup->findMyGroupJoinedAsMemberIdList($accountid, $onlyActive);
		}
		else {
			// 所属しているグループ(機関グループを除く)を取得
			$memberGroupIdList = $this->Mygroup->findMyGroupIdList($accountid, array(), $onlyActive);
		}
		foreach ($memberGroupIdList as $groupid) {
			if ($this->AutoGroup->isIdpGroup($groupid)) {
				continue;
			}
			array_push($groupIdList, $groupid);
		}

		// メンバーとして所属しているグループの上位グループ(機関グループを除く)を取得
		$upGroupIdList = $this->AddGroup->getTopIdList($groupIdList, $onlyActive);
		foreach ($upGroupIdList as $groupid) {
			if ($this->AutoGroup->isIdpGroup($groupid)) {
				continue;
			}
			array_push($groupIdList, $groupid);
		}

		// 管理者として所属しているグループ(機関グループを除く)を取得
		$adminGroupIdList = $this->Mygroup->findMyGroupJoinedAsAdminIdList($accountid, $onlyActive);
		foreach ($adminGroupIdList as $groupid) {
			if ($this->AutoGroup->isIdpGroup($groupid)) {
				continue;
			}
			array_push($groupIdList, $groupid);
		}

		if (empty($groupIdList)) {
			return array();
		}

		// 接続しているSPコネクタを検索
		$spcIdList = $this->AddGroup->find('list', array(
			'fields' => array('top_id', 'top_id'),
			'conditions' => array('bottom_id' => $groupIdList, 'top' => 1),
			'order' => array('top_id')));

		// サービスに紐付いていないSPコネクタは対象外
		$spConnectSpcIdList = $this->GroupSphost->find('list', array(
			'fields' => array('group_id', 'group_id'),
			'conditions' => array('group_id' => $spcIdList),
			'order' => array('group_id')));

		// ismemberof属性を提供しないSPコネクタは対象外
		$hasAttributeSpcIdList = $this->ProvideAttribute->find("list", array(
				'fields' => array('group_id', 'group_id'),
				'conditions' => array('group_id' => $spConnectSpcIdList, 'ismemberof' => 1)));

		return $hasAttributeSpcIdList;
	}

	/**
	 * 利用属性情報を検索
	 * @param $accountid アカウントID
	 * @param lang 使用言語
	 * @param $spcIdList 対象となるSPコネクタID一覧、指定がいないとアカウントIDに紐付くSPコネクタIDがこの関数内で設定される
	 * @param $groupIdList 対象となるグループID一覧、$spcIdListに接続するグループIDがこの関数内で設定される
	 * @param $merge
	 * @param $onlyMember 「管理者のみ」でも上位グループ経由で接続しているSPコネクタの同意状況を表示するために追加
	 */
	private function search($accountid, $lang, $spcIdList = array(), $groupIdList = array(), $merge = false,
			$onlyMember = true) {
		if (empty($spcIdList)) {
			$spcIdList = $this->getConnectSpcIdList($accountid, $onlyMember);
		}
		if (empty($spcIdList)) {
			return array();
		}

		// SPコネクタに同意を取っているかを検索
		$agreementList = $this->Agreement->find('all', array('conditions' => array(
			'account_id' => $accountid, 'group_id' => $spcIdList)));

		// SPコネクタの提供属性を検索
		$provideAttributeList = $this->ProvideAttribute->find('all', array(
			'conditions' => array('group_id' => $spcIdList)));

		// SPコネクタ名を検索
		$groupList = $this->Group->find('all', array(
			'conditions' => array('id' => $spcIdList)));

		//サービス名を検索
		$groupSpHostList = $this->GroupSphost->find('all', array(
			'conditions' => array('group_id' => $spcIdList),
			'order' => array('sp_id')));

		$resultList = array();
		$i = 0;
		foreach ($spcIdList as $spcId) {
			// SPコネクタ情報を設定
			$equal = false;
			foreach ($groupList as $group) {
				if ($group['Group']['id'] == $spcId) {
					$resultList[$i]['spc_name'] = $group['Group']['name'];
					$resultList[$i]['spc_name_en'] = $group['Group']['name_en'];
					$equal = true;
					// 活動状態の設定
					$resultList[$i]['active'] = ($group['Group']['active'] == 0) ? true : false;
					break;
				}
			}

			// 該当するSPコネクタがない場合は無視する
			if (!$equal) {
				continue;
			}

			// サービス名を取得
			$equal = false;
			foreach ($groupSpHostList as $groupSpHost) {
				if ($groupSpHost['GroupSphost']['group_id'] == $spcId) {
					if (!isset($resultList[$i]['sp_id']))
						$resultList[$i]['sp_id'] = '';
					if (!isset($resultList[$i]['spc_service_name']))
						$resultList[$i]['spc_service_name'] = array();
					if (!isset($resultList[$i]['spc_service_name_en']))
						$resultList[$i]['spc_service_name_en'] = array();
					if (!empty($resultList[$i]['sp_id']))
						$resultList[$i]['sp_id'] = $resultList[$i]['sp_id'].'_';
					$resultList[$i]['sp_id'] = $resultList[$i]['sp_id'].$groupSpHost['GroupSphost']['sp_id'];

					$name = $groupSpHost['GroupSphost']['service_name'];
					$nameEn = $groupSpHost['GroupSphost']['service_name_en'];
					if (!empty($groupSpHost['GroupSphost']['service_name'])) {
						$resultList[$i]['spc_service_name'][$name] = $name;
					} else if (!empty($groupSpHost['GroupSphost']['service_name_en'])) {
						$resultList[$i]['spc_service_name'][$nameEn] = $nameEn;
					} else {
						$host = $this->SpHost->getSpHost($groupSpHost['GroupSphost']['sp_id']);
						$resultList[$i]['spc_service_name'][$host] = $host;
					}
					if (!empty($groupSpHost['GroupSphost']['service_name_en'])) {
						$resultList[$i]['spc_service_name_en'][$nameEn] = $nameEn;
					} else if (!empty($groupSpHost['GroupSphost']['service_name'])) {
						$resultList[$i]['spc_service_name_en'][$name] = $name;
					} else {
						$host = $this->SpHost->getSpHost($groupSpHost['GroupSphost']['sp_id']);
						$resultList[$i]['spc_service_name'][$host] = $host;
					}
					$equal = true;
				}
			}

			// 該当するサービスがない場合も無視する
			if (!$equal) {
				unset($resultList[$i]);
				continue;
			}

			// ユーザの同意情報を設定
			$equal = false;
			foreach ($agreementList as $agreement) {
				if ($spcId != $agreement['Agreement']['group_id']) {
					continue;
				}
				$resultList[$i]['id'] = $agreement['Agreement']['id'];
				$resultList[$i]['account_id'] = $accountid;
				$resultList[$i]['group_id'] = $spcId;
				$resultList[$i]['eptid'] = $agreement['Agreement']['eptid'];;
				$resultList[$i]['name'] = $agreement['Agreement']['name'];;
				$resultList[$i]['mail'] = $agreement['Agreement']['mail'];;
				$resultList[$i]['eppn'] = $agreement['Agreement']['eppn'];;
				$resultList[$i]['organization'] = $agreement['Agreement']['organization'];;
				$resultList[$i]['idp'] = $agreement['Agreement']['idp'];;
				$resultList[$i]['introduction'] = $agreement['Agreement']['introduction'];;
				$resultList[$i]['language'] = $agreement['Agreement']['language'];;
				$equal = true;
				break;
			}
			if (!$equal) {
				$resultList[$i]['id'] = '';
				$resultList[$i]['account_id'] = $accountid;
				$resultList[$i]['group_id'] = $spcId;
				$resultList[$i]['ismemberof'] = '';
				$resultList[$i]['eptid'] = '';
				$resultList[$i]['name'] = '';
				$resultList[$i]['mail'] = '';
				$resultList[$i]['eppn'] = '';
				$resultList[$i]['organization'] = '';
				$resultList[$i]['idp'] = '';
				$resultList[$i]['introduction'] = '';
				$resultList[$i]['language'] = '';
			}

			// 必要ない項目を削除
			$equal = false;
			foreach ($provideAttributeList as $attribute) {
				if ($attribute['ProvideAttribute']['group_id'] != $spcId) {
					continue;
				}
				if (!$attribute['ProvideAttribute']['ismemberof'])
					unset($resultList[$i]['ismemberof']);
				if (!$attribute['ProvideAttribute']['eptid'])
					unset($resultList[$i]['eptid']);
				if (!$attribute['ProvideAttribute']['name'])
					unset($resultList[$i]['name']);
				if (!$attribute['ProvideAttribute']['mail'])
					unset($resultList[$i]['mail']);
				if (!$attribute['ProvideAttribute']['eppn'])
					unset($resultList[$i]['eppn']);
				if (!$attribute['ProvideAttribute']['organization'])
					unset($resultList[$i]['organization']);
				if (!$attribute['ProvideAttribute']['idp'])
					unset($resultList[$i]['idp']);
				if (!$attribute['ProvideAttribute']['introduction'])
					unset($resultList[$i]['introduction']);
				if (!$attribute['ProvideAttribute']['language'])
					unset($resultList[$i]['language']);
				$equal = true;
				break;
			}
			if (!$equal) {
				unset($resultList[$i]['ismemberof']);
				unset($resultList[$i]['eptid']);
				unset($resultList[$i]['name']);
				unset($resultList[$i]['mail']);
				unset($resultList[$i]['eppn']);
				unset($resultList[$i]['organization']);
				unset($resultList[$i]['idp']);
				unset($resultList[$i]['introduction']);
				unset($resultList[$i]['language']);
			}

			$i++;
		}

		// 戻すデータがなければここで返却
		if (empty($resultList)) {
			return $resultList;
		}

		// サービスでまとめる
		foreach ($resultList as $key => $row) {
			$sort_key[$key] = array($row['sp_id'], $row['group_id']);
		}
		array_multisort($sort_key, SORT_ASC, $resultList);

		$count = count($resultList);
		$beforeSpid = 0;
		$beforeNo = 0;
		for ($i = 0; $i < $count; $i++) {
			$spid = $resultList[$i]['sp_id'];
			if (!$merge || $spid != $beforeSpid || $spid == 0) {
				$beforeSpid = $spid;
				$beforeNo = $i;
				// サービス名 spc_service_name
				if (!isset($resultList[$beforeNo]['spc_service_name_list'])) {
					$resultList[$beforeNo]['spc_service_name_list'] = array();
				}
				$serviceNameList = $this->GroupSphost->getSpcServiceNameList($lang, $resultList[$i]);
				$resultList[$beforeNo]['spc_service_name_list'] = $resultList[$beforeNo]['spc_service_name_list'] + $serviceNameList;

				// SPコネクタ名 spc_name
				if (!isset($resultList[$beforeNo]['spc_name_list'])) {
					$resultList[$beforeNo]['spc_name_list'] = array();
				}
				if ($lang === 'ja') {
					if (isset($resultList[$i]['spc_name']) && $resultList[$i]['spc_name'])
						array_push($resultList[$beforeNo]['spc_name_list'], $resultList[$i]['spc_name']);
					else if (isset($resultList[$i]['spc_name_en']) && $resultList[$i]['spc_name_en'])
						array_push($resultList[$beforeNo]['spc_name_list'], $resultList[$i]['spc_name_en']);
				} else {
					if (isset($resultList[$i]['spc_name_en']) && $resultList[$i]['spc_name_en'])
						array_push($resultList[$beforeNo]['spc_name_list'], $resultList[$i]['spc_name_en']);
					else if (isset($resultList[$i]['spc_name']) && $resultList[$i]['spc_name'])
						array_push($resultList[$beforeNo]['spc_name_list'], $resultList[$i]['spc_name']);
				}

				// spc_id
				$spcid = $resultList[$i]['group_id'];
				if (!isset($resultList[$beforeNo]['spc_id_list'])) {
					$resultList[$beforeNo]['spc_id_list'] = array();
				}
				array_push($resultList[$beforeNo]['spc_id_list'], $spcid);

				// SPコネクタ 活動状態
				if (!isset($result[$beforeNo]['spc_service_active_list'])) {
					$resultList[$beforeNo]['spc_service_active_list'] = array();
				}
				foreach (array_keys($serviceNameList) as $key) {
					if (array_key_exists($key, $resultList[$beforeNo]['spc_service_active_list'])) {
						$resultList[$beforeNo]['spc_service_active_list'][$key] =
							$resultList[$beforeNo]['spc_service_active_list'][$key] || $resultList[$i]['active'];
					} else {
						$resultList[$beforeNo]['spc_service_active_list'][$key] = $resultList[$i]['active'];
					}
				}
				continue;
			}

			// まとめる
			if ($merge) { 
				// ismemberof
				if (isset($resultList[$i]['ismemberof'])) {
					if (isset($resultList[$beforeNo]['ismemberof']))
						$resultList[$beforeNo]['ismemberof'] = $resultList[$beforeNo]['ismemberof'] | $resultList[$i]['ismemberof'];
					else
						$resultList[$beforeNo]['ismemberof'] = $resultList[$i]['ismemberof'];
				}
				// eptid
				if (isset($resultList[$i]['eptid'])) {
					if (isset($resultList[$beforeNo]['eptid']))
						$resultList[$beforeNo]['eptid'] = $resultList[$beforeNo]['eptid'] | $resultList[$i]['eptid'];
					else
						$resultList[$beforeNo]['eptid'] = $resultList[$i]['eptid'];
				}
				// name
				if (isset($resultList[$i]['name'])) {
					if (isset($resultList[$beforeNo]['name']))
						$resultList[$beforeNo]['name'] = $resultList[$beforeNo]['name'] | $resultList[$i]['name'];
					else 
						$resultList[$beforeNo]['name'] = $resultList[$i]['name'];
				}
				// mail
				if (isset($resultList[$i]['mail'])) {
					if (isset($resultList[$beforeNo]['mail']))
						$resultList[$beforeNo]['mail'] = $resultList[$beforeNo]['mail'] | $resultList[$i]['mail'];
					else
						$resultList[$beforeNo]['mail'] = $resultList[$i]['mail'];
				}
				// eppn
				if (isset($resultList[$i]['eppn'])) {
					if (isset($resultList[$beforeNo]['eppn']))
						$resultList[$beforeNo]['eppn'] = $resultList[$beforeNo]['eppn'] | $resultList[$i]['eppn'];
					else
						$resultList[$beforeNo]['eppn'] = $resultList[$i]['eppn'];
				}
				// organization
				if (isset($resultList[$i]['organization'])) {
					if (isset($resultList[$beforeNo]['organization']))
						$resultList[$beforeNo]['organization'] = $resultList[$beforeNo]['organization'] | $resultList[$i]['organization'];
					else
						$resultList[$beforeNo]['organization'] = $resultList[$i]['organization'];
				}
				// idp
				if (isset($resultList[$i]['idp'])) {
					if (isset($resultList[$beforeNo]['idp']))
						$resultList[$beforeNo]['idp'] = $resultList[$beforeNo]['idp'] | $resultList[$i]['idp'];
					else
						$resultList[$beforeNo]['idp'] = $resultList[$i]['idp'];
				}
				// introduction
				if (isset($resultList[$i]['introduction'])) {
					if (isset($resultList[$beforeNo]['introduction']))
						$resultList[$beforeNo]['introduction'] = $resultList[$beforeNo]['introduction'] | $resultList[$i]['introduction'];
					else
						$resultList[$beforeNo]['introduction'] = $resultList[$i]['introduction'];
				}
				// language
				if (isset($resultList[$i]['language'])) {
					if (isset($resultList[$beforeNo]['language']))
						$resultList[$beforeNo]['language'] = $resultList[$beforeNo]['language'] | $resultList[$i]['language'];
					else
						$resultList[$beforeNo]['language'] = $resultList[$i]['language'];
				}

				// サービス名 spc_service_name
				if ($lang === 'ja') {
					if (isset($resultList[$i]['spc_service_name']) && $resultList[$i]['spc_service_name'])
						$serviceNameList = $resultList[$i]['spc_service_name'];
					else if(isset($resultList[$i]['spc_service_name_en']) && $resultList[$i]['spc_service_name_en'])
						$serviceNameList = $resultList[$i]['spc_service_name_en'];
				} else {
					if(isset($resultList[$i]['spc_service_name_en']) && $resultList[$i]['spc_service_name_en'])
						$serviceNameList = $resultList[$i]['spc_service_name_en'];
					else if (isset($resultList[$i]['spc_service_name']) && $resultList[$i]['spc_service_name'])
						$serviceNameList = $resultList[$i]['spc_service_name'];
				}
				$resultList[$beforeNo]['spc_service_name_list'] = $resultList[$beforeNo]['spc_service_name_list'] + $serviceNameList;

				// SPコネクタ名 spc_name
				if ($lang === 'ja') {
					if (isset($resultList[$i]['spc_name']) && $resultList[$i]['spc_name'])
						array_push($resultList[$beforeNo]['spc_name_list'], $resultList[$i]['spc_name']);
					else if (isset($resultList[$i]['spc_name_en']) && $resultList[$i]['spc_name_en'])
						array_push($resultList[$beforeNo]['spc_name_list'], $resultList[$i]['spc_name_en']);
				} else {
					if (isset($resultList[$i]['spc_name_en']) && $resultList[$i]['spc_name_en'])
						array_push($resultList[$beforeNo]['spc_name_list'], $resultList[$i]['spc_name_en']);
					else if (isset($resultList[$i]['spc_name']) && $resultList[$i]['spc_name'])
						array_push($resultList[$beforeNo]['spc_name_list'], $resultList[$i]['spc_name']);
				}

				// spc_id
				if (isset($resultList[$i]['group_id']) && $resultList[$i]['group_id'])
					array_push($resultList[$beforeNo]['spc_id_list'], $resultList[$i]['group_id']);

				// SPコネクタ 活動状態
				foreach (array_keys($serviceNameList) as $key) {
					if (array_key_exists($key, $resultList[$beforeNo]['spc_service_active_list'])) {
						$resultList[$beforeNo]['spc_service_active_list'][$key] =
							$resultList[$beforeNo]['spc_service_active_list'][$key] || $resultList[$i]['active'];
					} else {
						$resultList[$beforeNo]['spc_service_active_list'][$key] = $resultList[$i]['active'];
					}
				}

				unset($resultList[$i]);
			}
		}

		// このSPコネクタに接続している上位グループを設定
		$resultList = array_merge($resultList);
		$this->setTopGroupInfo($resultList, $groupIdList, $lang, $accountid, $onlyMember);

		// サービスでまとめているが、まとめた１つ１つのIDを設定
		$internal_spid_list = '';
		for ($i = 0; $i < count($resultList); $i++) {
			$id = $resultList[$i]['sp_id'];
			if (!empty($internal_spid_list)) {
				$internal_spid_list = $internal_spid_list.',';
			}
			$internal_spid_list = $internal_spid_list.$id;

			// SPコネクタIDを文字列で設定
			$internal_spcid_list = '';
			foreach ($resultList[$i]['spc_id_list'] as $id) {
				if (!empty($internal_spcid_list)) {
					$internal_spcid_list = $internal_spcid_list.',';
				}
				$internal_spcid_list = $internal_spcid_list.$id;
			}
			$resultList[$i]['internal_spcid_list'] = $internal_spcid_list;

		}
		for ($i = 0; $i < count($resultList); $i++) {
			$resultList[$i]['internal_spid_list'] = $internal_spid_list;
		}

		// 未同意のグループがあるかを設定
		for ($i = 0; $i < count($resultList); $i++) {
			$selectedCount = 0;
			foreach ($resultList[$i]['top_group_name_list'] as $key => $value) {
				if (in_array($key, $resultList[$i]['top_group_select_id_list']) ||
						(!$onlyMember && $this->isIndirectGroup($accountid, $key))) {
					$selectedCount++;
				}
			}
			if (count($resultList[$i]['top_group_name_list']) === $selectedCount) {
				$resultList[$i]['all_group_check'] = true;
			} else {
				$resultList[$i]['all_group_check'] = false;
			}
		}
//debug($resultList);
		return $resultList;
	}

	/**
	 * SPコネクタに接続しているグループ情報を設定する
	 * @param unknown_type $resultList
	 * @param $onlyMember true:メンバーのグループに関してのみ false:所属しているグループすべての情報を取得する
	 */
	private function setTopGroupInfo(&$resultList, $groupIdList, $lang, $accountid, $onlyMember = true) {
		if (empty($groupIdList)) {
			// 休止中グループも取得する
			$onlyActive = false;

			if ($onlyMember) {
				// メンバーとして所属しているグループ(機関グループを除く)を取得
				$memberGroupIdList = $this->Mygroup->findMyGroupJoinedAsMemberIdList($accountid, $onlyActive);
			}
			else {
				// 所属しているグループ(機関グループを除く)を取得
				$memberGroupIdList = $this->Mygroup->findMyGroupIdList($accountid, array(), $onlyActive);
			}
			foreach ($memberGroupIdList as $groupid) {
				if ($this->AutoGroup->isIdpGroup($groupid)) {
					continue;
				}
				array_push($groupIdList, $groupid);
			}

			// メンバーとして所属しているグループの上位グループ(機関グループを除く)を取得
			$upGroupIdList = $this->AddGroup->getTopIdList($groupIdList, $onlyActive);
			foreach ($upGroupIdList as $groupid) {
				if ($this->AutoGroup->isIdpGroup($groupid)) {
					continue;
				}
				array_push($groupIdList, $groupid);
			}

			// 管理者として所属しているグループ(機関グループを除く)を取得
			$adminGroupIdList = $this->Mygroup->findMyGroupJoinedAsAdminIdList($accountid, $onlyActive);
			foreach ($adminGroupIdList as $groupid) {
				if ($this->AutoGroup->isIdpGroup($groupid)) {
					continue;
				}
				array_push($groupIdList, $groupid);
			}
		}

		for ($i = 0; $i < count($resultList); $i++) {
			$spcIdList = $resultList[$i]['spc_id_list'];
			$topGroupNameList = array();

			if (!empty($groupIdList)) {
				// このSPコネクタに接続している上位グループ(id, name, name_en)を検索
				$topGroupIdList = $this->AddGroup->find('list', array(
						'fields' => array('bottom_id', 'bottom_id'),
						'conditions' => array(
							'top_id' => $spcIdList,
							'bottom_id' => $groupIdList)));
				if (!empty($topGroupIdList)) {
					$groupList = $this->Group->find('all', array(
						'fields' => array('id', 'name', 'name_en'),
						'conditions' => array('id' => $topGroupIdList)));

					foreach ($groupList as $group) {
						$id = $group['Group']['id'];
	
						if ($lang === 'ja') {
							if (!empty($group['Group']['name']))
								$topGroupNameList[$id] = $group['Group']['name'];
							else
								$topGroupNameList[$id] = $group['Group']['name_en'];
						} else {
							if (!empty($group['Group']['name_en']))
								$topGroupNameList[$id] = $group['Group']['name_en'];
							else
								$topGroupNameList[$id] = $group['Group']['name'];
						}
					}
				}
			}

			$resultList[$i]['top_group_name_list'] = $topGroupNameList;

			// 選択済みのグループを検索
			$ismemberofGroupIdList = $this->AgreementIsmemberof->find('list', array(
				'fields' => array('group_id', 'group_id'), 'conditions' => array(
					'spc_id' => $spcIdList,
					'account_id' => $resultList[$i]['account_id'])));
			$resultList[$i]['top_group_select_id_list'] = $ismemberofGroupIdList;
			
			// 同意項目がないSPコネクタを判定(CG用)
			$attributeExistCount = $this->ProvideAttribute->find("count", array(
				'conditions' => array(
					'group_id' => $spcIdList,
					'OR' => array('ismemberof' => 1, 'eptid' => 1, 'name' => 1, 'mail' => 1, 'eppn' => 1,
						'idp' => 1, 'introduction' => 1, 'language' => 1, 'organization' => 1))));
			if ($attributeExistCount == 0) {
				$resultList[$i]['nothing_agree_item'] = true;
			} else {
				$resultList[$i]['nothing_agree_item'] = false;
			}
		}
	}

	public function getNotAgreedToMail($accountIds) {
		$notAgreedAccountIds = array();
		foreach ($accountIds as $accountId) {
			if (!$this->agreeToMail($accountId)) {
				$notAgreedAccountIds[] = $accountId;
			}
		}
		return $notAgreedAccountIds;
	}

	private function agreeToMail($accountId) {
//		$spcIdList = $this->getConnectSpcIdList($accountId);
//		$attributeList = $this->getAttributeListForSave($spcIdList);
////		foreach ($spcIdList as $spcId) {
////			if (isset($attributeList['mail'.$spcId])) {
////				return true;
////			}
////		}
//
//		$meatmailScpId = Configure::read('meatmail_spcid');
//		if (empty($meatmailScpId)) {
//			return false;
//		}
//
//		if (isset($attributeList['mail'.$meatmailScpId])) {
//			return true;
//		}
//
//		return false;

		// meatmailのmail属性提供をOKしているかを判定
		$meatmailScpId = Configure::read('meatmail_spcid');
		if (empty($meatmailScpId)) {
			return false;
		}

		$targetAgreement = $this->Agreement->find('first', array(
			'conditions' => array('account_id' => $accountId, 'group_id' => $meatmailScpId)));
		if (empty($targetAgreement)) {
			return false;
		}
		if ($targetAgreement['Agreement']['mail']) {
			return true;
		}
		return false;
	}

	/**
	 * ユーザの同意情報にSPコネクタの同意情報を反映する
	 * @param $userAgreeList ユーザの同意情報
	 * @param $useSpcIdList 対象となるSPコネクタのID一覧
	 */
	public function reflectSPCAgreement($userAgreeList, $useSpcIdList, $id, $provideList = array()) {
		if (empty($provideList)) {
			$provideList = $this->ProvideAttribute->find('all', 
					array('conditions' => array('group_id' => $useSpcIdList)));
		}
//debug($useSpcIdList);
//debug($id);
//debug($provideList);
//debug($userAgreeList);

		for ($i = 0; $i < count($userAgreeList['Agreement']); $i++) {
			$uspcid = $userAgreeList['Agreement'][$i]['group_id'];
// debug("i=".$i);
// debug($userAgreeList['Agreement'][$i]);
			$agree = true;
			$providerNothing = true; // SPの同意属性が未設定かの判定
			foreach ($provideList as $provide) {
				if ($uspcid != $provide['ProvideAttribute']['group_id']) {
					continue;
				}

				// ismemberof
				if ($provide['ProvideAttribute']['ismemberof'] == 1) {
					$providerNothing = false;
					if (!isset($userAgreeList['Agreement'][$i]['top_group_select_id_list'][$id])) {
						$agree = false;
						break;
					}
				}
				// eptid
				if ($provide['ProvideAttribute']['eptid'] == 1) {
					$providerNothing = false;
					if (!isset($userAgreeList['Agreement'][$i]['eptid']) || $userAgreeList['Agreement'][$i]['eptid'] == 0) {
						$agree = false;
						break;
					}
				}
				// name
				if ($provide['ProvideAttribute']['name'] == 1) {
					$providerNothing = false;
					if (!isset($userAgreeList['Agreement'][$i]['name']) || $userAgreeList['Agreement'][$i]['name'] == 0) {
						$agree = false;
						break;
					}
				}
				// mail
				if ($provide['ProvideAttribute']['mail'] == 1) {
					$providerNothing = false;
					if (!isset($userAgreeList['Agreement'][$i]['mail']) || $userAgreeList['Agreement'][$i]['mail'] == 0) {
						$agree = false;
						break;
					}
				}
				// eppn
				if ($provide['ProvideAttribute']['eppn'] == 1) {
					$providerNothing = false;
					if (!isset($userAgreeList['Agreement'][$i]['eppn']) || $userAgreeList['Agreement'][$i]['eppn'] == 0) {
						$agree = false;
						break;
					}
				}
				// idp
				if ($provide['ProvideAttribute']['idp'] == 1) {
					$providerNothing = false;
					if (!isset($userAgreeList['Agreement'][$i]['idp']) || $userAgreeList['Agreement'][$i]['idp'] == 0) {
						$agree = false;
						break;
					}
				}
				// introduction
				if ($provide['ProvideAttribute']['introduction'] == 1) {
					$providerNothing = false;
					if (!isset($userAgreeList['Agreement'][$i]['introduction']) || $userAgreeList['Agreement'][$i]['introduction'] == 0) {
						$agree = false;
						break;
					}
				}
				// language
				if ($provide['ProvideAttribute']['language'] == 1) {
					$providerNothing = false;
					if (!isset($userAgreeList['Agreement'][$i]['language']) || $userAgreeList['Agreement'][$i]['language'] == 0) {
						$agree = false;
						break;
					}
				}
				// organization
				if ($provide['ProvideAttribute']['organization'] == 1) {
					$providerNothing = false;
					if (!isset($userAgreeList['Agreement'][$i]['organization']) || $userAgreeList['Agreement'][$i]['organization'] == 0) {
						$agree = false;
						break;
					}
				}
			}

			// どれかの属性に同意していない場合、グループID(ismemberof属性)をfalseに設定して、画面で制御する
			if (!$agree) {
				$userAgreeList['Agreement'][$i]['top_group_select_id_list'][$id] = false;

			}

			// SPの同意属性が未設定⇒CGでのSPコネクタ自動作成
			$userAgreeList['Agreement'][$i]['nothing_agree_item'] = $providerNothing;
		}
//debug($userAgreeList);
		return $userAgreeList;

//		if (!empty($userAgreeList) && !empty($userAgreeList['Agreement'])) {
//			$this->set('userAgreeList', $userAgreeList['Agreement']);
//		} else {
//			$this->set('userAgreeList', array());
//		}

	}

	/**
	 * 間接接続のみ(「管理者のみ」の上位グループで未参加）のグループかどうかを返す
	 * @param $accountid アカウントID
	 * @param $groupid 上位グループID
	 * @return boolean
	 */
	private function isIndirectGroup($accountid, $groupid) {
		if (empty($accountid)) {
			return false;
		}
		if (empty($groupid)) {
			return false;
		}

		if ($this->Mygroup->isMember($accountid, $groupid)) {	// グループに参加している
			return false;
		}

		$return = false;
		$bottomIdList = $this->AddGroup->getBottomIdList($groupid, false);
		foreach ($bottomIdList as $bgroupid) {
			if ($this->Mygroup->isMember($accountid, $bgroupid) &&
				$this->Mygroup->isAdminOnly($accountid, $bgroupid)) {	// 管理者のみ
				$return = true;
			}
			else {
				$return = false;
			}
		}
		return $return;
	}
}
?>
