<?php
/**
 * Manage Controller
 *
 * PHP versions 5
 *
 * @copyright     Copyright (c) 2011 - 2012, National Institute of Informatics,
 * @license       BSD License (http://creativecommons.org/licenses/BSD/)
 */
abstract class ManagesController extends AppController {
	var $name = 'Manages';
	var $uses = array('Group', 'GroupSphost', 'SpHost', 'AccountEppn', 'Mygroup', 'AutoGroup');
	var $components = array('ParameterReplacer') ;

	/**
	 * グループ追加メールを送信する。
	 * @param $toGroupid
	 * @param $fromGroupid
	 * @param $updown
	 * @param $permission
	 * @param $sp
	 * @param $connectSP
	 */
	function sedMailForAddGroup($toGroupid, $fromGroupid, $updown, $permission, $sp = false, $connectSP = false) {
		$groups = $this->Group->find('all', array(
			'fields' => array('id', 'name', 'name_en'),
			'conditions' => array('id' => array($toGroupid, $fromGroupid))));
		$toGroupName = '';
		$fromGroupName = '';
		$toGroupNameJp = '';
		$fromGroupNameJp = '';
		$toGroupNameEn = '';
		$fromGroupNameEn = '';
		foreach($groups as $group) {
			if ($toGroupid == $group['Group']['id']) {
				$toGroupName = $group['Group']['name'];
				if (!empty($group['Group']['name']))
					$toGroupNameJp = $group['Group']['name'];
				else
					$toGroupNameJp = $group['Group']['name_en'];
				if (!empty($group['Group']['name_en']))
					$toGroupNameEn = $group['Group']['name_en'];
				else
					$toGroupNameEn = $group['Group']['name'];
			}
			if ($fromGroupid == $group['Group']['id']) {
				$fromGroupName = $group['Group']['name'];
				if (!empty($group['Group']['name']))
					$fromGroupNameJp = $group['Group']['name'];
				else
					$fromGroupNameJp = $group['Group']['name_en'];
				if (!empty($group['Group']['name_en']))
					$fromGroupNameEn = $group['Group']['name_en'];
				else
					$fromGroupNameEn = $group['Group']['name'];
			}
		}
		$Mail = array(
			'toGroupName' => $toGroupName,
			'fromGroupName' => $fromGroupName,
			'toGroupNameJp' => $toGroupNameJp,
			'fromGroupNameJp' => $fromGroupNameJp,
			'toGroupNameEn' => $toGroupNameEn,
			'fromGroupNameEn' => $fromGroupNameEn,
		);

		$path = "";
		// 承認なし
		if ($permission == 1) {
			$template = "add_group_template";
			if (!$sp) {
				$path = "/admin/manage_groups/updown/";
			} else {
				$path = "/admin/manage_sp_groups/updown/";
			}
			// 承認あり
		} else {
			$template = "add_group_approval_template";
			if (!$sp) {
				if (!$connectSP) {
					$path = "/admin/manage_inspects/group_inspect/";
				} else {
					$path = "/admin/manage_inspects/group_inspect_sp/";
				}
			} else {
				$path = "/admin/manage_sp_groups/inspect/";
			}
		}

		$Mail['url'] = Router::url($path, true).$toGroupid;
		$this->sendMailAllAdmin($toGroupid, "add.group", $template, $Mail);
	}

	/**
	 * グループ追加の許可拒否メールを送信する。
	 * @param $toGroupid 申請先のグループID
	 * @param $fromGroupid 申請元のグループID
	 * @param $allow
	 * @param $sp
	 * @param $spState
	 */
	function sendMailAllowDeny($toGroupid, $fromGroupid, $allow, $sp = null, $spState = false) {
		$groups = $this->Group->find('all', array(
			'fields' => array('id', 'name', 'sp'),
			'conditions' => array('id' => array($toGroupid, $fromGroupid))));
		$Mail = array(
			'toGroupNameJp' => '',
			'toGroupNameEn' => '',
		);

		// 申請先がSPCかどうか
		$isToSPC = false;
		foreach($groups as $group) {
			if ($toGroupid == $group['Group']['id']) {
				$Mail['toGroupNameJp'] = $this->Group->getGroupNameJp($group);
				$Mail['toGroupNameEn'] = $this->Group->getGroupNameEn($group);
				$isToSPC = $group['Group']['sp'];
			}
			else if ($fromGroupid == $group['Group']['id']) {
				$Mail['fromGroupNameJp'] = $this->Group->getGroupNameJp($group);
				$Mail['fromGroupNameEn'] = $this->Group->getGroupNameEn($group);
			}
		}

		if ($allow) {
			$template = 'add_group_allow_template';
			if ($isToSPC) {
				$template = 'add_group_allow_sp_template';
			}
			$templateAdmin = 'add_group_allow_to_admin_template';
		} else {
			$template = 'add_group_deny_template';
			if ($isToSPC) {
				$template = 'add_group_deny_sp_template';
			}
			$templateAdmin = 'add_group_deny_to_admin_template';
		}

		$path = '/admin/manage_inspects/group_inspect_state/';
		$pathAdmin = '/admin/manage_inspects/group_inspect/';
		if ($sp) {
			$path = '/admin/manage_sp_groups/group_inspect_state/';
			$pathAdmin = '/admin/manage_sp_groups/inspect/';
		} else {
			if ($spState) {
				$path = '/admin/manage_inspects/group_inspect_sp_state/';
			}
		}
		$Mail['url'] = Router::url($path, true).$fromGroupid;
		$this->sendMailAllAdmin($fromGroupid, "add.group", $template, $Mail);

		// このグループ管理者に処理したことをメールする
		$Mail['url'] = Router::url($pathAdmin, true).$toGroupid;
		$this->sendMailToOtherAdmin($toGroupid, 'add.group', $templateAdmin, $Mail);
	}

	/**
	 * 管理者入会申請メールを送信する。
	 * @param $toGroupid
	 * @param $sp
	 */
	function sedMailForManager($toGroupid, $sp = true) {

		$groups = $this->Group->find('first', array(
			'fields' => array('id', 'name'),
			'conditions' => array('id' => $toGroupid)));
		$Mail = array(
			'groupName' => $groups['Group']['name'],
			'groupNameJp' => $this->getGroupNameForMail('ja', $groups),
			'groupNameEn' => $this->getGroupNameForMail('eng', $groups),
			'url' => Router::url('/admin/manage_inspects/join/'.$toGroupid, true),
		);
		$this->sendMailAllAdmin($toGroupid, "application.admin", 'application_admin_template', $Mail);
	}

	protected function getGroupNameForMail($lang, $Group) {
		$name = '';
		if ($lang === 'ja') {
			if (!empty($Group['Group']['name']))
				$name = $Group['Group']['name'];
			else
				$name = $Group['Group']['name_en'];
		} else {
			if (!empty($Group['Group']['name_en']))
				$name = $Group['Group']['name_en'];
			else
				$name = $Group['Group']['name'];
		}
		return $name;
	}

	/**
	 *
	 * @param $groups
	 * @param $optionName
	 * @param $updown
	 */
	function setAddGroupView($groups, $optionName, $updown) {
		$groupList = array();
		foreach($groups as $group) {
			$intro = $this->getIntroduction($group);
			// SPコネクタ > グループ接続の場合のみカット
			if ($updown == 'sp' && mb_strlen($intro) > 40) {
				$intro = mb_substr($intro, 0, 40)."...";
			}
			$approve = __("common.unnecessary", true);
			if ($updown == 'up') {
				if ($group['ManageGroup']['down_permission'] == 2)
				$approve = __("common.necessary", true);
			} else if ($updown == 'down') {
				if ($group['ManageGroup']['up_permission'] == 2)
				$approve = __("common.necessary", true);
			} else if ($updown == 'sp') {
				$approve = __("common.necessary", true);
			}

			$groupList[] = array(
				'id' => $group['ManageGroup']['id'],
				'name' => $this->getGroupName($group),
				'approve' => $approve,
				'introduction' => $intro,
				'spc_icon' => $group['ManageGroup']['spc_icon'],
			);
		}
		$this->set('groupList', $groupList);
	}

	function getGroupName($group) {
		foreach (array('Group', 'ManageGroup') as $modelName) {
			if (isset($group[$modelName])) {
				return $group[$modelName]['name'];
			}
		}
		return null;
	}

	function getIntroduction($group) {
		foreach (array('Group', 'ManageGroup') as $modelName) {
			if (isset($group[$modelName])) {
				return $group[$modelName]['introduction'];
			}
		}
		return null;
	}

	/**
	 * 使用言語から画面表示に必要でない情報を削除します。
	 * @param $entities
	 */
	function removeUnnecessaryLanguage(&$entities) {
		if (empty($entities)) {
			return;
		}
		$modelName = 'Group';
		if (isset($entities[0]['Group'])) {
			$modelName = 'Group';
		} else if (isset($entities[0]['ManageGroup'])) {
			$modelName = 'ManageGroup';
		} else if (isset($entities[0]['Account'])) {
			$modelName = 'Account';
		}
		if ($modelName === 'Account') {
			if (isset($entities[0])) {
				for ($i = 0; $i < count($entities); $i++) {
					if ($this->getLanguage() === 'ja') {
						if (!empty($entities[$i][$modelName]['name']))
							$entities[$i][$modelName]['name_en'] = '';
						if (!empty($entities[$i][$modelName]['introduction']))
							$entities[$i][$modelName]['introduction_en'] = '';
						if (!empty($entities[$i][$modelName]['university']))
							$entities[$i][$modelName]['university_en'] = '';
					} else {
						if (!empty($entities[$i][$modelName]['name_en']))
							$entities[$i][$modelName]['name'] = '';
						if (!empty($entities[$i][$modelName]['introduction_en']))
							$entities[$i][$modelName]['introduction'] = '';
						if (!empty($entities[$i][$modelName]['university_en']))
							$entities[$i][$modelName]['university'] = '';
					}
				}
			} else {
				if ($this->getLanguage() === 'ja') {
					if (!empty($entities[$modelName]['name']))
						$entities[$modelName]['name_en'] = '';
					if (!empty($entities[$modelName]['introduction']))
						$entities[$modelName]['introduction_en'] = '';
					if (!empty($entities[$modelName]['university']))
						$entities[$modelName]['university_en'] = '';
				} else {
					if (!empty($entities[$modelName]['name_en']))
						$entities[$modelName]['name'] = '';
					if (!empty($entities[$modelName]['introduction_en']))
						$entities[$modelName]['introduction'] = '';
					if (!empty($entities[$modelName]['university_en']))
						$entities[$modelName]['university'] = '';
				}
			}
		} else { // グループの場合はenを削除
			if (isset($entities[0])) {
				for ($i = 0; $i < count($entities); $i++) {
					$entities[$i][$modelName]['name_en'] = '';
					$entities[$i][$modelName]['introduction_en'] = '';
				}
			} else {
				$entities[$modelName]['name_en'] = '';
				$entities[$modelName]['introduction_en'] = '';
			}
		}
	}

	/**
	 * Nodeを設定する。
	 * @param $nodes
	 * @param $groupName
	 * @param $groupid
	 * @param $introduction
	 * @param $x
	 * @param $y
	 * @param $sp
	 * @param $self
	 * @param $auto
	 */
	function setNodes($nodes, $groupName, $groupid, $introduction = null, $x, $y, $sp, $self = null,
			$auto = null) {
		if (strlen($nodes) > 0) {
			$nodes = $nodes.", ";
		}

		$rgb = "rgb(255,0,0)";
		$shape = "sphere";
		if ($sp == 1) {
			$rgb = "rgb(255,128,255)";
			$shape = "square";
		}
		if ($self) {
			$rgb = "rgb(255,255,0)";
		}

		$intro = h($introduction);
		if ($intro != null) {
			if (mb_strlen($intro) > 50) {
				$intro = mb_substr($intro, 0, 50);
			}
			$intro = str_replace(array("\r\n","\r","\n"), '<br/>', $intro);
		}

		if ($auto) {
			$nodes = $nodes."{id: '".$groupName.
					"', color: '".$rgb."', shape: '".$shape."', size: 1, groupid: ".
			$groupid.", sp: ".$sp.", introduction: '".$intro."'}";
		} else {
			$nodes = $nodes."{id: '".$groupName.
				"', color: '".$rgb."', shape: '".$shape."', size: 1, x: ".$x.", y: ".$y.", groupid: ".
			$groupid.", sp: ".$sp.", introduction: '".$intro."'}";
		}
		return $nodes;
	}

	/**
	 * Edgesを設定する。
	 * @param $edges
	 * @param $id1
	 * @param $id2
	 * @param $headAllow
	 */
	function setEdges($edges, $id1, $id2, $headAllow) {
		if (strlen($edges) > 0) {
			$edges = $edges.", ";
		}

		$allow = "line";
		if ($headAllow) {
			$allow = "arrowHeadLine";
		}
		$edges = $edges."{id1: '".$id1.
			"', id2: '".$id2."', color: 'rgb(51,102,255)', width: '1', type: '".$allow."'}";

		return $edges;
	}

	/**
	 * 紐付くSPコネクタ情報を取得する。
	 * @param $groupid
	 */
	function getConnectedSpConnectorList($groupid) {
		$spinfoList = $this->Group->find('all', array(
			'fields' => array('id', 'name'),
			'conditions' => array(
				'active' => 0,
				'sp' => 1,
				"id in (SELECT top_id FROM add_groups where bottom_id in"
				." (SELECT top_id FROM add_groups where top = 0 and bottom_id = ? "
				.")"
				." union"
				." SELECT top_id FROM add_groups where top = 1 and bottom_id = ?)" => array($groupid, $groupid)
				))
			);
		$group = $this->Group->find('first', array('fields' => array('id', 'name', 'group_key'),
			'conditions' => array('id' => $groupid)));

		$spGroupIdList = array();
		foreach ($spinfoList as $info) {
			array_push($spGroupIdList, $info['Group']['id']);
		}
		$groupSpHostList = $this->GroupSphost->find('all', array('conditions' => array('group_id' => $spGroupIdList)));
		$spHostList = $this->SpHost->find('list');
		$spHostEntityIdList = $this->SpHost->find('list', array('fields' => array('id', 'entityid')));
		for ($i = 0; $i < count($groupSpHostList); $i++) {
			$serviceName = $this->GroupSphost->getServiceName($this->getLanguage(), $groupSpHostList[$i]);
			$groupSpHostList[$i]['GroupSphost']['service_name'] = $serviceName;
			if (empty($groupSpHostList[$i]['GroupSphost']['service_name'])) {
				$groupSpHostList[$i]['GroupSphost']['service_name'] = $spHostList[ $groupSpHostList[$i]['GroupSphost']['sp_id'] ];
			}

			// lead_urlのパラメータ置換する
			$groupSpHostList[$i]['GroupSphost']['lead_url'] =
				$this->ParameterReplacer->replaceGroupKey($group['Group']['group_key'], $groupSpHostList[$i]['GroupSphost']['lead_url']);
			$groupSpHostList[$i]['GroupSphost']['lead_url'] =
				$this->ParameterReplacer->replaceEntityId($groupSpHostList[$i]['GroupSphost']['lead_url']);

			// relogin_urlのパラメータ置換
			$groupSpHostList[$i]['GroupSphost']['relogin_url'] =
				$this->ParameterReplacer->replaceGroupKey($group['Group']['group_key'], $groupSpHostList[$i]['GroupSphost']['relogin_url']);
			$groupSpHostList[$i]['GroupSphost']['relogin_url'] =
				$this->ParameterReplacer->replaceEntityId($groupSpHostList[$i]['GroupSphost']['relogin_url']);
		}
		return $groupSpHostList;
	}

	/**
	 * グループを下に接続できるかを確認する。
	 * グループ：上位グループになっていない なら追加可能
	 * グループ、SPコネクタの両方:公開されているまたは、自分か管理者なら追加可能、接続されていないなら接続可能
	 * @param $myGroupid
	 * @param $selectedGroupId
	 * @param $sp
	 * @return validation results
	 */
	function validate_add_save_down($myGroupid, $selectedGroupId, $sp) {
		// 同じグループIDは接続できない
		if ($myGroupid == $selectedGroupId) {
			return false;
		}

		// 上位グループに申請しているかを確認
		$count = $this->AddGroupInspect->find('count', array('conditions' => array(
			'bottom_id' => $myGroupid,
			'join' => 0,
			'kind' => 0,  // 自分が下になりたい
			'status' => 0
		)));
		if ($count > 0) {
			$this->Session->setFlash(__('manage_groups.not.connect.group.alreadyapplication', true), 'message');
			$this->redirect('/admin/manage_groups/updown/'.$myGroupid);
			return;
		}

		$group = $this->Group->find('first', array(
			'fields' => array('id', 'public', 'active', 'independence', 'up_permission', 'sp'),
			'conditions' => array('id' => $selectedGroupId)));
		$mygroup = $this->Mygroup->find('first', array('fields' => array('groupid', 'admin'),
			'conditions' => array('groupid' => $selectedGroupId)));
		// 公開していなく、管理者でもない
		if ($group['Group']['public'] != 1) {
			if ($mygroup['Mygroup']['admin'] == 0) {
				return false;
			}
		}

		// 既に削除されていたら接続できない
		if ($group['Group']['active'] == 9) {
			return false;
		}

		if (!$sp) {
			// 独立していたら接続できない(グループのみ)
			if ($group['Group']['sp'] == 0 && $group['Group']['independence'] == 1) {
				return false;
			}

			// 下位グループなることを許可していないなら接続できない(グループのみ)
			if ($group['Group']['sp'] == 0 && $group['Group']['up_permission'] == 0) {
				return false;
			}
		}

		// 既に選択したグループと接続しているか確認
		$count = $this->AddGroup->find('count', array(
			'conditions' => array('bottom_id' => $selectedGroupId, 'top_id' => $myGroupid)));
		if ($count > 0) {
			return false;
		}

		if (!$sp) {
			// 選択したグループが上位グループならそのグループを下位に接続できない
			$count = $this->AddGroup->find('count', array('conditions' => array(
				'top_id' => $selectedGroupId)));
			if ($count > 0) {
				return false;
			}

			// 自グループが下位グループなら下位グループを接続できない
			$count = $this->AddGroup->find('count', array('conditions' => array(
				'bottom_id' => $myGroupid, 'top' => 0)));
			if ($count > 0) {
				return false;
			}
		}

		return true;
	}

	/**
	 * ページを制御する。
	 * @param integer $page 現在のページ数
	 * @param integer $count 検索結果数
	 * @param integer $limit １画面の最大表示件数
	 */
	function controllPage($page, $count, $limit) {
		if ($page == 0 || $page == 1) {
			$this->set("prev", false);
		} else {
			$this->set("prev", true);
		}
		if ($page == 0) {
			$this->set("next", false);
		} else {
			$next = $count / ($limit * $page);
			if ($next > 1) {
				$this->set("next", true);
			} else {
				$this->set("next", false);
			}
		}
	}

	public function setFavariteFiveGroup($id) {
		$top5GroupIdList = $this->findTopFiveGroupIdName();

		$this->set('selfGroupId', $id);
		$this->set('top5GroupIdList', $top5GroupIdList);
	}

	/**
	 * 自分が所属しているグループのTop 5のグループID/グループ名を取得する
	 */
	public function findTopFiveGroupIdName() {
		$mygroups = $this->getMyGroupList($withSPC=false);

		$top5GroupIdList = array();
		foreach (array_slice($mygroups, 0, 5, true) as $group) {
			$top5GroupIdList[$group['Group']['id']] = $group['Group']['name'];
		}

		return $top5GroupIdList;
	}

	/**
	 * 自分が所属しているグループの情報を取得する
         * @param withSPC boolean SPコネクタを含める否か
         */
	public function getMyGroupList($withSPC=true) {
		$mygroups_cond = array(
			'Mygroup.groupid = Group.id',
			'Mygroup.account_id' => $this->getAccountID());
		if ($withSPC != true) {
			$mygroups_cond[] = 'Group.sp = 0';
		}

		$mygroups = $this->Group->find('all', array(
			'conditions' => array(
				'Group.sp' => array(0, 1), 'NOT' => array('Group.active' => '9')),
			'fields' => array('Group.id', 'Group.name', 'Group.name_en', 'Group.introduction', 'Group.introduction_en',
				'Group.active', 'Group.public', 'Group.sp', 'Group.icon', 'Group.spc_icon',
				'Mygroup.admin', 'Mygroup.favorite',
				'Group.openmember', 'Group.created', 'count(Mygroup2.id) as count_member'),
			'order' => array('Mygroup.favorite DESC', 'sp', 'name'),
			'group' => array('Group.id'),
			'joins' => array(
				array(
					'type' => 'INNER', 'table' => 'mygroups', 'alias' => 'Mygroup',
					'conditions' => $mygroups_cond),
				array(
					'type' => 'INNER', 'table' => 'mygroups', 'alias' => 'Mygroup2',
					'conditions' => array(
						'Mygroup2.groupid = Group.id')))
		));

		// 多言語対応
		$this->removeUnnecessaryLanguage($mygroups);

		// メンバーである機関グループは表示しない
		$idpGroupIdList = $this->Mygroup->find('list', array(
			'fields' => array('groupid', 'groupid'),
			'conditions' => array(
				'account_id' => $this->getAccountID(),
				'admin' => '0'),
			'joins' => array(
				array(
					'type' => 'INNER', 'table' => 'auto_groups', 'alias' => 'AutoGroup',
					'conditions' => array(
						'AutoGroup.group_id = Mygroup.groupid',
						'AutoGroup.idp' => 'true'))),
		));

		foreach (array_keys($mygroups) as $i) {
			$groupid = $mygroups[$i]['Group']['id'];
			if (in_array($groupid, $idpGroupIdList)) {
				unset($mygroups[$i]);
			}
		}
		return $mygroups;
	}
}
?>
