<?php
/**
 * Statistics Controller
 *
 * PHP versions 5
 *
 * @copyright     Copyright (c) 2022, National Institute of Informatics,
 * @license       BSD License (http://creativecommons.org/licenses/BSD/)
 */
class StatisticsController extends AppController {
	var $name = 'Statistics';
	var $uses = array('AccountEppn', 'AddGroup', 'Mygroup', 'Group', 'AutoGroup', 'Account');
	var $components = array('NoticeUtils', 'Url');

	function beforeFilter() {
		parent::beforeFilter();
		// システム管理者のみがアクセス可能
		if (!$this->isSuperManager()) {
			$this->cakeError('cannotAccess', array('url' => $this->here));
		}
	}

	/**
	 * システム統計情報画面を表示する。
	 */
	function index() {
		$allAccounts = $this->Account->find('count', array(
			'conditions' => array(
				"id in (select account_id from account_eppns) and mail != '' and mail is not null"
			),
		));
		$this->set('allAccounts', $allAccounts);

		$group['general']['active'] = $this->Group->find('count', array(
			'conditions' => array(
				'active' => 0, 'sp' => 0,
				"id not in (select group_id from auto_groups)"
			),
		));

		$group['general']['private'] = $this->Group->find('count', array(
			'conditions' => array(
				'active' => 0, 'sp' => 0, 'public' => 0,
				"id not in (select group_id from auto_groups)"
			),
		));

		$group['general']['include'] = $this->Group->find('count', array(
			'conditions' => array(
				'active' => 0, 'sp' => 0, 'public' => 1,
				"id not in (select group_id from auto_groups)"
			),
		));

		$group['general']['exclude'] = $this->Group->find('count', array(
			'conditions' => array(
				'active' => 0, 'sp' => 0, 'public' => 2,
				"id not in (select group_id from auto_groups)"
			),
		));

		$group['general']['inactive'] = $this->Group->find('count', array(
			'conditions' => array(
				'active' => 1, 'sp' => 0,
				"id not in (select group_id from auto_groups)"
			),
		));

		$group['general']['deleted'] = $this->Group->find('count', array(
			'conditions' => array(
				'active' => 9, 'sp' => 0,
				"id not in (select group_id from auto_groups)"
			),
		));

		$group['idp']['active'] = $this->Group->find('count', array(
			'conditions' => array(
				'active' => 0, 'sp' => 0,
				"id in (select group_id from auto_groups)"
			),
		));

		$group['idp']['private'] = $this->Group->find('count', array(
			'conditions' => array(
				'active' => 0, 'sp' => 0, 'public' => 0,
				"id in (select group_id from auto_groups)"
			),
		));

		$group['idp']['include'] = $this->Group->find('count', array(
			'conditions' => array(
				'active' => 0, 'sp' => 0, 'public' => 1,
				"id in (select group_id from auto_groups)"
			),
		));

		$group['idp']['exclude'] = $this->Group->find('count', array(
			'conditions' => array(
				'active' => 0, 'sp' => 0, 'public' => 2,
				"id in (select group_id from auto_groups)"
			),
		));

		$group['idp']['inactive'] = $this->Group->find('count', array(
			'conditions' => array(
				'active' => 1, 'sp' => 0,
				"id in (select group_id from auto_groups)"
			),
		));

		$group['idp']['deleted'] = $this->Group->find('count', array(
			'conditions' => array(
				'active' => 9, 'sp' => 0,
				"id in (select group_id from auto_groups)"
			),
		));

		$this->set('group', $group);

		$sp['active']['all'] = $this->Group->find('count', array(
			'conditions' => array(
				'active' => 0, 'sp' => 1
			),
		));
		$sp['active']['public'] = $this->Group->find('count', array(
			'conditions' => array(
				'active' => 0, 'sp' => 1, 'public' => 1
			),
		));

		$sp['idp']['all'] = $this->Group->find('count', array(
			'conditions' => array(
				'active' => 0, 'sp' => 1, 'sp_type' => 1
			),
		));
		$sp['idp']['public'] = $this->Group->find('count', array(
			'conditions' => array(
				'active' => 0, 'sp' => 1, 'public' => 1, 'sp_type' => 1
			),
		));

		$sp['ap']['all'] = $this->Group->find('count', array(
			'conditions' => array(
				'active' => 0, 'sp' => 1, 'sp_type' => 2
			),
		));
		$sp['ap']['public'] = $this->Group->find('count', array(
			'conditions' => array(
				'active' => 0, 'sp' => 1, 'public' => 1, 'sp_type' => 2
			),
		));

		$sp['or']['all'] = $this->Group->find('count', array(
			'conditions' => array(
				'active' => 0, 'sp' => 1, 'sp_type' => 3
			),
		));
		$sp['or']['public'] = $this->Group->find('count', array(
			'conditions' => array(
				'active' => 0, 'sp' => 1, 'public' => 1, 'sp_type' => 3
			),
		));

		$sp['apAnd']['all'] = $this->Group->find('count', array(
			'conditions' => array(
				'active' => 0, 'sp' => 1, 'sp_type' => 4
			),
		));
		$sp['apAnd']['public'] = $this->Group->find('count', array(
			'conditions' => array(
				'active' => 0, 'sp' => 1, 'public' => 1, 'sp_type' => 4
			),
		));

		$sp['and']['all'] = $this->Group->find('count', array(
			'conditions' => array(
				'active' => 0, 'sp' => 1, 'sp_type' => 5
			),
		));
		$sp['and']['public'] = $this->Group->find('count', array(
			'conditions' => array(
				'active' => 0, 'sp' => 1, 'public' => 1, 'sp_type' => 5
			),
		));

		$sp['inactive']['all'] = $this->Group->find('count', array(
			'conditions' => array(
				'active' => 1, 'sp' => 1
			),
		));
		$sp['inactive']['public'] = $this->Group->find('count', array(
			'conditions' => array(
				'active' => 1, 'sp' => 1, 'public' => 1
			),
		));

		$this->set('sp', $sp);

		$private['active'] = $this->Group->find('count', array(
			'conditions' => array(
				'active' => 0, 'sp' => 2
			),
		));

		$private['idp'] = $this->AddGroup->find('count', array(
			'conditions' => array(
				"bottom_id IN (SELECT g.id FROM groups AS g, auto_groups AS ag WHERE g.id = ag.group_id AND g.active = 0 AND g.sp = 0)",
				"top_id IN (SELECT id FROM groups WHERE sp = 2 AND active = 0)"
			),
		));
		$private['group'] = $this->AddGroup->find('count', array(
			'conditions' => array(
				"bottom_id NOT IN (SELECT g.id FROM groups AS g, auto_groups AS ag WHERE g.id = ag.group_id AND g.active = 0 AND g.sp = 0)",
				"top_id IN (SELECT id FROM groups WHERE sp = 2 AND active = 0)"
			),
		));
		$private['private'] = $private['active'] - ($private['idp'] +  $private['group']);

		$private['inactive'] = $this->Group->find('count', array(
			'conditions' => array(
				'active' => 1, 'sp' => 2
			),
		));

		$this->set('private', $private);
	}

	/**
	 * 機関グループの一覧画面を表示する。
	 */
	function idps() {
		$idp = $this->AutoGroup->find('all', array(
			'fields' => array('id', 'group_id', 'entityid'),
			'conditions' => array(
				"group_id in (select id from groups where active = 0 and sp = 0)"
			),
			'order' => array('id ASC'),
		));
		$participants = Configure::read('participants');
		foreach ($idp as &$group) {
			$group_id = $group['AutoGroup']['group_id'];
			$group['name'] = $this->Group->getGroupName($group_id, 'ja');
			$entityId = $group['AutoGroup']['entityid'];
			$group['participant'] = in_array($entityId, $participants);
			$group['allMember'] = $this->Mygroup->find('count', array(
				'conditions' => array('groupid' => $group_id)));
			$group['accountMember'] = $this->Account->find('count', array(
				'conditions' => array(
					'is_temp' => 0,
					"mail != '' AND mail IS NOT NULL",
					'id IN (SELECT account_id FROM mygroups WHERE groupid = ?)' => $group_id
				),
			));
			$group['service'] = $this->Group->find('count', array(
				'conditions' => array(
					'sp' => 1,
					'active' => array(0, 1),
					'id IN (SELECT top_id FROM add_groups WHERE bottom_id = ?)' => $group_id
				),
			));
			$group['private'] = $this->AddGroup->find('count', array(
				'conditions' => array(
					'bottom_id' => $group_id,
					'top_id IN (SELECT id FROM groups WHERE sp = 2)'
				),
			));
		}

		$this->set('idp', $idp);
	}

	/**
	 * アカウント情報の一覧画面を表示する。
	 */
	function accounts() {
		if (isset($this->params['url']['page'])) {	// 前へ 次へ redirect時
			$page = (int)$this->params['url']['page'];
		}
		else {
			$page = 1;
		}
		if ($page <= 0) { $page = 1; }

		$count = $this->Account->find('count', array(
			'conditions' => array(
				"id in (select account_id from account_eppns) and mail != '' and mail is not null"
			),
		));
		$limit = Configure::read("page.memberlimit");
		$maxPage = ceil($count / $limit);
		if ($page > $maxPage) {
			$page = $maxPage;
		}

		$accountList = $this->Account->find('all', array(
			'fields' => array('id', 'name', 'name_en', 'mail'),
			'conditions' => array(
				"id in (select account_id from account_eppns) and mail != '' and mail is not null"
			),
			'order' => array('id ASC'),
			'limit' => $limit,
			'page' => $page,
		));
		// ページ制御
		$this->controllPage($page, $count, $limit);

		$records = array();
		foreach ($accountList as $account) {
			$eppnList = $this->AccountEppn->find('list', array(
				'fields' => array('eppn'),
				'conditions' => array( 'account_id' => $account['Account']['id'])
			));
			$record = array();
			$record['accountid'] = $account['Account']['id'];
			$record['name'] = $account['Account']['name'] . ' ( ' . $account['Account']['name_en'] . ' )';
			$record['mail'] = $account['Account']['mail'];
			$record['eppn'] = implode("</br>", $eppnList);
			$records[] = $record;
		}
		$this->set('records', $records);
		$this->set('page', $page);
	}

	/**
	 * ページを制御する。
	 * @param integer $page 現在のページ数
	 * @param integer $count 検索結果数
	 * @param integer $limit １画面の最大表示件数
	 */
	private function controllPage($page, $count, $limit) {
		if ($page == 0 || $page == 1) {
			$this->set("prev", false);
		} else {
			$this->set("prev", true);
		}
		if ($page == 0) {
			$this->set("next", false);
		} else {
			$next = $count / ($limit * $page);
			if ($next > 1) {
				$this->set("next", true);
			} else {
				$this->set("next", false);
			}
		}
	}

	/**
	 * アカウント情報の詳細画面を表示する。
	 */
	function account_details($accountId) {
		$account = $this->Account->find('first', array(
			'fields' => array('id', 'name', 'name_en', 'mail', 'university', 'university_en', 'introduction',
				'introduction_en', 'language', 'arrival'),
			'conditions' => array('id' => $accountId)));
		$eppnList = $this->AccountEppn->find('list', array(
			'fields' => array('eppn'),
			'conditions' => array('account_id' => $accountId)
		));
		$account['eppn'] = implode("</br>", $eppnList);
		$groupIds = $this->Mygroup->findMyGroupIdList($accountId, array(), true);
		$account['groupList'] = $this->Group->findGroupNameList($groupIds);

		$this->set('account', $account);
	}

	/**
	 * 一般グループの一覧画面を表示する。
	 */
	function groups() {
		if (isset($this->params['url']['page'])) {	// 前へ 次へ redirect時
			$page = (int)$this->params['url']['page'];
		}
		else {
			$page = 1;
		}
		if ($page <= 0) { $page = 1; }

		$count = $this->Group->find('count', array(
			'conditions' => array(
				'active' => 0, 'sp' => 0,
				"id not in (select group_id from auto_groups)"
			),
		));
		$limit = Configure::read("page.grouplimit");
		$maxPage = ceil($count / $limit);
		if ($page > $maxPage) {
			$page = $maxPage;
		}

		$groupList = $this->Group->find('all', array(
			'fields' => array('id', 'name'),
			'conditions' => array('active' => 0, 'sp' => 0, "id not in (select group_id from auto_groups)"),
			'order' => array('id ASC'),
			'limit' => $limit,
			'page' => $page,
		));
		// ページ制御
		$this->controllPage($page, $count, $limit);
		$records = array();
		foreach ($groupList as &$group) {
			$group_id = $group['Group']['id'];
			$group['allMember'] = $this->Mygroup->find('count', array(
				'conditions' => array('groupid' => $group_id)));
			$group['accountMember'] = $this->Account->find('count', array(
				'conditions' => array(
					'is_temp' => 0,
					"mail != '' AND mail IS NOT NULL",
					'id IN (SELECT account_id FROM mygroups WHERE groupid = ?)' => $group_id
				),
			));
		}
		$this->set('groupList', $groupList);
		$this->set('page', $page);
	}

}
?>
