<?php
/**
 * SystemAccount Controller
 *
 * PHP versions 5
 *
 * @copyright     Copyright (c) 2021 - 2022, National Institute of Informatics,
 * @license       BSD License (http://creativecommons.org/licenses/BSD/)
 */
class SystemAccountsController extends AppController {
	var $name = 'SystemAccounts';
	var $uses = array('SentNoticeMailLog', 'AccountEppn', 'Mygroup', 'Group', 'Transaction');
	var $components = array('NoticeUtils', 'Url');

	function beforeFilter() {
		parent::beforeFilter();
		// システム管理者のみがアクセス可能
		if (!$this->isSuperManager()) {
			$this->cakeError('cannotAccess', array('url' => $this->here));
		}
	}

	/**
	 * 一覧を表示する。
	 */
	function index() {
	}

	/**
	 * ２年間アクセスがなかったアカウントを表示する。
	 */
	function invalid() {
		$this->debugLog('SystemAccountsController::invalid data => '.var_export($this->data, true));
		if (isset($this->params['url']['page'])) {	// 前へ 次へ redirect時
			$page = (int)$this->params['url']['page'];
		}
		else if (isset($this->data['SystemAccount']['page'])) {	// submit時
			$page = $this->data['SystemAccount']['page'];
		}
		else {
			$page = 1;
		}
		if ($page <= 0) { $page = 1; }

		// 既に削除対象になっているアカウントのリスト
		$deleteIdList = $this->SentNoticeMailLog->find('list', array(
			'fields' => array('account_eppn_id'),
			'gruop' => 'account_eppn_id',
		));

		switch ($this->getSubmitName()) {
			case 'send':
				if (empty($this->data)) {
					break;
				}
				$eppnId = $this->data['SystemAccount']['eppnid'];
				$this->debugLog("SystemAccountsController::invalid : send notice mail (eppnId = $eppnId).");
				$this->Url->get("/map/batch_send_notice_mails/manual/$eppnId");
				$this->Session->setFlash("通知メールを再送しました。", 'message');
				$this->redirect('/system/accounts/invalid?page=' . $page);
				break;
			case 'delete':
				if (empty($this->data)) {
					break;
				}
				$accountId = $this->data['SystemAccount']['accountid'];
				$this->debugLog("SystemAccountsController::invalid : delete Account (accountid = $accountId).");
				$soloAdminList = array();
				$this->NoticeUtils->getSoloAdminGroups($accountId, $deleteIdList, $soloAdminList);
				foreach ($soloAdminList as $soloAdmin) {
					if ($soloAdmin['special']) {
						$this->Session->setFlash("管理者がこのアカウントのみの機関グループかSPコネクタがあるため削除できません。", 'message');
						$this->redirect('/system/accounts/invalid?page=' . $page);
					}
				}
				$this->Transaction->begin();
				if (!$this->NoticeUtils->deleteAccountGroup($accountId, true)) {
					$this->Transaction->rollback();
					$this->log("SystemAccountsController::invalid ERROR delete Account(id = $accountId) delete Error");
					$this->Session->setFlash("アカウントが削除できませんでした。", 'message');
					$this->redirect('/system/accounts/invalid?page=' . $page);
				}
				$this->Transaction->commit();
				$this->Session->setFlash("アカウントを削除しました。", 'message');
				$this->redirect('/system/accounts/invalid?page=' . $page);
				break;
			default:
				break;
		}
		$startTime = date('Y-m-d H:i:s');

		// 通知した人のリスト
		$idList = $this->SentNoticeMailLog->find('list', array(
			'fields' => array('account_eppn_id'),
			'group' => array('account_eppn_id'),
		));
//		$this->debugLog('SystemAccountsController::invalid idList => '.var_export($idList,true));
		$count = count($idList);
		$this->debugLog('SystemAccountsController::invalid count = '.$count);
		$maxPage = ceil($count / Configure::read("page.memberlimit"));
		if ($page > $maxPage) {
			$page = $maxPage;
		}

		$logList = $this->SentNoticeMailLog->find('all', array(
			'fields' => array('account_eppn_id', 'pending', 'MAX(kind) as max_kind', 'MAX(sent) as max_sent'),
			'conditions' => array('kind' => array(1,2)),
			'group' => array('account_eppn_id'),
			'order' => array('pending DESC', 'max_kind DESC', 'max_sent ASC'),
			'limit' => Configure::read("page.memberlimit"),
			'page' => $page,
		));
		// ページ制御
		$this->controllPage($page, $count, Configure::read("page.memberlimit"));

		$records = array();
		foreach ($logList as $log) {
			$deleteId = $log['SentNoticeMailLog']['account_eppn_id'];
			$account = $this->AccountEppn->find('first', array(
				'fields' => array('AccountEppn.id', 'AccountEppn.account_id', 'AccountEppn.eppn', 'AccountEppn.last_login_approx',
					'Account.name', 'Account.name_en', 'Account.mail'),
				'joins' =>array(array(
					'type'=>'INNER', 'table'=>'accounts', 'alias'=>'Account',
					'conditions' => array(
						'Account.id = AccountEppn.account_id',
					))
				),
				'conditions' => array( 'AccountEppn.id' => $deleteId)
			));
			$record = array();
			$record['id'] = $deleteId;
			$record['eppn'] = $account['AccountEppn']['eppn'];
			$record['accountid'] = $account['AccountEppn']['account_id'];
			$record['name'] = $this->getAccountName('ja', $account);
			$record['mail'] = $account['Account']['mail'];
			$record['last_login_approx'] = $account['AccountEppn']['last_login_approx'];
			$logs = $this->SentNoticeMailLog->find('all', array(
				'conditions' => array( 'account_eppn_id' => $deleteId),
				'order' => array('id ASC'),
			));
			$newest = 0;
			$pending = 0;
			foreach ($logs as $log) {
				$kind = $log['SentNoticeMailLog']['kind'];
				$sent = $log['SentNoticeMailLog']['sent'];
				$accessed = $log['SentNoticeMailLog']['accessed'];
				if ($kind == 1) {
					$record['first'] = $sent;
				}
				elseif ($kind == 2) {
					$record['second'] = $sent;
				}
				elseif ($kind == 3) {
					$record['manual'] = $sent;
				}
				if ($newest == 0 || strtotime($newest) < strtotime($accessed)) {
					$newest = $accessed;
				}
				if ($log['SentNoticeMailLog']['pending'] == 1) {
					$pending = 1;
				}
			}
			$record['accessed'] = $newest;
			$record['pending'] = $pending;
			$idList = $this->AccountEppn->find('list', array('fields' => array('id'),
					'conditions' => array('account_id' => $record['accountid'])));
			$unlink = 0;
			foreach ($idList as $id) {
				$idKey = array_search($id, $deleteIdList);
				if ($idKey === false) {	// 削除対象でない
					$unlink = 1;
					break;
				}
			}
			$soloAdminList = array();
			$special = 0;
			if (!$unlink) {
				$this->NoticeUtils->getSoloAdminGroups($record['accountid'], $deleteIdList, $soloAdminList);
				$name = '';
				foreach ($soloAdminList as &$soloAdmin) {
					$soloAdmin['name'] = $this->Group->getGroupName($soloAdmin['id'], 'ja');
					$soloAdmin['sp'] = $this->Group->isSpConnector($soloAdmin['id']);
					if ($soloAdmin['special']) {
						$special = 1;
					}
				}
			}
			$record['soloAdmin'] = $soloAdminList;
			$record['special'] = $special;
			$record['unlink'] = $unlink;
			$records[] = $record;
		}
		$this->set('records', $records);
		$this->data['SystemAccount']['page'] = $page;
	}

	/**
	 * ページを制御する。
	 * @param integer $page 現在のページ数
	 * @param integer $count 検索結果数
	 * @param integer $limit １画面の最大表示件数
	 */
	private function controllPage($page, $count, $limit) {
		if ($page == 0 || $page == 1) {
			$this->set("prev", false);
		} else {
			$this->set("prev", true);
		}
		if ($page == 0) {
			$this->set("next", false);
		} else {
			$next = $count / ($limit * $page);
			if ($next > 1) {
				$this->set("next", true);
			} else {
				$this->set("next", false);
			}
		}
	}
}
?>
