<?php
/**
 * Account Controller
 *
 * PHP versions 5
 *
 * @copyright     Copyright (c) 2011 - 2012, National Institute of Informatics,
 * @license       BSD License (http://creativecommons.org/licenses/BSD/)
 */
class AccountsController extends AppController {
	var $name = 'Accounts';
	var $uses = array('Account', 'Mygroup', 'Organization', 'ManageInspect', 'Invite',
		'AccountEppn', 'AccountIntegration', 'Transaction', 'Inspect', 'AddGroupInspect', 'Agreement', 'Group',
		'AutoGroup', 'GroupSphost', 'SentNoticeMailLog');
	var $components = array('Session', 'IdpTranslator', 'Csrf', 'NoticeUtils');
	var $helpers = array('Javascript', 'Common');

	function beforeFilter() {
		$this->Csrf->validatePost = false;
		parent::beforeFilter();
	}

	function beforeRender() {
		parent::beforeRender();
		$this->layout = 'account';
	}

	/**
	 * 招待中画面の表示
	 */
	function inviting() {
		// 招待中グループの確認
		$invitingList = $this->Invite->findInviting($this->getAccountID());
		$this->set('invitingList', $invitingList);

		$groupIdList = array();
		foreach ($invitingList as $invite) {
			array_push($groupIdList, $invite['Invite']['groupid']);
		}
		$groupList = $this->Group->find('all', array('conditions' => array('id' => $groupIdList)));
		$this->set('groupList', $groupList);

		// 招待中画面を表示済みに
		$this->Session->write('DisplayedInviting', true);
	}

	/**
	 * Create account information
	 * @param string $id account id
	 */
	public function create($id = null) {
		$this->_createAndEdit($id);
	}

	/**
	 * Edit account information
	 * @param string $id account id
	 */
	function edit($id = null) {
		$organizations = $this->Organization->find('list',
				array('fields' => array('name', 'name'),
					'conditions' => array('federation' => 1)));
		$this->set('organizations', $organizations);
		$organizations = $this->Organization->find('list',
				array('fields' => array('name_en', 'name_en'),
					'conditions' => array('federation' => 1)));
		$this->set('organizations_en', $organizations);
		$this->set('defalutLanguage', $this->getLanguage('accountLanguage'));

		$this->_createAndEdit($id);
	}

	/**
	 * Common function for creating and editing account information
	 * @param string $id account id
	 */
	private function _createAndEdit($id =  null) {
		// セッション変数のアカウントが存在しなければエラーにする
		if ($this->getAccountID()) {
			$account = $this->Account->find('first', array('fields' => array('id'),
				'conditions' => array('id' => $this->getAccountID())));
			if (empty($account)) {
				$this->log('AccountsController :: _createAndEdit : Account(' . $this->getAccountID() . ') is not exist.');
				$this->redirect('/logout?msgid=errors.illegal.access.message');
			}
		}

		if (empty($this->data)) {
			$this->show();
			return;
		}

		switch ($this->getSubmitName()) {
			case 'save': // 保存
				$this->_save();
				break;
			case 'arrival': // メール到達確認
				$this->confirmArraivalMail($this->data['Account']['mail'], $this->data['Account']['language']);
				$this->Session->setFlash(__("message.account.send.arraival.mail", true), 'message');
				return;
			case 'cancel': // キャンセル (ユーザ登録済み)
				$this->redirect('/top/index');
				return;
			case 'no_account_cancel': // キャンセル (ユーザ未登録)
				$this->redirect('/logout');
				return;
			case 'delete': // 削除
				if ($this->_delete($id)) {
					$this->redirect('/logout?msgid=message.categories.account.del');
				}
				return;
			default:
				break;
		}
	}

	private function _delete($id) {
		$mygroups = $this->Mygroup->find('all', array('fields' => array('groupid', 'admin'),
			'conditions' => array('account_id' => $this->getAccountID())));
		$adminIdList = array();
		$memberIdList = array();
		$groupIdList = array();
		foreach ($mygroups as $mygroup) {
			if ($mygroup['Mygroup']['admin'] == 1 || $mygroup['Mygroup']['admin'] == 2) {
				array_push($adminIdList, $mygroup['Mygroup']['groupid']);
			} else {
				array_push($memberIdList, $mygroup['Mygroup']['groupid']);
			}
			array_push($groupIdList, $mygroup['Mygroup']['groupid']);
		}

		// グループ名を検索
		$groupNames = $this->Group->find('all', array('fields' => array('id', 'name', 'sp'),
			'conditions' => array('id' => $groupIdList)));

		$error_messages = array();
		// グループ(SPコネクタ含む)の管理者が１人だったら、削除できない
		$adminGroupList = $this->Mygroup->find('all', array(
			'fields' => array('count(*) as count', 'groupid'),
			'group' => array('groupid'),
			'conditions' => array(
				'admin' => array(1, 2),
				'groupid' => $adminIdList)));
		foreach ($adminGroupList as $group) {
			if ($group['0']['count'] <= 1) {
				$groupName = $this->getGroupName($groupNames, $group['Mygroup']['groupid']);
				// mygroupsにあってgroupsにない場合は無視する
				if (empty($groupName)) {
					continue;
				}
				if ($this->isGroup($groupNames, $group['Mygroup']['groupid'])) {
					$error_messages[] = __("accounts.group.no.admin", true).'('.h($groupName).')';
				} else {
					$error_messages[] = __("accounts.sp.no.admin", true).'('.h($groupName).')';
				}
			}
		}

		// グループのメンバーで退会に申請が必要な場合は削除できない(グループのみ)
		$memberGroups = $this->Group->find('all', array('fields' => array('id', 'inspectquit'),
			'conditions' => array(
				'inspectquit' => 1,
				'active' => 0,
				'id' => $memberIdList)));
		foreach ($memberGroups as $group) {
			$groupName = sprintf('(%s)', $this->getGroupName($groupNames, $group['Group']['id']));
			$error_messages[] = __("accounts.group.inspect.quit", true).h($groupName);
		}

		// validate結果
		if (count($error_messages) >= 1) {
			$this->Session->setFlash(implode('<br/>', $error_messages), 'message');
			$this->show();
			return false;
		}

		$this->Transaction->begin();
		if (!$this->NoticeUtils->deleteAccount($id)) {
			$this->Transaction->rollback();
			$this->Session->setFlash(__("common.error.occurred", true), 'message');
			return false;
		}
		$this->Transaction->commit();
		return true;
	}

	private function _isMatchJoinUrlAndAddress($joinUrl, $address) {
		$result = preg_match('!^invite/rgst/([^/]+)/(.+)$!', $joinUrl, $matches);
		if (!$result) {
			return false;
		}
		$group_key = $matches[1];
		$random = $matches[2];

		$invite = $this->Invite->find('first', array(
			'conditions' => array(
				'address' => $address,
				'random' => $random,
				'status' => 0,
			)
		));
		if (!$invite) {
			return false;
		}
		$group = $this->Group->find('first', array(
			'fields' => array('group_key'),
			'conditions' => array(
				'id' => $invite['Invite']['groupid'],
			)
		));
		return $group && $group['Group']['group_key'] == $group_key;
	}

	private function _save() {
		// メール到達性確認
		$idpEmail = false;
		$beforeAccount = $this->Account->find('first', array(
			'conditions' => array('id' => $this->getAccountID())));
		// *新規作成または、メールアドレスが変更されていたらメール到達性確認をする
		// *入力されたメールアドレスとmail属性属性が同じなら到達確認済みとする
		if (empty($beforeAccount)
			|| $beforeAccount['Account']['mail'] != $this->data['Account']['mail']
			|| !$beforeAccount['Account']['arrival']) {
			if (!empty($this->data['Account']['mail'])
				&& $this->getMail() === $this->data['Account']['mail']) {
				$this->data['Account']['arrival'] = true;
				$idpEmail = true;
			} else {
				$this->data['Account']['arrival'] = false;
			}
			if ($this->Session->read('joinUrl') != "" && !$this->data['Account']['arrival']) {
				// 入力したメールアドレスと招待メールで送ったメールアドレスが一致したら到達性確認OKとする
				$joinUrl = $this->Session->read('joinUrl');
				if ($this->_isMatchJoinUrlAndAddress($joinUrl, $this->data['Account']['mail'])) {
					$this->data['Account']['arrival'] = true;
				}
			}
		}

		if (!empty($beforeAccount)) {
			$this->data['Account']['id'] = $this->getAccountID();
		}

		$this->Transaction->begin();
		// 新規作成または上書き（保存時に入力値チェック）
		$resultAccount = $this->Account->save($this->data['Account']);
		if (!$resultAccount) {
			$this->Transaction->rollback();
			$this->Session->setFlash(__("validation.top.notice", true), 'message');
			return;
		}

		// 新規作成ならeppnと紐付ける
		if (empty($beforeAccount)) {
			$resultAccountEppn = $this->AccountEppn->createAccountEppn(
				$this->Account->getLastInsertID(), $this->getEPPN(), $this->getEPTID(), $this->getIdentityProvider());
			if (!$resultAccountEppn) {
				$this->Transaction->rollback();
				$this->cakeError('systemError');
				return;
			}
		}
		$this->Transaction->commit();

		// 表示言語設定
		$language = $this->data['Account']['language'];
		$this->changeLanguage($language);
		$this->setAccountName($language, $resultAccount);

		// アカウント作成前に入会URLにアクセスしていたら、アカウント作成後に入会画面に戻る
		if ($this->Session->read('joinUrl') != "") {
			$joinUrl = $this->Session->read('joinUrl');
			$this->Session->delete('joinUrl');

			if (!$this->data['Account']['arrival']) {
				$this->confirmArraivalMail($this->data['Account']['mail'], $language);
				$this->Session->setFlash(
					__("message.categories.account.save", true)."<br/>".
					__("message.account.send.arraival.mail", true), 'message');
			} else {
				$this->Session->setFlash(__("message.categories.account.save", true), 'message');
			}
			$this->redirect('/'.$joinUrl);
		} else {
			// メール到達性確認により、表示メッセージを変更する
			if ((empty($beforeAccount)
				|| $beforeAccount['Account']['mail'] != $this->data['Account']['mail'])
				&& $this->getMail() != $this->data['Account']['mail']) {
				$this->confirmArraivalMail($this->data['Account']['mail'], $language);
				$this->Session->setFlash(
					__("message.categories.account.save", true)."<br/>".
					__("message.account.send.arraival.mail", true), 'message');
			} else {
				$this->Session->setFlash(__("message.categories.account.save", true), 'message');
			}
			if ($idpEmail) {
				$this->setFlash(__("message.arrival.not.mail", true));
			}
			$nextPage = $this->Session->read('nextPage');
			if (empty($nextPage)) {
				$nextPage = '/top/index';
			} else {
				$this->Session->delete('nextPage');
			}
			$this->redirect($nextPage);
		}
	}

	/**
	 * アカウント情報を表示する
	 */
	private function show() {
		$accountId = $this->getAccountID();
		$account = $this->Account->find('first', array(
			'conditions' => array('id' => $accountId)));
		$this->data = $account;
		if (empty($this->data)) {
			// Apacheの環境変数にユーザ情報をあれば設定する
			$this->data = array(
				'Account' => array(
					'name' => $this->getDisplayNameJa(),
					'name_en' => $this->getDisplayNameEn(),
					'mail' => $this->getMail(),
					'university' => $this->getOrganizationNameJa(),
					'university_en' => $this->getOrganizationNameEn(),
					'new' => true,
				)
			);
		} elseif (!$this->Account->hasAccountName($accountId) && !$this->Account->hasMailaddress($accountId)) {
			// Apacheの環境変数にユーザ情報をあれば設定する
			$this->data['Account']['name'] = $this->getDisplayNameJa();
			$this->data['Account']['name_en'] = $this->getDisplayNameEn();
			$this->data['Account']['mail'] = $this->getMail();
		} else {
			$this->data['Account']['new'] = false;
			if (!$this->data['Account']['arrival']) {
				$this->setFlash(__("message.account.not.arraival.mail", true), 'message');
			}

			$accoutEppnList = $this->AccountEppn->find('all', array(
				'conditions' => array('account_id' => $this->getAccountID())));
			if (count($accoutEppnList) <= 1) {
				return;
			}

			for ($i = 0; $i < count($accoutEppnList); $i++) {
				$accoutEppnList[$i]['loginuser'] = false;
				if (!empty($accoutEppnList[$i]['AccountEppn']['eppn'])) {
					if ($accoutEppnList[$i]['AccountEppn']['eppn'] === $this->getEPPN()) {
						$accoutEppnList[$i]['loginuser'] = true;
						continue;
					}
				}
				if (!empty($accoutEppnList[$i]['AccountEppn']['eptid'])) {
					if ($accoutEppnList[$i]['AccountEppn']['eptid'] === $this->getEPTID()) {
						$accoutEppnList[$i]['loginuser'] = true;
						continue;
					}
				}
			}
			$this->IdpTranslator->appendIdp($accoutEppnList, $this->getLanguage());
			$this->set('accoutEppnList', $accoutEppnList);
		}
	}

	private function confirmArraivalMail($mailAddress, $lang) {
		$key = $this->getRandom(15);
		$Mail['url'] = Router::url('/accounts/arrive/', true).$key;

		$this->sendMail('arrival.confirmation', $mailAddress, $lang,
			'arraival_email_template', $Mail);
		$account = $this->Account->find('first', array('fields' => array('id'),
			'conditions' => array('id' => $this->getAccountID())));
		$account['Account']['arrival_key'] = $key;
		$account['Account']['arrival_time'] = date("Y-m-d H:i:s");
		$this->Account->save($account);
	}

	private function getGroupName($groupNames, $groupid) {
		foreach ($groupNames as $group) {
			if ($group['Group']['id'] == $groupid) {
				return $group['Group']['name'];
			}
		}
		return '';
	}

	private function isGroup($groupNames, $groupid) {
		foreach ($groupNames as $group) {
			if ($group['Group']['id'] != $groupid) {
				continue;
			}
			if ($group['Group']['sp'] == 0) {
				return true;
			}
		}
		return false;
	}

	/**
	 * 到達性確認メールのURLクリックで、到達性OKにする
	 * @param string $key
	 */
	function arrive($key) {
		$account = $this->Account->find('first', array(
			'fields' => array('id', 'arrival', 'arrival_key', 'arrival_time'),
			'conditions' => array('id' => $this->getAccountID())));
		if (empty($account)) {
			$this->redirect('/accounts/edit');
			return;
		}

		if ($account['Account']['arrival']) {
			$this->setFlash(__("message.account.already.confirmed.arraival", true), 'message');
			$this->redirect('/accounts/edit');
			return;
		}

		if ($account['Account']['arrival_key'] != $key) {
			$this->setFlash(__("message.account.invalid.arraival.url", true), 'message');
			$this->redirect('/accounts/edit');
			return;
		}

		// +168 hour = a week :
		$limitTime = date('Y-m-d H:i:s', strtotime('+168 hour ' . $account['Account']['arrival_time']));
		if (strtotime(date('Y-m-d H:i:s')) > strtotime(date($limitTime))) {
			$this->setFlash(__("message.account.expire.arraival.url", true), 'message');
			$this->redirect('/accounts/edit');
			return;
		}

		$account['Account']['arrival'] = true;
		$this->Account->save($account);
		$this->setFlash(__("message.account.confirm.arraival", true), 'message');
		$this->redirect('/accounts/edit');
	}

	/**
	 * アカウント統合 ステップ０
	 * 再認証を求める。
	 */
	function integration() {
		$this->Session->destroy();
		$this->redirect('/accounts/integration_step1');
	}

	// 統合紐付けキーの有効性を検証する
	function validateTerm($random_key) {
		$date = date('Y-m-d H:i:s', strtotime('-3 minute ' . date("Y-m-d H:i:s")));
		$count = $this->AccountIntegration->find('count', array(
				'conditions' => array('random_key' => $random_key, 'created > \''.$date.'\'')));
		if ($count <= 0) {
			$this->redirect('/logout?msgid=message.account.expire.arraival.url');
		}
	}

	// 認証の有効性を検証する
	function validateAuth() {
		if (isset($_SERVER["Shib-Authentication-Instant"])) {
			$authTime = strtotime($_SERVER["Shib-Authentication-Instant"]);
			if ($authTime + 180 >= time()) { // 認証後3分以内ならOK
				return true;
			}
		}
		$this->redirect('/logout?msgid=message.account.expire.arraival.url');
	}

	/**
	 * アカウント統合 ステップ１
	 * 紐付け先のアカウントでログインする。
	 * @param $key 紐付けキー
	 */
	function integration_step1() {
		switch ($this->getSubmitName()) {
			case '':
				$random_key = $this->getRandom(20);
				for($i = 0; $i <= 10; $i++) {
					$count = $this->AccountIntegration->find('count', array(
						'conditions' => array('random_key' => $random_key)));
					if ($i == 10) {
						$this->cakeError('systemError');
						return;
					}
					if ($count >= 1) {
						$random_key = $this->getRandom(20);
						continue;
					}
					break;
				}

				// 元々ログインしているユーザの情報を保存(紐付け元)
				$integration['AccountIntegration']['random_key'] = $random_key;
				$integration['AccountIntegration']['account_id'] = $this->getAccountID();
				$integration['AccountIntegration']['status'] = 'yet';
				$this->AccountIntegration->save($integration);
				$this->data['Account']['random_key'] = $random_key;
				$this->render('/accounts/integration_step1');
				break;
			case 'logout':
				$random_key = $this->data['Account']['random_key'];
				$this->validateTerm($random_key);
				$this->Session->destroy();

				if ($this->Common->isDevelopment()) {
					$this->set('step2_url', '/accounts/integration_step2/'.$random_key);
					$this->render('integration_step1_dev');
					return;
				}

//				$this->redirect('/Shibboleth.sso/Logout?forceAuthn=true&return='.'https://'.env('HTTP_HOST').'/map/accounts/integration_step2/'.$random_key);
				$urlparam = 'https://'.env('HTTP_HOST').'/Shibboleth.sso/DS?forceAuthn=true&target=/map/accounts/integration_step2/'.$random_key;
				$urlparam = urlencode($urlparam);
				$this->redirect('https://'.env('HTTP_HOST').'/Shibboleth.sso/Logout?return='.$urlparam);
				break;
			case 'cancel':
			default:
				$this->setFlash(__("message.accounts.unify.cancel", true), 'message');
				$this->redirect('/accounts/edit');
				break;
		}
	}

	/**
	 * アカウント統合 ステップ２
	 * 紐付け元のアカウントでログインする。
	 */
	function integration_step2($random_key = null) {
		switch ($this->getSubmitName()) {
			case '':
				$this->validateAuth();
				$this->validateTerm($random_key);
				// キーの存在確認
				$integration = $this->AccountIntegration->find('first', array(
					'conditions' => array('random_key' => $random_key)));
				if (empty($integration)) {
					$this->redirect('/logout?msgid=message.groups.invalid.url');
				}

				// ここではまだ、紐付け先のアカウントでログインしている状態
				$eppn = $this->getEPPN();
				$eptid = $this->getEPTID();
				if (empty($eppn) && empty($eptid)) {
					$this->redirect('/logout?msgid=message.accounts.unify.not.acquire');
				}

				// 紐付け先と紐付け元が同一の場合はエラー
				$copyAccountId = $this->getAccountID();
				if (!empty($copyAccountId)) {
					if ($copyAccountId === $integration['AccountIntegration']['account_id']) {
						$this->redirect('/logout?msgid=message.accounts.unify.same.accounts');
					}
				}

				// 紐付け先の情報を保存
				$integrationSave['AccountIntegration']['id'] = $integration['AccountIntegration']['id'];
				if ($eptid) {
					$integrationSave['AccountIntegration']['copy_eptid'] = $this->getEPTID();
				}
				if ($eppn) {
					$integrationSave['AccountIntegration']['copy_eppn'] = $this->getEPPN();
				}
				$integrationSave['AccountIntegration']['copy_entityid'] = $this->getEntityID();
				$integrationSave['AccountIntegration']['copy_account_id'] = $this->getAccountID();
				$this->AccountIntegration->save($integrationSave);

				$this->data['Account']['random_key'] = $random_key;
				$this->render('/accounts/integration_step2');
				return;
			case 'logout':
				$this->validateTerm($this->data['Account']['random_key']);
				$this->Session->destroy();

				if ($this->Common->isDevelopment()) {
					$this->set('step3_url', '/accounts/integration_step3/'.$this->data['Account']['random_key']);
					$this->render('integration_step2_dev');
					return;
				}

//				$this->redirect('/Shibboleth.sso/Logout?forceAuthn=true&return='.'https://'.env('HTTP_HOST').'/map/accounts/integration_step3/'.$this->data['Account']['random_key']);
				$urlparam = 'https://'.env('HTTP_HOST').'/Shibboleth.sso/DS?forceAuthn=true&target=/map/accounts/integration_step3/'.$this->data['Account']['random_key'];
				$urlparam = urlencode($urlparam);
				$this->redirect('https://'.env('HTTP_HOST').'/Shibboleth.sso/Logout?return='.$urlparam);
				break;
			case 'cancel':
			default:
				$this->redirect('/logout?msgid=message.accounts.unify.cancel');
				break;
		}
	}

	/**
	 * アカウント統合 ステップ３
	 * 紐付け元、紐付け先の情報が異なる項目がある場合は、両方のアカウント情報を表示する。
	 * 無い場合は、どちらかの情報を表示する。
	 * 統合実行でアカウントを紐付ける。
	 */
	function integration_step3($random_key = null) {

		// validate random_key
		switch ($this->getSubmitName()) {
			case '':
				$this->validateAuth();
				if (empty($random_key)) {
					$this->redirect('/logout?msgid=message.accounts.unify.url.illegal');
					return;
				}
				$this->validateTerm($random_key);
				break;
			case 'execute_new_account':
			case 'execute_same_account':
			case 'execute_same_account':
				if (empty($this->data['Account']['random_key'])) {
					$this->redirect('/logout?msgid=message.accounts.unify.url.illegal');
					return;
				}
				$this->validateTerm($this->data['Account']['random_key']);
				break;
		}

		switch ($this->getSubmitName()) {
			case '':
				$this->step3_show($random_key);
				break;
			case 'execute_new_account': // 新規アカウントとの統合
				// 新規アカウントとの統合処理を実施
				$this->step3_execute_new_account($this->data['Account']['random_key']);
				break;
			case 'execute_same_account': // 既存アカウントとの統合(氏名、メールアドレス等が同一値)
				// 既存アカウントとの統合処理を実施
				$this->step3_execute_same_account($this->data['Account']['random_key']);
				break;
			case 'execute_diff_account': // 既存アカウントとの統合(氏名、メールアドレス等が異なる値)
				// 既存アカウントとの統合処理を実施
				$this->step3_execute_diff_account($this->data['Account']['random_key'], $this->data['Account']);
				break;
			case 'cancel':
			default:
				$this->redirect('/logout?msgid=message.accounts.unify.cancel');
				break;
		}
	}

	private function step3_show($random_key) {
		$this->validateTerm($random_key);

		// ⑨初めにログインしていたアカウントと紐付け元のアカウントが異なる場合はエラーとなる
		$integration = $this->AccountIntegration->find('first', array(
			'conditions' => array('random_key' => $random_key)));
		if (empty($integration) ||
			$integration['AccountIntegration']['account_id'] != $this->getAccountID()) {
			$this->redirect('/logout?msgid=message.accounts.unify.different.accounts');
		}

		$account = $this->Account->find('first',
			array('conditions' => array('id' => $integration['AccountIntegration']['account_id'])));
		$accountEppns = $this->AccountEppn->find('all', array(
				'fields' => array('id', 'eppn', 'eptid'),
				'conditions' => array('account_id' => $integration['AccountIntegration']['account_id'])));
		// 紐付け元のアカウント情報を表示する
		$eppnList = array();
		$eptidList = array();
		foreach ($accountEppns as $eppn) {
			if (!empty($eppn['AccountEppn']['eppn'])) {
				array_push($eppnList, $eppn['AccountEppn']['eppn']);
			}
			if (!empty($eppn['AccountEppn']['eptid'])) {
				array_push($eptidList, $eppn['AccountEppn']['eptid']);
			}
		}
		if (!empty($integration['AccountIntegration']['copy_eppn'])) {
			array_push($eppnList, $integration['AccountIntegration']['copy_eppn']);
		}
		if (!empty($integration['AccountIntegration']['copy_eptid'])) {
			array_push($eptidList, $integration['AccountIntegration']['copy_eptid']);
		}
		$account['Account']['eppnList'] = $eppnList;
		$account['Account']['eptidList'] = $eptidList;
		$this->set('account', $account);
		$this->data['Account']['random_key'] = $random_key;

		// 新規アカウントと紐付け
		// 新規ユーザ(eppn, eptid)との統合なので、アカウント情報の選択はしない
		if (empty($integration['AccountIntegration']['copy_account_id'])) {
			$this->render('/accounts/integration_step3');
			return;
		}

		// 紐付け元、紐付け先のアカウント情報に違いがあるかを調べる
		$copyAccount = $this->Account->find('first',
			 array('conditions' => array('id' => $integration['AccountIntegration']['copy_account_id'])));
		$selectAccount = array();
		$diff = $this->diffAccount($account, $copyAccount, $selectAccount);
		// 紐付け元と紐付け先が同一の値(氏名、メールアドレス等)なので、アカウント情報の選択はしない
		if (!$diff) {
			$this->render('/accounts/integration_step3_same');
			return;
		}

		// 紐付け元と紐付け先のアカウント情報(氏名等)に違いがあるので、選択画面を表示する
		$this->set('selectAccount', $selectAccount);
		$this->set('account_id', $integration['AccountIntegration']['account_id']);
		$this->render('/accounts/integration_step3_diff');
	}

	/**
	 * 新規アカントとの統合処理を行う
	 * @param string $random_key
	 */
	private function step3_execute_new_account($random_key) {
		$this->validateTerm($random_key);

		$integration = $this->AccountIntegration->find('first', array(
			'conditions' => array('random_key' => $random_key)));
		$this->step3_execute_validate($integration);

		$this->Transaction->begin();
		$resultEppn = $this->AccountEppn->createAccountEppn(
			$integration['AccountIntegration']['account_id'], $integration['AccountIntegration']['copy_eppn'],
			$integration['AccountIntegration']['copy_eptid'], $integration['AccountIntegration']['copy_entityid']);

		// 統合済みステータスに変更
		$integrationChange['AccountIntegration']['id'] = $integration['AccountIntegration']['id'];
		$integrationChange['AccountIntegration']['status'] = 'complete';
		$resultIntegration = $this->AccountIntegration->save($integrationChange);

		if (!$resultEppn || !$resultIntegration) {
			$this->Transaction->rollback();
			$this->Session->setFlash(__("common.error.occurred", true), 'message');
			$this->redirect('/accounts/edit');
		}
		$this->Transaction->commit();
		$this->setFlash(__("message.accounts.unify.settled", true), 'message');
		$this->redirect('/accounts/edit');
	}

	/**
	 * アカント統合バリデーション
	 * @param string $random_key
	 */
	private function step3_execute_validate($integration) {
		if (!empty($integration['AccountIntegration']['copy_eppn'])) {
			$count = $this->AccountEppn->find('count', array('conditions' => array(
				'account_id' => $integration['AccountIntegration']['account_id'],
				'eppn' => $integration['AccountIntegration']['copy_eppn'])));
			if (!empty($count)) {
				$this->redirect('/logout?msgid=message.accounts.unify.already');
			}
		}
		if (!empty($integration['AccountIntegration']['copy_eptid'])) {
			$count = $this->AccountEppn->find('count', array('conditions' => array(
				'account_id' => $integration['AccountIntegration']['account_id'],
				'eptid' => $integration['AccountIntegration']['copy_eptid'])));
			if (!empty($count)) {
				$this->redirect('/logout?msgid=message.accounts.unify.already');
			}
		}
	}

	/**
	 * 既存アカント(同一値)との統合処理を行う
	 * @param string $random_key
	 */
	private function step3_execute_same_account($random_key) {
		$integration = $this->AccountIntegration->find('first', array(
			'conditions' => array('random_key' => $random_key)));
		$this->step3_execute_validate($integration);

		if (empty($integration['AccountIntegration']['copy_account_id'])) {
			$this->cakeError('systemError');
			return;
		}

		$this->Transaction->begin();
		if (!$this->step3_execute_account_tran(
			$integration['AccountIntegration']['id'],
			$integration['AccountIntegration']['account_id'],
			$integration['AccountIntegration']['copy_account_id'])) {
			$this->Transaction->rollback();
			$this->Session->setFlash(__("common.error.occurred", true), 'message');
			$this->redirect('/accounts/edit');
		}
		$this->Transaction->commit();
		$this->setFlash(__("message.accounts.unify.settled", true), 'message');
		$this->redirect('/accounts/edit');
	}

	/**
	 * 既存アカント(違う値)との統合処理を行う
	 * @param string $random_key
	 */
	private function step3_execute_diff_account($random_key, $data) {
		$integration = $this->AccountIntegration->find('first', array(
			'conditions' => array('random_key' => $random_key)));
		$this->step3_execute_validate($integration);

		if (empty($integration['AccountIntegration']['copy_account_id'])) {
			$this->cakeError('systemError');
			return;
		}

		$chgAccount = $this->Account->find('first', array(
			'fields' => array('id', 'name', 'name_en', 'mail', 'university', 'university_en',
				'language', 'introduction', 'introduction_en', 'arrival', 'arrival_key', 'arrival_time'),
			'conditions' => array('id' => $integration['AccountIntegration']['account_id'])));
		$copyAccount = $this->Account->find('first', array(
			'fields' => array('id', 'name', 'name_en', 'mail', 'university', 'university_en',
				'language', 'introduction', 'introduction_en', 'arrival', 'arrival_key', 'arrival_time'),
			'conditions' => array('id' => $integration['AccountIntegration']['copy_account_id'])));
		// 選択した値を設定
		if ($data['name'] === $integration['AccountIntegration']['copy_account_id']) {
			$chgAccount['Account']['name'] = $copyAccount['Account']['name'];
		}
		if ($data['name_en'] === $integration['AccountIntegration']['copy_account_id']) {
			$chgAccount['Account']['name_en'] = $copyAccount['Account']['name_en'];
		}
		if ($data['mail'] === $integration['AccountIntegration']['copy_account_id']) {
			$chgAccount['Account']['mail'] = $copyAccount['Account']['mail'];
			$chgAccount['Account']['arrival'] = $copyAccount['Account']['arrival'];
			$chgAccount['Account']['arrival_key'] = $copyAccount['Account']['arrival_key'];
			$chgAccount['Account']['arrival_time'] = $copyAccount['Account']['arrival_time'];
		}
		if ($data['university'] === $integration['AccountIntegration']['copy_account_id']) {
			$chgAccount['Account']['university'] = $copyAccount['Account']['university'];
		}
		if ($data['university_en'] === $integration['AccountIntegration']['copy_account_id']) {
			$chgAccount['Account']['university_en'] = $copyAccount['Account']['university_en'];
		}
		if ($data['language'] === $integration['AccountIntegration']['copy_account_id']) {
			$chgAccount['Account']['language'] = $copyAccount['Account']['language'];
		}
		if ($data['introduction'] === $integration['AccountIntegration']['copy_account_id']) {
			$chgAccount['Account']['introduction'] = $copyAccount['Account']['introduction'];
		}
		if ($data['introduction_en'] === $integration['AccountIntegration']['copy_account_id']) {
			$chgAccount['Account']['introduction_en'] = $copyAccount['Account']['introduction_en'];
		}

		$this->Transaction->begin();
		// 選択した値で更新
		if (!$this->Account->save($chgAccount)) {
			$this->Transaction->rollback();
			$this->Session->setFlash(__("common.error.occurred", true), 'message');
			$this->redirect('/accounts/edit');
		}

		// 紐付け先のaccount_idを紐付け元のaccount_idに変更
		if (!$this->step3_execute_account_tran(
			$integration['AccountIntegration']['id'],
			$integration['AccountIntegration']['account_id'],
			$integration['AccountIntegration']['copy_account_id'])) {
			$this->Transaction->rollback();
			$this->Session->setFlash(__("common.error.occurred", true), 'message');
			$this->redirect('/accounts/edit');
		}

		$this->Transaction->commit();
		$this->setFlash(__("message.accounts.unify.settled", true), 'message');
		$this->redirect('/accounts/edit');
	}

	private function step3_execute_account_tran($integration_id, $account_id, $copy_account_id) {
		// account_eppnsを変更
		$result = $this->AccountEppn->updateAll(
			array('account_id' => $account_id),
			array('account_id' => $copy_account_id));
		if (!$result)
			return false;

		// invitesを変更
		$result = $this->Invite->updateAll(
			array('account_id' => $account_id),
			array('account_id' => $copy_account_id));
		if (!$result)
			return false;

		// mygroups
		$result = $this->Mygroup->updateAll(
			array('account_id' => $account_id),
			array('account_id' => $copy_account_id));
		if (!$result)
			return false;

		// inspects
		$result = $this->Inspect->updateAll(
			array('account_id' => $account_id),
			array('account_id' => $copy_account_id));
		if (!$result)
			return false;

		// add_group_inspects
		$result = $this->AddGroupInspect->updateAll(
			array('account_id' => $account_id),
			array('account_id' => $copy_account_id));
		if (!$result)
			return false;

		// 統合先アカウントを削除
		$result = $this->Account->delete($copy_account_id);
		if (!$result)
			return false;

		// 統合済みステータスに変更
		$integrationChange['AccountIntegration']['id'] = $integration_id;
		$integrationChange['AccountIntegration']['status'] = 'complete';
		$result = $this->AccountIntegration->save($integrationChange);
		if (!$result)
			return false;

		return true;
	}

	/**
	 *
	 * @param array $account 紐付け元アカウント情報
	 * @param array $copyAccount 紐付け先アカウント情報
	 * @param array $newAccount 紐付け選択アカウント情報
	 */
	private function diffAccount($account, $copyAccount, &$newAccount) {
		$diff = false;
		$accountId = $account['Account']['id'];
		$copyAccountId = $copyAccount['Account']['id'];
		// name
		if (!empty($account['Account']['name']) && !empty($copyAccount['Account']['name'])) {
			if ($account['Account']['name'] != $copyAccount['Account']['name']) {
				$newAccount['Account']['name'] = array(
						$accountId => $account['Account']['name'],
						$copyAccountId => $copyAccount['Account']['name']);
				$diff = true;
			} else {
				$newAccount['Account']['name_value'] = $account['Account']['name'];
				$newAccount['Account']['name'] = $accountId;
			}
		} else if (!empty($account['Account']['name'])) {
			$newAccount['Account']['name_value'] = $account['Account']['name'];
			$newAccount['Account']['name'] = $accountId;
		} else {
			$newAccount['Account']['name_value'] = $copyAccount['Account']['name'];
			$newAccount['Account']['name'] = $copyAccountId;
		}

		// name_en
		if (!empty($account['Account']['name_en']) && !empty($copyAccount['Account']['name_en'])) {
			if ($account['Account']['name_en'] != $copyAccount['Account']['name_en']) {
				$newAccount['Account']['name_en'] = array(
					$accountId => $account['Account']['name_en'],
					$copyAccountId => $copyAccount['Account']['name_en']);
				$diff = true;
			} else {
				$newAccount['Account']['name_en_value'] = $account['Account']['name_en'];
				$newAccount['Account']['name_en'] = $accountId;
			}
		} else if (!empty($account['Account']['name_en'])) {
			$newAccount['Account']['name_en_value'] = $account['Account']['name_en'];
			$newAccount['Account']['name_en'] = $accountId;
		} else {
			$newAccount['Account']['name_en_value'] = $copyAccount['Account']['name_en'];
			$newAccount['Account']['name_en'] = $copyAccountId;
		}

		// mail
		if (!empty($account['Account']['mail']) && !empty($copyAccount['Account']['mail'])) {
			if ($account['Account']['mail'] != $copyAccount['Account']['mail']) {
				$newAccount['Account']['mail'] = array(
					$accountId => $account['Account']['mail'],
					$copyAccountId => $copyAccount['Account']['mail']);
				$diff = true;
			} else {
				$newAccount['Account']['mail_value'] = $account['Account']['mail'];
				$newAccount['Account']['mail'] = $accountId;
			}
		} else if (!empty($account['Account']['mail'])) {
			$newAccount['Account']['mail_value'] = $account['Account']['mail'];
			$newAccount['Account']['mail'] = $accountId;
		} else {
			$newAccount['Account']['mail_value'] = $copyAccount['Account']['mail'];
			$newAccount['Account']['mail'] = $copyAccountId;
		}

		// university
		if (!empty($account['Account']['university']) && !empty($copyAccount['Account']['university'])) {
			if ($account['Account']['university'] != $copyAccount['Account']['university']) {
				$newAccount['Account']['university'] = array(
					$accountId => $account['Account']['university'],
					$copyAccountId => $copyAccount['Account']['university']);
				$diff = true;
			} else {
				$newAccount['Account']['university_value'] = $account['Account']['university'];
				$newAccount['Account']['university'] = $accountId;
			}
		} else if (!empty($account['Account']['university'])) {
			$newAccount['Account']['university_value'] = $account['Account']['university'];
			$newAccount['Account']['university'] = $accountId;
		} else {
			$newAccount['Account']['university_value'] = $copyAccount['Account']['university'];
			$newAccount['Account']['university'] = $copyAccountId;
		}

		// university_en
		if (!empty($account['Account']['university_en']) && !empty($copyAccount['Account']['university_en'])) {
			if ($account['Account']['university_en'] != $copyAccount['Account']['university_en']) {
				$newAccount['Account']['university_en'] = array(
					$accountId => $account['Account']['university_en'],
					$copyAccountId => $copyAccount['Account']['university_en']);
				$diff = true;
			} else {
				$newAccount['Account']['university_en_value'] = $account['Account']['university_en'];
				$newAccount['Account']['university_en'] = $accountId;
			}
		} else if (!empty($account['Account']['university_en'])) {
			$newAccount['Account']['university_en_value'] = $account['Account']['university_en'];
			$newAccount['Account']['university_en'] = $accountId;
		} else {
			$newAccount['Account']['university_en_value'] = $copyAccount['Account']['university_en'];
			$newAccount['Account']['university_en'] = $copyAccountId;
		}

		// language
		if (isset($account['Account']['language']) && $account['Account']['language'] === 'ja') {
			$accountLang = __("common.japanese", true);
		} else {
			$accountLang = __("common.english", true);
		}
		if (isset($copyAccount['Account']['language']) && $copyAccount['Account']['language'] === 'ja') {
			$copyAccountLang = __("common.japanese", true);
		} else {
			$copyAccountLang = __("common.english", true);
		}
		if (!empty($account['Account']['language']) && !empty($copyAccount['Account']['language'])) {
			if ($account['Account']['language'] != $copyAccount['Account']['language']) {
				$newAccount['Account']['language'] = array(
				$accountId => $accountLang,
				$copyAccountId => $copyAccountLang);
				$diff = true;
			} else {
				$newAccount['Account']['language_value'] = $accountLang;
				$newAccount['Account']['language'] = $accountId;
			}
		} else if (!empty($account['Account']['language'])) {
			$newAccount['Account']['language_value'] = $accountLang;
			$newAccount['Account']['language'] = $accountId;
		} else {
			$newAccount['Account']['language_value'] = $copyAccountLang;
			$newAccount['Account']['language'] = $copyAccountId;
		}

		// introduction
		if (!empty($account['Account']['introduction']) && !empty($copyAccount['Account']['introduction'])) {
			if ($account['Account']['introduction'] != $copyAccount['Account']['introduction']) {
				$newAccount['Account']['introduction'] = array(
					$accountId => $account['Account']['introduction'],
					$copyAccountId => $copyAccount['Account']['introduction'],
					);
				$diff = true;
			} else {
				$newAccount['Account']['introduction_value'] = $account['Account']['introduction'];
				$newAccount['Account']['introduction'] = $accountId;
			}
		} else if (!empty($account['Account']['introduction'])) {
			$newAccount['Account']['introduction_value'] = $account['Account']['introduction'];
			$newAccount['Account']['introduction'] = $accountId;
		} else {
			$newAccount['Account']['introduction_value'] = $copyAccount['Account']['introduction'];
			$newAccount['Account']['introduction'] = $copyAccountId;
		}

		// introduction_en
		if (!empty($account['Account']['introduction_en']) && !empty($copyAccount['Account']['introduction_en'])) {
			if ($account['Account']['introduction_en'] != $copyAccount['Account']['introduction_en']) {
				$newAccount['Account']['introduction_en'] = array(
				$accountId => $account['Account']['introduction_en'],
				$copyAccountId => $copyAccount['Account']['introduction_en']);
				$diff = true;
			} else {
				$newAccount['Account']['introduction_en_value'] = $account['Account']['introduction_en'];
				$newAccount['Account']['introduction_en'] = $accountId;
			}
		} else if (!empty($account['Account']['introduction_en'])) {
			$newAccount['Account']['introduction_en_value'] = $account['Account']['introduction_en'];
			$newAccount['Account']['introduction_en'] = $accountId;
		} else {
			$newAccount['Account']['introduction_en_value'] = $copyAccount['Account']['introduction_en'];
			$newAccount['Account']['introduction_en'] = $copyAccountId;
		}
		return $diff;
	}

	/**
	 * アカウント統合を解除する。
	 * @param $accountEppnId
	 */
	function release($accountEppnId) {
		if (empty($accountEppnId)) {
			$this->redirect('/logout?msgid=errors.illegal.access.message');
			return;
		}

		$accountEppn = $this->AccountEppn->find('first', array(
			'fields' => array('id', 'account_id', 'eppn', 'eptid', 'entityid'),
			'conditions' => array('id' => $accountEppnId)));
		if (empty($accountEppn)) {
			$this->redirect('/logout?msgid=errors.illegal.access.message');
			return;
		}

		// 自分の$accountEppnIdかを確認
		$count = $this->AccountEppn->find('count', array('conditions' => array(
			'id' => $accountEppnId, 'account_id' => $this->getAccountID())));
		if ($count == 0) {
			$this->redirect('/logout?msgid=errors.illegal.access.message');
			return;
		}

		// アカウントと１つしか紐付いていない場合もエラー
		$count = $this->AccountEppn->find('count', array(
			'conditions' => array('account_id' => $accountEppn['AccountEppn']['account_id'])));
		if ($count <= 1) {
			$this->redirect('/logout?msgid=errors.illegal.access.message');
			return;
		}

		// ログインしているユーザのデータは解除できない
		if (!empty($accountEppn['AccountEppn']['eppn'])) {
			if ($accountEppn['AccountEppn']['eppn'] === $this->getEPPN()) {
				$this->Session->setFlash(__("message.accounts.unify.cannot.release", true), 'message');
				$this->redirect('/accounts/edit');
				return;
			}
		}
		if (!empty($accountEppn['AccountEppn']['eptid'])) {
			if ($accountEppn['AccountEppn']['eptid'] === $this->getEPTID()) {
				$this->Session->setFlash(__("message.accounts.unify.cannot.release", true), 'message');
				$this->redirect('/accounts/edit');
				return;
			}
		}

		// 機関グループに参加している場合は、退会処理をする
		$message = __("message.accounts.unify.done.release", true);
		$idpEntityid = $accountEppn['AccountEppn']['entityid'];
		$count = $this->AccountEppn->find('count', array('conditions' => array(
			'account_id' => $this->getAccountID(), 'entityid' => $idpEntityid)));
		if ($count == 1) {	// 他に同じentityidが無い
			$autoGroup = $this->AutoGroup->findIdPGroup($idpEntityid);
			if (!empty($autoGroup)) {
				$idpGroupId = $autoGroup['AutoGroup']['group_id'];
				$mygroup = $this->Mygroup->find('first', array('conditions' => array('groupid' => $idpGroupId,
								'account_id' => $this->getAccountID())));
				if (!empty($mygroup)) {	// 機関グループに入会している
					$delMygroup = true;
					if ($mygroup['Mygroup']['admin'] != 0) {	// 管理者
						// 他に管理者がいるか確認
						$count = $this->Mygroup->find('count', array('conditions' =>
							array('NOT' => array('account_id' => $this->getAccountID()),
								'groupid' => $idpGroupId, 'admin' => array(1, 2))));
						if ($count == 0) {
							// 他に管理者がいない
							$delMygroup = false;
							if ($mygroup['Mygroup']['admin'] == 2) {
								// 管理者+メンバー」なら「管理者only」に変更
								$mygroup['Mygroup']['admin'] = 1;
								unset($mygroup['Mygroup']['modified']);
								$this->Mygroup->save($mygroup);
							}
							$groupName = $this->Group->getGroupName($idpGroupId, 'ja');
							$message .= sprintf(__("message.accounts.unify.only.admin", true),
								$groupName);
						}
					}
					if ($delMygroup) {
						// プライベートサービスの管理者情報を削除
						$privateServiceIdList = $this->GroupSphost->getPrivateServiceId($idpGroupId);
						$this->Mygroup->deleteAll(array('groupid' => $privateServiceIdList,
										'account_id' => $this->getAccountID()));
						// Myグループ情報を削除
						$this->Mygroup->deleteAll(array('groupid' => $idpGroupId,
										'account_id' => $this->getAccountID()), false);
					}
				}
			}
		}

		// 解除する
		$this->AccountEppn->delete($accountEppnId);
		$this->Session->setFlash($message, 'message');
		$this->redirect('/accounts/edit');
	}

	/**
	 * アカウント削除通知メールに記載されたURLのアクセス確認
	 * @param $random
	 */
	function check($random) {
		// randomからlogを検索
		$rLog = $this->SentNoticeMailLog->find('first', array(
			'conditions' => array('random' => array($random))
		));
		$logAccountEppnId = $rLog['SentNoticeMailLog']['account_eppn_id'];
		if (empty($rLog)) {	// randomに対応するlogデータが無い
			// ePPNからlogを検索
			$eLogs = $this->SentNoticeMailLog->find('all', array(
				'conditions' => array('account_eppn_id in (select id from account_eppns where eppn = ?)' => array($this->getEPPN()))
			));
			if (!empty($eLogs)) {	// アクセスしたePPNに対応したlogデータがある
				$this->Session->setFlash(__("message.check.url.invalid", true), 'message');
			}
		}
		else {	// randomに対応するlogデータがある
			$accountEppn = $this->AccountEppn->find('first', array(
				'fields' => array('id', 'eppn', 'entityid'),
				'conditions' => array('id' => $logAccountEppnId)
			));
			$logAccountEppn = $accountEppn['AccountEppn']['eppn'];
			if ($logAccountEppn == $this->getEPPN()) {	// アクセス確認OK
				$this->log('AccountsController:check delete SentNoticeMailLog :' . print_r($rLog, true), "info");
				$result = $this->SentNoticeMailLog->deleteAll(array('account_eppn_id' => $rLog['SentNoticeMailLog']['account_eppn_id']), false);
				if (!$result) {
					$error = $this->getDataSource()->error;
					$this->log('AccountsController:check ERROR SentNoticeMailLog deleteAll Error '.$error);
					$this->cakeError('systemError');
					return;
				}
				$this->Session->setFlash(__("message.check.confirmed.access", true), 'message');
			}
			else {	// アクセス確認NG（ePPN違い）
				$rLog['SentNoticeMailLog']['accessed'] = date('Y-m-d H:i:s');
				$this->SentNoticeMailLog->save($rLog);
				$accoutEppnList = array($accountEppn);
				$this->IdpTranslator->appendIdp($accoutEppnList, $this->getLanguage());
				$this->Session->setFlash(sprintf(__("message.check.eppn.wrong", true), $accoutEppnList[0]['AccountEppn']['idp'], $logAccountEppn), 'message');
			}
		}
		$this->redirect('/accounts/edit');
	}
}
?>
