<?php
/**
 * Agreement Controller
 *
 * PHP versions 5
 *
 * @copyright     Copyright (c) 2011 - 2012, National Institute of Informatics,
 * @license       BSD License (http://creativecommons.org/licenses/BSD/)
 */
class AgreementsController extends AppController {
	var $name = 'Agreements';
	var $uses = array('Agreement', 'Transaction', 'Mygroup', 'Group', 'AddGroup', 'AgreementEntrustment');
	var $components = array('Session', 'TakerAgreement', 'Csrf');
	var $helpers = array('Javascript', 'Common');

	function beforeFilter() {
		$this->Csrf->validatePost = false;
		parent::beforeFilter();
	}

	function index() {
		$this->Session->delete('AgreementMessage');
		switch ($this->getSubmitName()) {
			case '':
				$this->show();
				break;
			case 'agree':
				$this->agree();
				break;
			case 'cancel':
				$this->cancel();
				break;
		}
	}

	private function show() {
		$data = $this->TakerAgreement->servData($this->getAccountID(), $this->getLanguage(), array(), array(), true, false);
		$accountid = $this->getAccountID();
		foreach ($data['Agreement'] as &$agree) {
			// SPコネクタに接続しているグループのいずれかにメンバーとして所属しているかの設定
			$agree['member'] = $this->AddGroup->isMemberConnectedSP($accountid, $agree['group_id']);
		}
$this->debugLog('AgreementsController::show $data = '.print_r($data, true));
		$this->data = $data;
		// 「管理者のみ」で上位グループ経由で接続しているSPコネクタがあるグループを取得
		$adminGroupIdList = $this->Mygroup->find('list', array(
				'fields' => array('groupid', 'groupid'),
				'conditions' => array('account_id' => $this->getAccountID(), 'admin ' => 1,
					'groupid in (select id from groups where sp = 0)',
					'groupid not in (select group_id from auto_groups where idp = \'true\')'),
		));
		$adminGroupId = array();
		foreach ($adminGroupIdList as $groupid) {
			// 「管理者のみ」として所属しているグループの上位グループを取得
			$groupIdList = $this->AddGroup->getTopIdList($groupid, false);
			foreach ($groupIdList as $gid) {
				$upSpcIdList = $this->AddGroup->getConnectedSPCIdList($gid, false, true);
				if (!empty($upSpcIdList)) {	// 接続しているSPコネクタがある
					$adminGroupId[] = $groupid;
					break;
				}
			}
		}
		$adminGroupId = array_unique($adminGroupId);
		$adminGroupList = array();
		foreach ($adminGroupId as $groupid) {
			$group['id'] = $groupid;
			$group['name'] = $this->Group->getGroupName($groupid, 'ja');
			$adminGroupList[] = $group;
		}
		$this->set('adminGroupList', $adminGroupList);

		// 委任グループを取得
		$agreeGroupId = $this->AgreementEntrustment->find('list', array(
			'fields' => array('group_id', 'group_id'),
			'conditions' => array('account_id' => $this->getAccountID())));
		$agreeGroupList = array();
		foreach ($agreeGroupId as $groupid) {
			$agreeGroupList[$groupid] = $this->Group->getGroupName($groupid, 'ja');
		}
		$this->set('agreeGroupList', $agreeGroupList);
	}

	private function agree() {
		$this->debugLog('agree : data = ' . print_r($this->data, true));
		$this->Transaction->begin();
		$result = $this->TakerAgreement->saveAgreed($this->data['Agreement'], $this->getAccountID());
		if (!$result) {
			$this->Transaction->rollback();
			$this->cakeError('systemError');
		}
		if (!empty($this->data['Agreement']['delegation_list'])) { // 委任登録があった
			// 委任を解除したグループを取得
			$undelegateGroupId = array();
			$beforeAgreeGroupId = explode(',' , $this->data['Agreement']['delegation_list']);
			if (empty($this->data['Agreement']['agree_group_list'])) {	// 全部の委任を削除
				$undelegateGroupId = $beforeAgreeGroupId;
			}
			else {	// 一部削除
				$undelegateGroupId = array_diff($beforeAgreeGroupId, $this->data['Agreement']['agree_group_list']);
			}
			if (!empty($undelegateGroupId)) {
				$this->log('agree : delete AgreementEntrustment group id(s) = ' . print_r($undelegateGroupId, true), "debug");
				// 委任を削除
				$result = $this->AgreementEntrustment->deleteAll(
					array('account_id' => $this->getAccountID(), 'group_id' => $undelegateGroupId), false);
				if (!$result) {
					$this->Transaction->rollback();
					$this->cakeError('systemError');
				}
			}
		}
		$this->Transaction->commit();

		$this->setFlash(__("message.agreement.use.attribute", true));
		$this->forward();
	}

	private function cancel() {
		$this->forward();
	}

	private function forward() {
		$url = $this->Session->read('previousAgreementUrl');
		$this->redirect('/mygroups/view');
	}
}
?>
