<?php
/**
 * Images Controller
 *
 * PHP versions 5
 *
 * @copyright     Copyright (c) 2016, SOUM Corporation.
 * @copyright     Copyright (c) 2011 - 2012, National Institute of Informatics,
 * @license       BSD License (http://creativecommons.org/licenses/BSD/)
 */
class ImagesController extends AppController {
	var $name = 'Images';
	var $uses = array('Group', 'IconImage');

	/**
	 *
	 * @param integer $size thumbnail size
	 * @param integer $id group ID
	 */
	public function group($size, $id = null) {
		if (is_null($id)) {
			exit;
		}
		$group = $this->getGroup($id);
		$iconType = 'image/'.$group['Group']['icon_type'];
		if ($group['Group']['icon'] == 0) {
			$filepath = $this->getUploadGroupImageFilepath($size, $id);
			$this->IconImage->replaceIconImage($id, $size, $filepath);
			$iconType = mime_content_type($filepath);
		} else {
			$filepath = $this->getGroupImageFilepath($size, $group['Group']['icon']);
		}
		if (file_exists($filepath)) {
			header('Content-Type: '.$iconType);
			header('Content-Length: '.filesize($filepath));
//			$this->setCacheHeader();

			ob_clean();
			flush();
			readfile($filepath);
		}
		exit;
	}

	/**
	 * 画像のキャッシュ時間を12時間に設定する
	 */
	private function setCacheHeader() {
		$date =  date('D M d Y H:i:s' , strtotime('+12 hour')).' GMT+0900';
		header('Cache-Control: max-age=43200');
		header('Expires: '.$date);
	}

	protected function getUploadGroupImageFilepath($size, $group_id) {
		return sprintf('%s%s/%s', _GROUP_IMAGE_THUMBNAIL_DIRECTORY, $size, $group_id);
	}

	protected function getGroupImageFilepath($size, $icon) {
		return IMAGES.sprintf('group/%d/icon_group_%02d.png', $size, $icon);
	}

	/**
	 *
	 * @param integer $size thumbnail size
	 * @param integer $id group ID
	 */
	public function sp_group($size, $id = null) {
		if (is_null($id)) {
			exit;
		}
		$group = $this->getGroup($id);
		$iconType = 'image/'.$group['Group']['spc_icon_type'];
		if (empty($group['Group']['spc_icon'])) {
			$filepath = $this->getUploadSpGroupImageFilepath($size, $id);
			$error = $this->IconImage->replaceIconImage($id, $size, $filepath);
			$iconType = mime_content_type($filepath);
		} else {
			$filepath = $this->getSpGroupImageFilepath($size, $group['Group']['spc_icon']);
		}
		if (file_exists($filepath)) {
			header('Content-Type: '.$iconType);
			header('Content-Length: '.filesize($filepath));
//			$this->setCacheHeader();

			ob_clean();
			flush();
			readfile($filepath);
		}
		exit;
	}

	private function getGroup($id) {
		return $this->Group->find('first', array(
			'conditions' => array('Group.id' => $id),
			'fields' => array('Group.id', 'Group.icon', 'Group.icon_type', 'Group.spc_icon', 'Group.spc_icon_type'),
		));
	}

	protected function getUploadSpGroupImageFilepath($size, $group_id) {
		return sprintf('%s%s/%s', _SP_GROUP_IMAGE_THUMBNAIL_DIRECTORY, $size, $group_id);
	}

	protected function getSpGroupImageFilepath($size, $icon) {
		return IMAGES.sprintf('sp/icon_%s.png', $icon);
	}

	/**
	 * アイコンイメージがあればtrueを返す
	 *
	 * @param $group_id グループID
	 * @param $size サムネイルサイズ
	 * @return boolean 該当アイコンイメージがあればtrue
	 */
	public function has_user_icon($size, $id) {
		return ($this->IconImage->existIconImage($id, $size));
	}

	/**
	 * ユーザアイコンを表示する
	 *
	 * @param $group_id グループID
	 * @param $size サムネイルサイズ
	 */
	public function user_icon($size, $id) {
		if (is_null($id)) {
			exit;
		}
		if ($this->Group->isGroup($id)) {
			$filepath = $this->getUploadGroupImageFilepath($size, $id);
		}
		else {
			$filepath = $this->getUploadSpGroupImageFilepath($size, $id);
		}
		$this->IconImage->replaceIconImage($id, $size, $filepath);
		if (file_exists($filepath)) {
			header('Content-Type: '.mime_content_type($filepath));
			header('Content-Length: '.filesize($filepath));

			ob_clean();
			flush();
			readfile($filepath);
		}
		exit;
	}
}
?>
