<?php
/**
 * Invite Controller
 * Invite members to a group.
 * PHP versions 5
 *
 * @copyright     Copyright (c) 2011 - 2012, National Institute of Informatics,
 * @license       BSD License (http://creativecommons.org/licenses/BSD/)
 */
App::import('Controller', 'Manages');

class InviteController extends ManagesController {
	var $name = 'Invite';
	var $uses = array('Invite', 'Mygroup', 'Group', 'Account', 'GroupSphost', 'SpHost', 'AccountEppn',
			'AddGroup', 'Transaction', 'Agreement', 'AgreementEntrustment');
	var $components = array('TakerAgreement', 'Url', 'ParameterReplacer', 'JoinOperator', 'AgreementOperator', 'Csrf');

	function beforeFilter() {
		$this->Csrf->validatePost = false;
		parent::beforeFilter();
	}

	/**
	 *
	 * @param integer $groupid group id
	 * @param string $random random number
	 */
	private function validateRegister($groupid, $random) {
		$Invite = $this->Invite->find('first', array(
			'fields' => array('id', 'account_id', 'groupid', 'random', 'address', 'term', 'status', 'created', 'invited_account_id'),
			'conditions' => array('random' => $random, 'groupid' => $groupid)));
		if (empty($Invite)) {
			return false;
		}
		if ($Invite['Invite']['status'] == 2) {
			$this->Session->setFlash(__("message.groups.invalid.url", true), 'message');
			return false;
		}

		// 有効期限チェック：１週間)
		if ($Invite['Invite']['term'] === "time") {
			$created = $Invite['Invite']['created'];
			// +168 hour = a week :
			$limitTime = date('Y-m-d H:i:s', strtotime('+168 hour ' . $created));
			if (strtotime(date('Y-m-d H:i:s')) > strtotime(date($limitTime))) {
				$this->Session->setFlash(__("message.groups.expires.url", true), 'message');
				return false;
			}
		}

		// 有効期限：ワンタイム
		if ($Invite['Invite']['term'] === "onetime") {
			// 既にURLにアクセス済みか検索
			$Invite['Invite']['message'] = "";
			if ($Invite['Invite']['status'] == 1) {
				$this->Session->setFlash(__('validation.used.url', true), 'message');
				return false;
			}
		}

		// 既にグループに登録済みか検索
		if ($this->Group->isIdPGroup($groupid)) {
			$conditions = array('groupid' => $groupid, 'account_id' => $this->getAccountID(),
				'admin' => array(1, 2));
		} else {
			$conditions = array('groupid' => $groupid, 'account_id' => $this->getAccountID());
		}
		$count = $this->Mygroup->find('count', array('conditions' => $conditions));
		if ($count > 0) {
			$this->Session->setFlash(__('group_lists.message.already.join', true), 'message');
			return false;
		}

		// 招待時に選択したアカウントと異なっていたらエラー
		if ($Invite['Invite']['invited_account_id']
			&& $Invite['Invite']['invited_account_id'] != $this->getAccountID()) {
			$this->Session->setFlash(__('errors.cannot.access.message', true), 'message');
			return false;
		}

		return true;
	}


	function rgst($group_key, $random) {
		if (empty($group_key)) {
			$this->cakeError('notFound', array());
			return;
		}

		// グループKeyからIDを検索
		$group = $this->Group->find('first', array('fields' => array('id'),
			'conditions' => array('group_key' => $group_key)));
		if (empty($group)) {
			$this->cakeError('notFound', array());
			return;
		}
		$id = $group['Group']['id'];
		$this->register($id, $random);
	}

	/**
	 *
	 * @param integer $groupid group id
	 * @param string $random random number
	 */
	function register($groupid, $random = null) {
		switch ($this->getSubmitName()) {
			case '': // 確認画面の表示
				$this->register_show($groupid, $random);
				break;
			case 'join': // 入会
				if (isset($this->data['Invite']['agreement']) && empty($this->data['Invite']['agreement'][0])) {
					$this->data['Invite']['error_agreement'] = true;
					$this->Session->setFlash(__("message.agreement.sending.attribute", true), 'message');
					$this->register_show($groupid, $this->data['Invite']['random']);
					return;
				}
				$this->join($groupid, $this->data['Invite']['random']);
				break;
			case 'cancel': // キャンセル
				$this->redirect('/mygroups/view');
				return;
		}
	}

	private function register_show($groupid, $random) {
		$group = $this->Group->find('first', array('conditions' => array('id' => $groupid)));
		if (empty($group)) {
			$this->cakeError('notFound', array());
			return;
		}

		$this->removeUnnecessaryLanguage($group);
		$this->set('group', $group);
		$this->set('groupid', $groupid);
		$this->set('random', $random);

		if (!$this->validateRegister($groupid, $random)) {
			return $this->render('error');
		}

		$idp = $this->Group->isIdPGroup($groupid);
		$spc = $this->Group->isSpConnector($groupid);
		$this->set('idp', $idp);

		if (!($idp || $spc)) {
			// 自グループIDと上位グループに接続しているSPコネクタのIDを取得
			$groupIdList = $this->AddGroup->getTopIdMyIdList($groupid, $onlyActive=false);
			$upperGroup = 0;
			if (count($groupIdList) >= 2) { // 上位グループあり
				$upperGroup = 1;
			}
			$this->set('upperGroup', $upperGroup);
			$spcIdList = $this->AddGroup->getConnectedSPCIdList($groupIdList, $onlyActive=false, true);

			// 同意している属性を設定
			if (empty($spcIdList)) {
				$agreedList = array('Agreement' => array());
			} else {
				$lang = $this->getLanguage();
				$agreedList = $this->TakerAgreement->servData($this->getAccountID(), $lang, $spcIdList, $groupIdList, true);
			}
			$this->set('agreedList', $agreedList);
		}

		$this->data['Invite']['id'] = $groupid;
		$this->render('register');
	}

	/**
	 * 入会する。
	 * @param integer $groupid group id
	 * @param string $random random number
	 */
	private function join($groupid, $random) {
		$this->Transaction->begin();
		$Group = $this->execute_join($groupid);
		if (!$Group) {
			$this->Transaction->rollback();
			$this->log("InviteController join error. groupid:".$groupid."  random:".$random, "error");
			$this->cakeError('systemError');
			return;
		}
		$this->Transaction->commit();

		$message = sprintf(__("invite.message.join.finish", true), $Group['Group']['name']);
		$this->Session->setFlash($message, 'message');


		if ($Group['Group']['sp'] == 0) {
			$this->redirect('/group_lists/detail/'.$Group['Group']['id']);
		} else {
			$this->redirect('/sp_groups/detail/'.$Group['Group']['id']);
		}
	}

	/**
	 * 入会処理をする。
	 * @param integer $groupid group id
	 * @param string $random random number
	 */
	private function execute_join($groupid) {
		$random = $this->data['Invite']['random'];

		// 直接手でURLにアクセスされた場合の対処
		if (!$this->validateRegister($groupid, $random)) {
			$this->redirect('/invite/register/'.$groupid.'/'.$random);
			return;
		}

		$Invite = $this->Invite->find('first', array(
			'conditions' => array('random' => $random, 'groupid' => $groupid)));

		// グループを検索
		$Group = $this->Group->find('first', array('conditions' => array('Group.id' => $groupid)));

		$idp = $this->Group->isIdPGroup($groupid);

		$admin = 0;
		if ($idp) {
			$Mygroup = $this->Mygroup->find('first', array(
				'conditions' => array(
					'account_id' => $this->getAccountID(),
					'groupid' => $groupid)));

			if ($Mygroup) {
				// 機関グループの管理者+メンバーとして設定する
				unset($Mygroup['Mygroup']['modified']);
				$admin = 2;
			} else {
				$Mygroup['Mygroup']['account_id'] = $this->getAccountID();
				$Mygroup['Mygroup']['groupid'] = $groupid;

				// 機関グループの管理者として設定する
				$admin = 1;
			}
		} else {
			// グループ参加状態にする
			$Mygroup['Mygroup']['account_id'] = $this->getAccountID();
			$Mygroup['Mygroup']['groupid'] = $groupid;

			// SPコネクタの管理者として設定する
			if ($Group['Group']['sp']) {
				$admin = 1;
			}
		}
		$Mygroup['Mygroup']['admin'] = $admin;
		$result = $this->Mygroup->save($Mygroup);
		if (!$result) {
			return false;
		}
		// 機関グループは接続するプライベートサービスも管理者にする
		if ($idp) {
			$privateServiceIdList = $this->GroupSphost->getPrivateServiceId($groupid);
			foreach ($privateServiceIdList as $psid) {
				$psMygroup = array(
					'groupid' => $psid,
					'account_id' => $this->getAccountID(),
					'admin' => 1);
				$this->Mygroup->create();
				$this->Mygroup->save(array('Mygroup' => $psMygroup));
			}
		}

		$account = $this->Account->find('first', array('fields' => array('mail'),
			'conditions' => array('id' => $this->getAccountID())));

		// ワイタイム
		if ($Invite['Invite']['term'] === "onetime") {
			$Invite['Invite']['status'] = 1;
			$Invite['Invite']['counter'] = 1;

			// メールの到達性が未確認のメールアドレスなら、確認済みにする
			 $this->Account->updateAll(
				array('arrival' => 1),
				array('arrival' => 0, 'mail' => $Invite['Invite']['address']));

			// メールアドレス（個人）で入会した人のアカウントID保存
			if (empty($Invite['Invite']['invited_account_id'])) {
				$Invite['Invite']['joined_account_id'] = $this->getAccountID();
			}
		} elseif ($Invite['Invite']['term'] === "time") {
			// 有効期限が期間の場合は、招待メールアドレスと
			// 入会者のメールアドレスが一致した場合に、入会済みにする
			if ($Invite['Invite']['status'] == 0 && $Invite['Invite']['address'] == $account['Account']['mail']) {
				$Invite['Invite']['status'] = 1;
			}

			// 入会URLで入会した件数に加算
			$Invite['Invite']['counter'] ++;
		}

//		// 登録アカウントのメールアドレスと招待のメールアドレスが同じなら仮アカウント削除
//		if ($Invite['Invite']['temp_account_id'] && $Invite['Invite']['address'] == $account['Account']['mail']) {
		// 仮アカウントを削除
		if ($Invite['Invite']['temp_account_id']) {
			$this->Account->delete($Invite['Invite']['temp_account_id']);
			$this->Mygroup->deleteByAccountId($Invite['Invite']['temp_account_id'], $groupid);
			$this->Agreement->deleteAll(array('account_id' => $Invite['Invite']['temp_account_id']));
			$this->AgreementEntrustment->deleteAll(array('account_id' => $Invite['Invite']['temp_account_id']));
			$Invite['Invite']['temp_account_id'] = null;
		}

		$result = $this->Invite->save($Invite);
		if (!$result)
			return false;

		$this->set('Invite', $Invite);

		$spinfoList = $this->getConnectedSpConnectorList($groupid);
		$this->set('spinfoList', $spinfoList);

		$group['Group']['id'] = $groupid;
		$this->set('group', $group);

		// 自グループID＋上位グループIDを検索
		$groupIdList = $this->AddGroup->getTopIdMyIdList($groupid, $onlyActive=false);
		foreach ($groupIdList as $gid) {
			// このグループに接続しているSPコネクタのIDを取得
			$spcIdList = $this->AddGroup->getConnectedSPCIdList($gid, $onlyActive=false);
			$attributeList = $this->TakerAgreement->getAttributeListForSave($spcIdList);

			// 利用属性を登録
			$result = $this->TakerAgreement->saveAttribute(
					$gid, $attributeList, $spcIdList, $this->getAccountID());
			if (!$result) {
				return false;
			}
		}

		// 委任登録
		if (isset($this->data['Invite']['entrustment'])) {
			$entrustment = false;
			if (!empty($this->data['Invite']['entrustment'][0])
				&& $this->data['Invite']['entrustment'][0] == true) {
				$entrustment = true;
			}
			$result = $this->JoinOperator->entrust(
				$groupid, $this->getAccountID(), $entrustment);
			if (!$result) {
				return false;
			}
		}
		return $Group;
	}

	/**
	 * 同一メールアドレスのデータも更新する
	 * @param string $address email address
	 * @param integer $groupid group id
	 */
	private function saveInviteSameAddress($address, $groupid) {
		$result = $this->Invite->updateAll(
			array('status' => 1),
			array('address' => $address, 'status' => 0, 'groupid' => $groupid));
		if (!$result) {
			return false;
		}
		return true;
	}

	/**
	 *
	 * @param string $groupid group id
	 */
	function join_finish($groupid) {
		$Group = $this->Group->find('first', array('fields' => array('id', 'name'),
			'conditions' => array('Group.id' => $groupid)));
		if (!$Group) {
			$this->cakeError('notFound', array());
			return;
		}

		$message = sprintf(__("invite.message.join.finish", true), $Group['Group']['name']);
		$this->Session->setFlash($message, 'message');
		$this->redirect('/group_lists/detail/'.$groupid);
	}
}
?>
