<?php
/**
 * Menu Controller
 * top menu, group menu, admin menu
 * PHP versions 5
 *
 * @copyright     Copyright (c) 2011 - 2012, National Institute of Informatics,
 * @license       BSD License (http://creativecommons.org/licenses/BSD/)
 */
App::import('Controller', 'Manages');

class MenusController extends ManagesController {
	var $name = 'Menus';
	var $uses = array('Group', 'SpHost', 'GroupSphost', 'SpAdministrator',
			'Mygroup', 'ManageInspect', 'AddGroupInspect', 'AgreementConnect', 'AutoGroup');
	var $components = array("Session") ;

	function beforeFilter() {
		$this->accessLog();
	}

	/**
	 * 左メニュー情報を取得する。
	 * @param $groupid group ID
	 * @param $view view name
	 * @param $group_key
	 */
	function left($groupid, $view = null, $group_key = null) {
		// グループ情報を取得
		$Group = $this->Group->find('first', array(
				'fields' => array('name', 'group_key', 'contactmember', 'contactadmin', 'active',
					'inspectjoin', 'inspectquit', 'openmember', 'sp', 'public'),
				'conditions' => array('Group.id' => $groupid)));
		if ($Group['Group']['sp']) {
			return $this->leftSp($groupid, $view, $Group);
		} else {
			return $this->leftGroup($groupid, $view, $Group, $group_key);
		}
	}

	/**
	 *
	 * @param integer $groupid
	 * @param integer $view
	 * @param array $Group
	 */
	private function leftGroup($groupid, $view = null, $Group, $group_key) {
		// 管理権限を持っているか確認
		$isAdmin = $this->isGroupAdmin($groupid);
		$inspectJoin = $Group['Group']['inspectjoin'];

		$isInspectQuit = false;
		if ($isAdmin) {
			if ($Group['Group']['inspectquit'] == 1) {
				$isInspectQuit = true;
			}
		}

		$joinCount = 0; // 入会申請数
		$quitCount = 0; // 退会申請数
		$inspectCount = 0;
		$stateCount = 0;
		$inspectSpCount = 0;
		$stateSpCount = 0;
		$history = false; // 審査履歴画面の表示の有無
		$spHistory = false; // SPコネクタ接続審査履歴画面の表示の有無
		$groupHistory = false; // グループ接続審査履歴画面の表示の有無
		$agreementCount = 0; // 利用属性変更の未同意数
		if ($isAdmin) {
			// 入会審査者数、退会審査者数を検索
			$options = array('fields' => array('kind', 'status', 'count(*) as count'),
						'conditions' => array('groupid' => $groupid),
						'group' => array("kind", "status"),
						);
			$inspects = $this->ManageInspect->find('all', $options);
			foreach($inspects as $inspect) {
				if ($inspect['ManageInspect']['kind'] == 0 && $inspect['ManageInspect']['status'] == 0)
					$joinCount = $inspect[0]['count'];
				elseif ($inspect['ManageInspect']['kind'] == 1 && $inspect['ManageInspect']['status'] == 0)
					$quitCount = $inspect[0]['count'];
				if ($inspect['ManageInspect']['status'] != 0 && !empty($inspect[0]['count'])) {
					$history = true;
				}
			}

			$inspectCount = $this->getInspectCount($groupid, 0);
			$stateCount = $this->getStateCount($groupid, 0);
			$inspectSpCount = $this->getInspectCount($groupid, 1);
			$stateSpCount = $this->getStateCount($groupid, 1);

			$addInspectCount = $this->AddGroupInspect->find('all', array(
				'fields' => array('top', 'count(*) as count'),
				'conditions' => array('to_id' => $groupid, 'status' => array(1, 2)),
				'group' => array('top')
				));
			foreach($addInspectCount as $inspect) {
				if ($inspect['AddGroupInspect']['top'] == 0 && !empty($inspect[0]['count'])) {
					$groupHistory = true;
				} else if ($inspect['AddGroupInspect']['top'] == 1 && !empty($inspect[0]['count'])) {
					$spHistory = true;
				}
			}

			// 利用属性変更の同意
			$agreementCount = $this->AgreementConnect->find('count', array('conditions' => array(
					'OR' => array('top_group_id' => $groupid, 'bottom_group_id' => $groupid),
					'status' => 0)));
		}

		// 入会しているかを確認
		$joined = false;
		$count = $this->Mygroup->find('count', array('conditions' => array(
				'account_id' => $this->getAccountID(),
				'groupid' => $groupid)));
		if ($count > 0) {
			$joined = true;
		}

		// 機関グループであるかを確認
		$isIdPGroup = $this->Group->isIdPGroup($groupid);

		$rd = false;
		$rdGroupid = $this->Session->read('rd_groupid');
		if (!empty($rdGroupid) && $rdGroupid == $groupid) {
			$rd = true;
		}

		return array(
			'sp' => false,
			'idp' => $isIdPGroup,
			'groupid' => $groupid,
			'group_key' => $Group['Group']['group_key'],
			'rd' => $rd,
			'view' => $view,
			'contactmember' => $Group['Group']['contactmember'],
			'contactadmin' => $Group['Group']['contactadmin'],
			'active' => $Group['Group']['active'],
			'public' => $Group['Group']['public'],
			'manager' => $isAdmin,
			'joinCount' => $joinCount,
			'quitCount' => $quitCount,
			'inspectCount' => $inspectCount,
			'stateCount' => $stateCount,
			'inspectSpCount' => $inspectSpCount,
			'stateSpCount' => $stateSpCount,
			'inspectJoin' => $inspectJoin,
			'isInspectQuit' => $isInspectQuit,
			'openMember' => $Group['Group']['openmember'],
			'joined' => $joined,
			'history' => $history,
			'spHistory' => $spHistory,
			'groupHistory' => $groupHistory,
			'agreementCount' => $agreementCount,
			'groupName' => $Group['Group']['name'],
		);
	}

	/**
	 *
	 * @param integer $groupid
	 * @param integer $view
	 * @param array $Group
	 */
	private function leftSp($groupid, $view = null, $Group) {
		// グループ管理者のみ、管理メニューを表示する
		$manage = false;
		$inspectCount = 0;
		$stateCount = 0;
		$openMember = false;
		$spAdmin = false; // SP管理者かどうか
		$joinCount = 0; // 入会申請数
		$history = false; // 審査履歴画面の表示の有無
		$groupHistory = false; // グループ接続審査履歴画面の表示の有無

		if ($groupid != null) {
			$manage = $this->isGroupAdmin($groupid);
			$openMember = $Group['Group']['openmember'];
		}

		if (!$manage) {
			// 管理者になるリンクの表示／非表示
			$groupSpHosts = $this->GroupSphost->find('all', array('fields' => array('sp_id'),
				'conditions' => array('group_id' => $groupid)));
			$idList = array();
			foreach($groupSpHosts as $host) {
				array_push($idList, $host['GroupSphost']['sp_id']);
			}

			// このグループのSPホスト名を取得
			if (count($idList) != 0) {
				$spHosts = $this->SpHost->find('all', array(
					'conditions' => array('id' => $idList, 'status' => 0)));

				// SP管理者(ログインユーザ)のentityidと、このSPコネクタの対象SPのentityidが一致
				$spAdministrators = $this->SpAdministrator->find('all', array('fields' => array('entityid'),
					'conditions' => array('OR' => array(
					'eppn' => $this->getEPPN(), 'eptid' => $this->getEPTID()))));
				foreach($spHosts as $host) {
					foreach($spAdministrators as $administrator) {
						if ($host['SpHost']['entityid'] === $administrator['SpAdministrator']['entityid']) {
							// このグループの管理権限を持っている
							$spAdmin = true;
							break;
						}
					}
				}
			}
		} else {
			// 入会審査者数、退会審査者数を検索
			$options = array('fields' => array('kind', 'status', 'count(*) as count'),
						'conditions' => array('groupid' => $groupid),
						'group' => array("kind", "status"),
						);
			$inspects = $this->ManageInspect->find('all', $options);
			foreach($inspects as $inspect) {
				if ($inspect['ManageInspect']['kind'] == 0 && $inspect['ManageInspect']['status'] == 0)
					$joinCount = $inspect[0]['count'];
				elseif ($inspect['ManageInspect']['kind'] == 0 && $inspect['ManageInspect']['status'] == 1)
					$history = true;
			}

			$stateCount = $this->getStateAllCount($groupid);

			$addInspectCount = $this->AddGroupInspect->find('all', array(
				'fields' => array('status', 'count(*) as count'),
				'conditions' => array('to_id' => $groupid),
				'group' => array('status')
				));
			foreach($addInspectCount as $inspect) {
				if ($inspect['AddGroupInspect']['status'] == 0 && !empty($inspect[0]['count'])) {
					$inspectCount = $inspectCount + $inspect[0]['count'];
				} elseif (($inspect['AddGroupInspect']['status'] == 1 || $inspect['AddGroupInspect']['status'] == 2)
					&& !empty($inspect[0]['count'])) {
					$groupHistory = true;
				}
			}
		}

		// 入会しているかを確認
		$joined = false;
		$count = $this->Mygroup->find('count', array('conditions' => array(
				'account_id' => $this->getAccountID(),
				'groupid' => $groupid)));
		if ($count > 0) {
			$joined = true;
		}

		return array(
			'sp' => true,
			'groupid' => $groupid,
			'group_key' => $Group['Group']['group_key'],
			'view' => $view,
			'manage' => $manage,
			'inspectCount' => $inspectCount,
			'stateCount' => $stateCount,
			'openMember' => $openMember,
			'spAdmin' => $spAdmin,
			'joinCount' => $joinCount,
			'joined' => $joined,
			'history' => $history,
			'groupHistory' => $groupHistory,
		);
	}

	/**
	 * トップメニュー情報を取得する。
	 * @param boolean $account true:アカウント作成済み false:アカウント未作成
	 */
	function top($account = 'true') {
		$count = $this->SpAdministrator->find('count', array('conditions' => array(
				'OR' => array('eppn' => $this->getEPPN(), 'eptid' => $this->getEPTID()))));
		$spmanager = false;
		if ($count > 0) {
			$spmanager = true;
		}
		$super = $this->isSuperManager();
		return array('spmanager' => $spmanager, 'account' => $account, 'super' => $super);
	}

	/**
	 * グループ名を取得します。
	 * @param $groupid
	 */
	function groupName($groupid) {
		$Group = $this->Group->find('first', array(
				'fields'=>array('name'),
				'conditions' => array('Group.id' => $groupid)));
		return array('name' => $Group['Group']['name']);
	}

	/**
	 *
	 * @param integer $groupid
	 * @param boolean $sp
	 */
	private function getInspectCount($groupid, $sp) {
		// グループ承認数を取得
//		$inspectCount = $this->AddGroupInspect->find('count', array(
//			'conditions' => array('to_id' => $groupid, 'status' => 0, 'top' => $sp),
//			));
		$inspectCount = $this->Group->find('count', array('conditions' => array(
			'active' => array(0, 1),
			'id in (select from_id from add_group_inspects where to_id = ? and status = 0 and top = ?)' => array($groupid, $sp))));
		return $inspectCount;
	}

	/**
	 *
	 * @param integer $groupid
	 * @param boolean $sp
	 */
	private function getStateCount($groupid, $sp) {
		// グループ申請数を取得
//		$stateCount = $this->AddGroupInspect->find('count', array(
//			'conditions' => array('from_id' => $groupid, 'status' => 0, 'top' => $sp),
//			));
		$stateCount = $this->Group->find('count', array('conditions' => array(
			'active' => array(0, 1),
			'id in (select to_id from add_group_inspects where from_id = ? and status = 0 and top = ?)' => array($groupid, $sp))));
		return $stateCount;
	}

	/**
	 *
	 * @param integer $groupid
	 */
	private function getStateAllCount($groupid) {
		// グループ申請数を取得
//		$stateCount = $this->AddGroupInspect->find('count', array('conditions' => array(
//			'from_id' => $groupid, 'status' => 0)));
		$stateCount = $this->Group->find('count', array('conditions' => array(
			'active' => array(0, 1),
			'id in (select to_id from add_group_inspects where from_id = ? and status = 0)' => array($groupid))));
		return $stateCount;
	}

	/**
	 * MYグループを表示する。
	 */
	function mygroups() {
		if (is_null($this->getAccountID())) {
			$mygroups = array();
		} else {
			// 自分が所属しているグループを検索
			$mygroups = $this->getMyGroupList();
		}
		return array('mygroups' => $mygroups);
	}

	/**
	 * TOP5と自グループ以外のMYグループを表示する。
	 * @param $groupid カレントのグループID
	 */
	function other_mygroups($groupid) {
		if (is_null($this->getAccountID())) {
			return array('mygroups' => array());
		}
		if (empty($groupid)) {
			return array('mygroups' => array());
		}

		// 自分が所属しているグループを検索
		$mygroups = $this->getMyGroupList($withSPC = false);

		$exclusionGroupIdList = $this->findTopFiveGroupIdName();
		foreach ($mygroups as $key => $group) {
			if (array_key_exists($group['Group']['id'], $exclusionGroupIdList)) {
				unset($mygroups[$key]);
			}
		}

		if (isset($this->params['actionName'])) {
			return array('mygroups' => $mygroups, 'actionName' => $this->params['actionName'],
				'selectedGroupId' => $groupid);
		}
		return array('mygroups' => $mygroups, 'selectedGroupId' => $groupid);
	}
}
?>
