<?php
/**
 * SP Connector Controller
 *
 * PHP versions 5
 *
 * @copyright     Copyright (c) 2016, SOUM Corporation.
 * @license       BSD License (http://creativecommons.org/licenses/BSD/)
 */
App::import('Controller', 'Manages');
App::import('Core', 'Xml');

class PrivateServicesController extends ManagesController {
	var $name = 'PrivateServices';
	var $uses = array('SpGroup', 'Mygroup', 'AddGroup', 'AutoGroup',
			'GroupSphost', 'Transaction', 'ProvideAttribute', 'Tag');
	var $helpers = array('Javascript', 'Attribute', 'Base64Url');

	function beforeFilter() {
		parent::beforeFilter();
		$groupid = $this->_getGroupIdFromURL();
		$this->_checkAccessible($groupid);
	}

	protected function _checkAccessible($groupid) {
		if ($this->params['action'] === 'create') {
			return;
		}
		if ($this->params['action'] === 'edit'
			&& is_null($groupid)) {
			return;
		}
		if (is_null($groupid) || !$this->Group->isPrivateService($groupid)) {
			$this->cakeError('notFound', array());
			return;
		}
		parent::_checkAccessible($groupid);
	}

	function create() {
		$this->set_candidate_clone();
		$this->set_candidate_groups();

		$candidateTagNames = array_values($this->Tag->getPublicTagNames($this->getAccountID()));
		$this->set('candidateTagNames', $candidateTagNames);
		switch ($this->getSubmitName()) {
			case '':
				$this->data['PrivateService']['sp_type'] = 0;

				break;
			case 'clone':
				if ($this->set_data($this->data['Clone']) === false) {
					$this->Session->setFlash(__("message.input.error", true), 'message');
					return false;
				}
				unset($this->data['PrivateService']['spc_icon']);
				break;
			case 'fill':
				$filename = $this->get_uploaded_filename($this->data['Metadata']);
				if ($filename === false) {
					$this->Session->setFlash(__("message.input.error", true), 'message');
					return false;
				}

				if (($services = $this->read_metadata($filename)) === false) {
					$this->Session->setFlash(__("message.input.error", true), 'message');
					return false;
				}
				// metadataの先頭一件のみ使用する
				$this->data['PrivateService'] = $services[0]['PrivateService'];
				$this->data['GroupSphost'] = $services[0]['GroupSphost'];
				break;
			case 'save':
				$this->Transaction->begin();
				$group_id = $this->save();
				if ($group_id === 'validate_error') {
					$this->Transaction->rollback();
					$this->Session->setFlash(__("message.input.error", true), 'message');
					return false;
				} elseif (!$group_id) {
					$this->Transaction->rollback();
					$this->Session->setFlash(__("common.error.occurred", true), 'message');
					return false;
				}
				$this->Transaction->commit();
				$this->redirect('/top/index#'.$group_id);
				break;
			case 'cancel':
			default:
				$this->redirect('/top/index');
				break;
		}
	}

	function edit($group_id=null) {
		$this->set_candidate_groups($group_id);

		$candidateTagNames = array_values($this->Tag->getPublicTagNames($this->getAccountID()));
		$this->set('candidateTagNames', $candidateTagNames);
		switch ($this->getSubmitName()) {
			case '':
				if ($this->isGroupAdmin($group_id) === false) {
					$this->Session->setFlash(__("message.input.error", true), 'message');
					return false;
				}
				if ($this->set_data($group_id) === false) {
					$this->Session->setFlash(__("message.input.error", true), 'message');
					return false;
				}

				break;
			case 'save':
				$this->Transaction->begin();
				$group_id = $this->update();
				if ($group_id === 'validate_error') {
					$this->Transaction->rollback();
					$this->Session->setFlash(__("message.input.error", true), 'message');
					return false;
				} elseif (!$group_id) {
					$this->Transaction->rollback();
					$this->Session->setFlash(__("common.error.occurred", true), 'message');
					return false;
				}
				$this->Transaction->commit();

				$this->redirect('/top/index#'.$group_id);
				break;
			case 'delete':
				$this->Transaction->begin();
				$result = $this->delete();
				if ($result === false) {
					$this->Transaction->rollback();
					$this->Session->setFlash(__("common.error.occurred", true), 'message');
					return false;
				}
				$this->Transaction->commit();
				$this->redirect('/top/index');
				break;
			case 'cancel':
			default:
				$this->redirect('/top/index#'.$group_id);
				break;
		}
	}

	private function set_data($group_id) {
		if ($this->isGroupAdmin($group_id) === false) {
			return false;
		}

		$private_service = $this->SpGroup->findById($group_id);
		$this->data['PrivateService'] = $private_service['SpGroup'];

		if (!isset($this->data['PrivateService']['sp_type'])) {
			$this->data['PrivateService']['sp_type'] = 0;
		}

		$join_group_id = $this->AddGroup->field('bottom_id', array('top_id' => $group_id));
		if ($join_group_id === false) {
			$join_group_id = null;
		} elseif ($this->AutoGroup->isIdPGroup($join_group_id)) {
			$this->data['JoinGroup']['idp_group_id'] = $join_group_id;
		} else {
			$this->data['JoinGroup']['group_id'] = $join_group_id;
		}

		$group_sphost = $this->GroupSphost->find('first',
			array('conditions' => array('group_id' => $group_id)));
		$this->data['GroupSphost'] = $group_sphost['GroupSphost'];

		$tagNames = array_values($this->Tag->getPublicTagNames(null, $group_id));
		$this->set('tagNames', $tagNames);

		return true;
	}

	private function set_candidate_clone() {
		$admin_groups = $this->Mygroup->find('list', array(
			'fields' => array('groupid'),
			'conditions' => array(
				'account_id' => $this->getAccountID(),
				'admin' => array(1, 2))));
		$groups = $this->SpGroup->find('list', array(
			'fields' => array('id', 'name'),
			'conditions' => array(
				'id' => $admin_groups,
				'sp' => 2)));
		$this->set('candidateClone', $groups);
	}

	private function set_candidate_groups($group_id=null) {
		$admin_groups = $this->Mygroup->find('list', array(
			'fields' => array('groupid'),
			'conditions' => array(
				'account_id' => $this->getAccountID(),
				'admin' => array(1, 2))));
                if ($this->isSuperManager() && $group_id != null) {
			$join_group = $this->AddGroup->find('list', array(
				'fields' => array('bottom_id'),
				'conditions' => array(
					'top_id' => $group_id)));
			$admin_groups[] = array_shift($join_group);
                }
		$idp_groups = $this->AutoGroup->find('list', array(
			'fields' => array('group_id'),
			'conditions' => array(
				'idp' => 'true',
				'group_id' => $admin_groups)));
		$admin_groups = array_diff($admin_groups, $idp_groups);
		$groups = $this->SpGroup->find('list', array(
			'fields' => array('id'),
			'conditions' => array(
				'id' => $admin_groups,
				'sp' => 0)));
		$this->set('candidateIdpGroups', $this->SpGroup->findGroupNameList($idp_groups));
		$this->set('candidateGroups', $this->SpGroup->findGroupNameList($groups));
	}

	private function update() {
		$group_id = $this->data['PrivateService']['id'];
		if ($this->SpGroup->field('id', array('id' => $group_id)) === false) {
			$this->Session->setFlash(__("message.input.error", true), 'message');
			return false;
		}
		if ($this->isGroupAdmin($group_id) === false) {
			$this->Session->setFlash(__("message.input.error", true), 'message');
			return false;
		}
		return $this->save($group_id);
	}

	private function save($group_id = null) {
		if (empty($this->data)) {
			return false;
		}
		$is_edit = false;
		if (!is_null($group_id)) {
			$is_edit = true;
		}

		if ($this->data['PrivateService']['sp_type'] == 0) {
			$this->data['PrivateService']['sp_type'] = null;
			$this->data['JoinGroup']['group_id'] = null;
			$this->data['JoinGroup']['idp_group_id'] = null;
		}
		if (empty($this->data['JoinGroup']['group_id'])) {
			$this->data['JoinGroup']['group_id'] = null;
		}
		if (empty($this->data['JoinGroup']['idp_group_id'])) {
			$this->data['JoinGroup']['idp_group_id'] = null;
		}

		$group_id = $this->save_private_service($is_edit);
		if ($group_id === false) {
			return 'validate_error';
		}

		$result = $this->save_group_sphosts($group_id, $is_edit);
		if (!$result) {
			return false;
		}

		if (!$is_edit) {
			$result = $this->save_provide_attribute($group_id);
			if (!$result) {
				return false;
			}
		}

		$result = $this->Tag->updatePublicTags($this->getAccountID(), $group_id, $this->data['tagNames']);
		if (!$result) {
			return false;
		}

		$result = $this->save_add_group_and_mygroup($group_id, $is_edit);
		if (!$result) {
			return false;
		}

		return $group_id;
	}

	private function get_join_group_id() {
		$join_group_id = null;
		if ($this->data['PrivateService']['sp_type'] == 1) {
			$join_group_id = $this->data['JoinGroup']['idp_group_id'];
		} elseif ($this->data['PrivateService']['sp_type'] == 2) {
			$join_group_id = $this->data['JoinGroup']['group_id'];
		}
		return $join_group_id;
	}

	private function validate_join_group_id($join_group_id) {
		if (!is_null($this->data['PrivateService']['sp_type'])
		    && is_null($join_group_id)) {
			$this->AutoGroup->validationErrors['group_id'] = __('validation.selected', true);
			return false;
		}
		if (is_null($this->data['PrivateService']['sp_type'])
		    && !is_null($join_group_id)) {
			return false;
		}
		$is_idp = $this->AutoGroup->isIdPGroup($join_group_id);
		if ($this->data['PrivateService']['sp_type'] == 1
		    && !$is_idp) {
			return false;
		}
		if ($this->data['PrivateService']['sp_type'] == 2
		    && $is_idp) {
			return false;
		}

		return true;
	}

	private function save_private_service($is_edit) {
		$join_group_id = $this->get_join_group_id();
		if ($this->validate_join_group_id($join_group_id) == false) {
			return false;
		}

		$field_list = null;
		if ($is_edit) {
			$field_list = array(
				'name',
				'introduction',
				'active',
				'sp_type',
				'spc_icon',
				'spc_icon_type',
				'spc_icon_logo',
				'icon',
				'icon_type',
				'file');
		} else {
			$this->data['PrivateService']['inspectjoin'] = 1;
			$this->data['PrivateService']['group_key'] = String::uuid();
			$this->data['PrivateService']['public'] = 0;
			$this->data['PrivateService']['openmember'] = 0;
			$this->data['PrivateService']['down_permission'] = 2;
			$this->data['PrivateService']['group_response'] = 1;
		}
		// icon設定に必要
		$this->data['PrivateService']['sp'] = 2;
		// PrivateService(実態はGroups)に登録
		$this->SpGroup->create();
		$result = $this->SpGroup->save(
			array('SpGroup' => $this->data['PrivateService']),
			array('fieldList' => $field_list));
		if (!$result) {
			return false;
		}

		if (!$is_edit) {
			$group_id = $this->SpGroup->getLastInsertID();
		} else {
			$group_id = $this->data['PrivateService']['id'];
		}
		return $group_id;
	}

	private function save_group_sphosts($group_id, $is_edit) {
		// GroupSphostsに登録
		$group_sp = array(
			'group_id' => $group_id,
			'service_name' => $this->data['PrivateService']['name'],
			'lead_url' => $this->data['GroupSphost']['lead_url']);
		$field_list = null;
		if ($is_edit) {
			$group_sphost_id = $this->GroupSphost->field('id', array('group_id' => $group_id));
			if ($group_sphost_id === false) {
				return false;
			}
			$group_sp['id'] = $group_sphost_id;
			$field_list = array(
				'group_id',
				'service_name',
				'lead_url');
		}
		$this->GroupSphost->create();
		return $this->GroupSphost->save(
			array('GroupSphost' => $group_sp),
			array('fieldList' => $field_list));
	}

	private function save_provide_attribute($group_id) {
		$attribute = array(
			'group_id' => $group_id,
			'ismemberof' => 0,
			'eptid' => 0,
			'name' => 0,
			'mail' => 0,
			'eppn' => 0,
			'organization' => 0,
			'idp' => 0,
			'language' => 0,
			'introduction' => 0);
		$this->ProvideAttribute->create();
		return $this->ProvideAttribute->save(array('ProvideAttribute' => $attribute));
	}

	private function save_add_group_and_mygroup($group_id, $is_edit) {
		// 接続するグループを取得する。
		$join_group_id = $this->get_join_group_id();

		if (!$is_edit) {
			// プライベートサービスを作成し、作成者を管理者として登録
			$mygroup = array(
				'groupid' => $group_id,
				'account_id' => $this->getAccountID(),
				'admin' => 1);
			$this->Mygroup->create();
			if (!$this->Mygroup->save(array('Mygroup' => $mygroup))) {
				return false;
			}
			// 作成時に接続するグループが存在しない場合、
			// 個人用プライベートサービスのため後の処理は不要
			if (is_null($join_group_id)) {
				return true;
			}
		}

		// 接続するグループの管理者であることを確認する。
		if (!is_null($join_group_id)) {
			$is_admin = $this->Mygroup->field('admin', array(
				'groupid' => $join_group_id,
				'account_id' => $this->getAccountID()));
			if ($is_admin == 0) {
				return false;
			}
		}

		// 現在接続しているグループを取得する。
		$addGroup = $this->AddGroup->find('first', array(
			'conditions' => array('top_id' => $group_id)
		));

		if (is_array($addGroup)) {
			if (is_null($join_group_id)) {
				// グループ用・機関グループ用プライベートサービスから個人用プライベートサービスへの変更
				if ($this->AddGroup->delete($addGroup['AddGroup']['id']) &&
					$this->Mygroup->deleteAll(array(
						'groupid' => $group_id,
						'NOT' => array('account_id' => $this->getAccountID())))) {
					return true;
				}
				return false;
			} else {
				if ($addGroup['AddGroup']['bottom_id'] == $join_group_id) {
					// 接続グループを変更しないため処理はなし
					return true;
				}

				// 接続グループを$join_group_idのグループに変更
				if (!$this->Mygroup->deleteAll(array(
					'groupid' => $group_id,
					'NOT' => array('account_id' => $this->getAccountID())))) {
					return false;
				}

				$admins = $this->Mygroup->find('list', array(
					'fields' => array('account_id'),
					'conditions' => array(
						'groupid' => $join_group_id,
						'admin' => array(1, 2),
						'NOT' => array('account_id' => $this->getAccountID()))));
				foreach ($admins as $admin) {
					$mygroup = array(
						'groupid' => $group_id,
						'account_id' => $admin,
						'admin' => 1);
					$this->Mygroup->create();
					if ($this->Mygroup->save(array('Mygroup' => $mygroup)) == false) {
						return false;
					}
				}

				if (!$this->AddGroup->delete($addGroup['AddGroup']['id'])) {
					return false;
				}

				$newAddGroup = array(
					'top_id' => $group_id,
					'bottom_id' => $join_group_id,
					'top' => 1,
					'bottom' => 0);
				$this->AddGroup->create();
				return $this->AddGroup->save(array('AddGroup' => $newAddGroup));
			}
		} else {
			if (is_null($join_group_id)) {
				// 個人用プライベートサービスのままのため処理はなし
				return true;
			} else {
				// グループ用・機関グループ用プライベートサービスの新規作成、
				// もしくは個人用プライベートサービスからグループ用・機関グループ用プライベートサービスへの変更
				$admins = $this->Mygroup->find('list', array(
					'fields' => array('account_id'),
					'conditions' => array(
						'groupid' => $join_group_id,
						'admin' => array(1, 2),
						'NOT' => array('account_id' => $this->getAccountID()))));
				foreach ($admins as $admin) {
					$mygroup = array(
						'groupid' => $group_id,
						'account_id' => $admin,
						'admin' => 1);
					$this->Mygroup->create();
					if ($this->Mygroup->save(array('Mygroup' => $mygroup)) == false) {
						return false;
					}
				}

				$addGroup = array(
					'top_id' => $group_id,
					'bottom_id' => $join_group_id,
					'top' => 1,
					'bottom' => 0);
				$this->AddGroup->create();
				return $this->AddGroup->save(array('AddGroup' => $addGroup));
			}
		}

	}

	private function delete() {
		$group_id = $this->data['PrivateService']['id'];
		if ($this->SpGroup->field('id', array('id' => $this->data['PrivateService']['id'])) === false) {
			$this->Session->setFlash(__("message.input.error", true), 'message');
			return false;
		}
		if ($this->isGroupAdmin($group_id) === false) {
			$this->Session->setFlash(__("message.input.error", true), 'message');
			return false;
		}

		$result = $this->SpGroup->delete($group_id);
		if ($result === false) {
			return false;
		}
		$result = $this->GroupSphost->deleteAll(array('group_id' => $group_id));
		if ($result === false) {
			return false;
		}
		$result = $this->ProvideAttribute->deleteAll(array('group_id' => $group_id));
		if ($result === false) {
			return false;
		}
		$result = $this->Mygroup->deleteAll(array('groupid' => $group_id));
		if ($result === false) {
			return false;
		}
		$result = $this->AddGroup->deleteAll(array('top_id' => $group_id));
		if ($result === false) {
			return false;
		}
		$result = $this->Tag->deleteAll(array('group_id' => $group_id));
		if ($result === false) {
			return false;
		}
		return true;
	}

	private function get_uploaded_filename($file_param) {
		if ((isset($file_param['error']) && $file_param['error'] == 0) ||
			(!empty($file_param['tmp_name']) && $file_param['tmp_name'] != 'none')) {
			if (is_uploaded_file($file_param['tmp_name'])) {
				return $file_param['tmp_name'];
			}
		}
		if ($_SERVER['CONTENT_LENGTH'] > $this->return_bytes(ini_get('upload_max_filesize'))) {
			$this->SpGroup->validationErrors['metadata'] = __('private_services.error.filesize', true);
		}
		return false;
	}

	private function return_bytes($val) {
		$val = trim($val);
		$last = strtolower($val[strlen($val)-1]);
		switch($last) {
			case 'g':
				$val *= 1024;
			case 'm':
				$val *= 1024;
			case 'k':
				$val *= 1024;
		}
		return $val;
	}

	private function metadataToXml($metaPath) {
		try{
			if (is_null($metaPath) ||empty($metaPath)) {
				return false;
			}
			$metaXml = '';
			if ($fp = fopen($metaPath, 'rb')) {
				while (!feof($fp)) {
					$metaXml = $metaXml.fgets($fp);
				}
				fclose($fp);
			}
			if (empty($metaXml)) {
				return false;
			}
			$xml = new Xml($metaXml);
			$xmlAsArray = $xml->toArray();
			$xmlValidUntil = $xmlAsArray['EntitiesDescriptor']['validUntil'];
			return array('xmlAsArray' => $xmlAsArray, 'validUntil' => $xmlValidUntil);
		} catch (Exception $e) {
			$this->log('ERROR occurred. '.$e->getMessage());
			return false;
		}
	}

	private function format($cert) {
		$cert = str_replace(array(" ", "\r\n","\r","\n"), '', $cert);
		return $cert;
	}

	private function get_service($entity) {
		if (!is_array($entity)) {
			return false;
		}
		$service = array();

		$entityID = $entity['entityID'];
		preg_match("/^.+:\/\/([^\/|:]+)/", $entityID, $match);

		$names = array();
		// DisplayName > ServiceName > OrganizationDisplayName
		if (isset($entity['SPSSODescriptor']['Extensions']['UIInfo']['DisplayName'])) {
			$names = $entity['SPSSODescriptor']['Extensions']['UIInfo']['DisplayName'];
		} else if (isset($entity['SPSSODescriptor']['AttributeConsumingService']['ServiceName'])) {
			$names = $entity['SPSSODescriptor']['AttributeConsumingService']['ServiceName'];
		} else if (isset($entity['Organization']['OrganizationDisplayName'])) {
			$names = $entity['Organization']['OrganizationDisplayName'];
		}

		$service['ja'] = '';
		$service['en'] = '';
		$service['entityID'] = $entityID;
//		$service['name'] = $match[1];
		$hostName = "";
		if (isset($match[1]) ) {
			$hostName = $match[1];
		} else {
			$hostName = $entityID;
		}
		$service['name'] = $hostName;
		if (isset($names[0])) {
			foreach($names as $name) {
				if (isset($name['xml:lang']) && $name['xml:lang'] == 'ja') {
					$service['ja'] = $name['value'];
				} else if (isset($name['xml:lang']) && $name['xml:lang'] == 'en') {
					$service['en'] = $name['value'];
				}
			}
		} else {
			if (isset($names['xml:lang']) && $names['xml:lang'] == 'ja') {
				$service['ja'] = $names['value'];
			} else if (isset($names['xml:lang']) && $names['xml:lang'] == 'en') {
				$service['en'] = $names['value'];
			}
		}
		if ($service['ja'] == '') {
			$service['ja'] = $service['en'];
		}

		// set X509Certificate
		if (!isset($entity['SPSSODescriptor']['KeyDescriptor'][1])) {
			$service['certificate'] = $this->format($entity['SPSSODescriptor']['KeyDescriptor']['KeyInfo']['X509Data']['X509Certificate']);
		} else {
			$service['certificate'] = $this->format($entity['SPSSODescriptor']['KeyDescriptor'][0]['KeyInfo']['X509Data']['X509Certificate']);
			$service['certificate2'] = $this->format($entity['SPSSODescriptor']['KeyDescriptor'][1]['KeyInfo']['X509Data']['X509Certificate']);
		}

		// 説明を設定
		if (isset($entity['SPSSODescriptor']['Extensions']['UIInfo']['Description']['value'])) {
			$description = $entity['SPSSODescriptor']['Extensions']['UIInfo']['Description']['value'];
			if (!empty($description)) {
				$service['description'] = trim($description);
			}
		}
		if (isset($entity['SPSSODescriptor']['AttributeConsumingService']['ServiceDescription']['value'])) {
			$serviceDescription = $entity['SPSSODescriptor']['AttributeConsumingService']['ServiceDescription']['value'];
			if (!empty($serviceDescription)) {
				if (!isset($service['description'])) {
					$service['description'] = trim($serviceDescription);
				}
			}
		}
		return $service;
	}

	private function read_metadata($filename = null) {
		$metadata = $this->metadataToXml($filename);
		if ($metadata === false) {
			return false;
		}

		$validUntil = $metadata['validUntil'];
		$xmlAsArray = $metadata['xmlAsArray'];
		if (!isset($xmlAsArray['EntitiesDescriptor'])
		    && isset($xmlAsArray['EntityDescriptor'])) {
			$xmlAsArray = array('EntitiesDescriptor' => $xmlAsArray);
		}
		$serviceList = array();
		if (!empty($xmlAsArray['EntitiesDescriptor']['EntityDescriptor']['SPSSODescriptor'])) {
			$service = $this->get_service($xmlAsArray['EntitiesDescriptor']['EntityDescriptor']);
			if ($service !== false) {
				$serviceList[] = $service;
			}
		} else {
			foreach ($xmlAsArray['EntitiesDescriptor']['EntityDescriptor'] as $entity) {
				if (!empty($entity['SPSSODescriptor'])) {
					$service = $this->get_service($entity);
					if ($service === false) {
						continue;
					}
					$serviceList[] = $service;
				}
			}
		}

		$services = array();
		foreach ($serviceList as $service) {
			$private_service = array();
			$group_sphost = array();

			$private_service['name'] = $service['ja'];
			$group_sphost['service_name'] = $service['ja'];
			$private_service['sp_type'] = 0;

			// entiryIdからURLを取得
			$pattern = '/((?:https?|http):\/\/[-_.!~*\'()a-zA-Z0-9;?:@&=+$,%#]+)/';
			preg_match($pattern, $service['entityID'], $matches);
			if (isset($matches[0])) {
				$group_sphost['lead_url'] = $matches[0];
			}

			if (isset($service['description'])) {
				$private_service['introduction'] = substr($service['description'], 0, 256);
			}

			$services[] = array(
				'PrivateService' => $private_service,
				'GroupSphost' => $group_sphost);
		}

		return $services;
	}
}
?>
