<?php
/**
 * SP Connector List Controller
 *
 * PHP versions 5
 *
 * @copyright     Copyright (c) 2011 - 2012, National Institute of Informatics,
 * @license       BSD License (http://creativecommons.org/licenses/BSD/)
 */
App::import('Controller', 'GroupLists');

class SpGroupListsController extends GroupListsController {
	var $name = 'SpGroupLists';

	function beforeFilter() {
		parent::beforeFilter();
		$groupid = $this->_getGroupIdFromURL();
		$this->_checkAccessible($groupid);
	}

	protected function _checkAccessible($groupid) {
		if (is_null($groupid) || !$this->Group->isSpConnector($groupid)) {
			$this->cakeError('notFound', array());
			return;
		}
		parent::_checkAccessible($groupid);
	}

	/**
	 * 管理者一覧を表示する。
	 * @param $id
	 * @param $page
	 */
	function administrators($id, $page = 'all') {
		$this->memberlist('all', $id, 'SpGroupList');
		$this->render('members');
	}

	/**
	 *
	 * @param integer $id group ID
	 */
	function join_confirm_admin($id) {
		$validate = $this->validate_confirm_admin($id);
		if (!$validate) {
			$this->cakeError('cannotAccess', array('url' => $this->here));
			return;
		}

		$this->join_confirm($id);
		$this->render('/group_lists/join_confirm');
	}

	/**
	 *
	 * @param integer $id group ID
	 */
	function quit_admin($id) {
		$this->data['SpGroupList']['id'] = $id;
		$this->quit($id);
		$this->render('quit');
	}

	/**
	 *
	 * @param integer $groupid
	 */
	private function validate_confirm_admin($groupid) {
		if ($this->isGroupAdmin($groupid)) {
			return true;
		}

		$groupSpHosts = $this->GroupSphost->find('all', array('fields' => array('sp_id'),
				'conditions' => array('group_id' => $groupid)));
		$idList = array();
		foreach($groupSpHosts as $host) {
			array_push($idList, $host['GroupSphost']['sp_id']);
		}

		// このグループのSPホスト名を取得
		if (count($idList) != 0) {
			$spHosts = $this->SpHost->find('all', array('conditions' => array('id' => $idList, 'status' => 0)));

			// SP管理者(ログインユーザ)の管理しているホストと、このグループのSPホストが一致
			$spAdministrators = $this->SpAdministrator->find('all', array('fields' => array('entityid'),
					'conditions' => array('OR' => array(
						'eppn' => $this->getEPPN(), 'eptid' => $this->getEPTID()))));
			foreach($spHosts as $host) {
				foreach($spAdministrators as $administrator) {
					if ($host['SpHost']['entityid'] === $administrator['SpAdministrator']['entityid']) {
						// このグループの管理権限を持っている
						return true;
					}
				}
			}
		}

		return false;
	}
}
?>
