<?php
/**
 * Tag Controller
 *
 * PHP versions 5
 *
 * @copyright     Copyright (c) 2016, SOUM Corporation.
 * @license       BSD License (http://creativecommons.org/licenses/BSD/)
 */
App::import('Controller');

class TagsController extends AppController {
	var $name = 'Tags';
	var $uses = array('Tag', 'GroupSphost', 'Transaction');
	var $helpers = array('Base64Url');

	function beforeFilter() {
		parent::beforeFilter();
		$groupid = $this->_getGroupIdFromURL();
		$this->_checkAccessible($groupid);
	}

	protected function _checkAccessible($groupid) {
		if (is_null($groupid)
			|| !($this->Group->isSpConnector($groupid) || $this->Group->isPrivateService($groupid))) {
			$this->cakeError('notFound', array());
			return;
		}
		parent::_checkAccessible($groupid);
	}

	/**
	 * SPコネクタのタグの編集を行う。
	 * @param $groupid
	 */
	function edit($groupid) {
		// Top画面と同様に先にマッチするサービス名を表示する
		$groupSphost = $this->GroupSphost->find('first',
			array('conditions' => array('group_id' => $groupid)));
		if (!$groupSphost) {
			$this->cakeError('notFound', array());
			return;
		}
		$accountid = $this->getAccountID();
		switch ($this->getSubmitName()) {
			case '':
				$this->set('serviceName', $this->GroupSphost->getServiceName($this->getLanguage(), $groupSphost));
				$this->set('groupId', $groupid);
				$tagNames = array_values($this->Tag->getUserTagNames($accountid, $groupid));
				$this->set('tagNames', $tagNames);
				$candidateTagNames = array_values($this->Tag->getUserTagNames($accountid));
				$this->set('candidateTagNames', $candidateTagNames);
				break;
			case 'save':
				$this->Transaction->begin();
				$result = $this->Tag->updateUserTags($accountid, $groupid, $this->data['tagNames']);
				if (!$result) {
					$this->Transaction->rollback();
					$this->Session->setFlash(__("common.error.occurred", true), 'message');
					$this->redirect('/mygroups/view');
					return;
				}
				$this->Transaction->commit();

				$this->redirect('/#'.$groupid);
				break;
			case 'cancel':
			default:
				$this->redirect('/#'.$groupid);
				break;
		}
	}
}
?>
