<?php
/**
 * AddGroup Model
 *
 * PHP versions 5
 *
 * @copyright     Copyright (c) 2011 - 2012, National Institute of Informatics,
 * @license       BSD License (http://creativecommons.org/licenses/BSD/)
 */
App::import('Model','Group');
App::import('Model','Mygroup');
App::import('Model','AutoGroup');

class AddGroup extends AppModel {
	var $name = 'AddGroup';

	/**
	 * 自グループが接続しているSPコネクタID一覧を取得する。
	 * @param $groupid グループID
	 * @param $onlyAcitve 稼働中のSPコネクタ or 全SPコネクタ
	 */
	function getConnectedSPCIdList($groupid, $onlyActive=true, $notPrivate=false) {
		$spcIdList = $this->find('list', array(
			'fields' => array('top_id', 'top_id'),
			'conditions' => array('bottom_id' => $groupid, 'top' => 1)));
		if ($onlyActive) {
			$Group = new Group;
			$spcIdList = $Group->find('list', array(
				'fields' => array('id', 'id'),
				'conditions' => array('id' => $spcIdList, 'active' => 0)));
		}
		if ($notPrivate) {
			$Group = new Group;
			$spcIdList = $Group->find('list', array(
				'fields' => array('id', 'id'),
				'conditions' => array('id' => $spcIdList, 'sp' => 1)));
		}
		return $spcIdList;
	}

	/**
	 * 自グループIDと上位グループID一覧を取得する。
	 * @param $groupid グループID
	 * @param $onlyAcitve 稼働中のグループ or 全グループ
	 */
	function getTopIdMyIdList($groupid, $onlyActive=true) {
		$upGroupIdList = $this->getTopIdList($groupid, $onlyActive);
		array_push($upGroupIdList, $groupid);
		return $upGroupIdList;
	}

	/**
	 * 指定されたグループIDに接続している上位グループID一覧を取得する
	 * @param $groupid グループID
	 * @param $onlyAcitve 稼働中のグループ or 全グループ
	 */
	function getTopIdList($groupid, $onlyActive=true) {
		$upGroupIdList = $this->find('list', array(
			'fields' => array('top_id', 'top_id'),
			'conditions' => array('bottom_id' => $groupid, 'top' => 0)));
		if ($onlyActive) {
			$Group = new Group;
			$upGroupIdList = $Group->find('list', array(
				'fields' => array('id', 'id'),
				'conditions' => array('id' => $upGroupIdList, 'active' => 0)));
		}
		return $upGroupIdList;
	}

	/**
	 * 自グループIDと下位グループID一覧を取得する。
	 * @param $groupid グループID
	 * @param $onlyAcitve 稼働中のグループ or 全グループ
	 */
	function getBottomIdMyIdList($groupid, $onlyActive=true) {
		$groupIdList = $this->getBottomIdList($groupid, $onlyActive);
		array_push($groupIdList, $groupid);
		return $groupIdList;
	}

	/**
	 * 指定されたグループIDに接続している下位グループID一覧を取得する。
	 * @param $groupid グループID
	 * @param $onlyAcitve 稼働中のグループ or 全グループ
	 */
	function getBottomIdList($groupid, $onlyActive=true) {
		$groupIdList = $this->find('list', array(
			'fields' => array('bottom_id', 'bottom_id'),
			'conditions' => array('top_id' => $groupid, 'top' => 0)));
		if ($onlyActive) {
			$Group = new Group;
			$groupIdList = $Group->find('list', array(
				'fields' => array('id', 'id'),
				'conditions' => array('id' => $groupIdList, 'active' => 0)));
		}
		return $groupIdList;
	}

	/**
	 * 渡されたグループIDから指定されたSPコネクタに直接接続しているグループのみに絞り込む。
	 */
	function getGroupIdListConnectedSpc($spcid, $groupIdList) {
		$list = $this->find('list', array(
			'fields' => array('bottom_id', 'bottom_id'),
			'conditions' => array('top_id' => $spcid, 'top' => 1, 'bottom_id' => $groupIdList)));
		return $list;
	}

	/**
	 * グループIDが直接SPコネクタに接続しているかを判定する。
	 */
	function isDirectConnectSPConnetor($spcid, $groupid) {
		$count = $this->find('count', array(
			'conditions' => array('top_id' => $spcid, 'top' => 1, 'bottom_id' => $groupid)));
		if ($count > 0) {
			return true;
		}
		return false;
	}

	/**
	 * SPコネクタとIdPグループを自動接続する
	 */
	public function saveAuto($spcId, $groupId) {
		$entity = array();
		$entity['top_id'] = $spcId;
		$entity['bottom_id'] = $groupId;
		$entity['top'] = 1;
		$entity['bottom'] = 0;
		$entity['auto'] = 1;
		$this->create();
		$result = $this->save($entity);
		if (!$result) {
			return false;
		}
		return true;
	}

	/**
	 * 指定されたSPコネクタIDに自動接続しているIdPグループID一覧を検索する
	 */
	public function findIdPGroupIdList($spcid) {
		$idList = $this->find('list', array(
				'fields' => array('bottom_id', 'bottom_id'),
				'conditions' => array('top_id' => $spcid, 'top' => 1, 'auto' => 1)));
		return $idList;
	}

	/**
	 * SPコネクタとIdPグループの接続を切断する
	 */
	public function deleteIdPGroup($spcid, $idpGroupId) {
		$result = $this->deleteAll(array(
				'top_id' => $spcid, 'bottom_id' => $idpGroupId, 'top' => 1, 'auto' => 1), false);
		if (!$result) {
			return false;
		}
		return true;
	}

	/**
	 * 各ハイブリッド型SPCに接続しているグループIDをリストを取得します。
	 * @param $userid ユーザID
	 */
	public function findHybridSpcList($userid) {
		$list = $this->find('all', array(
				'fields' => array('top_id', 'bottom_id'),
				'conditions' => array(
					'top' => 1,
					'top_id in (select id from groups where sp_type=3)',
					'bottom_id in (select groupid from mygroups where account_id = ?)' => $userid ),
				'order' => array('top_id', 'bottom_id')));
		$result = array();
		foreach ($list as $entity) {
			$topid = $entity['AddGroup']['top_id'];
			$bottomid = $entity['AddGroup']['bottom_id'];
			if (!isset($result[ $topid ])) {
				$result[ $topid ] = array();
			}
			array_push($result[ $topid ], $bottomid);
		}

		return $result;
	}

	/**
	 * SPコネクタに接続しているグループのいずれかにメンバーとして所属しているかを取得します。
	 * @param $account_id アカウントID
	 */
	public function isMemberConnectedSP($account_id, $spc_id) {
		$groupIdList = $this->find('list', array(
			'fields' => array('bottom_id', 'bottom_id'),
			'conditions' => array('top_id' => $spc_id, 'top' => 1)));
		$Mygroup = new Mygroup;
		$AutoGroup = new AutoGroup;
		foreach ($groupIdList as $groupid) {
			if ($AutoGroup->isIdpGroup($groupid)) {
				continue;
			}
			// メンバー
			if ($Mygroup->isMember($account_id, $groupid) && !$Mygroup->isAdminOnly($account_id, $groupid)) {
				return true;
			}
			// 下位グループ
			$bottomIdList = $this->getBottomIdList($groupid, $onlyActive=false);
			foreach ($bottomIdList as $bottomid) {
				if ($AutoGroup->isIdpGroup($bottomid)) {
					continue;
				}
				// メンバー
				if ($Mygroup->isMember($account_id, $bottomid) && !$Mygroup->isAdminOnly($account_id, $bottomid)) {
					return true;
				}
			}
		}
		return false;
	}
}
?>
