<?php
/**
 * AgreementConnect Model
 *
 * PHP versions 5
 *
 * @copyright     Copyright (c) 2011 - 2012, National Institute of Informatics,
 * @license       BSD License (http://creativecommons.org/licenses/BSD/)
 */
class AgreementConnect extends AppModel
{
    var $name = 'AgreementConnect';

    function findNonAgreementCount($groupid) {
    	$count = $this->find('count', array('conditions' => array(
			'OR' => array('top_group_id' => $groupid , 'bottom_group_id' => $groupid), 'status' => 0)));
    	return $count;
    }

    /**
     * 対象SP一覧を取得する
     * @param $spHostList SPホスト一覧
     * @param $groupSphostList SPコネクタと接続しているSP一覧
     */
    function getTargetSpList($spHostList, $groupSphostList) {
    	$resultList['targetSpListJp'] = array();
    	$resultList['targetSpListEn'] = array();
    	$resultList['addSpList'] = array();
    	$i = 0;
		foreach ($spHostList as $sphost) {
			$resultList['targetSpListJp'][$i] = array();
			$resultList['targetSpListEn'][$i] = array();
			$resultList['targetSpListJp'][$i]['hostname'] = $sphost['SpHost']['name'];
			$resultList['targetSpListEn'][$i]['hostname'] = $sphost['SpHost']['name'];
			array_push($resultList['addSpList'], $sphost['SpHost']['name']);

			foreach ($groupSphostList as $groupSp) {
				if ($sphost['SpHost']['id'] === $groupSp['GroupSphost']['sp_id']) {
					if (isset($groupSp['GroupSphost']['service_name_en']))
						$groupSp['GroupSphost']['service_name_en'] = '';

					if (!empty($groupSp['GroupSphost']['service_name'])) {
						$resultList['targetSpListJp'][$i]['service_name'] = $groupSp['GroupSphost']['service_name'];
					} else {
						$resultList['targetSpListJp'][$i]['service_name'] = $groupSp['GroupSphost']['service_name_en'];
					}

					if (!empty($groupSp['GroupSphost']['service_name_en'])) {
						$resultList['targetSpListEn'][$i]['service_name'] = $groupSp['GroupSphost']['service_name_en'];
					} else {
						$resultList['targetSpListEn'][$i]['service_name'] = $groupSp['GroupSphost']['service_name'];
					}
					break;
				}
			}
			$i++;
		}

		return $resultList;
    }

    function getAddAttributeNameList($add_attributes) {
    	$resultList['jp'] = array();
    	$resultList['en'] = array();
    	$attributeList = explode(',', $add_attributes);
    	foreach ($attributeList as $attribute) {
	    	if ($attribute === 'ismemberof') {
				array_push($resultList['jp'], __("common.group.id.ja", true));
				array_push($resultList['en'], __("common.group.id.eng", true));
			} else if ($attribute === 'eptid') {
				array_push($resultList['jp'], __("common.eptid", true));
				array_push($resultList['en'], __("common.eptid", true));
			} else if ($attribute === 'name') {
				array_push($resultList['jp'], __("common.name.jp", true));
				array_push($resultList['en'], __("common.name.en", true));
			} else if ($attribute === 'mail') {
				array_push($resultList['jp'], __("common.mailaddress.ja", true));
				array_push($resultList['en'], __("common.mailaddress.eng", true));
			} else if ($attribute === 'eppn') {
				array_push($resultList['jp'], __("common.eppn", true));
				array_push($resultList['en'], __("common.eppn", true));
			}
    	}
    	return $resultList;
    }

    function getAttributeList($id) {
		$entity = $this->find('first', array('conditions' => array('id' => $id)));
		$list = explode(',', $entity['AgreementConnect']['add_attributes']);
debug($entity);
		$resultList = array();
		foreach ($list as $attribute) {
			if ($attribute === 'eptid') {
				$resultList['eptid'] = 1;
			} else if ($attribute === 'name') {
				$resultList['name'] = 1;
			} else if ($attribute === 'mail') {
				$resultList['mail'] = 1;
			} else if ($attribute === 'eppn') {
				$resultList['eppn'] = 1;
			} else if ($attribute === 'idp') {
				$resultList['idp'] = 1;
			} else if ($attribute === 'introduction') {
				$resultList['introduction'] = 1;
			} else if ($attribute === 'language') {
				$resultList['language'] = 1;
			} else if ($attribute === 'organization') {
				$resultList['organization'] = 1;
			}
		}

		return $resultList;
    }

    function getUpdateAttributeList($agreement, $attributeList) {
    	$resultList = array();
    	foreach ($attributeList as $attribute => $value) {
    		if (!$value)
    			continue;

	    	if ($attribute === 'eptid') {
				if (empty($agreement) || !$agreement['Agreement']['eptid']) {
					$resultList['eptid'] = 1;
				}
			}
			if ($attribute === 'name') {
				if (empty($agreement) || !$agreement['Agreement']['name']) {
					$resultList['name'] = 1;
				}
			}
			if ($attribute === 'mail') {
				if (empty($agreement) || !$agreement['Agreement']['mail']) {
					$resultList['mail'] = 1;
				}
			}
			if ($attribute === 'eppn') {
				if (empty($agreement) || !$agreement['Agreement']['eppn']) {
					$resultList['eppn'] = 1;
				}
			}
			if ($attribute === 'idp') {
				if (empty($agreement) || !$agreement['Agreement']['idp']) {
					$resultList['idp'] = 1;
				}
			}
			if ($attribute === 'introduction') {
				if (empty($agreement) || !$agreement['Agreement']['introduction']) {
					$resultList['introduction'] = 1;
				}
			}
			if ($attribute === 'language') {
				if (empty($agreement) || !$agreement['Agreement']['language']) {
					$resultList['language'] = 1;
				}
			}
			if ($attribute === 'organization') {
				if (empty($agreement) || !$agreement['Agreement']['organization']) {
					$resultList['organization'] = 1;
				}
			}
    	}

    	return $resultList;
    }

    /**
     * add_attributesの値を生成します。
     */
	function generateAddAttributes($addAttributes) {
		// 追加になった属性を設定
		$attributeList = array();
		if (isset($addAttributes['ismemberof']) && $addAttributes['ismemberof']) {
			array_push($attributeList, 'ismemberof');
		}
		if (isset($addAttributes['eptid']) && $addAttributes['eptid']) {
			array_push($attributeList, 'eptid');
		}
		if (isset($addAttributes['name']) && $addAttributes['name']) {
			array_push($attributeList, 'name');
		}
		if (isset($addAttributes['mail']) && $addAttributes['mail']) {
			array_push($attributeList, 'mail');
		}
		if (isset($addAttributes['eppn']) && $addAttributes['eppn']) {
			array_push($attributeList, 'eppn');
		}
		if (isset($addAttributes['idp']) && $addAttributes['idp']) {
			array_push($attributeList, 'idp');
		}
		if (isset($addAttributes['introduction']) && $addAttributes['introduction']) {
			array_push($attributeList, 'introduction');
		}
		if (isset($addAttributes['language']) && $addAttributes['language']) {
			array_push($attributeList, 'language');
		}
		if (isset($addAttributes['organization']) && $addAttributes['organization']) {
			array_push($attributeList, 'organization');
		}
		return $attributeList;
	}
}
?>
