<?php
/**
 * GroupSphost Model
 *
 * PHP versions 5
 *
 * @copyright     Copyright (c) 2016, SOUM Corporation.
 * @copyright     Copyright (c) 2011 - 2012, National Institute of Informatics,
 * @license       BSD License (http://creativecommons.org/licenses/BSD/)
 */
App::import('Model','SpHost');
App::import('Model','AddGroup');
App::import('Model','Group');
App::import('Model','SpGroup');
App::import('Component', 'ParameterReplacer');

class GroupSphost extends AppModel {
	var $name = 'GroupSphost';
	var $useTable = 'group_sphosts';
	var $SpHost;
	var $AddGroup;
	var $parameterReplacer;

	var $validate = array(
		'lead_url' => array(
			'maxlength' => array(
				'rule' => array('maxLength', '1024'),
				'message' => 'validation.maxlength',
				'last' => true
			),
			'website' => array(
				'rule' => array('url', true),
				'allowEmpty' => true,
				'message' => 'validation.url.incorrect.format',
				'last' => true
			),
		),

		'lead_non_map_url' => array(
			'maxlength' => array(
				'rule' => array('maxLength', '1024'),
				'message' => 'validation.maxlength',
				'last' => true
			),
			'website' => array(
				'rule' => array('url', true),
				'allowEmpty' => true,
				'message' => 'validation.url.incorrect.format',
				'last' => true
			),
		),

		'callback_url' => array(
			'maxlength' => array(
				'rule' => array('maxLength', '1024'),
				'message' => 'validation.maxlength',
				'last' => true
			),
			'website' => array(
				'rule' => array('url', true),
				'allowEmpty' => true,
				'message' => 'validation.url.incorrect.format',
				'last' => true
			),
		),

		'relogin_url' => array(
			'maxlength' => array(
				'rule' => array('maxLength', '1024'),
				'message' => 'validation.maxlength',
				'last' => true
			),
			'website' => array(
				'rule' => array('url', true),
				'allowEmpty' => true,
				'message' => 'validation.url.incorrect.format',
				'last' => true
			),
		),
	);


	function __construct() {
		parent::__construct();
		$this->SpHost = new SpHost;
		$this->AddGroup = new AddGroup();
		$this->Group = new Group();
		$this->SpGroup = new SpGroup();
		$this->parameterReplacer = new ParameterReplacerComponent();
	}

	function getServiceName($lang, $entity) {
		$serviceName = '';
		if ($lang === 'ja') {
			if (!empty($entity['GroupSphost']['service_name']))
				$serviceName = $entity['GroupSphost']['service_name'];
			else if (!empty($entity['GroupSphost']['service_name_en']))
				$serviceName = $entity['GroupSphost']['service_name_en'];
		} else {
			if (!empty($entity['GroupSphost']['service_name_en']))
				$serviceName = $entity['GroupSphost']['service_name_en'];
			else if (!empty($entity['GroupSphost']['service_name']))
				$serviceName = $entity['GroupSphost']['service_name'];
		}

		if (empty($serviceName)) {
			$serviceName = $this->getSpHost($entity['GroupSphost']['sp_id']);
		}

		return $serviceName;
	}

	function getSpcServiceNameList($lang, $list) {
		$nameList = array();
		if ($lang === 'ja') {
			if (isset($list['spc_service_name'])) {
				foreach ($list['spc_service_name'] as $name) {
					$nameList[$name] = $name;
				}
			} else if (isset($list['spc_service_name_en'])) {
				foreach ($list['spc_service_name_en'] as $name) {
					$nameList[$name] = $name;
				}
			}
		} else {
			if (isset($list['spc_service_name_en'])) {
				foreach ($list['spc_service_name_en'] as $name) {
					$nameList[$name] = $name;
				}
			} else if (isset($list['spc_service_name'])) {
				foreach ($list['spc_service_name'] as $name) {
					$nameList[$name] = $name;
				}
			}
		}

		return $nameList;
	}

	private function getSpHost($spid) {
		$host = $this->SpHost->getSpHost($spid);
		return $host;
	}

	function getServiceNameList($spcid) {
		$resultList['jp'] = array();
		$resultList['en'] = array();
		$list = $this->find('all', array('conditions' => array('group_id' => $spcid)));
		foreach ($list as $entity) {
			if (!empty($entity['GroupSphost']['service_name'])) {
				array_push($resultList['jp'], $entity['GroupSphost']['service_name']);
			} elseif (!empty($entity['GroupSphost']['service_name_en'])) {
				array_push($resultList['jp'], $entity['GroupSphost']['service_name_en']);
			}

			if (!empty($entity['GroupSphost']['service_name_en'])) {
				array_push($resultList['en'], $entity['GroupSphost']['service_name_en']);
			} elseif (!empty($entity['GroupSphost']['service_name'])) {
				array_push($resultList['en'], $entity['GroupSphost']['service_name']);
			}
		}
		return $resultList;
	}


	/**
	 * サービスのリンクを取得します。
	 * 自グループがSPコネクタに接続している場合は、$groupIDは自グループキーで置換される
	 * @param $groupid グループID
	 * @param $lang 使用言語
	 * @param $groupIdSpcIdList groupid毎の接続spcid一覧
	 * @param $spHostList SPホスト名一覧
	 */
	function getServiceLink($groupid, $lang, $option = false,
				$iconList = array(), $groupIdSpcIdList = array(), $spHostList = array()) {
		if (empty($groupIdSpcIdList)) {
			$spcIdList = $this->AddGroup->getConnectedSPCIdList($groupid);
		} else {
			if (isset($groupIdSpcIdList[$groupid])) {
				$spcIdList = $groupIdSpcIdList[$groupid];
			} else {
				$spcIdList = array();
			}
		}
//debug($spcIdList);
		$resultList = $this->getNameAndLeadUrl($spcIdList, $groupid, $lang, $option, $spHostList);

		// SPのURLをリダイレクト用に変換
		foreach ($resultList as $spcid => $result) {
			// 元のサービスのURLも残す
			$orgUrl = $resultList[$spcid][1];
			$resultList[$spcid]['orgUrl'] = $orgUrl;
			// リダイレクト用URL
			$url = Router::url('/', true)."redirect/sp/".$spcid."/".$groupid;
			$resultList[$spcid][1] = $url;
		}

		// アイコンを設定
		$spcIconList = $this->SpGroup->findIconList();
		if (empty($iconList)) {
			foreach ($resultList as $spcid => $result) {
				if (!isset($spcIconList[$spcid])) {
					continue;
				}
				$resultList[$spcid][3] = $spcIconList[$spcid];
			}
		} else {
			foreach ($resultList as $spcid => $result) {
				if (isset($iconList[$spcid])) {
					$resultList[$spcid][3] = $iconList[$spcid];
				} else {
					if (!isset($spcIconList[$spcid])) {
						continue;
					}
					$resultList[$spcid][3] = $spcIconList[$spcid];
				}
			}
		}

		return $resultList;
	}

	/**
	 * サービスのURLを取得します
	 * @param $groupid グループID
	 * @param $spcid SPコネクタID
	 */
	function getServiceURL($groupid, $spcid) {
		$result = $this->getNameAndLeadUrl($spcid, $groupid, 'java', false);
		if (!empty($result)) {
			return $result[$spcid][1];
		}
		return "";
	}

	/**
	 * 接続していないサービスのリンクを取得します。
	 * 自グループがSPコネクタに接続している場合は、$groupIDは自グループキーで置換される
	 * @param $groupid グループID
	 * @param $lang 使用言語
	 */
	function getServiceLinkNotConnect($groupid, $lang, $option = false) {
		$spcIdList = $this->AddGroup->getConnectedSPCIdList($groupid);
		$notConnectIdList = $this->Group->find('list', array(
			'fields' => array('id', 'id'),
			'conditions' => array('sp' => 1, 'active' => 0, 'public' => 1,  'NOT' => array('id' => $spcIdList))));
		$resultList = $this->getNameAndLeadUrl($notConnectIdList, $groupid, $lang, $option);

		foreach ($resultList as $spcid => $result) {
			// 接続していないのでリダイレクト用URLなし
			$resultList[$spcid]['orgUrl'] = $resultList[$spcid][1];
		}

		// アイコンを設定
		$iconList = $this->SpGroup->findIconList();
		foreach ($resultList as $spcid => $result) {
			if (!isset($iconList[$spcid])) {
				continue;
			}
			$resultList[$spcid][3] = $iconList[$spcid];
		}

		return $resultList;
	}

	/**
	 * 接続していないSPコネクタを取得する
	 * @param $groupid グループID
	 * @param $lang 使用言語
	 */
	function getNotConnectSPC($groupid) {
		$spcIdList = $this->AddGroup->getConnectedSPCIdList($groupid);
		$notConnectIdList = $this->SpGroup->find('list', array(
			'fields' => array('id', 'name'),
			'conditions' => array('sp' => 1, 'active' => 0, 'public' => 1,  'NOT' => array('id' => $spcIdList)),
			'order' => array('name')));

		// アイコンを設定
		$iconList = $this->SpGroup->findIconList();
		$resultList = array();
		$i = 0;
		foreach ($notConnectIdList as $spcid => $name) {
			$resultList[$i] = array(
				'spcid' => $spcid, 'icon' => $iconList[$spcid], 'name' => $name);
			$i++;

		}
		return $resultList;
	}

	/**
	 * @param unknown $spcIdList
	 * @param unknown $groupid
	 * @param unknown $lang
	 * @param unknown $option
	 * @param unknown $spHostList SPホスト名一覧
	 * @return multitype:|multitype:multitype:string unknown  multitype:NULL unknown  multitype:unknown Ambigous <string, unknown>  multitype:NULL unknown mixed  multitype:unknown string
	 */
	private function getNameAndLeadUrl($spcIdList, $groupid, $lang, $option, $spHostList = array()) {
		if (empty($spcIdList)) {
			return array();
		}

		$resultList = array();
		$spResultList = array();
//		$list = $this->find('all', array('conditions' => array('group_id' => $spcIdList)));

		// 名前順で取得する
		$list = $this->find('all', array('conditions' =>
						array('group_id' => $spcIdList,
						      'NOT' => array('lead_url' => "")),
			'order' => array('service_name ASC')));

		// SPホスト名一覧を取得
		if (empty($spHostList)) {
			$spHostList = $this->SpHost->getSpHostList();
		}

		$this_group_key = null;
		foreach ($list as $entity) {
			$spcid = $entity['GroupSphost']['group_id'];
			$serviceName = $entity['GroupSphost']['service_name'];

			if (!preg_match('/^(http:|https:)/i', $entity['GroupSphost']['lead_url'])) {
				$spHost = $this->SpHost->find('first', array('conditions' => array('id' => $entity['GroupSphost']['sp_id'])));
				if ($option) {
					//array_push($resultList, array($spHost['SpHost']['name'].'　('.$spHost['SpHost']['entityid'].')'));
					array_push($resultList, array($serviceName, '', $spcid));
					$spResultList[$spcid] = array($serviceName, '', $spcid);
				} else {
					//array_push($resultList, array($spHost['SpHost']['name'], '', $spcid));
					array_push($resultList, array($serviceName, '', $spcid));
					$spResultList[$spcid] = array($serviceName, '', $spcid);
				}
				continue;
			}

			// $groupid がこのSPと接続していたら、$groupIDをこのグループで置換してリンクにする
			$connectGroupKeyList = array();
			if (preg_match('/\$groupID/i', $entity['GroupSphost']['lead_url'])) {
				$spcid = $entity['GroupSphost']['group_id'];
				if (is_null($this_group_key)) {
					$group = $this->Group->find('first', array('conditions' => array('id' => $groupid)));
					$this_group_key = $group['Group']['group_key'];
				}
				array_push($connectGroupKeyList, $this_group_key);
			} else {
				array_push($connectGroupKeyList, null);
			}

			foreach ($connectGroupKeyList as $group_key) {
				$link = $entity['GroupSphost']['lead_url'];
				if (!is_null($group_key)) {
					$link = $this->parameterReplacer->replaceGroupKey($group_key, $link);
				}
				$link = $this->parameterReplacer->replaceEntityId($link);
				$spcid = $entity['GroupSphost']['group_id'];


				if ($lang === 'ja') {
					if (!empty($entity['GroupSphost']['service_name'])) {
						array_unshift ($resultList, array($entity['GroupSphost']['service_name'], $link, $spcid));
						$spResultList[$spcid] = array($entity['GroupSphost']['service_name'], $link, $spcid);
					} elseif (!empty($entity['GroupSphost']['service_name_en'])) {
						array_unshift ($resultList, array($entity['GroupSphost']['service_name_en'], $link, $spcid));
						$spResultList[$spcid] = array($entity['GroupSphost']['service_name_en'], $link, $spcid);
					} else {
//						$host = $this->SpHost->getSpHost($entity['GroupSphost']['sp_id']);
						$host = '';
						if (isset($entity['GroupSphost']['sp_id'])) {
							$host = $spHostList[ $entity['GroupSphost']['sp_id'] ];
						}
						array_push($resultList, array($host, $link, $spcid));
						$spResultList[$spcid] = array($host, $link, $spcid);
					}
				} else {
					if (!empty($entity['GroupSphost']['service_name_en'])) {
						array_unshift ($resultList, array($entity['GroupSphost']['service_name_en'], $link, $spcid));
						$spResultList[$spcid] = array($entity['GroupSphost']['service_name_en'], $link, $spcid);
					} elseif (!empty($entity['GroupSphost']['service_name'])) {
						array_unshift ($resultList, array($entity['GroupSphost']['service_name'], $link, $spcid));
						$spResultList[$spcid] = array($entity['GroupSphost']['service_name'], $link, $spcid);
					} else {
						//$host = $this->SpHost->getSpHost($entity['GroupSphost']['sp_id']);
						$host = '';
						if (isset($entity['GroupSphost']['sp_id'])) {
							$host = $spHostList[ $entity['GroupSphost']['sp_id'] ];
						}
						array_push($resultList, array($host, $link, $spcid));
						$spResultList[$spcid] = array($host, $link, $spcid);
					}
				}
				if (isset($entity['GroupSphost']['lead_non_map_url'])) {
					$spResultList[$spcid]['lead_non_map_url'] = $entity['GroupSphost']['lead_non_map_url'];
				}
			}
		}
//		return $resultList;
		return $spResultList;
	}

	/**
	 * グループに紐付いたサービス情報を検索します。
	 * @param $groupid グループID
	 * @param $lang 使用言語
	 */
	function findList($spcid) {
		$entityList = $this->find('all', array('conditions' => array(
			'group_id' => $spcid, )));

		return $entityList;
	}

	/**
	 * sp_id一覧を取得します。
	 */
	function getSpIdList() {
		$list = $this->find('list', array('fields' => array('sp_id', 'sp_id')));
		return $list;
	}

	/**
	 * 自動生成用のgroup_sphosts作成します。
	 * @param $groupId SPコネクタID（グループID）
	 * @param $spid SPID
	 * @param $serviceName サービス名
	 * @param $spUrl SPのURL
	 */
	public function createAuto($groupId, $spid, $serviceName, $spUrl) {
		$groupSphost = array();
		$groupSphost['GroupSphost']['group_id'] = $groupId;
		$groupSphost['GroupSphost']['sp_id'] = $spid;
		$groupSphost['GroupSphost']['service_name'] = $serviceName;
		$groupSphost['GroupSphost']['lead_url'] = $spUrl;
		$groupSphost['GroupSphost']['auto'] = 1;
		$this->create();
		$result = $this->save($groupSphost);
		return $result;
	}

	/**
	 * SPIDと紐付いているSPコネクタIDを取得します。(１つのみに限定)
	 */
	public function getSpcId($spId) {
		$entity = $this->find('first', array('conditions' => array('sp_id' => $spId)));
		if (empty($entity)) {
			return false;
		}
		return $entity['GroupSphost']['group_id'];
	}

	/**
	 * SPIDと紐付いているSPコネクタID一覧を取得します。
	 */
	public function getSpcIdList($spId) {
		$list = $this->find('list', array(
				'fields' => array('group_id', 'group_id'),
				'conditions' => array('sp_id' => $spId)));
		if (empty($list)) {
			return array();
		}
		return $list;
	}

	public function getAutoSpcIdList() {
		$list = $this->find('list', array(
			'fields' => 'group_id',
			'conditions' => array(
				'auto' => 1,
			)
		));
		if (empty($list)) {
			return array();
		}
		return $list;
	}

	/**
	 * 接続しているプライベートサービスのグループIDを取得します。
	 * @param $groupid グループID
	 */
	function getPrivateServiceId($groupid) {
		$serviceLinkList = $this->getServiceLink($groupid, 'eng', true);
		$list = $this->Group->find('list', array(
			'fields' => array('id'),
			'conditions' => array(
				'id' => array_keys($serviceLinkList),
				'sp' => 2)));
		if (empty($list)) {
			return array();
		}
		return $list;
	}
}
?>
