<?php
/**
 * SentNoticeMailLog Model
 *
 * PHP versions 5
 *
 * @copyright     Copyright (c) 2019, National Institute of Informatics,
 * @license       BSD License (http://creativecommons.org/licenses/BSD/)
 */
class SentNoticeMailLog extends AppModel {
	var $name = 'SentNoticeMailLog';

	public function createSentNoticeMailLog($eppnId, $kind) {
		$sentNoticeMailLog = array(
			'account_eppn_id' => $eppnId,
			'kind' => $kind,
		);
		$this->create();
		$result = false;
		$count = 0;
		$max = 5; // 最大試行回数
		$random = false;
		do {
			$now = date('Y-m-d H:i:s');
			$random = $this->getUniqueRandom();
			$sentNoticeMailLog['random'] = $random;
			$sentNoticeMailLog['sent'] = $now;

			$result = $this->save($sentNoticeMailLog);
			if (!$result) {
				$error = $this->getDataSource()->error;
				$err = explode(':', $error);
				if ($err[0] != 1062) { // Duplicate以外のエラーは終了
					$this->log('SentNoticeMailLog#createSentNoticeMailLog INSERT Error '.$error);
					return false;
				}
				if ($count >= $max) {
					$this->log('SentNoticeMailLog#createSentNoticeMailLog INSERT retry max Error ');
					return false;
				}
			}
			$count++;
		} while (!$result);
		return $random;
	}

	private function getUniqueRandom()
	{
		for ($i = 0; $i < 10; $i++) {
			$random = $this->getRandom(50);
			if (!$this->existsUniqueKey($random)) {
				return $random;
			}
		}
		return false;
	}

	private function existsUniqueKey($random)
	{
		$count = $this->find('count', array(
			'conditions' => array(
				'random' => $random,
			)
		));
		return $count >= 1;
	}

}
?>
