<?php
/**
 * SpcSort Model
 *
 * PHP versions 5
 *
 * @copyright     Copyright (c) 2011 - 2012, National Institute of Informatics,
 * @license       BSD License (http://creativecommons.org/licenses/BSD/)
 */
class SpcSort extends AppModel
{
    var $name = 'SpcSort';

    /**
     * ユーザのSPコネクタのソート順を取得します。
     * @param $userid
     */
	function getSortType($userid, $groupid) {
		$entity = $this->find('first', array(
			'conditions' => array('account_id' => $userid, 'groupid' => $groupid)));

		// 未設定の場合は名前順
		if (empty($entity)) {
			return 'name';
		}
		if (empty($entity['SpcSort']['sort_type'])) {
			return 'name';
		}

		return $entity['SpcSort']['sort_type'];
	}

	/**
	 * ユーザのSPコネクタのソート順をグループIDごとにで取得します。
	 * @param $userid
	 */
	function getSortTypeList($userid) {
		$sortTypeList = $this->find('list', array(
				'fields' => array('groupid', 'sort_type'),
				'conditions' => array('account_id' => $userid)));
		return $sortTypeList;
	}

	/**
	 * SPコネクタのソート順を保存します。
	 */
	function updateType($userid, $groupid, $type) {
		$spcSort =  $this->find('first', array(
			'conditions' => array('account_id' => $userid, 'groupid' => $groupid)));
		if (empty($spcSort)) {
			$this->create();
			$spcSort = array();
			$spcSort['SpcSort']['account_id'] = $userid;
		}
		$spcSort['SpcSort']['modified'] = null;
		$spcSort['SpcSort']['groupid'] = $groupid;
		$spcSort['SpcSort']['sort_type'] = $type;
		$this->save($spcSort);
	}

	/**
	 * SPコネクタのソート順を保存します。
	 */
	function saveType($userid, $groupid, $type) {
		$this->create();
		$spcSort = array();
		$spcSort['SpcSort']['account_id'] = $userid;
		$spcSort['SpcSort']['modified'] = null;
		$spcSort['SpcSort']['groupid'] = $groupid;
		$spcSort['SpcSort']['sort_type'] = $type;
		$this->save($spcSort);
	}

	/**
	 * 機関グループの設定をグループに反映します。
	 */
	function copyIdpGroupToTargetGroup($idpGroupId, $targetGroupId, $accountid) {
		$idpGroup = $this->find('first', array(
				'conditions' => array('groupid' => $idpGroupId, 'account_id' => $accountid)));
		if (empty($idpGroup)) {
			return;
		}

		$sortType = $idpGroup['SpcSort']['sort_type'];
		if (empty($sortType)) {
			return;
		}

		$targetGroup = $this->find('first', array('conditions' =>
				array('groupid' => $targetGroupId, 'account_id' => $accountid)));
		if (!empty($targetGroup)) {
			$targetGroup['SpcSort']['sort_type'] = $sortType;
			$targetGroup['SpcSort']['modified'] = NULL;
			$result = $this->save($targetGroup);
		} else {
			$this->saveType($accountid, $targetGroupId, $sortType);
		}
	}
}
?>