<?php
/**
 * UserSpSort Model
 *
 * PHP versions 5
 *
 * @copyright     Copyright (c) 2011 - 2012, National Institute of Informatics,
 * @license       BSD License (http://creativecommons.org/licenses/BSD/)
 */
class UserSpSort extends AppModel
{
	var $name = 'UserSpSort';

	/**
	 * 左側のSPコネクタIDを表示順の配列で取得します。
	 * @param $userid
	 * @param $flip
	 * @return flip:true  -> [SPコネクタのID] => 表示順
	 *         flip:false -> [表示順] => SPコネクタのID
	 */
	function findLeftSpIdList($userid, $flip = true) {
		$entity = $this->find('first', array(
			'fields' => array('left_spc_id_list'),
			'conditions' => array('account_id' => $userid)));
		if (!empty($entity)) {
			$spcIdList = $entity['UserSpSort']['left_spc_id_list'];
			$list = explode(',', $spcIdList);
			if ($flip) {
				$list = array_flip($list);
			}
			return $list;
		}
		return '';
	}

	/**
	 * 右側のSPコネクタIDを表示順の配列で取得します。
	 * @param $userid
	 * @param $flip
	 * @return flip:true  -> [SPコネクタのID] => 表示順
	 *         flip:false -> [表示順] => SPコネクタのID
	 */
	function findRightSpIdList($userid, $flip = true) {
		$entity = $this->find('first', array(
			'fields' => array('right_spc_id_list'),
			'conditions' => array('account_id' => $userid)));
		if (!empty($entity)) {
			$spcIdList = $entity['UserSpSort']['right_spc_id_list'];
			$list = explode(',', $spcIdList);
			if ($flip) {
				$list = array_flip($list);
			}
			return $list;
		}
		return '';
	}

	/**
	 * 中央のSPコネクタIDを表示順の配列で取得します。
	 * @param $userid
	 * @param $flip
	 * @return flip:true  -> [SPコネクタのID] => 表示順
	 *         flip:false -> [表示順] => SPコネクタのID
	 */
	function findCenterSpIdList($userid, $flip = true) {
		$entity = $this->find('first', array(
			'fields' => array('center_spc_id_list'),
			'conditions' => array('account_id' => $userid)));
		if (!empty($entity)) {
			$spcIdList = $entity['UserSpSort']['center_spc_id_list'];
			$list = explode(',', $spcIdList);
			if ($flip) {
				$list = array_flip($list);
			}
			return $list;
		}
		return '';
	}

	/**
	 * SPコネクタIDを表示順の配列で取得します。（横１列パターン）
	 * @param $userid
	 * @param $flip
	 * @return flip:true  -> [SPコネクタのID] => 表示順
	 *         flip:false -> [表示順] => SPコネクタのID
	 */
	function findSideSpIdList($userid, $groupid, $flip = true) {
		$entity = $this->find('first', array(
				'fields' => array('spc_id_list'),
				'conditions' => array('account_id' => $userid, 'groupid' => $groupid)));
		if (!empty($entity)) {
			$spcIdList = $entity['UserSpSort']['spc_id_list'];
			$list = explode(',', $spcIdList);
			if ($flip) {
				$list = array_flip($list);
			}
			return $list;
		}
		return '';
	}

	/**
	 * 表示順を全て更新します。(縦３列パターン)
	 * @param $userid ユーザID
	 * @param $groupid idpグループのグループID
	 */
	function updateVerticalAll($userid, $groupid, $leftSpcidList, $rightSpcidList, $centerSpcidList) {
		$entity = $this->find('first', array(
				'conditions' => array('groupid' => $groupid, 'account_id' => $userid)));
				if (empty($entity)) {
			$this->create();
			$entity['UserSpSort']['account_id'] = $userid;
			$entity['UserSpSort']['groupid'] = $groupid;
		}

		$entity['UserSpSort']['left_spc_id_list'] = $leftSpcidList;
		$entity['UserSpSort']['right_spc_id_list'] = $rightSpcidList;
		$entity['UserSpSort']['center_spc_id_list'] = $centerSpcidList;
		$entity['UserSpSort']['modified'] = null;
		$result = $this->save($entity);
		if (!$result) {
			return false;
		}
		return true;
	}

	/**
	 * 表示順を全て更新します。(横１列パターン)
	 * @param $userid ユーザID
	 * @param $groupid idpグループのグループID
	 */
	function updateSideAll($userid, $groupid, $spcidList) {
		$entity = $this->find('first', array(
				'conditions' => array('groupid' => $groupid, 'account_id' => $userid)));
		if (empty($entity)) {
			$this->create();
			$entity['UserSpSort']['account_id'] = $userid;
			$entity['UserSpSort']['groupid'] = $groupid;
		}
	
		$entity['UserSpSort']['spc_id_list'] = $spcidList;
		$entity['UserSpSort']['modified'] = null;
		$result = $this->save($entity);
		if (!$result) {
			return false;
		}
		return true;
	}

	/**
	 * 左側と右側と中央を順番を１つにまとめたSPコネクタIDを表示順を取得します。
	 * @param $userid ユーザID
	 */
	function getAllSpIdList($userid) {
		$left = $this->findLeftSpIdList($userid, false);
		$right = $this->findRightSpIdList($userid, false);
		$center = $this->findCenterSpIdList($userid, false);
		$resultIdList = array();
		$i = 0;
		$leftEnd = false;
		$rightEnd = false;
		$centerEnd = false;
		for (;;) {
			if (!$leftEnd && isset($left[$i])) {
				$spcid = $left[$i];
				$resultIdList += array($spcid => $spcid);
			} else {
				$leftEnd = true;
			}
			if (!$rightEnd && isset($right[$i])) {
				$spcid = $right[$i];
				$resultIdList += array($spcid => $spcid);
			} else {
				$rightEnd = true;
			}
			if (!$centerEnd && isset($center[$i])) {
				$spcid = $center[$i];
				$resultIdList += array($spcid => $spcid);
			} else {
				$centerEnd = true;
			}

			$i++;
			if ($leftEnd && $rightEnd && $centerEnd) {
				break;
			}
		}
		return $resultIdList;
	}

	/**
	 * 表示順を保存します。
	 * @param $userid ユーザID
	 * @param $groupid idpグループのグループID
	 */
	function saveSpcIdList($userid, $groupid, $spcidList) {
		$this->create();
		$entity['UserSpSort']['account_id'] = $userid;
		$entity['UserSpSort']['groupid'] = $groupid;
		$entity['UserSpSort']['spc_id_list'] = $spcidList;
		$entity['UserSpSort']['modified'] = null;
		$result = $this->save($entity);
		if (!$result) {
			return false;
		}
		return true;
	}

	/**
	 * 機関グループの設定をグループに反映します。
	 */
	function copyIdpGroupToTargetGroup($idpGroupId, $targetGroupId, $accountid) {
		$idpGroup = $this->find('first', array('conditions' =>
				array('groupid' => $idpGroupId, 'account_id' => $accountid)));
		if (empty($idpGroup)) {
			return;
		}

		$spcIdList = $idpGroup['UserSpSort']['spc_id_list'];
		if (empty($spcIdList)) {
			return;
		}

		$targetGroup = $this->find('first', array('conditions' =>
					array('groupid' => $targetGroupId, 'account_id' => $accountid)));
		if (!empty($targetGroup)) {
			$targetGroup['UserSpSort']['spc_id_list'] = $spcIdList;
			$targetGroup['UserSpSort']['modified'] = NULL;
			$this->save($targetGroup);
		} else {
			$this->saveSpcIdList($accountid, $targetGroupId, $spcIdList);
		}
	}
}
?>