<?php
/**
 * ManageGroups Controller Test
 *
 * PHP versions 5
 *
 * @copyright     Copyright (c) 2011 - 2012, National Institute of Informatics,
 * @license       BSD License (http://creativecommons.org/licenses/BSD/)
 */
require_once('app'.DS.'tests'.DS.'cases'.DS.'controllers'.DS.'controllerTestCase.php');
App::import('Model', 'Group');
App::import('Model', 'AddGroup');

class ManageGroupsControllerTest extends ControllerTestCase  {
	function startCase() {
		parent::startCase();
		$this->Group =& ClassRegistry::init('Group');
		$this->AddGroup =& ClassRegistry::init('AddGroup');
	}

	function endCase() {
		parent::endCase();
		unset($this->Group);
		unset($this->AddGroup);
	}

	function startTest($method) {
		parent::startTest($method);
		$this->deleteData();
	}

	function endTest($method) {
		parent::endTest($method);
		$this->deleteData();
	}

	private function deleteData() {
		$group = $this->Group->find('first', array('conditions' => array('group_key' => 'GroupListsControllerTest_SP')));
		if (!empty($group)) {
			$this->Group->delete($group['Group']['id']);
		}
		$group = $this->Group->find('first', array('conditions' => array('group_key' => 'GroupListsControllerTest_GROUP')));
		if (!empty($group)) {
			$this->Group->delete($group['Group']['id']);
		}
	}

	/**
	 * SPコネクタを接続できることを確認
	 */
	function test_add_up_save_can_connect() {
		// SPコネクタを作成
		$group['Group'] = array(
				'Test GroupLists' => 'name', 'group_key' => 'GroupListsControllerTest_SP',
				'public' => 1,
				'inspectjoin' => 1,
				'down_permission' => 1,
				'active' => 0,
				'sp' => 1);
		$this->Group->create();
		$result = $this->Group->save($group);
		$spcId = $this->Group->getLastInsertID();
		$this->assertTrue($result);

		// グループを作成
		$group['Group'] = array(
				'Test GroupLists' => 'name', 'group_key' => 'GroupListsControllerTest_GROUP',
				'public' => 1,
				'inspectjoin' => 1,
				'down_permission' => 1,
				'active' => 0,
				'sp' => 0);
		$this->Group->create();
		$result = $this->Group->save($group);
		$groupId = $this->Group->getLastInsertID();
		$this->assertTrue($result);

		$data = array('ManageGroup' => array(
			'spGroup' => $spcId,
			'spc' => true,
			'selectedSpc' => $spcId));
		$result = $this->testAction('/admin/manage_groups/add_up_sp_save/'.$groupId,
			array('return' => 'vars', 'data' => $data));
		$addGroup = $this->AddGroup->find('first', array('conditions' => array(
			'top_id' => $spcId, 'bottom_id' => $groupId)));
		$addId = $this->AddGroup->getLastInsertID();

		$this->assertTrue($addGroup);
		$this->assertEqual('1', $addGroup['AddGroup']['top']);
		$this->assertEqual('0', $addGroup['AddGroup']['bottom']);
		$this->assertTrue($addGroup['AddGroup']['created']);
		$this->AddGroup->delete($addId);
	}

	/**
	 * SPコネクタを接続できないことを確認
	 */
	function test_add_up_save_cannot_connect() {
		// SPコネクタを非公開設定で作成
		$group['Group'] = array(
				'Test GroupLists' => 'name', 'group_key' => 'GroupListsControllerTest_SP',
				'public' => 0,
				'inspectjoin' => 1,
				'down_permission' => 1,
				'active' => 0,
				'sp' => 1);
		$this->Group->create();
		$result = $this->Group->save($group);
		$spcId = $this->Group->getLastInsertID();
		$this->assertTrue($result);

		// グループを作成
		$group['Group'] = array(
				'Test GroupLists' => 'name', 'group_key' => 'GroupListsControllerTest_GROUP',
				'public' => 1,
				'inspectjoin' => 1,
				'down_permission' => 1,
				'active' => 0,
				'sp' => 0);
		$this->Group->create();
		$result = $this->Group->save($group);
		$groupId = $this->Group->getLastInsertID();
		$this->assertTrue($result);

		$data = array('ManageGroup' => array(
			'spGroup' => $spcId,
			'spc' => true,
			'selectedSpc' => $spcId));
		$result = $this->testAction('/admin/manage_groups/add_up_sp_save/'.$groupId,
			array('return' => 'vars', 'data' => $data));
		$addGroup = $this->AddGroup->find('first', array('conditions' => array(
			'top_id' => $spcId, 'bottom_id' => $groupId)));
		$addId = $this->AddGroup->getLastInsertID();

		$this->assertFalse($addGroup);
		if (!empty($addId)) {
			$this->AddGroup->delete($addId);
		}
	}
}
?>

