<?php
/**
 * IdpGroupCreatorShell
 *
 * PHP versions 5
 *
 * @copyright     Copyright (c) 2016, SOUM Corporation.
 * @copyright     Copyright (c) 2011 - 2012, National Institute of Informatics,
 * @license       BSD License (http://creativecommons.org/licenses/BSD/)
 */
App::import('Core', 'Xml');
App::import('Shell', 'Metadata');

/**
 * メタデータを元にIdPグループを自動作成します。
 *
 * 実行例）
 * windows)
 *  前提: E:\xampp\phpにPATHを通す
 *  $ cd E:\xampp\htdocs\map\cake\console
 *  $ cake -app E:\xampp\htdocs\map\app idp_group_creator E:\xampp\htdocs\map\app\tmp\metadata\gakunin-test-metadata.xml
 *  $ cake -app E:\xampp\htdocs\map\app idp_group_creator E:\xampp\htdocs\map\app\tmp\metadata\gakunin-metadata.xml
 *  $ cake -app E:\xampp\htdocs\map\app idp_group_creator E:\xampp\htdocs\map\app\tmp\metadata\federation-metadata.xml
 * linux)
 *  $ export TERM=vt100
 *  $ cd /usr/local/map/cake/console
 *  $ ./cake -app /usr/local/map/app idp_group_creator
 *  $ ./cake -app /usr/local/map/app idp_group_creator /etc/shibboleth/metadata/federation-metadata.xml
 */
class IdpGroupCreatorShell extends MetadataShell {

	var $uses = array('Transaction', 'IdpAdministrator', 'AccountEppn', 'AutoGroupKeySequence',
			'Group',  'Mygroup', 'AutoGroup');

	function main() {
		$this->log('IdpGroupCreatorShell start', "batch");

		$metaPath = $this->args[0];
		if (empty($metaPath)) {
			$this->log('IdpGroupCreatorShell ERROR  There are no parameters. ', "batch");
			$this->log('IdpGroupCreatorShell end', "batch");
			var_dump('There are no parameters.');
			return;
		}

		try{
			$this->Transaction->begin();

			$idpList = $this->getIdpEntityIdList($metaPath);
			foreach ($idpList as $idp) {
				$entityid = $idp['entityID'];
				$name = $idp['name'];
// 				preg_match("/^.+:\/\/([^\/|:]+)/", $entityid, $match);
// 				$name = $match[1];
// 				if (empty($name)) {
// 					continue;
// 				}

				if ($this->_isExistAutoGroupIdP($entityid)) {
					// グループ名をホスト名にしていたら、entityidで上書きする
					$this->updateGroupName($entityid, $name);
					$this->updateAutoGroup($entityid, $idp['scope'], $idp['cert']);
					// 新規追加はしない
					continue;
				}
				$this->createIdPGroup($entityid, $idp['scope'], 'idp', $name, $idp['cert']);
			}

			$this->Transaction->commit();
		} catch (Exception $e) {
			$this->Transaction->rollback();
			var_dump($e->getMessage());
			$this->log('IdpGroupCreatorShell ERROR occurred. '.$e->getMessage(), "batch");
		}
		$this->log('IdpGroupCreatorShell end', "batch");
	}

	private function updateGroupName($entityid, $name) {
		// IdP管理者の検索
		$autoGroup = $this->AutoGroup->findIdPGroup($entityid);
		if (empty($autoGroup)) {
			return;
		}

		$this->Group->updateGroupName($autoGroup['AutoGroup']['group_id'], $name);
	}

	private function updateAutoGroup($entityid, $scope, $cert=array()) {
		$autoGroup = $this->AutoGroup->findIdPGroup($entityid);
		if (empty($autoGroup)) {
			return;
		}
		$autoGroup['AutoGroup']['certificate'] = $cert[0];
		$autoGroup['AutoGroup']['certificate2'] = $cert[1];
		$autoGroup['AutoGroup']['scope'] = $scope;
		$this->AutoGroup->create();
		$result = $this->AutoGroup->save($autoGroup, false);
		if (!$result) {
			$this->log('IdpGroupCreatorShell ERROR  AutoGroup Error', "batch");
			$this->log($error, "error");
			return false;
		}
		return true;
	}

	/**
	 * IdPのエンティティID一覧を取得します。
	 */
	private function getIdpEntityIdList($metaPath) {
		$metadata = $this->metadataToXml($metaPath);
		if (!$metadata) {
			$this->log('IdpGroupCreatorShell ERROR  Metadata is empty. ', "batch");
			$this->log('IdpGroupCreatorShell end', "batch");
			return false;
		}
	
		$idpList = array();
		$xmlAsArray = $metadata['xmlAsArray'];
		foreach ($xmlAsArray['EntitiesDescriptor']['EntityDescriptor'] as $entity) {
//debug($entity);
			if (empty($entity['IDPSSODescriptor'])) {
				continue;
			}

			$idpScope = null;
			if (isset($entity['IDPSSODescriptor']['Extensions']['Scope']['value'])) {
				$idpScope = $entity['IDPSSODescriptor']['Extensions']['Scope']['value'];
			}
			$idpName = $entity['entityID'];
			if (isset($entity['IDPSSODescriptor']['Extensions']['UIInfo']['DisplayName']['0'])) {
				$idpNameEn = '';
				$idpNameJa = '';
				foreach ($entity['IDPSSODescriptor']['Extensions']['UIInfo']['DisplayName'] as $displayName) {
					if ($displayName['xml:lang'] === 'en') {
						$idpNameEn = $displayName['value'];
					} elseif ($displayName['xml:lang'] === 'ja')  {
						$idpNameJa = $displayName['value'];
					}
				}
				if (!empty($idpNameEn)) {
					$idpName = $idpNameEn;
				} elseif(!empty($idpNameJa)) {
					$idpName = $idpNameJa;
				} else {
					$idpName = $entity['entityID'];
				}
			}
			$cert = array();
			if (!isset($entity['IDPSSODescriptor']['KeyDescriptor'][1])) {
				$cert[0] = $this->format($entity['IDPSSODescriptor']['KeyDescriptor']['KeyInfo']['X509Data']['X509Certificate']);
				$cert[1] = null;
			} else {
				$cert[0] = $this->format($entity['IDPSSODescriptor']['KeyDescriptor'][0]['KeyInfo']['X509Data']['X509Certificate']);
				$cert[1] = $this->format($entity['IDPSSODescriptor']['KeyDescriptor'][1]['KeyInfo']['X509Data']['X509Certificate']);
			}

			$idp = array();
			$idp['scope'] = $idpScope;
			$idp['name'] = $idpName;
			$idp['entityID'] = $entity['entityID'];
			$idp['cert'] = $cert;
			array_push($idpList, $idp);
		}
		return $idpList;
	}

	private function format($cert) {
		$cert = str_replace(array(" ", "\r\n","\r","\n"), '', $cert);
		return $cert;
	}

	/**
	 * IdPグループの作成
	 * @param unknown $entityid
	 * @param unknown $attributeName
	 * @return boolean
	 */
	private function createIdPGroup($entityid, $scope, $attributeName, $name, $cert=array()) {
		// IdP管理者の検索
		$idpAdminEppnList = $this->IdpAdministrator->find('list', array(
				'fields' => array('eppn', 'eppn'),
				'conditions' => array('entityid' => $entityid)));
		$accountIdList = array();
		if (empty($idpAdminEppnList)) {
			$this->log('IdpGroupCreatorShell INFO  IdP Administrator does not exist', "batch");
		} else {
			$accountIdList = $this->AccountEppn->find('list', array(
					'fields' => array('account_id', 'account_id'),
					'conditions' => array('eppn' => $idpAdminEppnList)));
		}
		if (empty($accountIdList)) {
			// SPコネクタの管理者がいないので、システム管理者がSPコネクタの管理者となる
			$accountIdList = $this->Mygroup->getSystemAdminAccountIdList();
			if (empty($accountIdList)) {
				$this->log('IdpGroupCreatorShell ERROR  System Administrator does not exist', "batch");
				return false;
			}
		}
	
		// group_keyを生成
		$group_key = '';
		for ($i = 0; $i < 20; $i++) {
			$newid = $this->AutoGroupKeySequence->findNextId();
			if (strlen($newid) == 1) {
				$newid = '00'.$newid;
			} elseif (strlen($newid) == 2) {
				$newid = '0'.$newid;
			}
			$group_key = $attributeName.'-'.$newid;
			$count = $this->Group->find('count', array('conditions' => array(
					'group_key' => $group_key, 'name' => $group_key, 'name_en' => $group_key)));
			if ($count == 0) {
				break;
			}
			if ($i == 10) {
				$this->AutoGroupKeySequence->upId();
			}
		}
	
		// Group
		$groupName = $name;
		$group = array(
				'Group' => array(
						'group_key' => $group_key,
						'name' => $groupName,
						'active' => 0,
						'public' => 0,
						'openmember' => 0,
						'inspectjoin' => 2,
						'inspectquit' => 0,
						'independence' => 1,
						'down_permission' => 0,
						'up_permission' => 0,
						'group_response' => 0,
						'sp' => 0,
				),
		);
		$group['Group']['introduction'] = __("message.auto.group.idp.introduction.ja", true);

		$this->Group->create();
		$result = $this->Group->save($group, false);
		if (!$result) {
			$this->log('IdpGroupCreatorShell ERROR  Group Error', "batch");
			return false;
		}
		$groupId = $this->Group->getLastInsertID();
	
		// Mygroup
		foreach ($accountIdList as $accountid) {
			$mygroup = array(
					'Mygroup' => array(
							'account_id' => $accountid,
							'groupid' => $groupId,
							'admin' => 1, // 自動グループは管理者のみの権限とする
					),
			);
			$this->Mygroup->create();
			$result = $this->Mygroup->save($mygroup);
			if (!$result) {
				$this->log('IdpGroupCreatorShell ERROR  Mygroup Error', "batch");
				$this->log($error, "error");
				return false;
			}
		}
	
		// AutoGroup
		$autoGrup = array(
				'AutoGroup' => array(
						'idp' => 'true',
						'entityid' => $entityid,
						'group_id' => $groupId,
						'status' => 'active',
						'certificate' => $cert[0],
						'certificate2' => $cert[1],
						'scope' => $scope,
				),
		);
		$this->AutoGroup->create();
		$result = $this->AutoGroup->save($autoGrup);
		if (!$result) {
			$this->log('IdpGroupCreatorShell ERROR  AutoGroup Error', "batch");
			$this->log($error, "error");
			return false;
		}
	
		return true;
	}

	private function _isExistAutoGroupIdP($entityid) {
		$count = $this->AutoGroup->find('count', array(
				'conditions' => array('entityid' => $entityid, 'idp' => 'true', 'status' => 'active')));
		if ($count >= 1) {
			return true;
		}
		return false;
	}
}
?>
