<?php
/**
 * OrganizationShell
 *
 * PHP versions 5
 *
 * @copyright     Copyright (c) 2011 - 2012, National Institute of Informatics,
 * @license       BSD License (http://creativecommons.org/licenses/BSD/)
 */
App::import('Core', 'Xml');
App::import('Shell', 'Metadata');

/**
 * メタデータから組織名を取得しデータベースに登録する。
 * https://metadata.gakunin.nii.ac.jp/gakunin-metadata.xml から メタデータを取得し、
 *   <OrganizationName xml:lang="en">, <OrganizationName xml:lang="ja">,
 *   <OrganizationDisplayName xml:lang="en">, <OrganizationDisplayName xml:lang="ja">
 *   をorganizationテーブルへ反映します。
 *
 * 実行例）
 * windos)
 *  前提: C:\xampp\phpにPATHを通す
 *  $ cd C:\xampp\htdocs\map\cake\console
 *  $ cake -app C:\xampp\htdocs\map\app organization C:\xampp\htdocs\map\app\tmp\metadata\gakunin-test-metadata.xml
 *
 * linux)
 *  $ export TERM=vt100
 *  $ cd /usr/local/map/cake/console
 *  $ ./cake -app /usr/local/map/app organization /etc/shibboleth/metadata/federation-metadata.xml
 */
class OrganizationShell extends MetadataShell {

	var $uses = array('Organization');

	function main() {
		$this->log('OrganizationShell start', "batch");

		try{
			$metaPath = $this->args[0];
			$metadata = $this->metadataToXml($metaPath);
			if (!$metadata) {
				$this->log('OrganizationShell end', "batch");
				return;
			}
			$validUntil = $metadata['validUntil'];
			$xmlAsArray = $metadata['xmlAsArray'];

			$organizationList = array();
			$i = 0;
			foreach ($xmlAsArray['EntitiesDescriptor']['EntityDescriptor'] as $entity) {
				if (!empty($entity['IDPSSODescriptor'])) {
					if (!isset($entity['Organization']['OrganizationName'])) {
						continue;
					}

					// entityID
					$organizationList[$i]['entityid'] = $entity['entityID'];

					// OrganizationName
					$names = $entity['Organization']['OrganizationName'];
					$organizationList[$i]['name']['ja'] = '';
					$organizationList[$i]['name']['en'] = '';
					if (array_key_exists(0, $names)) {
						foreach($names as $name) {
							if (isset($name['xml:lang']) && $name['xml:lang'] == 'ja') {
								$organizationList[$i]['name']['ja'] = $name['value'];
							} else if (isset($name['xml:lang']) && $name['xml:lang'] == 'en') {
								$organizationList[$i]['name']['en'] = $name['value'];
							}
						}
					} else {
						if (isset($names['xml:lang']) && $names['xml:lang'] == 'ja') {
							$organizationList[$i]['name']['ja'] = $names['value'];
						} else if (isset($names['xml:lang']) && $names['xml:lang'] == 'en') {
							$organizationList[$i]['name']['en'] = $names['value'];
						}
					}

					// OrganizationDisplayName
					$names = $entity['Organization']['OrganizationDisplayName'];
					$organizationList[$i]['display']['ja'] = '';
					$organizationList[$i]['display']['en'] = '';
					if (array_key_exists(0, $names)) {
						foreach($names as $name) {
							if (isset($name['xml:lang']) && $name['xml:lang'] == 'ja') {
								$organizationList[$i]['display']['ja'] = $name['value'];
							} else if (isset($name['xml:lang']) && $name['xml:lang'] == 'en') {
								$organizationList[$i]['display']['en'] = $name['value'];
							}
						}
					} else {
						if (isset($names['xml:lang']) && $names['xml:lang'] == 'ja') {
							$organizationList[$i]['display']['ja'] = $names['value'];
						} else if (isset($names['xml:lang']) && $names['xml:lang'] == 'en') {
							$organizationList[$i]['display']['en'] = $names['value'];
						}
					}

					$i++;
				}
			}

			$saveNameList = array();
			foreach ($organizationList as $organization) {
				if (empty($organization['name']['ja']) && empty($organization['name']['en'])
					&& empty($organization['display']['ja']) && empty($organization['display']['en'] )) {
					$this->log('WARNING  skipped registring ' . $organization['entityid'], "batch");
					continue;
				}
				$result = $this->Organization->find('first', array(
					'conditions' => array('entityid' => $organization['entityid'])));

				$entity['Organization']['id'] = $result['Organization']['id'];
				$entity['Organization']['name'] = $organization['name']['ja'];
				$entity['Organization']['name_en'] = $organization['name']['en'];
				$entity['Organization']['display_name'] = $organization['display']['ja'];
				$entity['Organization']['display_name_en'] = $organization['display']['en'];
				$entity['Organization']['entityid'] = $organization['entityid'];
				$entity['Organization']['valid_util'] = $validUntil;
				$entity['Organization']['federation'] = 1;

				$this->Organization->create();
				$this->Organization->save($entity);
			}
		} catch (Exception $e) {
			$this->log('OrganizationShell ERROR occurred. '.$e->getMessage(), "batch");
		}
		$this->log('OrganizationShell end', "batch");
	}
}
?>