DROP TABLE IF EXISTS `sequence`;
CREATE TABLE `sequence` (`id` INT NOT NULL);
INSERT INTO `sequence` VALUES (0);

DROP TABLE IF EXISTS `tmp_ismemberof`;
CREATE TABLE `tmp_ismemberof` (
  `id` INT UNSIGNED NOT NULL,
  `aq_id` INT UNSIGNED NOT NULL,
  `groupid` INT UNSIGNED NOT NULL,
  `count` SMALLINT UNSIGNED NOT NULL,
  `name` VARCHAR(200) DEFAULT NULL,
  `sp` INT(1) DEFAULT NULL,
  `valid` INT(1) DEFAULT 0,
  `created` DATETIME DEFAULT NULL,
  PRIMARY KEY  (`id`, `count`)
) ENGINE=InnoDB DEFAULT CHARSET=utf8;

DROP TABLE IF EXISTS `tmp_ismemberof_groupid`;
CREATE TABLE `tmp_ismemberof_groupid` (
  `id` INT UNSIGNED NOT NULL,
  `groupid` INT UNSIGNED NOT NULL,
  `created` DATETIME DEFAULT NULL
) ENGINE=InnoDB DEFAULT CHARSET=utf8;



DROP PROCEDURE IF EXISTS getIsMemberOf;

DELIMITER //
CREATE PROCEDURE getIsMemberOf(IN in_principalname VARCHAR(256), IN in_entityid VARCHAR(256), IN in_prefix VARCHAR(100))
BEGIN
    DECLARE eod INT;
    DECLARE myId INT;
    DECLARE myName VARCHAR(100);
    DECLARE myResponse INT;
    DECLARE mySp INT;
    DECLARE myActive INT;
    DECLARE myTopId INT;
    DECLARE myBottomId INT;
    DECLARE myLastLoginApprox DATETIME;
    DECLARE myDiffDate INT;
    DECLARE myUpdateLogin INT;
    DECLARE myNowDate DATETIME;
    DECLARE myEppnId INT;

    DECLARE cur_addgroups CURSOR FOR
        /* sp- uni- uni(belong) */
        SELECT top_id, bottom_id
          FROM add_groups
         WHERE bottom_id IN (SELECT top_id
                               FROM add_groups
                              WHERE top = 0
                                AND bottom_id IN (SELECT groupid FROM mygroups WHERE account_id = @accountId))
           AND bottom_id NOT IN (SELECT group_id FROM auto_groups WHERE idp = 'true')
           AND (SELECT COUNT(*) FROM sp_hosts
                               WHERE id IN (SELECT sp_id FROM group_sphosts WHERE group_id = top_id)
                                 AND entityid = in_entityid) > 0
         UNION
        /* sp - uni(belong) */
        SELECT top_id, bottom_id
          FROM add_groups
         WHERE top = 1
           AND bottom_id IN (SELECT groupid FROM mygroups WHERE account_id = @accountId)
           AND bottom_id NOT IN (SELECT group_id FROM auto_groups WHERE idp = 'true')
           AND (SELECT COUNT(*) FROM sp_hosts
                               WHERE id IN (SELECT sp_id FROM group_sphosts WHERE group_id = top_id)
                                 AND entityid = in_entityid) > 0;

    DECLARE cur_groups CURSOR FOR
        SELECT id, group_key, group_response, sp, active
          FROM groups
         WHERE id IN (SELECT groupid FROM tmp_ismemberof_groupid WHERE id = @seq_ag)
           AND active = 0;

    DECLARE CONTINUE HANDLER FOR NOT FOUND SET eod = 1;

    /* idを生成 */
    UPDATE sequence SET id = LAST_INSERT_ID(id+1);
    SELECT LAST_INSERT_ID() INTO @seq_ag;

    /* アカウントIDを検索 */
    SET @accountId = 0;
    SET myLastLoginApprox = NULL;
    SET myEppnId = 0;
    SELECT account_id,last_login_approx,id INTO @accountId,myLastLoginApprox,myEppnId FROM account_eppns WHERE eppn = in_principalname OR eptid = in_principalname;

    /* ユーザの最終ログイン日時を記録する */
    IF myEppnId > 0 THEN
        SET myUpdateLogin = 1;
        SET myNowDate = CONVERT_TZ(NOW(),'GMT','+9:00');
        SET myDiffDate = DATEDIFF(myNowDate,myLastLoginApprox);
        IF myDiffDate <= 7 THEN
            SET myUpdateLogin = 0;
        END IF;
        /* ログイン日が1週間より前なら更新 */
        IF myUpdateLogin = 1 THEN
            UPDATE account_eppns SET last_login_approx = myNowDate WHERE id = myEppnId;
        END IF;
    END IF;

    SET eod = 0;

    OPEN cur_addgroups;
    loop_addgroups: LOOP
        FETCH cur_addgroups INTO myTopId, myBottomId;
        IF eod = 1 THEN
            LEAVE loop_addgroups;
        END IF;

        INSERT INTO tmp_ismemberof_groupid(id, groupid, created) VALUES (@seq_ag, myTopId, NOW());
        INSERT INTO tmp_ismemberof_groupid(id, groupid, created) VALUES (@seq_ag, myBottomId, NOW());
    END LOOP;
    CLOSE cur_addgroups;

    /* idを生成 */
    UPDATE sequence SET id = LAST_INSERT_ID(id+1);
    SELECT LAST_INSERT_ID() INTO @seq;

    SET eod = 0;
    SET @prefix = in_prefix;
    SET @response = 0;
    SET @count = 0;

    OPEN cur_groups;
    loop_groups: LOOP
        FETCH cur_groups INTO myId, myName, myResponse, mySp, myActive;
        IF eod = 1 THEN
            LEAVE loop_groups;
        END IF;

        /* SPコネクタgroup_responseを取得 */
        IF mySp = 1 THEN
            IF myResponse > @response THEN
              SET @response = myResponse;
            END IF;
        END IF;
        /* グループ名を取得 */
        IF mySp = 1 THEN
            INSERT INTO tmp_ismemberof(id, groupid, count, name, sp, created) VALUES (@seq, 0, @count, CONCAT(@prefix, 'sp/', myName), 1, NOW());
        ELSEIF mySp = 0 THEN
            /* 管理者／メンバー判定 */
            SELECT COUNT(*) INTO @adminCount FROM mygroups WHERE account_id = @accountId AND groupid = myId AND admin = 1;
            SELECT COUNT(*) INTO @bottomGroupsMemberCount FROM mygroups WHERE account_id = @accountId
                                                                          AND groupid IN (SELECT bottom_id FROM add_groups WHERE top_id = myId AND top = 0)
                                                                          AND admin IN (0, 2);
            SELECT COUNT(*) INTO @adminMemberCount FROM mygroups WHERE account_id = @accountId AND groupid = myId AND admin = 2;
            IF @adminCount > 0 THEN
                INSERT INTO tmp_ismemberof(id, groupid, count, name, sp, created) VALUES (@seq, myId, @count, CONCAT(@prefix, 'gr/', myName, '/admin'), 0, NOW());
                IF @bottomGroupsMemberCount > 0 THEN
                    SET @count = @count + 1;
                    INSERT INTO tmp_ismemberof(id, groupid, count, name, sp, created) VALUES (@seq, myId, @count, CONCAT(@prefix, 'gr/', myName), 0, NOW());
                END IF;
            ELSE
                IF @adminMemberCount > 0 THEN
                    INSERT INTO tmp_ismemberof(id, groupid, count, name, sp, created) VALUES (@seq, myId, @count, CONCAT(@prefix, 'gr/', myName), 0, NOW());
                    SET @count = @count + 1;
                    INSERT INTO tmp_ismemberof(id, groupid, count, name, sp, created) VALUES (@seq, myId, @count, CONCAT(@prefix, 'gr/', myName, '/admin'), 0, NOW());
                ELSE
                    INSERT INTO tmp_ismemberof(id, groupid, count, name, sp, created) VALUES (@seq, myId, @count, CONCAT(@prefix, 'gr/', myName), 0, NOW());
                END IF;
            END IF;
        END IF;

        SET @count = @count + 1;
    END LOOP;
    CLOSE cur_groups;

    /* 要求があったSPを検索 */
    SET @sphostId = 0;
    SELECT id INTO @sphostId FROM sp_hosts WHERE entityid = in_entityid;

    /* グループのみ */
    IF @response = 0 THEN
        SELECT name FROM tmp_ismemberof
                   WHERE id = @seq
                     AND sp IN (0)
                     AND groupid IN (SELECT group_id FROM agreement_ismemberofs
                                                    WHERE account_id = @accountId
                                                      AND spc_id IN (SELECT group_id FROM group_sphosts WHERE sp_id = @sphostId));
    /* SPコネクタのみ  groupid=0:SPコネクタ */
    ELSEIF @response = 1 THEN
        SELECT name FROM tmp_ismemberof WHERE id = @seq AND sp IN (1);
    /* グループ、SPコネクタの両方 groupid=0:SPコネクタ*/
    ELSEIF @response = 2 THEN
        SELECT name FROM tmp_ismemberof
                   WHERE id = @seq
                     AND sp IN (0, 1)
                     AND (groupid = 0 OR groupid IN (SELECT group_id FROM agreement_ismemberofs
                                                                    WHERE account_id = @accountId
                                                                      AND spc_id IN (SELECT group_id FROM group_sphosts WHERE sp_id = @sphostId)));
    END IF;

    DELETE FROM tmp_ismemberof_groupid WHERE id = @seq_ag;
    DELETE FROM tmp_ismemberof WHERE id = @seq;
END
//
DELIMITER ;



DROP PROCEDURE IF EXISTS getIsMemberOfByEmailAddress;

DELIMITER //
CREATE PROCEDURE getIsMemberOfByEmailAddress(IN in_principalname VARCHAR(256), IN in_entityid VARCHAR(256), IN in_prefix VARCHAR(100))
BEGIN
    DECLARE eod INT;
    DECLARE myId INT;
    DECLARE myName VARCHAR(100);
    DECLARE myResponse INT;
    DECLARE mySp INT;
    DECLARE myActive INT;
    DECLARE myTopId INT;
    DECLARE myBottomId INT;
    DECLARE myAccountId INT;

    DECLARE cur_addgroups CURSOR FOR
        /* sp- uni- uni(belong) */
        SELECT top_id, bottom_id
          FROM add_groups
         WHERE bottom_id IN (SELECT top_id
                               FROM add_groups
                              WHERE top = 0
                                AND bottom_id IN (SELECT groupid FROM mygroups WHERE account_id = @accountId AND admin = 0))
           AND bottom_id NOT IN (SELECT group_id FROM auto_groups WHERE idp = 'true')
           AND (SELECT COUNT(*) FROM sp_hosts
                               WHERE id IN (SELECT sp_id FROM group_sphosts WHERE group_id = top_id)
                                 AND entityid = in_entityid) > 0
         UNION
        /* sp - uni(belong) */
        SELECT top_id, bottom_id
          FROM add_groups
         WHERE top = 1
           AND bottom_id IN (SELECT groupid FROM mygroups WHERE account_id = @accountId AND admin = 0)
           AND bottom_id NOT IN (SELECT group_id FROM auto_groups WHERE idp = 'true')
           AND (SELECT COUNT(*) FROM sp_hosts
                               WHERE id IN (SELECT sp_id FROM group_sphosts WHERE group_id = top_id)
                                 AND entityid = in_entityid) > 0;

    DECLARE cur_groups CURSOR FOR
        SELECT id, group_key, group_response, sp, active
          FROM groups
         WHERE id IN (SELECT groupid FROM tmp_ismemberof_groupid WHERE id = @seq_ag)
           AND active = 0;

    DECLARE cur_tmpaccounts CURSOR FOR
        SELECT accounts.id
          FROM accounts
          JOIN invites
            ON accounts.id = invites.temp_account_id
         WHERE (accounts.is_temp IN (1, 3) AND accounts.mail = in_principalname)
            OR (accounts.is_temp = 2       AND accounts.mail = in_principalname
                                           AND invites.invited_account_id IN (SELECT id
                                                                                FROM accounts
                                                                               WHERE is_temp = 0
                                                                                 AND arrival = 1 AND mail = in_principalname));

    DECLARE CONTINUE HANDLER FOR NOT FOUND SET eod = 1;

    /* idを生成 */
    UPDATE sequence SET id = LAST_INSERT_ID(id+1);
    SELECT LAST_INSERT_ID() INTO @seq_aq;

    UPDATE sequence SET id = LAST_INSERT_ID(id+1);
    SELECT LAST_INSERT_ID() INTO @seq_ag;

    SET eod = 0;
    OPEN cur_tmpaccounts;
    loop_tmpaccounts: LOOP
        FETCH cur_tmpaccounts INTO myAccountId;
        IF eod = 1 THEN
            LEAVE loop_tmpaccounts;
        END IF;

        SET @accountId = myAccountId;

        OPEN cur_addgroups;
        loop_addgroups: LOOP
            FETCH cur_addgroups INTO myTopId, myBottomId;
            IF eod = 1 THEN
                LEAVE loop_addgroups;
            END IF;

            INSERT INTO tmp_ismemberof_groupid(id, groupid, created) VALUES (@seq_ag, myTopId, NOW());
            INSERT INTO tmp_ismemberof_groupid(id, groupid, created) VALUES (@seq_ag, myBottomId, NOW());
        END LOOP;
        CLOSE cur_addgroups;

        /* idを生成 */
        UPDATE sequence SET id = LAST_INSERT_ID(id+1);
        SELECT LAST_INSERT_ID() INTO @seq;

        SET eod = 0;
        SET @prefix = in_prefix;
        SET @response = 0;
        SET @count = 0;

        OPEN cur_groups;
        loop_groups: LOOP
            FETCH cur_groups INTO myId, myName, myResponse, mySp, myActive;
            IF eod = 1 THEN
                LEAVE loop_groups;
            END IF;

            /* SPコネクタgroup_responseを取得 */
            IF mySp = 1 THEN
                IF myResponse > @response THEN
                  SET @response = myResponse;
                END IF;
            END IF;
            /* グループ名を取得 */
            IF mySp = 1 THEN
                INSERT INTO tmp_ismemberof(id, aq_id, groupid, count, name, sp, created) VALUES (@seq, @seq_aq, 0, @count, CONCAT(@prefix, 'sp/', myName), 1, NOW());
            ELSEIF mySp = 0 THEN
                INSERT INTO tmp_ismemberof(id, aq_id, groupid, count, name, sp, created) VALUES (@seq, @seq_aq, myId, @count, CONCAT(@prefix, 'gr/', myName), 0, NOW());
            END IF;

            SET @count = @count + 1;
        END LOOP;
        CLOSE cur_groups;

        /* 要求があったSPを検索 */
        SET @sphostId = 0;
        SELECT id INTO @sphostId FROM sp_hosts WHERE entityid = in_entityid;

        /* グループのみ */
        IF @response = 0 THEN
            UPDATE tmp_ismemberof SET valid = 1
                                WHERE id = @seq
                                  AND aq_id = @seq_aq
                                  AND sp IN (0)
                                  AND groupid IN (SELECT group_id FROM agreement_ismemberofs
                                                                 WHERE account_id = @accountId
                                                                   AND spc_id IN (SELECT group_id FROM group_sphosts WHERE sp_id = @sphostId));
        /* SPコネクタのみ  groupid=0:SPコネクタ */
        ELSEIF @response = 1 THEN
            UPDATE tmp_ismemberof SET valid = 1
                                WHERE id = @seq
                                  AND aq_id = @seq_aq
                                  AND sp IN (1);
        /* グループ、SPコネクタの両方 groupid=0:SPコネクタ*/
        ELSEIF @response = 2 THEN
            UPDATE tmp_ismemberof SET valid = 1
                                WHERE id = @seq
                                  AND aq_id = @seq_aq
                                  AND sp IN (0, 1)
                                  AND (groupid = 0 OR groupid IN (SELECT group_id FROM agreement_ismemberofs
                                                                                 WHERE account_id = @accountId
                                                                                   AND spc_id IN (SELECT group_id FROM group_sphosts WHERE sp_id = @sphostId)));
        END IF;

        DELETE FROM tmp_ismemberof_groupid WHERE id = @seq_ag;

        SET eod = 0;
    END LOOP;

    SELECT DISTINCT name FROM tmp_ismemberof WHERE aq_id = @seq_aq AND valid = 1;
    DELETE FROM tmp_ismemberof WHERE aq_id = @seq_aq;
END
//
DELIMITER ;

EXIT
